package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._

/** A one step  (x1,x2,x3,...)_l -> y_l distributin regression.
  * @version $Id: DistributionRegression1Step.scala,v 1.5 2015/11/28 08:07:19 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
class DistributionRegression1Step(val dim:Int,val sample:Array[OneYObservationPoint],val Q:OrthogonalPolynomialsABasis)
{
  val w=new Array[Array[Double]](sample.length)
  val N=sample(0).x.length
;  for(l <- 0 until sample.length) {
    w(l)=Q.B.calculateMomentsFromSample(dim-1,sample(l).x)
    if(sample(l).x.length!=N) throw new RuntimeException("l="+l+" N="+N+" sample(l).x.length="+sample(l).x.length)
  }

  val yQQ=new Array[Double](dim*dim)
  val QQ=new Array[Double](dim*dim)
  val momY=new Array[Double](dim)

;  for(j <- 0 until dim)
  {
    for(k <- 0 until dim )
    {
      var s=0.0;
      var sy=0.0;
      for(l <- 0 until w.length)
      {
        val v=w(l)(j)*w(l)(k)
        s+=v
        sy+=v*sample(l).y
      }
      yQQ(j*dim+k)=sy
      QQ(j*dim+k)=s
    }
    var sy=0.0;
    for(l <- 0 until w.length)
    {
      sy+=w(l)(j)*sample(l).y
    }
    momY(j)=sy
  }

  val ev=Q.EV.getGEV_Generalized(dim,yQQ,dim,QQ,dim,true);
  val IQQ=Linsystems.getInvertedMatrix(dim,dim,QQ)

  def getYLSQ(x:Double)={
    val mx0=Q.getX0Moments(dim,x).map(_ * N)
    TMatr.getAMatrB(mx0,momY,IQQ,dim)
  }
  def getYRN(x:Double)={
    val mx0=Q.getX0Moments(dim,x).map(_ * N)
    TMatr.getAMatrA(TMatr.getMatr_x_A(dim,IQQ,dim,mx0),yQQ,dim)/
    TMatr.getAMatrA(mx0,IQQ,dim)
  }

  def getWno2(x:Double)={
    val mx0=Q.getX0Moments(dim,x).map(_ * N)
    val w=new Array[Double](dim)
    for(k <- 0 until dim)
    {
      w(k)=TMatr.mulVV(dim,mx0,0,ev,dim+k*dim)
    }
    //System.err.println("at x="+x+" Q(x0)G^-1 Q(x0)="+TMatr.getAMatrA(mx0,IQQ,dim))
    w
  }
  def getYkW(x:Double)={
    val yk=new Array[Double](dim)
    val w=getWno2(x)
    for(k <- 0 until dim) {
      yk(k)=ev(k)
      w(k)=Math.pow(w(k),2)
    }
    //System.err.println("at x="+x+" sum wk="+w.sum)
    (yk,w)
  }
  def getYkP(x:Double)={
    val(yk,w)=getYkW(x)
    (yk,w.map(_ / w.sum))
  }

  def getYkP(x:Array[Double])={
    val r=Array.ofDim[Double](1+x.length,dim)
    val mx=x.map( (a)=>Q.getX0Moments(dim,a).map(_ * N))
    for(k <- 0 until dim)
    {
      r(0)(k)=ev(k)
      for( m <- 1 to x.length)
      {
        r(m)(k)=Math.pow(TMatr.mulVV(dim,mx(m-1),0,ev,dim+k*dim),2)
      }
    }
    for( m <- 1 to x.length)
    {
      System.err.println("x0="+x(m-1)+" Q(x0)G^-1 Q(x0)="+TMatr.getAMatrA(mx(m-1),IQQ,dim)+" sum wk="+r(m).sum)
      r(m)=r(m).map( _ / r(m).sum)
    }
    r
  }
}

