package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.itch.*;
import com.polytechnik.utils.*;

/** A class to output some data.
    java -server -Xmx6500000K com/polytechnik/algorithms/ExampleBookDisbalance /rdata/itch/itch4/S012309-v4.txt.gz AAPL
 *  @version $Id: ExampleBookDisbalance.java,v 1.14 2016/03/17 00:35:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ExampleBookDisbalance extends DumpData2Trader {

ExampleBookDisbalance(final String [] tickerstowatch,final ProcessTick tickprocessor){
    super(tickerstowatch,tickprocessor);
}
static final double BOOK_EDGE_PRICE_CUT=1.0;
static final boolean SKIP_RADON_NIKODYM=false;
static final double TAUTIMESCALE=TraderInterface.TIME_SCALE;
static final OrthogonalPolynomialsABasis M=new OrthogonalPolynomialsChebyshevBasis()/* new OrthogonalPolynomialsLegendreBasis()*/;
    
static class CalculateBookEdgeBasics {
    long vol=0;
    double price=Double.NaN,tau=Double.NaN,p_average=Double.NaN,sigma=Double.NaN;
    void calculate(final long currenttime,final int no,final BookItem [] orders)
    {
	double p=Double.NaN,pv_tot=0;
	long v=0,tv=0,vtot=0;
	for(int k=0;k<no;k++)
	{
	    final BookItem o=orders[k];
	    if(k==0 || p==o.price){p=o.price;v+=o.shares;tv+=(currenttime-o.time)*o.shares;}
	    vtot+=o.shares;
	    pv_tot+=o.price*o.shares;
	}
	this.vol=v;
	this.price=p;
	this.tau=(tv/TAUTIMESCALE)/v;
	this.p_average=pv_tot/vtot;
	double s=0;
	for(int k=0;k<no;k++)
	{
	    final BookItem o=orders[k];
	    s+=Math.abs(o.price-this.p_average)*o.shares;
	}
	this.sigma=s/vtot;
    }
    double convertPriceToBasis(final double p)
    {
	return (p-p_average)/sigma;
    }
    double convertPriceFromBasis(final double pb)
    {
	return pb*sigma+p_average;
    }
} // END OF class CalculateBookEdgeBasics


static class CalculateBookEdge {
    final CalculateBookEdgeBasics priceconverter=new CalculateBookEdgeBasics();
    final int DIM;
    CalculateBookEdge(final int DIM)
    {
	this.DIM=DIM;
    }
    double vedge=Double.NaN,tauedge=Double.NaN,dvdp=Double.NaN,dvdp_A=Double.NaN;
    double [] psample=new double [0],wsample=new double [0],tauwsample=new double [0];
    void calculate(final long currenttime,final int no,final BookItem [] orders)
    {
	if(orders.length>psample.length)
	{
	    psample=new double [orders.length];
	    wsample=new double [orders.length];
	    tauwsample=new double [orders.length];
	}
	priceconverter.calculate(currenttime,no,orders);
	for(int k=0;k<no;k++)
	{
	    final BookItem o=orders[k];
	    psample[k]=priceconverter.convertPriceToBasis(o.price);
	    wsample[k]=o.shares;
	    tauwsample[k]=o.shares*(currenttime-o.time)/TAUTIMESCALE;
	}
	final double bpedge=priceconverter.convertPriceToBasis(priceconverter.price),
	    pend=no>0?orders[no-1].price:Double.NaN,
	    bpend=priceconverter.convertPriceToBasis(pend);
	final double [] pmoments=M.B.calculateMomentsFromSample(2*DIM-1,psample,wsample,no),
	    taupmoments=M.B.calculateMomentsFromSample(2*DIM-1,psample,tauwsample,no);
	final double [] QQp=M.getQQMatr(DIM,pmoments),QQtaup=M.getQQMatr(DIM,taupmoments);
	final double [] pedge=M.getX0Moments(DIM,bpedge);
	final double [] QQmatr=QQp.clone(),xpedge=pedge.clone();
	final int [] ipvt=new int [DIM];
	final int pinfo=Linsystems.dgefa_m1(QQmatr,DIM,DIM,ipvt);
	if(pinfo!=0)
	{
	    java.util.Arrays.fill(xpedge,Double.NaN);
	}
	else
	{
	    Linsystems.dgesl_m1(QQmatr,DIM,DIM,ipvt,xpedge,1);
	}
	this.vedge=1/TMatr.mulVV(xpedge,pedge);
	this.tauedge=TMatr.getAMatrA(xpedge,QQtaup,DIM)*this.vedge;

	final double [] ppmoments=new double [2*DIM-1];
	for(int k=0;k<ppmoments.length;k++)
	{
	    final double [] x=new double [k+1];
	    x[k]=1;
	    final double [] intgr=M.B.getIntegrated(k,x);
	    ppmoments[k]=M.fx(intgr,bpend)-M.fx(intgr,bpedge);
	}
	// to have real "dp", not "scaled".
	TMatr.scaleVector(ppmoments,priceconverter.sigma);
	if(bpedge>bpend) TMatr.scaleVector(ppmoments,-1);
	final double [] QQpp=M.getQQMatr(DIM,ppmoments);
	final double [] QQppmatr=QQpp.clone(),xppedge=pedge.clone();
	final int ppinfo=Linsystems.dgefa_m1(QQppmatr,DIM,DIM,ipvt);
	if(ppinfo!=0)
	{
	    java.util.Arrays.fill(xppedge,Double.NaN);
	}
	else
	{
	    Linsystems.dgesl_m1(QQppmatr,DIM,DIM,ipvt,xppedge,1);
	}
	this.dvdp=TMatr.getAMatrA(xppedge,QQp,DIM)/TMatr.getAMatrA(xppedge,QQpp,DIM);
	this.dvdp_A=TMatr.getAMatrA(xpedge,QQp,DIM)/TMatr.getAMatrA(xpedge,QQpp,DIM);

	// ========= TEST =================
	if(false)
	{ // Test matching with Radau.
	    final double [] pp=TMatr.sumAcB(pmoments.length-1,M.getXFMomentsFromMoments(pmoments),-bpedge,pmoments);
	    if(bpedge>bpend) TMatr.scaleVector(pp,-1);
	    final double [][] qq=M.getQuadraturesFromMoments_EV(DIM-1,pp);
	    final double [] nodes=qq[2];
	    final double vedge_from_Radau=TMatr.mulVV(nodes,pmoments)/M.fx(nodes,bpedge);
	    System.err.println("no="+no+" this.vedge="+this.vedge+" vedge_from_Radau="+vedge_from_Radau+" verge_from_QQ="+(1/TMatr.getAMatrA(xpedge,QQp,DIM)));
	    System.err.println("ppmoments[0]="+ppmoments[0]+" pedge-pend="+(priceconverter.price-pend));
	}
	// ========= END TEST =================
    }
} // END OF class CalculateBookEdge

/** Print data.
 */
public static class PrintBookEdges implements ProcessTick {

final CalculateBookEdgeBasics cbuy=new CalculateBookEdgeBasics(),csell=new CalculateBookEdgeBasics();
final CalculateBookEdge rbuy,rsell;
PrintBookEdges(final int DIM){
    this.rbuy=new CalculateBookEdge(DIM);
    this.rsell=new CalculateBookEdge(DIM);
}
protected long n_lines=0;
static int getIndexPriceCut(final int no,final BookItem [] orders,final double price_cut,final int buy_sell_indicator)
{
    for(int k=0;k<no;k++)
    {
	final BookItem o=orders[k];
	if(buy_sell_indicator==BookItem.BOOK_BUY)
	{
	    if(!(o.price>=price_cut)) return k;
	}
	else if(buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    if(!(o.price<=price_cut)) return k;
	}
	else throw new RuntimeException("buy_sell_indicator="+buy_sell_indicator);
    }
    return no;
}

public void processData(final OrdersAndTrades ot,final long currenttime,final int eventid,
			final int no_buy,final BookItem [] buy,
			final int no_sell,final BookItem [] sell,
			final int no_trades,final Trade [] trades)
{
    // OK, now we have 3 arrays:
    // trades, no_trades elements: executed trades
    // buy, no_buy elements: buy side of the book
    // sell, no_sell elements: sell side of the book}
    
    cbuy.calculate(currenttime,no_buy,buy);
    if(!SKIP_RADON_NIKODYM) rbuy.calculate(currenttime,getIndexPriceCut(no_buy,buy,cbuy.price-BOOK_EDGE_PRICE_CUT,BookItem.BOOK_BUY),buy);
    csell.calculate(currenttime,no_sell,sell);
    if(!SKIP_RADON_NIKODYM) rsell.calculate(currenttime,getIndexPriceCut(no_sell,sell,csell.price+BOOK_EDGE_PRICE_CUT,BookItem.BOOK_SELL),sell);
    Trade t=null;
    // skip non-trade events, they would give duplicated trade records.
    if(eventid==TraderInterface.UPD_EXECUTIONORDER_TRADE || eventid==TraderInterface.UPD_EXECUTIONOTH_TRADE)
    {
	// Last trade. if printing for all eventid get possible duplicates.
	t=no_trades>0?trades[no_trades-1]:null;
    }

    if(n_lines<=0)
    {
	final String h=
	    "|ticker\tcurrenttime\teventid\texe_time\texe_price_last\texe_shares\texe_matchnumber\texe_buysellindicator\tbook_p_best_buy\tbook_v_best_buy\tbook_tau_best_buy\tbook_p_best_sell\tbook_v_best_sell\tbook_tau_best_sell"+
	    (!SKIP_RADON_NIKODYM?
	     "\tvedge_buy\ttauedge_buy\tdvdp_buy\tdvdp_A_buy\tvedge_sell\ttauedge_sell\tdvdp_sell\tdvdp_A_sell":
	     "")+
	    "";
	int cn=0;
	for(String sh:h.split("\t"))
	{
	    System.out.print(sh+":"+(++cn)+"\t");
	}
	System.out.println();
    }
    n_lines++;

    // print data
    System.out.println(ot.ticker+"\t"+currenttime+"\t"+eventid+"\t"+(t!=null?t.time:0)+
		       "\t"+(t!=null?t.price:(no_trades>0?trades[no_trades-1].price:Double.NaN))+"\t"+(t!=null?t.shares:0)+
		       "\t"+(t!=null?t.matchnumber:0)+
		       "\t"+(t!=null?t.buy_sell_indicator:BookItem.BOOK_UNKNOWN)+
		       "\t"+cbuy.price+"\t"+cbuy.vol+"\t"+cbuy.tau+
		       "\t"+csell.price+"\t"+csell.vol+"\t"+csell.tau+
		       (!SKIP_RADON_NIKODYM?
			"\t"+rbuy.vedge+"\t"+rbuy.tauedge+"\t"+rbuy.dvdp+"\t"+rbuy.dvdp_A+
			"\t"+rsell.vedge+"\t"+rsell.tauedge+"\t"+rsell.dvdp+"\t"+rsell.dvdp_A
			:"")+
		       "\t");
}
} // END OF class PrintBookEdges

public static void main(String [] args)	throws Exception
{
    if(args.length!=2)
    {
	System.err.println("ExampleBookDisbalance itch_file ticker");
    }
    else
    {
	final int DIM=7;
	final long t0=System.currentTimeMillis();
	final String fname=args[0];
	final String [] tickers=new String []{args[1]};
	final PrintBookEdges tickprocessor=new PrintBookEdges(DIM);
	new ExampleBookDisbalance(tickers,tickprocessor).runParser(fname);
	System.err.println("Processed file "+fname+" saved "+tickprocessor.n_lines+
			   " lines to stdout for "+java.util.Arrays.toString(tickers)+
			   ". Finished in "+((System.currentTimeMillis()-t0)/1000L)+
			   " seconds. by $Id: ExampleBookDisbalance.java,v 1.14 2016/03/17 00:35:57 mal Exp $");
    }
}
} // END OF class ExampleBookDisbalance
