package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._

/** A simple test (x1,x2,x3,...)_l -> y_l distribution dependence.
  * @version $Id: ExampleDistributionDependence.scala,v 1.42 2015/11/27 13:08:08 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  * A library from http://www.ioffe.ru/LNEPS/malyshkin/code_polynomials_quadratures.zip is required.
  */
class ExampleDistributionDependence(val dimX:Int,val dimY:Int,
  val RANDOMNESS:Double,
  val Q:OrthogonalPolynomialsABasis) {
    

  class XScaler(val obsr: Array[Double],val flag_donotscale:Boolean){
    val(minX,maxX)={
      var minx=Double.NaN
      var maxx=Double.NaN
      for(ox <- obsr)
      {
        if(!(ox>minx)) minx=ox
        if(!(ox<maxx)) maxx=ox
      }
      //System.err.println("minx="+minx+" maxx="+maxx)
      (minx,maxx)
    }
    val MIN= -1.0
    val MAX=  1.0
    def getXScaled(x:Double)= if(flag_donotscale) x else (MAX-MIN)*(x-minX)/(maxX-minX)+MIN
    def getScaledObs()={
      val o=new Array[Double](obsr.length);
      for(j <- 0 until obsr.length) o(j)=getXScaled(obsr(j))
      o
    }
    def getXScaledBack(xscaled:Double)=if(flag_donotscale) xscaled else (maxX-minX)*(xscaled-MIN)/(MAX-MIN)+minX
  }


  class OneWithWeight(val y:Double,val dim:Int,obsr: Array[Double],wei: Array[Double],val flag_donotscale:Boolean){
    val NinKxx=obsr.length;
    val scaler=new XScaler(obsr,flag_donotscale)
    val moments=Q.B.calculateMomentsFromSample(2*dim-1-1+1/* +1 for quadrature */,scaler.getScaledObs,wei)
    val IQQ=Linsystems.getInvertedMatrix(dim,dim,Q.getQQMatr(dim,moments))
    def getW_fromQQ(x:Double)=TMatr.getAMatrA(Q.getX0Moments(dim,scaler.getXScaled(x)),IQQ,dim)
    val K=Q.getKK(dim,IQQ,dim)
    def getW_fromF(x:Double)=Q.fx(K,scaler.getXScaled(x))
    def getW(x:Double)=getW_fromQQ(x)
  }

  /** The method calculates K(x,x) from x observations assuming the weights=1.
    */
  def getOneYWFromObservation(o: OneYObservationPoint):OneWithWeight={
    if(o.x.length<dimX) throw new RuntimeException("o.x.length="+o.x.length+"<dimX="+dimX)
    ;
    /*
    {
      val xxS=new OneWithWeight(o.y,dimX,o.x,null,true)
      val xxN=new OneWithWeight(o.y,dimX,o.x,null,false)
      System.err.println("minx="+xxS.scaler.minX+" maxx="+xxS.scaler.maxX+"\txxS.getW("+(0.5*(xxS.scaler.minX+xxS.scaler.maxX))+")="+xxS.getW(0.5*(xxS.scaler.minX+xxS.scaler.maxX))+
        "\txxN.getW("+(0.5*(xxN.scaler.minX+xxN.scaler.maxX))+")="+xxN.getW(0.5*(xxN.scaler.minX+xxN.scaler.maxX)))
    }
     */
    new OneWithWeight(o.y,dimX,o.x,null,false)
  }

   /** For each y-observation the method calculates K(x,x) from x observations.
    */
  def getObserwationsWithK(o: Array[OneYObservationPoint]):Array[OneWithWeight]={
    val ow=new Array[OneWithWeight](o.length)
    for(l <- 0 until o.length)
    {
      ow(l)=getOneYWFromObservation(o(l))
    }
    ow
  }

  class OneWithWeightAndQuadrature(y:Double,dim:Int,obsr: Array[Double],wei: Array[Double],flag_donotscale:Boolean)
      extends OneWithWeight(y,dim,obsr,wei,flag_donotscale){
    val (qx,qw)={val q=Q./*getQuadraturesFromMoments_EV*/getQuadraturesFromMoments_Direct(dim,moments);
      for(m <- 0 until q(0).length) q(0)(m)=scaler.getXScaledBack(q(0)(m))
      (q(0),q(1))
    }
  }

  /** Get y distribution 1/K(y,y) for given x, here we assume the weight for 
    *  each y_l is 1/K_l(x,x), where the K_l(x,x) is the repoducing kernel
    *  calculated for (x1,x2,x3,...)_l sample. If flag_noX=true,
    *  then all the weights are assumed to be equal to 1.
    */
  def getYdistatX(x:Double,w: Array[OneWithWeight],flag_noX:Boolean)={
    if(w.length<dimY) throw new RuntimeException("w.length="+w.length+"<dimY="+dimY)
    val yo=new Array[Double](w.length)
    val wo=new Array[Double](w.length)
    for( l <- 0 until w.length){
      yo(l)=w(l).y
      // negative weights are the sign of rounding errors.
      // you may be better off using Q(x)G^{-1}Q(x) rather than expanded to x space polynomial.
      wo(l)=if(flag_noX) 1 else {val fw=w(l).getW(x); if(fw>0) 1/fw else 0}
      //System.err.println("wo("+l+")="+wo(l))
      //if(wo(o)<0) throw new RuntimeException("wo("+o+")="+wo(o))
    }
    //System.err.println("at x="+x+" dimY="+dimY+" yo.length="+yo.length)
    new OneWithWeightAndQuadrature(x,dimY,yo,wo,false)
  }

  val RANDOM=new java.util.Random(123)
  val MINY = -1.0
  val MAXY = 1.0

  /** Create simplistic observations x=y+RANDOMNESS*random
    */
  def getSample(): Array[OneYObservationPoint]={
    val oX=1000
    val oY=10000
    val ob=new Array[OneYObservationPoint](oY)
    for( l <- 0 until ob.length)
    {
      val y=MINY+(MAXY-MINY)*RANDOM.nextDouble()
      val ox=new Array[Double](oX)
      for( j <- 0 until ox.length)
      {
        ox(j)=y+RANDOMNESS*(2*RANDOM.nextDouble()-1)
      }
      ob(l)=new OneYObservationPoint(y,ox)
    }
    ob
  }

  def printTwoStepYDistribution(xs:Array[Double],sample:Array[OneYObservationPoint])
  {
    val sampleK=getObserwationsWithK(sample)
    val Xfixed=new Array[OneWithWeightAndQuadrature](1+xs.length)
    val fnamenase="OUT.R="+RANDOMNESS+".";
    val fnameK=fnamenase+"K.dat"
    System.err.print("Writing "+fnameK);
    val of=new java.io.PrintWriter(fnameK)
    of.print("|y\tnoX")
    Xfixed(0)=getYdistatX(Double.NaN,sampleK,true)
    for(jx <- 1 until Xfixed.length)
    {
      Xfixed(jx)=getYdistatX(xs(jx-1),sampleK,false)
      of.print("\tx="+Xfixed(jx).y+"("+Xfixed(jx).NinKxx+"Yobs)")
    }
    of.println()

    val step=(MAXY-MINY)/5000.0
    for(y <- MINY to MAXY+step/2 by step)
    {
      of.print(y)
      for(jx <- 0 until Xfixed.length)
      {
        val wy=Xfixed(jx)
        of.print("\t"+(1/wy.getW(y)))
      }
      of.println()
    }
    System.err.println(" DONE");
    of.close()

    for(jx <- 0 until Xfixed.length)
    {
      val fname=fnamenase+"X="+Xfixed(jx).y+".Q.dat"
      System.err.print("Writing "+fname);
      val o=new java.io.PrintWriter(fname)
      val wy=Xfixed(jx)      
      o.println("|y\tWeight (sum="+wy.qw.sum+" ; moms[0]="+wy.moments(0)+")")
      for(i <- 0 until wy.qx.length)
      {
        o.println(""+wy.qx(i)+"\t"+wy.qw(i))
      }
      o.close()
      System.err.println(" DONE");
    }
  }



def printOneStepYDistribution(xs:Array[Double],sample:Array[OneYObservationPoint])
{
  val fnamenase="OUT.R="+RANDOMNESS+".";
  val yd=new DistributionRegression1Step(dimX,sample,Q)

  val fnameK=fnamenase+"IN.dat"
  System.err.print("Writing "+fnameK);
  val of=new java.io.PrintWriter(fnameK)
  of.print("|y\tLSQ\nRN")
  val step=(MAXY-MINY)/5000.0
  for(x <- MINY-0.5 to MAXY+0.5+step/2 by step)
  {
    of.println(x+"\t"+yd.getYLSQ(x)+"\t"+yd.getYRN(x))
  }
  of.close()
  System.err.println(" DONE");

  for(jx <- 0 until xs.length)
  {
    val (y,w)=yd.getYkP(xs(jx))
    val fname=fnamenase+"X="+xs(jx)+".S.dat"
    System.err.print("Writing "+fname);
    val o=new java.io.PrintWriter(fname)
    o.println("|y\tWeight")
    for(i <- 0 until w.length)
    {
      o.println(""+y(i)+"\t"+w(i))
    }
    o.close()
    System.err.println(" DONE");
  }

  val r=yd.getYkP(xs)
  val fnameP=fnamenase+".XS.S.dat"
  System.err.print("Writing "+fnameP);
  val o=new java.io.PrintWriter(fnameP)
  o.print("|y")
  for( j <- 0 until xs.length) o.print("\txs="+xs(j))
  o.println()
  for( k <- 0 until r(0).length)
  {
    o.print(r(0)(k))
    for( j <- 0 until xs.length) o.print("\t"+r(j+1)(k))
    o.println()
  }
  o.close()
  System.err.println(" DONE");
}


}

object ExampleDistributionDependence {
  def main(args: Array[String]) {
    val E=new ExampleDistributionDependence(10,10,0.5,new OrthogonalPolynomialsChebyshevBasis()/* new OrthogonalPolynomialsLegendreBasis()*/)
    val sample=E.getSample()
    // two--step solution
    E.printTwoStepYDistribution(Array((E.MINY+E.MAXY)/2-0.25*(E.MAXY-E.MINY),(E.MINY+E.MAXY)/2-0*(E.MAXY-E.MINY),(E.MINY+E.MAXY)/2+0.25*(E.MAXY-E.MINY)),sample)
    // one--step solution
    E.printOneStepYDistribution(Array(-0.5,0,0.5),sample)
  }
}
