package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._
import com.polytechnik.realtimebook._

/** A simple example calculating I(p) in price space.
  * @version $Id: ExampleNoSupplyDemand.scala,v 1.17 2016/02/04 06:56:55 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  * A library from http://www.ioffe.ru/LNEPS/malyshkin/code_polynomials_quadratures.zip is required.
  */
class Observations() {
  val Q=new OrthogonalPolynomialsHermiteEBasis()
    /*new OrthogonalPolynomialsLegendreBasis()*/ /*new OrthogonalPolynomialsMonomialsBasis()*/

  val T_START=(9*3600+30*60)*TraderInterface.TIME_SCALE // 9:30
  val T_END=(16*3600+0*60)*TraderInterface.TIME_SCALE // 16:00
  val p=new scala.collection.mutable.ArrayBuffer[Double]()
  val t=new scala.collection.mutable.ArrayBuffer[Long]()
  val dv=new scala.collection.mutable.ArrayBuffer[Long]()

  def readPoint(pr:Double,tr:Long, dvr:Long)=
  {
    p+=pr
    t+=tr
    dv+=dvr
  }
  def readPointTInterval(pr:Double,tr:Long, dvr:Long)=
  {
    if(T_START<=tr && tr<= T_END) {readPoint(pr,tr,dvr);1}
    else 0
  }
  def readObservationsFromFile(fname:String)
  {
    var at:String=null
    var n=0
    var nr=0
    for (ln <- scala.io.Source.fromFile(fname).getLines){
      n+=1
      val a=ln.split("\t")
      if(at==null) at=a(0)
      else if(!a(0).equals(at)) throw new RuntimeException("line:"+n+" ticker mismatch at="+at+" a(0)="+a(0))
      nr+=readPointTInterval(a(2).toDouble,a(1).toLong,a(3).toLong)
    }
    System.err.println("file="+fname+" read "+n+" lines read ; generated "+nr+" records")
  }

  class Histogram(val p_min:Double ,val p_max:Double,val ph:Array[(Double,Double)],val vh:Array[Long]){
    def printToFile(fname:String)={
      System.err.print("Writing "+fname)
      val of=new java.io.PrintWriter(fname)
      of.println("| p_min="+p_min+" p_max="+p_max)
      of.println("|left\trigh\tv")
      for(k <-0 until ph.length)
      {
        of.println(""+ph(k)._1+"\t"+ph(k)._2+"\t"+vh(k))
      }
      of.close()
      System.err.println(" DONE")
    }
  }
  def getHistogram(n:Int)={
    val p_min=p.min
    val p_max=p.max
    val dp=(p_max-p_min)/n
    val ph=new Array[(Double,Double)](n)
    var pk=p_min
    for(k <- 0 until n)
    {
      ph(k)=(pk,pk+dp)
      pk+=dp
    }
    val vh=new Array[Long](n)
    for(j <- 0 until dv.length){
      val pj=p(j)
      for(k <- 0 until n)
      {
        if(ph(k)._1 <=pj && (k==n-1||pj<ph(k)._2))
        {
          vh(k)+=dv(j)
        }
      }
    }
    System.err.println("Histogram: dv.sum="+dv.sum+" vh.sum="+vh.sum);
    new Histogram(p_min,p_max,ph,vh)
  }

  def getPOffsetAndScale()={
    var spv=0.0
    var sv=0l
    for(j <- 0 until dv.length){
      spv+=p(j)*dv(j)
      sv+=dv(j)
    }
    val p_offset=spv/sv
    spv=0.0
    for(j <- 0 until dv.length){
      spv+=Math.abs((p(j)-p_offset)*dv(j))
    }
    (p_offset,spv/sv)
  }
  class MomentsData(val p_offset:Double,val scale:Double,val pt:Array[Double],val pv:Array[Double]){
    def convertPxToP(px:Double)=px*scale+p_offset
    def convertPToPx(p:Double)=(p-p_offset)/scale
  }
  def getMoments(n:Int)={
    val (p_offset,scale)=getPOffsetAndScale()
    val pt=new Array[Double](n)
    val pv=new Array[Double](n)
    val r=new Array[Double](n)
    for(j <- 0 until dv.length){
      val px=(p(j)-p_offset)/scale
      val dtx=if(j<=0) 0 else (t(j)-t(j-1))
      val dvx=dv(j)
      for(k <- 0 until n)
      {
        r(k)=1
        val q=Q.fx(r,px)
        pt(k)+=q*dtx
        pv(k)+=q*dvx
        r(k)=0
      }
    }
    new MomentsData(p_offset,scale,pt,pv)
  }
  class PsiStates(val dim:Int){
    val moments=getMoments(2*dim-1+1)
    val QQt=Q.getQQMatr(dim,moments.pt.slice(0,2*dim-1))
    val QQv=Q.getQQMatr(dim,moments.pv.slice(0,2*dim-1))
    val QQpv=Q.getQQMatr(dim,Q.getXFMomentsFromMoments(moments.pv))
    val ev=Q.EV.getGEV_Generalized(dim,QQv,dim,QQt,dim,true);
    val IQQt=Linsystems.getInvertedMatrix(dim,dim,QQt)
    val QQnom=TMatr.getMatr_x_Matr(dim,IQQt,dim,TMatr.getMatr_x_Matr(dim,QQv,dim,IQQt,dim),dim)
    def getIP(px: Double)={
      val pP=Q.getX0Moments(dim,px)
      TMatr.getAMatrA(pP,QQnom,dim)/TMatr.getAMatrA(pP,IQQt,dim)
    }
   
    def getPsi2Val(ind:Int,px:Double)={
      val psi=new Array[Double](dim)
      System.arraycopy(ev,dim+ind*dim,psi,0,dim)
      val psi2=Math.pow(Q.fx(psi,px),2)
      (psi2,psi2/TMatr.getAMatrA(Q.getX0Moments(dim,px),IQQt,dim))
    }
    def getPxatPsi(ind:Int)={
      TMatr.getAMatrA(dim,ev,dim+ind*dim,QQpv,dim)/TMatr.getAMatrA(dim,ev,dim+ind*dim,QQv,dim)
    }
    def getDynamicImpactEV(ind:Int)={
      assert(ind==0 || ind==(dim-1))
      val psi=new Array[Double](dim)
      System.arraycopy(ev,dim+ind*dim,psi,0,dim)
      val ie=ev(ind)
      val pe=TMatr.getAMatrA(psi,QQpv,dim)/ie
      val QQph=new Array[Double]((dim-1)*(dim-1))
      val QQih=new Array[Double]((dim-1)*(dim-1))
      var o=0
      for(k <- 0 until dim if k!=ind )
      {
        for(l <- 0 until dim  if l!=ind )
        {
          val pv=TMatr.getAMatrB(dim,ev,dim+k*dim,ev,dim+l*dim,QQpv,dim)
          val I=if(k==l) ev(k) else 0.0
          val c=if(k==l) 1.0 else 0.0
          QQph(o)=pv-pe*I
          QQih(o)=I-ie*c
          if(ind==(dim-1))
          {
            QQph(o)= -QQph(o)
            QQih(o)= -QQih(o)
          }
          o=o+1
        }
      }
      //System.err.println("ind="+ind+" ev="+java.util.Arrays.toString(ev.slice(0,dim))+" QQih="+java.util.Arrays.toString(QQih));
      val dimpact=Q.EV.getGEV_Generalized(dim-1,QQph,dim-1,QQih,dim-1,false);
      (pe,ie,dimpact)
    }

    def getIVarPDynamicImpact(ind:Int)={
      assert(0<=ind && ind<dim)
      val psi=new Array[Double](dim)
      System.arraycopy(ev,dim+ind*dim,psi,0,dim)
      val ie=ev(ind)
      val pe=TMatr.getAMatrA(psi,QQpv,dim)/ie
      val Dpsi=TMatr.scaleVector(Q.B.getDifferentiated(psi.length-1,psi),1/moments.scale)
      val D2psi=TMatr.scaleVector(Q.B.getDifferentiated(Dpsi.length-1,Dpsi),1/moments.scale)
      //java.util.Arrays.fill(D2psi,0)
      val rq=RayleighQuotient.getRQVariationValues(psi,Dpsi,D2psi,QQv,QQt,dim)
      System.err.println("rq="+java.util.Arrays.toString(rq)+" D2cut="+(2*(TMatr.getAMatrA(Dpsi,QQv,dim)-ie*TMatr.getAMatrA(Dpsi,QQt,dim))))
      (pe,ie,rq)
     }

    def getDynamicImpactLIN(ind:Int)={
      assert(0<=ind && ind<dim)
      val psi=new Array[Double](dim)
      System.arraycopy(ev,dim+ind*dim,psi,0,dim)
      val ie=ev(ind)
      val pe=TMatr.getAMatrA(psi,QQpv,dim)/ie
      val d=dim-1
      val QQph=new Array[Double](d*d)
      val Aih=new Array[Double](d)
      var o=0
      var vo=0
      for(k <- 0 until dim if k!=ind )
      {
        for(l <- 0 until dim  if l!=ind )
        {
          val pv=TMatr.getAMatrB(dim,ev,dim+k*dim,ev,dim+l*dim,QQpv,dim)
          val I=if(k==l) ev(k) else 0.0
          //System.err.println("I="+I+" <psi"+k+"|I|psi"+l+">="+TMatr.getAMatrB(dim,ev,dim+k*dim,ev,dim+l*dim,QQv,dim))
          QQph(o)=pv-pe*I
          o=o+1
        }
        Aih(vo)=TMatr.getAMatrB(dim,ev,dim+k*dim,ev,dim+ind*dim,QQpv,dim)
        vo=vo+1
      }
      val rAih=Aih.clone()
      val rQQph=QQph.clone()
      val ipvt=new Array[Int](d)
      val pinfo=Linsystems.dgefa_m1(rQQph,d,d,ipvt)
      if(pinfo!=0)
      {
	java.util.Arrays.fill(rAih,Double.NaN);
      }
      else
      {
        TMatr.scaleVector(rAih,-1)
        Linsystems.dgesl_m1(rQQph,d,d,ipvt,rAih,1);
      }
      val dp=(2*TMatr.mulVV(rAih,Aih)+TMatr.getAMatrA(rAih,QQph,d))/ie
      if(false) // ================ DEBUG
      {
        System.err.println("TMatr.mulVV(rAih,Aih)="+TMatr.mulVV(rAih,Aih)+" TMatr.getAMatrA(rAih,QQph,d)="+TMatr.getAMatrA(rAih,QQph,d)+" must differ with '-' sign ; x:dp="+dp+" reg dp="+(dp*moments.scale))
        System.err.println("ind="+ind+" ev="+java.util.Arrays.toString(ev.slice(0,dim)))
        var ot=0
        var vot=0
        val psik=new Array[Double](dim)
        val psil=new Array[Double](dim)
        for(k <- 0 until dim if k!=ind )
        {
          System.arraycopy(ev,dim+k*dim,psik,0,dim)
          for(l <- 0 until dim  if l!=ind )
          {
            System.arraycopy(ev,dim+l*dim,psil,0,dim)
            val rr=RayleighQuotient.getRQVariationValues(psi,psik,new Array[Double](0),psi,psil,new Array[Double](0),QQpv,QQv,dim)
            System.err.println("rr[2]/2="+(rr(2)/2)+" QQph/lam="+(QQph(ot)/ie))
            ot=ot+1
          }
          val r1=RayleighQuotient.getRQVariationValues(psi,psik,null,QQpv,QQv,dim)
          System.err.println("r1[1]="+r1(1)+" 2*Aih/lam="+(2*Aih(vot)/ie))
          vot=vot+1
        }
      }  // ================ END DEBUG
      (pe,ie,pe+dp)
    }


    def printToFile(fname:String,p_min:Double,p_max:Double,npoints:Int)={
      val pstep=(p_max-p_min)/(npoints-1)
      

      System.err.print("Writing "+fname)
      val of=new java.io.PrintWriter(fname)
      of.println("| IL="+ev(0)+" IH="+ev(dim-1))
      of.println("| p_IL="+moments.convertPxToP(getPxatPsi(0))+" p_IH="+moments.convertPxToP(getPxatPsi(dim-1)))
    ;
      {
      val(pe_IL,ie_IL,dimpact_IL)=getDynamicImpactEV(0)
      val(pe_IH,ie_IH,dimpact_IH)=getDynamicImpactEV(dim-1)
      of.println("|IL: p="+moments.convertPxToP(pe_IL)+" I="+ie_IL+" dimpact="+java.util.Arrays.toString(dimpact_IL.map(_ * moments.scale )))
      of.println("|IH: p="+moments.convertPxToP(pe_IH)+" I="+ie_IH+" dimpact="+java.util.Arrays.toString(dimpact_IH.map(_ * moments.scale )))
      }
      ;
      {
        val(pe_IL,ie_IL,pImp_IL)=getDynamicImpactLIN(0)
        val(pe_IH,ie_IH,pImp_IH)=getDynamicImpactLIN(dim-1)
        of.println("|(lin)IL: p="+moments.convertPxToP(pe_IL)+" I="+ie_IL+" pImp="+moments.convertPxToP(pImp_IL))
        of.println("|(lin)IH: p="+moments.convertPxToP(pe_IH)+" I="+ie_IH+" pImp="+moments.convertPxToP(pImp_IH))
      }
      ;
      {
        val (peL,ieL,rqL)=getIVarPDynamicImpact(0)
        val (peH,ieH,rqH)=getIVarPDynamicImpact(dim-1)
        System.err.println("ieH="+ieH)
      }


      of.println("|p\tI(p)\tpsiL^2(p)\t<psiL|psipi>^2\tpsiH^2(p)\t<psiH|psipi>^2")

      var pp=p_min;
      for(k <- 0 until npoints)
      {
        val px=moments.convertPToPx(pp)
        val(psi2L,ppsi2L)=getPsi2Val(0,px)
        val(psi2H,ppsi2H)=getPsi2Val(dim-1,px)
        of.println(""+pp+"\t"+
          (getIP(px)*TraderInterface.TIME_SCALE) // multiply to have regular time scale (seconds)
          +"\t"+
          (psi2L*TraderInterface.TIME_SCALE)
          +"\t"+
          (ppsi2L)
          +"\t"+
          (psi2H*TraderInterface.TIME_SCALE)
          +"\t"+
          (ppsi2H)
        )
        pp+=pstep
      }
      of.close()
      System.err.println(" DONE")
    }
  }
}


object ExampleNoSupplyDemand {
  def main(args: Array[String]) {
    if(args.length!=1)
    {
      System.err.println("usage: ExampleNoSupplyDemand filetoread");
      System.exit(1)
    }
    val o=new Observations();
    o.readObservationsFromFile(args(0))
    val h=o.getHistogram(20);
    h.printToFile("histogram.csv")
    val m=new o. PsiStates(10)
    m.printToFile("psi.csv",h.p_min-1,h.p_max+1,10000)
  }
}
