package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Item 4.6.2 from the manual.
 *  @version $Id: CROSSTRADEMESSAGEMessageProcessor.java,v 1.23 2012/08/15 16:47:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CROSSTRADEMESSAGEMessageProcessor extends MessageProcessor {

    static final int
        CROSS_TYPE_NASDAQ_OPENING_CROSS='O',
        CROSS_TYPE_NASDAQ_CLOSING_CROSS='C',
        CROSS_TYPE_CROSS_FOR_IPO_AND_HALTED='H',
        CROSS_TYPE_NASDAQ_CROSS_NETWORK='I',
        CROSS_TYPE_EMC_CROSS='E',
	CROSS_TYPE_REGULAR_NO_CROSS=' ';

     // all false by default
    static final boolean [] validate_cross_type=new boolean [MAX_N_CHAR_ARRAY_SIZE];
    static {
	validate_cross_type[CROSS_TYPE_NASDAQ_OPENING_CROSS]=true;
        validate_cross_type[CROSS_TYPE_NASDAQ_CLOSING_CROSS]=true;
        validate_cross_type[CROSS_TYPE_CROSS_FOR_IPO_AND_HALTED]=true;
        validate_cross_type[CROSS_TYPE_NASDAQ_CROSS_NETWORK]=true;
        validate_cross_type[CROSS_TYPE_EMC_CROSS]=true;
	// we use CROSS_TYPE_REGULAR_NO_CROSS only internally.
	//validate_cross_type[CROSS_TYPE_REGULAR_NO_CROSS]=true;
    }


    static class CROSSTRADEMESSAGEMessage extends ITCHMessage {
	final long shares;
	final String stock;
	final double price;
	final long matchnumber;
	final int cross_type;


	CROSSTRADEMESSAGEMessage(final long time,
				 final long shares,
				 final String stock,
				 final double price,
				 final long matchnumber,
				 final int cross_type)
	{
	    super(time);
	    this.shares=shares;
	    this.stock=stock;
	    this.price=price;
	    this.matchnumber=matchnumber;
	    this.cross_type=cross_type;
	}
	String dumpData()
	{
	    return "CROSSTRADEMESSAGEMessageProcessor:"+com.polytechnik.realtimebook.BookItem.getTimeDiag(time)+":"+
		" shares="+shares+
		" price="+price+
		" matchnumber="+matchnumber+
		" stock="+stock+
		" cross_type="+cross_type;
	}
    }


    CROSSTRADEMESSAGEMessageProcessor(final ITCHParser parser)
    {
	super('Q',new int []{1,5,13,21,25,33,34},parser);
    }


    public CROSSTRADEMESSAGEMessage getMessage(final byte [] buf,final int b_start)
    {
	final int v_cross_type=getChar(buf,b_start,6);
	validateChar(v_cross_type,validate_cross_type,"CROSS TYPE",buf,b_start);

	return new CROSSTRADEMESSAGEMessage(
	    this.parser.getTime(),
	    getLong(buf,b_start,2),
	    getString(buf,b_start,3),
	    getPrice(buf,b_start,4),
	    getLong(buf,b_start,5),
	    v_cross_type);
    }

    public void processMessage(final byte [] buf,final int b_start)
    {
	this.parser.executeCrossTrade(getMessage(buf,b_start));
    }
}
