package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** A class to output some data.
    java -server -Xmx6500000K com/polytechnik/itch/DumpData2Trader /rdata/itch/itch4/S012309-v4.txt.gz
 *  @version $Id: DumpData2Trader.java,v 1.19 2016/03/17 00:35:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DumpData2Trader extends DummyTrader {

public interface ProcessTick {
    void processData(final OrdersAndTrades ot,final long currenttime,final int eventid,
		     final int no_buy,final BookItem [] buy,
		     final int no_sell,final BookItem [] sell,
		     final int no_trades,final Trade [] trades);
} // END OF interface ProcessTick

/** Print data class.
 */
public static class PrintData implements ProcessTick {
final double TAUTIMESCALE=TraderInterface.TIME_SCALE;
protected long n_lines=0;
public void processData(final OrdersAndTrades ot,
			final long currenttime,
			final int eventid,
			final int nb,final BookItem [] b,
			final int ns,BookItem [] s,
			final int nt,final Trade [] trades)
{
    // OK, now we have 3 arrays:
    // trades, nt elements: executed trades
    // b, nb elements: buy side of the book
    // s, ns elements: sell side of the book

    Trade t=null;
    // skip non-trade events, they would give duplicated trade records.
    if(eventid==TraderInterface.UPD_EXECUTIONORDER_TRADE || eventid==TraderInterface.UPD_EXECUTIONOTH_TRADE)
    {
	// Last trade. if printing for all eventid get possible duplicates.
	t=nt>0?trades[nt-1]:null;
    }
    // best price & volume
    double p_buy=Double.NaN,p_sell=Double.NaN;
    long v_buy=0,v_sell=0,tv_buy=0,tv_sell=0;
    for(int k=0;k<nb;k++)
    {
	final BookItem o=b[k];
	if(k==0 || p_buy==o.price){p_buy=o.price;v_buy+=o.shares;tv_buy+=(currenttime-o.time)*o.shares;}
    }
    for(int k=0;k<ns;k++)
    {
	final BookItem o=s[k];
	if(k==0 || p_sell==o.price){p_sell=o.price;v_sell+=o.shares;tv_sell+=(currenttime-o.time)*o.shares;}
    }
    final double tau_buy=(tv_buy/TAUTIMESCALE)/v_buy,tau_sell=(tv_sell/TAUTIMESCALE)/v_sell;

    if(n_lines<=0)
    {
	final String h="|ticker\tcurrenttime\teventid\texe_time\texe_price_last\texe_shares\texe_matchnumber\texe_buysellindicator\tbook_p_best_buy\tbook_v_best_buy\tbook_tau_best_buy\tbook_p_best_sell\tbook_v_best_sell\tbook_tau_best_sell";
	int cn=0;
	for(String sh:h.split("\t"))
	{
	    System.out.print(sh+":"+(++cn)+"\t");
	}
	System.out.println();
    }
    n_lines++;

    System.out.println(ot.ticker+"\t"+currenttime+"\t"+eventid+"\t"+(t!=null?t.time:0)+
		       "\t"+(t!=null?t.price:(nt>0?trades[nt-1].price:Double.NaN))+"\t"+(t!=null?t.shares:0)+
		       "\t"+(t!=null?t.matchnumber:0)+
		       "\t"+(t!=null?t.buy_sell_indicator:BookItem.BOOK_UNKNOWN)+
		       "\t"+p_buy+"\t"+v_buy+"\t"+tau_buy+
		       "\t"+p_sell+"\t"+v_sell+"\t"+tau_sell+"\t");
    if(false){
	// diag, dump entire book
	dumpOrdersAndCheckConstrains(nb,b,BookItem.BOOK_BUY);
	dumpOrdersAndCheckConstrains(ns,s,BookItem.BOOK_SELL);
	// diag, dump trades
	dumpTradesAndCheckConstrains(nt,trades);
    }

}
}// END OF class PrintData

    private final java.util.HashMap<String,String> k=new java.util.HashMap<String,String>();
    final ProcessTick tickprocessor;
    public DumpData2Trader(final String [] tickerstowatch,final ProcessTick tickprocessor)
    {
	this.tickprocessor=tickprocessor;
	for(String x :tickerstowatch)
	{
	    this.k.put(x,x);
	}
    }

    protected final Distribution<BookItem>
	buy=new Distribution<BookItem>(),
	sell=new Distribution<BookItem>();
    protected final Distribution<Trade>
	savedtrades=new Distribution<Trade>();

protected void copyDataToArrays(final OrdersAndTrades ot)
{
    // copy orders
    ot.orderbook.copyOrdersSeparateBuySell(this.buy,this.sell);
    // copy trades
    ot.tradebook.copyTrades(this.savedtrades);
}

protected void clearArrays()
{
    this.buy.clearAll();
    this.sell.clearAll();
    this.savedtrades.clearAll();
}

public static void dumpTradesAndCheckConstrains(final int nt,final Trade [] trds)
{
    // Iterate backward, latest trades are at array end.
    for(int k=nt;--k>=0;)
    {
	final Trade t=trds[k];
	System.err.println("k="+k+"\tt.time="+t.time+"\tt.price="+t.price+"\tt.shares="+t.shares+"\tt.matchnumber="+t.matchnumber+"\tt.buy_sell_indicator="+t.buy_sell_indicator);

	// check ordering constrains
	if(true)
	{
	    if(k>0)
	    {
		final BookItem p=trds[k-1];
		if(t.time<p.time) throw new RuntimeException("t.time="+t.time+" p.time="+p.time);
	    }
	}
    }
}
public static void dumpOrdersAndCheckConstrains(final int no,final BookItem [] ords,final int order_type)
{
    for(int k=0;k<no;k++){
	final BookItem o=ords[k];
	System.err.println("k="+k+"\tprice="+o.price+"\ttime="+o.time+"\torderreferencenumber="+o.orderreferencenumber+"\tshares="+o.shares+"\tbuy_sell_indicator="+o.buy_sell_indicator);

	// check ordering constrains
	if(true)
	{
	    if(o.buy_sell_indicator!=order_type) throw new RuntimeException("o.buy_sell_indicator="+o.buy_sell_indicator+" order_type="+order_type);
	    if(k>0){final BookItem p=ords[k-1];
		if(o.price==p.price){
		    if(o.time<p.time) throw new RuntimeException("o.time="+o.time+" p.time="+p.time);
		} else {
		    if(order_type==BookItem.BOOK_BUY){
			if(o.price>p.price) throw new RuntimeException("o.price="+o.price+" p.price="+p.price);
		    } else if(order_type==BookItem.BOOK_SELL){
			if(o.price<p.price) throw new RuntimeException("o.price="+o.price+" p.price="+p.price);
		    } else throw new RuntimeException("unknown order_type="+order_type);
		}
	    }
	}
    }
}

   public void update(final OrdersAndTrades ot,
		      final long currenttime,
		      final int eventid){
       if(this.k.get(ot.ticker)==null)
       {
	   // skip ticker
	   ot.clearBefore(currenttime);
	   return;
       }
       //System.err.println("TICKER="+ot.ticker+" currenttime="+currenttime+" N trades in buffer="+ot.tradebook.getSize()+" eventid="+eventid+" last trade="+(ot.tradebook.getLastAddedElement()!=null?ot.tradebook.getLastAddedElement().time:-1)+" saved time="+(ot.getData()!=null?((Trade)ot.getData()).time:-1));
       // dump data
       copyDataToArrays(ot);
       tickprocessor.processData(ot,currenttime,eventid,buy.getSize(),buy.getBuffer(),sell.getSize(),sell.getBuffer(),savedtrades.getSize(),savedtrades.getBuffer());
       clearArrays();

       // The DummyTrader.update clears old trades, according to the parameter DummyTrader.TIME_KEEP_TRADES.
       // Comment the line below to have all trades in buffer available.
       super.update(ot,currenttime,eventid);
   }


public static void main(String [] args)	throws Exception
{
    if(args.length<2)
    {
	System.err.println("DumpData2Trader itch_file ticker_1 ticker_2 ...");
    }
    else
    {
	final long t0=System.currentTimeMillis();
	int n=0;
	final String [] tickers=new String [args.length-1];
	final String fname=args[0];
	for(int i=1;i<args.length;i++){tickers[i-1]=args[i];}
	final PrintData tickprocessor=new PrintData();
	new DumpData2Trader(tickers,tickprocessor).runParser(fname);
	System.err.println("Processed file "+fname+" saved "+tickprocessor.n_lines+
			   " lines to stdout for "+java.util.Arrays.toString(tickers)+
			   ". Finished in "+((System.currentTimeMillis()-t0)/1000L)+
			   " seconds. by $Id: DumpData2Trader.java,v 1.19 2016/03/17 00:35:58 mal Exp $");
    }
}
} // END OF class DumpData2Trader
