package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** A class to output some data.
    java -server -Xmx6500000K com/polytechnik/itch/DummyTrader /rdata/itch/itch4/S012309-v4.txt.gz
 *  @version $Id: DumpDataTrader.java,v 1.5 2011/08/19 23:14:19 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DumpDataTrader extends DummyTrader {


    private final java.util.HashMap<String,String> k=new java.util.HashMap<String,String>();
    DumpDataTrader(final String [] tickerstowatch)
    {
	for(String x :tickerstowatch)
	{
	    this.k.put(x,x);
	}
    }

    private static class FPR implements com.polytechnik.utils.TObjectProcedure<Trade> {
	OrdersAndTrades ot=null;
	long nlines=0;
	Object bprev=null;
	void setOrdersAndTrades(final OrdersAndTrades ot){this.ot=ot;bprev=null;}
	public boolean execute(final Trade b)
	{
	    //System.err.println("b.time="+b.time);
	    if(b!=ot.tradebook.getLastAddedElement())
	    {
		if(b!=bprev) throw new RuntimeException("this should never happen");
		return false;
	    }
	    if(b==ot.getData()){return false;} // already processed trade
	    bprev=ot.getData();
	    ot.setData(b); // new trade
	    nlines++;
	    // PRINT DATA
	    System.out.println(ot.ticker+"\t"+b.time+"\t"+b.price+"\t"+b.shares+"\t"+
			       (b.best_buy!=null?b.best_buy.price:"?")+"\t"+
			       (b.best_sell!=null?b.best_sell.price:"?")+"\t"+
			       b.origtime+"\t"+b.buy_sell_indicator+"\t");
	    return true;
	}
    };
   final FPR foreach_proc=new FPR();
   public void update(final OrdersAndTrades ot,
		      final long currenttime,
		      final int eventid){
       if(this.k.get(ot.ticker)==null)
       {
	   // skip ticker
	   ot.clearBefore(currenttime);
	   return;
       }
       //System.err.println("TICKER="+ot.ticker+" currenttime="+currenttime+" N trades in buffer="+ot.tradebook.getSize()+" eventid="+eventid+" last trade="+(ot.tradebook.getLastAddedElement()!=null?ot.tradebook.getLastAddedElement().time:-1)+" saved time="+(ot.getData()!=null?((Trade)ot.getData()).time:-1));
       // dump data
       foreach_proc.setOrdersAndTrades(ot);
       ot.tradebook.backwardForEachEntry(foreach_proc,0);
       super.update(ot,currenttime,eventid);
   }


    public static void main(String [] args)
	throws Exception
    {
	if(args.length<2)
	{
	    System.err.println("DumpDataTrader itch_file ticker_1 ticker_2 ...");
	}
	else
	{
	    final long t0=System.currentTimeMillis();
	    int n=0;
	    final String [] tickers=new String [args.length-1];
	    final String fname=args[0];
	    for(int i=1;i<args.length;i++){tickers[i-1]=args[i];}
	    final DumpDataTrader d=new DumpDataTrader(tickers);
	    d.runParser(fname);
	    System.err.println("Processed file "+fname+" saved "+d.foreach_proc.nlines+
			       " lines to stdout for "+java.util.Arrays.toString(tickers)+
			       ". Finished in "+((System.currentTimeMillis()-t0)/1000L)+
			       " seconds. by $Id: DumpDataTrader.java,v 1.5 2011/08/19 23:14:19 mal Exp $");
	}
    }
}
