package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** Item 4.3.4 from the manual.
 *  @version $Id: MARKETPARTICIPANTMessageProcessor.java,v 1.28 2012/09/23 17:46:37 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class MARKETPARTICIPANTMessageProcessor extends MessageProcessor {

    static final int
	MARKET_MAKER_MODE_NORMAL='N',
        MARKET_MAKER_MODE_PASSIVE='P',
        MARKET_MAKER_MODE_SYNDICATE='S',
        MARKET_MAKER_MODE_PRE_SYNDICATE='R',
        MARKET_MAKER_MODE_PENALTY='L';

    static final int
	MARKET_PARTICIPANT_STATE_ACTIVE='A',
        MARKET_PARTICIPANT_STATE_EXCUSED_WITHDRAWN='E',
        MARKET_PARTICIPANT_STATE_WITHDRAWN='W',
        MARKET_PARTICIPANT_STATE_SUSPENDED='S',
        MARKET_PARTICIPANT_STATE_DELETED='D';

    // all false by default
    private static final boolean []
	validate_market_maker_mode=new boolean [MAX_N_CHAR_ARRAY_SIZE],
	validate_market_participant_state=new boolean [MAX_N_CHAR_ARRAY_SIZE];

    static {

	validate_market_maker_mode[MARKET_MAKER_MODE_NORMAL]=true;
        validate_market_maker_mode[MARKET_MAKER_MODE_PASSIVE]=true;
        validate_market_maker_mode[MARKET_MAKER_MODE_SYNDICATE]=true;
        validate_market_maker_mode[MARKET_MAKER_MODE_PRE_SYNDICATE]=true;
        validate_market_maker_mode[MARKET_MAKER_MODE_PENALTY]=true;

	validate_market_participant_state[MARKET_PARTICIPANT_STATE_ACTIVE]=true;
        validate_market_participant_state[MARKET_PARTICIPANT_STATE_EXCUSED_WITHDRAWN]=true;
        validate_market_participant_state[MARKET_PARTICIPANT_STATE_WITHDRAWN]=true;
        validate_market_participant_state[MARKET_PARTICIPANT_STATE_SUSPENDED]=true;
        validate_market_participant_state[MARKET_PARTICIPANT_STATE_DELETED]=true;

    }

    static class MARKETPARTICIPANTMessage extends ITCHMessage {
	final String mpid;
	final String stock;
	final int primary_market_maker;
	final int market_maker_mode;
	final int market_participant_state;

	MARKETPARTICIPANTMessage(final long time,
				 final String mpid,
				 final String stock,
				 final int primary_market_maker,
				 final int market_maker_mode,
				 final int market_participant_state)

	{
	    super(time);
	    this.mpid=mpid;
	    this.stock=stock;
	    this.primary_market_maker=primary_market_maker;
	    this.market_maker_mode=market_maker_mode;
	    this.market_participant_state=market_participant_state;
	}
    }

    private final TraderInterface trader;
    MARKETPARTICIPANTMessageProcessor(final ITCHParser parser,
				      final TraderInterface trader)
    {
	super('L',new int []{1,5,9,17,18,19,20},parser);
	this.trader=trader;
    }


    public MARKETPARTICIPANTMessage getMessage(final byte [] buf,final int b_start)
    {
	final int v_primary_market_maker=getChar(buf,b_start,4);
	validateChar(v_primary_market_maker,validate_yes_no,"PRIMARY MARKET MAKER",buf,b_start);

	final int v_market_maker_mode=getChar(buf,b_start,5);
	validateChar(v_market_maker_mode,validate_market_maker_mode,"MARKET MAKER MODE",buf,b_start);

	final int v_market_participant_state=getChar(buf,b_start,6);
	validateChar(v_market_participant_state,validate_market_participant_state,"MARKET PARTICIPANT STATE",buf,b_start);

	return new MARKETPARTICIPANTMessage(this.parser.getTime(),
					    getString(buf,b_start,2),
					    getString(buf,b_start,3),
					    v_primary_market_maker,
					    v_market_maker_mode,
					    v_market_participant_state);
    }

    public void processMessage(final byte [] buf,final int b_start)
    {
	final MARKETPARTICIPANTMessage m=getMessage(buf,b_start);
	final OrdersAndTrades ot=this.parser.tradesbuilder.initTicker(m.stock);
	if(true) //synchronized(ot.ticker)
	{
	    this.trader.setMarketMaker(
		ot,
		m.time,
		this.parser.eliminateMultipleInstances(m.mpid),
		m.primary_market_maker==G_YES,
		m.market_maker_mode==MARKET_MAKER_MODE_NORMAL&&
		m.market_participant_state==MARKET_PARTICIPANT_STATE_ACTIVE);
	}
    }

}
