package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Item 4.7 from the manual.
 *  @version $Id: NOIIMessageProcessor.java,v 1.9 2010/08/16 14:29:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class NOIIMessageProcessor extends MessageProcessor {

    static final int
        IMBALANCE_DIRECTION_BUY='B',
        IMBALANCE_DIRECTION_SELL='S',
        IMBALANCE_DIRECTION_NO='N',
        IMBALANCE_DIRECTION_INSUFFICIENT='O';


     // all false by default
    private static final boolean [] validate_imbalance_direction=new boolean [MAX_N_CHAR_ARRAY_SIZE];
    static {
	validate_imbalance_direction[IMBALANCE_DIRECTION_BUY]=true;
        validate_imbalance_direction[IMBALANCE_DIRECTION_SELL]=true;
        validate_imbalance_direction[IMBALANCE_DIRECTION_NO]=true;
        validate_imbalance_direction[IMBALANCE_DIRECTION_INSUFFICIENT]=true;
    }


    static class NOIIMessage extends ITCHMessage {
	final long paired_shares;
	final long imbalance_shares;
	final int direction;
	final String stock;
	final double fair_price,near_price,ref_price;
	final int cross_type;
	final int price_variation_indicator;


	NOIIMessage(final long time,
		    final long paired_shares,
		    final long imbalance_shares,
		    final int direction,
		    final String stock,
		    final double fair_price,
		    final double near_price,
		    final double ref_price,
		    final int cross_type,
		    final int price_variation_indicator)
	{
	    super(time);
	    this.paired_shares=paired_shares;
	    this.imbalance_shares=imbalance_shares;
	    this.direction=direction;
	    this.stock=stock;
	    this.fair_price=fair_price;
	    this.near_price=near_price;
	    this.ref_price=ref_price;
	    this.cross_type=cross_type;
	    this.price_variation_indicator=price_variation_indicator;
	}
    }


    NOIIMessageProcessor(final ITCHParser parser)
    {
	super('I',new int []{1,5,13,21,22,30,34,38,42,43,44},parser);
    }


    public NOIIMessage getMessage(final byte [] buf,final int b_start)
    {
	final int v_direction=getChar(buf,b_start,4);
	validateChar(v_direction,validate_imbalance_direction,"IMBALANCE DIRECTION",buf,b_start);

	final int v_cross_type=getChar(buf,b_start,9);
	validateChar(v_cross_type,CROSSTRADEMESSAGEMessageProcessor.validate_cross_type,"CROSS TYPE",buf,b_start);
	return new NOIIMessage(
	    this.parser.getTime(),
	    getLong(buf,b_start,2),
	    getLong(buf,b_start,3),
	    v_direction,
	    getString(buf,b_start,5),
	    getPrice(buf,b_start,6),
	    getPrice(buf,b_start,7),
	    getPrice(buf,b_start,8),
	    v_cross_type,
	    getChar(buf,b_start,10));
    }

    public void processMessage(final byte [] buf,final int b_start)
    {
	final NOIIMessage m=getMessage(buf,b_start);
	// I am not sure what to do with a messages of this type
    }
}
