package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** Item 4.3.1 from the manual.
 *  @version $Id: STOCKDIRECTORYMessageProcessor.java,v 1.29 2012/09/23 17:46:37 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class STOCKDIRECTORYMessageProcessor extends MessageProcessor {

    static final int
        NYSE='N',
	NYSE_AMEX='A',
	NYSE_ARCA='P',
	NASDAQ_GLOBAL_SELECT='Q',
	NASDAQ_GLOBAL_MARKET='G',
	NASDAQ_CAPITAL_MARKET='S',
	BATS_BZX='Z',
	MARKET_CATEGORY_N_A=' ';

    static final int
	FINANCIAL_STATUS_INDICATOR_DEFICIENT='D',
	FINANCIAL_STATUS_INDICATOR_DELINQUENT='E',
	FINANCIAL_STATUS_INDICATOR_BANKRUPT='Q',
	FINANCIAL_STATUS_INDICATOR_SUSPENDED='S',
	FINANCIAL_STATUS_INDICATOR_DEFICIENT_AND_BANKRUPT='G',
	FINANCIAL_STATUS_INDICATOR_DEFICIENT_AND_DELINQUENT='H',
	FINANCIAL_STATUS_INDICATOR_DELINQUENT_AND_BANKRUPT='J',
        FINANCIAL_STATUS_INDICATOR_DEFICIENT_DELINQUENT_BANKRUPT='K',
        FINANCIAL_STATUS_INDICATOR_IN_COMPLIANCE=' ';


    // all false by default
    private static final boolean []
	validate_market_category=new boolean [MAX_N_CHAR_ARRAY_SIZE],
	validate_financial_status_indicator=new boolean [MAX_N_CHAR_ARRAY_SIZE];
    static {
	validate_market_category[NYSE]=true;
	validate_market_category[NYSE_AMEX]=true;
	validate_market_category[NYSE_ARCA]=true;
	validate_market_category[NASDAQ_GLOBAL_SELECT]=true;
	validate_market_category[NASDAQ_GLOBAL_MARKET]=true;
	validate_market_category[NASDAQ_CAPITAL_MARKET]=true;
	validate_market_category[BATS_BZX]=true;
	validate_market_category[MARKET_CATEGORY_N_A]=true;


	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_DEFICIENT]=true;
	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_DELINQUENT]=true;
	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_BANKRUPT]=true;
	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_SUSPENDED]=true;
	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_DEFICIENT_AND_BANKRUPT]=true;
	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_DEFICIENT_AND_DELINQUENT]=true;
	validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_DELINQUENT_AND_BANKRUPT]=true;
        validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_DEFICIENT_DELINQUENT_BANKRUPT]=true;
        validate_financial_status_indicator[FINANCIAL_STATUS_INDICATOR_IN_COMPLIANCE]=true;

    }

    static class STOCKDIRECTORYMessage extends ITCHMessage {
	final String stock;
	final int market_category;
	final int financial_status_indicator;
	final int round_lot_size;
	final int round_lot_only;

	STOCKDIRECTORYMessage(final long time,
			      final String stock,
			      final int market_category,
			      final int financial_status_indicator,
			      final int round_lot_size,
			      final int round_lot_only)
	{
	    super(time);
	    this.stock=stock;
	    this.market_category=market_category;
	    this.financial_status_indicator=financial_status_indicator;
	    this.round_lot_size=round_lot_size;
	    this.round_lot_only=round_lot_only;
	}
    }


    private final TraderInterface trader;
    STOCKDIRECTORYMessageProcessor(final ITCHParser parser,
				   final TraderInterface trader)
    {
	super('R',new int []{1,5,13,14,15,19,20},parser);
	this.trader=trader;
    }


    public STOCKDIRECTORYMessage getMessage(final byte [] buf,final int b_start)
    {
	final int v_market_category=getChar(buf,b_start,3);
	validateChar(v_market_category,validate_market_category,"MARKET CATEGORY",buf,b_start);

	final int v_financial_status_indicator=getChar(buf,b_start,4);
	validateChar(v_financial_status_indicator,validate_financial_status_indicator,"FINANCIAL STATUS",buf,b_start);
	final int v_round_lot_only=getChar(buf,b_start,6);
	validateChar(v_round_lot_only,validate_yes_no,"ROUND LOT ONLY",buf,b_start);

	return new STOCKDIRECTORYMessage(this.parser.getTime(),
					 getString(buf,b_start,2),
					 v_market_category,
					 v_financial_status_indicator,
					 getInteger(buf,b_start,5),
					 v_round_lot_only);
    }


    public void processMessage(final byte [] buf,final int b_start)
    {
	final STOCKDIRECTORYMessage m=getMessage(buf,b_start);
	final OrdersAndTrades ot=this.parser.tradesbuilder.initTicker(m.stock);
	if(true) //synchronized(ot.ticker)
	{
	    this.trader.setInstrument(
		ot,
		m.time,
		m.financial_status_indicator==FINANCIAL_STATUS_INDICATOR_IN_COMPLIANCE,
		m.round_lot_size,
		m.round_lot_only==G_YES,
		this.parser.eliminateMultipleInstances(String.valueOf((char)m.market_category)));
	}
    }
}
