package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** Item 4.3.2 from the manual.
 *  @version $Id: STOCKTRADINGACTIONMessageProcessor.java,v 1.28 2016/03/23 17:29:19 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class STOCKTRADINGACTIONMessageProcessor extends MessageProcessor {

    static final int
	TRADING_STATE_HALTED='H',
	TRADING_STATE_PAUSED='P',
	TRADING_STATE_HALTED_NASDAQ='V',
	TRADING_STATE_QUOTATION_ONLY_PERIOD='Q',
	TRADING_STATE_QUOTATION_ONLY_NASDAQ_PERIOD='R',
        TRADING_STATE_TRADING_ON_NASDAQ='T';

    // all false by default
    private static final boolean []
	validate_trading_state=new boolean [MAX_N_CHAR_ARRAY_SIZE];
    static {
	validate_trading_state[TRADING_STATE_HALTED]=true;
	validate_trading_state[TRADING_STATE_PAUSED]=true;
	validate_trading_state[TRADING_STATE_HALTED_NASDAQ]=true;
	validate_trading_state[TRADING_STATE_QUOTATION_ONLY_PERIOD]=true;
	validate_trading_state[TRADING_STATE_QUOTATION_ONLY_NASDAQ_PERIOD]=true;
        validate_trading_state[TRADING_STATE_TRADING_ON_NASDAQ]=true;
    }


    static class STOCKTRADINGACTIONMessage extends ITCHMessage {
	final String stock;
	final int trading_state;
	final int reserved;
	final String reason;

	STOCKTRADINGACTIONMessage(final long time,
				  final String stock,
				  final int trading_state,
				  final int reserved,
				  final String reason)

	{
	    super(time);
	    this.stock=stock;
	    this.trading_state=trading_state;
	    this.reserved=reserved;
	    this.reason=reason;
	}
    }

    private final TraderInterface trader;
    STOCKTRADINGACTIONMessageProcessor(final ITCHParser parser,
				       final TraderInterface trader)
    {
	super('H',new int []{1,5,13,14,15,19},parser);
	this.trader=trader;
    }


    public STOCKTRADINGACTIONMessage getMessage(final byte [] buf,final int b_start)
    {
	final int v_trading_state=getChar(buf,b_start,3);
	validateChar(v_trading_state,validate_trading_state,"TRADING STATE",buf,b_start);

	return new STOCKTRADINGACTIONMessage(this.parser.getTime(),
					     getString(buf,b_start,2),
					     v_trading_state,
					     getChar(buf,b_start,4),
					     getString(buf,b_start,5));
    }

    public void processMessage(final byte [] buf,final int b_start)
    {
	final STOCKTRADINGACTIONMessage m=getMessage(buf,b_start);
	final OrdersAndTrades ot=this.parser.tradesbuilder.initTicker(m.stock);
	if(true) //synchronized(ot.ticker)
	{
	    this.trader.allowTrading(ot,m.time,m.trading_state==TRADING_STATE_TRADING_ON_NASDAQ?TraderInterface.SEC_TRADING_OK:TraderInterface.SEC_TRADING_DISABLED,"STOCKTRADINGACTION:STATE="+((char)m.trading_state)+";"+m.reason);
	}
    }
}
