package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** Item 4.2 from the ITCH2 manual.
 *  @version $Id: SYSTEMEVENTMessageProcessor.java,v 1.21 2009/10/06 16:01:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class SYSTEMEVENTMessageProcessor extends MessageProcessor {

    static final int
        START_OF_MESSAGES='O',
        START_OF_SYSTEM_HOURS='S',
        START_OF_MARKET_HOURS='Q',
        END_OF_MARKET_HOURS='M',
        END_OF_SYSTEM_HOURS='E',
        END_OF_MESSAGES='C',

        EMERGENCY_MARKET_CONDITION_HALT='A',
        EMERGENCY_MARKET_CONDITION_QUOTE_ONLY_PERIOD='R',
        EMERGENCY_MARKET_CONDITION_RESUMPTION='B';

    // all false by default
    static final boolean [] validate_eventcode=new boolean [MAX_N_CHAR_ARRAY_SIZE];
    static {
	validate_eventcode[START_OF_MESSAGES]=true;
        validate_eventcode[START_OF_SYSTEM_HOURS]=true;
        validate_eventcode[START_OF_MARKET_HOURS]=true;
        validate_eventcode[END_OF_MARKET_HOURS]=true;
        validate_eventcode[END_OF_SYSTEM_HOURS]=true;
        validate_eventcode[END_OF_MESSAGES]=true;
        validate_eventcode[EMERGENCY_MARKET_CONDITION_HALT]=true;
        validate_eventcode[EMERGENCY_MARKET_CONDITION_QUOTE_ONLY_PERIOD]=true;
        validate_eventcode[EMERGENCY_MARKET_CONDITION_RESUMPTION]=true;
    }

    static class SYSTEMEVENTMessage extends ITCHMessage {
	final int eventcode;

	SYSTEMEVENTMessage(final long time,
			   final int eventcode)
	{
	    super(time);
	    this.eventcode=eventcode;
	}
    }

    private final TraderInterface trader;
    SYSTEMEVENTMessageProcessor(final ITCHParser parser,
				final TraderInterface trader)
    {
	super('S',new int []{1,5,6},parser);
	this.trader=trader;
    }


    public SYSTEMEVENTMessage getMessage(final byte [] buf,final int b_start)
    {
	final int ecode=getChar(buf,b_start,2);
	validateChar(ecode,validate_eventcode,"EVENT CODE",buf,b_start);
	return new SYSTEMEVENTMessage(this.parser.getTime(),ecode);
    }


    boolean start_of_system_hours=false;
    boolean start_of_market_hours=false;
    boolean emc_halt=false;
    boolean emc_quote_only=false;

    public void processMessage(final byte [] buf,final int b_start)
    {
	final SYSTEMEVENTMessage m=getMessage(buf,b_start);
	switch(m.eventcode)
	{
	case START_OF_SYSTEM_HOURS:
	    start_of_system_hours=true;
	    break;
	case START_OF_MARKET_HOURS:
	    start_of_market_hours=true;
	    break;
	case EMERGENCY_MARKET_CONDITION_RESUMPTION:
	    emc_halt=false;
	    emc_quote_only=false;
	    break;
	case END_OF_MARKET_HOURS:
	    start_of_system_hours=false;
	    break;
	case END_OF_SYSTEM_HOURS:
	    start_of_system_hours=false;
	    break;
	case EMERGENCY_MARKET_CONDITION_HALT:
	    emc_halt=true;
	    break;
	case EMERGENCY_MARKET_CONDITION_QUOTE_ONLY_PERIOD:
	    emc_quote_only=true;
	    break;
	}
	this.trader.allowTrading(
	    m.time,
	    start_of_system_hours &
	    start_of_market_hours &
	    !emc_halt &
	    !emc_quote_only);
    }
}
