package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A convertor of SoupBinTCP to plain ITCH4.
 * https://www.nasdaqtrader.com/content/technicalsupport/specifications/TradingProducts/soupBin30_ouch40.pdf
 * specification.
 *  @version $Id: SoupBinTCP.java,v 1.89 2011/10/15 15:18:07 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class SoupBinTCP {

    private static final boolean PRG_DEBUG=true;

    private String session_id=null;
    private long seq_next;
    private boolean flag_logged_in,flag_session_end;
    private final StreamProcessor parser;
    private final String login,password,ip;
    private final int port;
    private final boolean read_only_stream;
    public SoupBinTCP(final StreamProcessor parser,
		      final String login,
		      final String password,
		      final String ip,
		      final int port,
		      final boolean read_only_stream)
    {
	this.session_id=null;
	this.seq_next=1;
	this.parser=parser;
	this.login=login;
	this.password=password;
	this.ip=ip;
	this.port=port;
	this.read_only_stream=read_only_stream;
	this.flag_logged_in=read_only_stream;
	this.flag_session_end=false;
    }



    private void sendLogin(final java.io.OutputStream out) throws java.io.IOException
    {
	this.flag_logged_in=false;
	out.write(0);
	out.write(47);
	out.write('L');
	if(true)
	{
	    final byte [] b=this.login.getBytes();
	    if(b.length>6)
	    {
		throw new IllegalArgumentException();
	    }
	    out.write(b,0,b.length);
	    for(int i=b.length;i<6;i++)
	    {
		out.write(' ');
	    }
	}

	if(true)
	{
	    final byte [] b=this.password.getBytes();
	    if(b.length>10)
	    {
		throw new IllegalArgumentException();
	    }
	    out.write(b,0,b.length);
	    for(int i=b.length;i<10;i++)
	    {
		out.write(' ');
	    }
	}

	if(this.session_id==null)
	{
	    for(int i=0;i<10;i++)
	    {
		out.write(' ');
	    }
	}
	else
	{
	    final byte [] b=this.session_id.getBytes();
	    if(b.length>10)
	    {
		throw new IllegalStateException("Session="+this.session_id);
	    }
	    for(int i=b.length;i<10;i++)
	    {
		out.write(' ');
	    }
	    out.write(b,0,b.length);
	}

	if(true)
	{
	    final byte [] b=String.valueOf(this.seq_next).getBytes();
	    if(b.length>20)
	    {
		throw new IllegalStateException("seq_next="+this.seq_next);
	    }
	    for(int i=b.length;i<20;i++)
	    {
		out.write(' ');
	    }
	    out.write(b,0,b.length);
	}

	out.flush();
	if(PRG_DEBUG)
        {
	    System.err.println("LOGIN "+this.login+"/"+this.password+
			       " sent to "+this.ip+":"+this.port+
			       " seq_next="+this.seq_next+
			       " session_id="+this.session_id+
			       " now="+new java.util.Date());
	}
    }

    private void sendHB(final java.io.OutputStream out) throws java.io.IOException
    {
	out.write(0);
	out.write(1);
	out.write('R');
	//out.flush();
    }

    private void sendLogout(final java.io.OutputStream out) throws java.io.IOException
    {
	this.flag_logged_in=false;
	out.write(0);
	out.write(1);
	out.write('O');
	out.flush();
	if(PRG_DEBUG)
        {
	    System.err.println("LOGOUT sent to "+this.ip+":"+this.port+
			       " seq_next="+this.seq_next+
			       " session_id="+this.session_id+
			       " now="+new java.util.Date());
	}
    }

    static final int MESSAGE_PADDING=2;
    static final int INTERNAL_BUFFER_SIZE=5*1024*1024; // bytes
    static final int GZIP_BUFFER_SIZE=1024*128;

    private int processDataChunk(final byte [] buf,final int b_start,final int b_len)
    {
	if(b_len<SoupBinTCP.MESSAGE_PADDING)
	{
	    return 0; // empty string
	}
	// First two bytes are message length.
	final int m_length=MessageProcessor.getShortIntegerFromBytes(buf,b_start);
	if(m_length<=0)
	{
	    throw new IllegalStateException("m_length="+m_length);
	}
	final int lenp=m_length+SoupBinTCP.MESSAGE_PADDING;
	if(lenp>b_len)
	{
	    // to few data read, try next time
	    return 0;
	}
	final int mb_start=b_start+SoupBinTCP.MESSAGE_PADDING;
	// first character is message type
	final int t=buf[mb_start]&0xff;
	if(t=='+')
	{
	    if(PRG_DEBUG)
	    {
		System.err.println("DEBUGGING:"+
				   MessageProcessor.getStringFromBytesOfLen(buf,mb_start+1,m_length-1)+
				   " session="+this.session_id+" seq_next="+this.seq_next+
				   " now="+new java.util.Date());
	    }
	    return lenp;
	}
	if(!this.flag_logged_in)
	{
	    if(t=='J')
	    {
		final int c=m_length==2?buf[mb_start+1]:'?';
		if(PRG_DEBUG)
		{
		    System.err.println("LOGIN REJECTED"+
				       " CODE="+(c)+" ("+((char)c)+")"+
				       " session="+this.session_id+" seq_next="+this.seq_next+
				       " now="+new java.util.Date());
		}
		throw new RuntimeException("Failed to login CODE="+(c)+" ("+((char)c)+")");
	    }
	    if(t!='A' || m_length!=31)
	    {
		throw new RuntimeException("Failed to login");
	    }
	    // login OK
	    this.session_id=MessageProcessor.getStringFromBytesOfLen(buf,mb_start+1,10);
	    final String seq_start=MessageProcessor.getStringFromBytesOfLen(buf,mb_start+11,20);
	    if(PRG_DEBUG)
	    {
		System.err.println("LOGIN OK session_id="+this.session_id+
				   " sequence_start="+seq_start+
				   " now="+new java.util.Date());
	    }
	    final long sq=Long.parseLong(seq_start.trim());
	    if(sq!=this.seq_next)
	    {
		//System.err.println(
		throw new RuntimeException(
		    "Requested ID mismatch for session_id="+this.session_id+
		    " sequence_start="+seq_start+
		    " is different from requested="+this.seq_next+
		    " some records may be missed"+
		    " now="+new java.util.Date());
	    }
	    this.seq_next=sq;
	    this.flag_logged_in=true;
	}
	else
	{
	    // already logged in.
	    if(t=='S')
	    {
		this.seq_next++;
		// Skip type ('S') from buffer to obtain ITCH message actual length.
		final int itch_actual_len=m_length-1;
		if(itch_actual_len<=0)
		{
		    this.flag_logged_in=false;
		    // session end
                    if(PRG_DEBUG)
                    {
                        System.err.println("SESSION END ZERO LENGTH RECEIVED session_id="+this.session_id+
					   " seq_next="+this.seq_next+" now="+new java.util.Date());
                    }
		    return -1;
		}
		// SoupBin byte layout [soup bin len b1][soup bin len b2][soup bin type][itch message]
		// Convert to [not used][itch len b1][itch len b2][itch message], 1 byte is removed.
		// Push length to the buffer.
		buf[mb_start-1]=(byte)((itch_actual_len&0xff00)>>>8);
		buf[mb_start]=(byte)(itch_actual_len&0xff);
		// actully execute parser
		// offset is now b_start+1=mb_start-1,length=itch_actual_len+2(2 bytes itch length+itch message).
		final int r=this.parser.process(buf,mb_start-1,itch_actual_len+2);
		if(r+1!=lenp) // 1 byte is removed when soup bin buffer was conveted to itch.
		{
		    throw new RuntimeException("Failed to process, broken invariant.");
		}
	    }
	    else if(t=='H')
	    {
		// heartbeat, do nothing
	    }
	    else if(t=='Z')
	    {
		this.flag_logged_in=false;
		this.flag_session_end=true;
                if(PRG_DEBUG)
                {
                    System.err.println("SESSION END MESSAGE RECEIVED session_id="+this.session_id+
				       " seq_next="+this.seq_next+" now="+new java.util.Date());
                }
		return -1; // session end
	    }
	    else if(this.read_only_stream && t=='A')
	    {
		// ignore login OK message
	    }
	    else
	    {
		throw new RuntimeException("Unknown message type="+t+" ("+
			((char)t)+") or length="+m_length);
	    }
	}
	return lenp;
    }


    public long processStream() throws java.io.IOException
    {
	final long t0=System.currentTimeMillis();
	if(PRG_DEBUG)
        {
	    System.err.println("Reading data now="+new java.util.Date(t0));
	}
	long n=0,br=0;
	final byte [] buf=new byte [INTERNAL_BUFFER_SIZE];
	final int thr_move=2+((buf.length*2)/3);
	int n_connection_breaks=0;

    final int N_CONNECTION_ATTEMPTS=5;
    int n_connection_attempts_left=this.read_only_stream?1:N_CONNECTION_ATTEMPTS;
    for(;;)
    {
     n_connection_attempts_left--;
     java.net.Socket s=null;
     try {
	s=new java.net.Socket(this.ip,this.port);
	final java.io.InputStream r=s.getInputStream();
	final java.io.OutputStream out;
	if(!this.read_only_stream)
	{
	    out=s.getOutputStream();
	    sendLogin(out);
	    sendHB(out);
	    // restore the number after success
	    n_connection_attempts_left=N_CONNECTION_ATTEMPTS;
	}
	else
	{
	    out=null;
	}

	long next_hb=t0;
        gloop:for(int b_start=0,n_left=buf.length,b_len=0;;)
	{

	    if(b_start>=thr_move)
	    {
		// move to the beginning;
		System.arraycopy(buf,b_start,buf,0,b_len);
		n_left+=b_start;
		b_start=0;
	    }

	    if(n_left<=0)
	    {
		throw new RuntimeException("n_left="+n_left+" must be positive");
	    }
	    if(out!=null)
	    {
		final long t=System.currentTimeMillis();
		if(t>=next_hb)
		{
		    next_hb=t+100L; // 100 msec for HB
		    sendHB(out);
		}
		if(false)
		{ // Test debug connection drop.
		  // Drop connection possibly every 5 minutes.
		  if((t/1000L)%300L==0)
		  {
		      System.err.println("DEBUG: Force connection drop at t="+t+
					 " session="+this.session_id+
					 " seq_next="+this.seq_next);
		      try {Thread.sleep(1000L);} catch(final InterruptedException ie){}
		      out.close();
		  }
		}
	    }
	    final int syms_read=r.read(buf,b_start+b_len,n_left);
	    if(syms_read<0)
	    {
		break;
	    }
	    n_left-=syms_read;
	    b_len+=syms_read;
	    if(b_start+b_len+n_left!=buf.length)
	    {
		throw new RuntimeException("Broken Invariant");
	    }

	    for(;;)
	    {
		final int syms_processed=processDataChunk(buf,b_start,b_len);
		if(syms_processed==0)
		{
		    // need to read more data
		    break;
		}
		else if(syms_processed<0)
		{
		    // finish session, parser sent "END" signal.
		    break gloop;
		}
		n++;
		br+=syms_processed;
		b_start+=syms_processed;
		b_len-=syms_processed;
		if(PRG_DEBUG)
		{
		    if(n%100000==0)
		    {
			System.err.println("Processed "+n+" SoupBinTCP records; "+
					   br+" bytes read; session="+
					   this.session_id+" seq_next="+this.seq_next+
					   " now="+new java.util.Date());
		    }
		}
	    }
	}
	n_connection_attempts_left=0;
	if(out!=null)
	{
	    if(!this.flag_session_end)
	    {
		sendLogout(out);
	    }
	    out.close();
	}
	r.close();
	s.close();
     } catch(final java.net.SocketException se) {
	 if(this.flag_session_end)
	 {
	     if(PRG_DEBUG)
	     {
		 System.err.println("Exit after session="+
				    this.session_id+
				    " end message received."+
				    " n="+n+" SoupBinTCP records; "+br+
				    " bytes read; seq_next="+this.seq_next+
				    " n_connection_breaks="+n_connection_breaks+
				    " now="+new java.util.Date());
	     }
	     break;
	 }
	 n_connection_breaks++;
	 System.err.println("An error in stream connection at "+n+" SoupBinTCP records; "+
			    br+" bytes read; session="+
			    this.session_id+" seq_next="+this.seq_next+
			    " n_connection_breaks="+n_connection_breaks+
			    " now="+new java.util.Date());
	 if(s!=null)
	 {
	     s.close();
	 }
	 if(n_connection_attempts_left<=0||!this.flag_logged_in||this.read_only_stream)
	 {
	     throw se;
	 }
	 if(PRG_DEBUG)
	 {
	     // print stack trace
	     se.printStackTrace(System.err);
	     System.err.println("Trying to re-establish session="+
				this.session_id+" seq_next="+this.seq_next+
				" n_connection_attempts_left="+n_connection_attempts_left);
	 }
	 continue; // try again
     }
     break;
    }
    if(PRG_DEBUG)
    {
	final long t1=System.currentTimeMillis();
	System.err.println("TOTAL "+n+" SoupBinTCP recors read; bytes="+br+"; for session="+
			   this.session_id+"; time diff="+((t1-t0)/1000L)+
			   " seconds. n_connection_breaks="+
			   n_connection_breaks+"; started="+new java.util.Date(t0)+
			   " now="+new java.util.Date(t1));
    }
    return n;
    }


    /** Unit test. */
    public static void main(String [] args) throws java.io.IOException
    {
	System.err.println("Processing stream by $Id: SoupBinTCP.java,v 1.89 2011/10/15 15:18:07 mal Exp $. now="+new java.util.Date());
	if(args.length!=1)
	{
	    System.err.println("Usage: SoupBinTCP ITCH_config");
	    System.exit(1);
	}

	final String configfile=args[0];
	// Save to a file using .gz compression.
	final String fname="out."+System.currentTimeMillis()+"-v41.txt.gz";
	System.err.println("SoupBinTCP::ITCH SoupBinTCP data read using config="+configfile+" to be saved to file="+fname);
	final java.io.OutputStream out=new java.util.zip.GZIPOutputStream(new java.io.FileOutputStream(fname),SoupBinTCP.GZIP_BUFFER_SIZE);

	final java.util.ResourceBundle itchconfig=
	    new java.util.PropertyResourceBundle(
		new java.io.FileInputStream(configfile));

	new SoupBinTCP(new StreamProcessorDumpData(out,new ITCHParser(new DummyTrader())),
		       itchconfig.getString("ITCH_login"),
		       itchconfig.getString("ITCH_password"),
		       itchconfig.getString("ITCH_IP"),
		       Integer.parseInt(itchconfig.getString("ITCH_port")),
		       false).processStream();

	out.close();
	System.err.println("SoupBinTCP::ITCH SoupBinTCP data read using config="+configfile+" is saved to file="+fname);

	System.err.println("DONE processing stream by $Id: SoupBinTCP.java,v 1.89 2011/10/15 15:18:07 mal Exp $. now="+new java.util.Date());
	System.err.flush();
    }

}
