package com.polytechnik.itch;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** Test spread & other stats.
 *  @version $Id: TestStats.java,v 1.15 2012/09/23 17:46:37 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TestStats extends DummyTrader {


    private final java.util.Map<String,CalculateBuySell> sbs=
	new java.util.HashMap<String,CalculateBuySell>();

    public void update(final OrdersAndTrades ot,
		       final long currenttime,
		       final int eventid){

	CalculateBuySell bs=sbs.get(ot.ticker);
	if(bs==null)
	{
	    bs=new CalculateBuySell(ot.ticker);
	    sbs.put(ot.ticker,bs);
	}

	Trade tl=ot.tradebook.getLastAddedElement();
	bs.setTrade(tl);
	bs.setTick();

	final long tb=currenttime-TIME_KEEP_TRADES;
	if(tb>0)
	{
	    ot.clearBefore(tb);
	}
	//ot.clearBefore(currenttime);
    }

    private static final java.util.Comparator<CalculateBuySell>
	cmpbyticker=new java.util.Comparator<CalculateBuySell>(){
		public int compare(final CalculateBuySell o1,final CalculateBuySell o2)
		                  {return o1.ticker.compareTo(o2.ticker);}
         };
    java.util.List<CalculateBuySell> getOrderedByTickerStats()
    {
	final java.util.ArrayList<CalculateBuySell> ts=
	    new java.util.ArrayList<CalculateBuySell>(sbs.values());
	java.util.Collections.sort(ts,cmpbyticker);
	return ts;
    }


private static class CalculateBuySell {
    final String ticker;
    long volume_total=0,trades_total=0,events_total=0,volume_sum_spread=0;
    double sum_spread=0,sum_price=0,sum_volatility=0;
    CalculateBuySell(final String ticker)
    {
	this.ticker=ticker;
    }

    Trade tt=null;
    void setTrade(final Trade t)
    {
	if(t==null || t==tt) return;
	tt=t;
	volume_total+=t.shares;
	trades_total++;
	sum_price+=t.price*t.shares;
	sum_volatility+=(t.price*t.price)*t.shares;


	double spread=Double.NaN;
	if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	{
	    if(t.best_sell!=null)
		spread=t.best_sell.price-t.price;
	}
	else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    if(t.best_buy!=null)
		spread=t.price-t.best_buy.price;
	}
	else if(t.best_buy!=null && t.best_sell!=null)
	{
	    spread=t.best_sell.price-t.best_buy.price;
	}
	if(!Double.isNaN(spread))
	{
	    sum_spread+=spread*t.shares;
	    volume_sum_spread+=t.shares;
	}
    }
    void setTick()
    {
	events_total++;
    }
    void printDiag(final java.io.PrintStream out)
    {
	final double
	    aver_price=sum_price/volume_total,
	    volatility=Math.sqrt(sum_volatility/volume_total-aver_price*aver_price),
	    aver_spread=sum_spread/volume_sum_spread;
	out.println(ticker+"\t"+
		    aver_price+"\t"+
		    volatility+"\t"+
		    aver_spread+"\t"+
		    volume_total+"\t"+
		    trades_total+"\t"+
		    events_total+"\t");
    }
    static void printHeader(final java.io.PrintStream out,
			    final String [] sources)
    {
	out.print("SOURCES:\t");
	for(String s:sources)
	{
	    out.print(s);
	    out.print("\t");
	}
	out.println();
	out.println();
	out.println();
	out.println("TICKER\tA_PRICE\tA_VOLATILITY\tA_SPREAD\tVOL\tTRADES\tEVENTS\t");
    }
} // END of class CalculateBuySell

public static void main(String [] args) throws java.io.IOException
{
    if(args.length<1)
    {
	System.err.println("TestStats.main itch_file_1 itch_file_2 ... ");
	System.exit(1);
    }

    final long t0=System.currentTimeMillis();
    final TestStats t=new TestStats();
    int n=0;
    for(int i=0;i<args.length;i++)
    {
	n++;
	final String file=args[i];
	final long tt0=System.currentTimeMillis();
	System.err.println("Processing file #"+n+"="+file+" by $Id: TestStats.java,v 1.15 2012/09/23 17:46:37 mal Exp $");
	t.runParser(file);
	final long tt1=System.currentTimeMillis();
	System.err.println("Processing file #"+n+"="+file+" is DONE in "+(tt1-tt0)+
			   " milliseconds by $Id: TestStats.java,v 1.15 2012/09/23 17:46:37 mal Exp $");
    }

    // dump results
    CalculateBuySell.printHeader(System.out,args);
    for(CalculateBuySell k:t.getOrderedByTickerStats())
    {
	k.printDiag(System.out);
    }


    System.err.println("Processed "+n+" files="+java.util.Arrays.toString(args)+" in "+
		       ((System.currentTimeMillis()-t0)/1000L)+
		       " seconds. by $Id: TestStats.java,v 1.15 2012/09/23 17:46:37 mal Exp $");
}

}
