/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: ddot.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Ddot.java,v 1.4 2012/12/20 14:37:25 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Ddot {

// *> \brief \b DDOT
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       DOUBLE PRECISION FUNCTION DDOT(N,DX,INCX,DY,INCY)
// * 
// *       .. Scalar Arguments ..
// *       INTEGER INCX,INCY,N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION DX(*),DY(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DDOT forms the dot product of two vectors.
// *>    uses unrolled loops for increments equal to one.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level1
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>     jack dongarra, linpack, 3/11/78.
// *>     modified 12/3/93, array(1) declarations changed to array(*)
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
// *     ..

public static double ddot (int n,
double [] dx, int _dx_offset,
int incx,
double [] dy, int _dy_offset,
int incy)  {

double ddot= 0.0d;
double dtemp= 0.0d;
int i= 0;
int ix= 0;
int iy= 0;
int m= 0;
int mp1= 0;
ddot = 0.0e0;
dtemp = 0.0e0;
/***
*
*        code for unequal increments or equal increments
*          not equal to 1
*
***/
if ((n <= 0)) {
    return ddot;//Dummy.go_to("com/polytechnik/lapack/Ddot",999999);
}
    ix = 1;
iy = 1;
if ((incx < 0)) {
    ix = (((((-(n))+1))*incx)+1);
}
    if ((incy < 0)) {
    iy = (((((-(n))+1))*incy)+1);
}
    {
for (i = 1; i <= n; i++) {
dtemp = (dtemp+(dx[(ix-(1))+ _dx_offset]*dy[(iy-(1))+ _dy_offset]));
ix = (ix+incx);
iy = (iy+incy);
// 
//Dummy.label("com/polytechnik/lapack/Ddot",1000000);
}              //  Close for() loop. 
}
ddot = dtemp;
//Dummy.go_to("com/polytechnik/lapack/Ddot",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Ddot",999999);
return ddot;
   }
} // End class.
