/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgebak.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgebak.java,v 1.5 2012/12/22 06:45:13 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dgebak {

// *> \brief \b DGEBAK
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DGEBAK + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEBAK( JOB, SIDE, N, ILO, IHI, SCALE, M, V, LDV,
// *                          INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOB, SIDE
// *       INTEGER            IHI, ILO, INFO, LDV, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   SCALE( * ), V( LDV, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEBAK forms the right or left eigenvectors of a real general matrix

// *> by backward transformation on the computed eigenvectors of the
// *> balanced matrix output by DGEBAL.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOB
// *> \verbatim
// *>          JOB is CHARACTER*1
// *>          Specifies the type of backward transformation required:
// *>          = 'N', do nothing, return immediately;
// *>          = 'P', do backward transformation for permutation only;
// *>          = 'S', do backward transformation for scaling only;
// *>          = 'B', do backward transformations for both permutation and

// *>                 scaling.
// *>          JOB must be the same as the argument JOB supplied to DGEBAL.
// *> \endverbatim
// *>
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'R':  V contains right eigenvectors;
// *>          = 'L':  V contains left eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of rows of the matrix V.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>          The integers ILO and IHI determined by DGEBAL.
// *>          1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.
// *> \endverbatim
// *>
// *> \param[in] SCALE
// *> \verbatim
// *>          SCALE is DOUBLE PRECISION array, dimension (N)
// *>          Details of the permutation and scaling factors, as returned

// *>          by DGEBAL.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of columns of the matrix V.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension (LDV,M)
// *>          On entry, the matrix of right or left eigenvectors to be
// *>          transformed, as returned by DHSEIN or DTREVC.
// *>          On exit, V is overwritten by the transformed eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] LDV
// *> \verbatim
// *>          LDV is INTEGER
// *>          The leading dimension of the array V. LDV >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleGEcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Decode and Test the input parameters
*
***/

public static void dgebak (final char job,
final char side,
int n,
int ilo,
int ihi,
double [] scale, int _scale_offset,
int m,
double [] v, int _v_offset,
int ldv,
intW info)  {

boolean leftv= false;
boolean rightv= false;
int i= 0;
int ii= 0;
int k= 0;
double s= 0.0d;
//job = job.substring((1)-1,1);
//side = side.substring((1)-1,1);
rightv = com.polytechnik.lapack.Lsame.lsame(side,'R');
// *
leftv = com.polytechnik.lapack.Lsame.lsame(side,'L');
info.val = 0;
if (((((!com.polytechnik.lapack.Lsame.lsame(job,'N')) && (!com.polytechnik.lapack.Lsame.lsame(job,'P'))) && (!com.polytechnik.lapack.Lsame.lsame(job,'S'))) && (!com.polytechnik.lapack.Lsame.lsame(job,'B'))))  {
    info.val = -1;
}
else if (((!rightv) && (!leftv)))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if (((ilo < 1) || (ilo > Math.max(1, n))))  {
    info.val = -4;
}              // Close else if()
else if (((ihi < Math.min(ilo, n)) || (ihi > n)))  {
    info.val = -5;
}              // Close else if()
else if ((m < 0))  {
    info.val = -7;
}              // Close else if()
else if ((ldv < Math.max(1, n)))  {
    info.val = -9;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGEBAK",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dgebak",999999);
/***
*
*     Quick return if possible
*
***/
}
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dgebak",999999);
}
    if ((m == 0)) {
	return;//Dummy.go_to("com/polytechnik/lapack/Dgebak",999999);
}
    // *
if (com.polytechnik.lapack.Lsame.lsame(job,'N')) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dgebak",999999);
}
    /***
*
*     Backward balance
*
***/
if ((ilo == ihi)) {
    //Dummy.go_to("com/polytechnik/lapack/Dgebak",30);
}
else /* label30 */ {
    // *
if ((com.polytechnik.lapack.Lsame.lsame(job,'S') || com.polytechnik.lapack.Lsame.lsame(job,'B')))  {
    if (rightv)  {
    {
for (i = ilo; i <= ihi; i++) {
s = scale[(i-(1))+ _scale_offset];
com.polytechnik.lapack.Dscal.dscal(m,s,v,(i-(1))+(1-(1)) * (ldv)+ _v_offset,ldv);
//Dummy.label("com/polytechnik/lapack/Dgebak",10);
}              //  Close for() loop. 
}
// *
}
if (leftv)  {
    {
for (i = ilo; i <= ihi; i++) {
s = (1.0/scale[(i-(1))+ _scale_offset]);
com.polytechnik.lapack.Dscal.dscal(m,s,v,(i-(1))+(1-(1)) * (ldv)+ _v_offset,ldv);
//Dummy.label("com/polytechnik/lapack/Dgebak",20);
}              //  Close for() loop. 
}
// *
}
/***
*
*     Backward permutation
*
*     For  I = ILO-1 step -1 until 1,
*              IHI+1 step 1 until N do --
*
***/
}
} //label30: Dummy.label("com/polytechnik/lapack/Dgebak",30);

if ((com.polytechnik.lapack.Lsame.lsame(job,'P') || com.polytechnik.lapack.Lsame.lsame(job,'B')))  {
    if (rightv)  {
    {
label40:for (ii = 1; ii <= n; ii++) {
i = ii;
if (((i >= ilo) && (i <= ihi))) {
    continue label40;//Dummy.go_to("com/polytechnik/lapack/Dgebak",40);
}
    if ((i < ilo)) {
    i = (ilo-ii);
}
    k = (int)(scale[(i-(1))+ _scale_offset]);
if ((k == i)) {
    continue label40;//Dummy.go_to("com/polytechnik/lapack/Dgebak",40);
}
    com.polytechnik.lapack.Dswap.dswap(m,v,(i-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,v,(k-(1))+(1-(1)) * (ldv)+ _v_offset,ldv);
//Dummy.label("com/polytechnik/lapack/Dgebak",40);
}              //  Close for() loop. 
}
// *
}
if (leftv)  {
    {
label50:for (ii = 1; ii <= n; ii++) {
i = ii;
if (((i >= ilo) && (i <= ihi))) {
    continue label50;//Dummy.go_to("com/polytechnik/lapack/Dgebak",50);
}
    if ((i < ilo)) {
    i = (ilo-ii);
}
    k = (int)(scale[(i-(1))+ _scale_offset]);
if ((k == i)) {
    continue label50;//Dummy.go_to("com/polytechnik/lapack/Dgebak",50);
}
    com.polytechnik.lapack.Dswap.dswap(m,v,(i-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,v,(k-(1))+(1-(1)) * (ldv)+ _v_offset,ldv);
//Dummy.label("com/polytechnik/lapack/Dgebak",50);
}              //  Close for() loop. 
}
}
// *
}
/***
*
*     End of DGEBAK
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgebak",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dgebak",999999);
return;
   }
} // End class.
