/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgehd2.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgehd2.java,v 1.3 2012/12/16 16:11:30 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dgehd2 {

// *> \brief \b DGEHD2 reduces a general square matrix to upper Hessenberg 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DGEHD2 + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEHD2( N, ILO, IHI, A, LDA, TAU, WORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IHI, ILO, INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEHD2 reduces a real general matrix A to upper Hessenberg form H by

// *> an orthogonal similarity transformation:  Q**T * A * Q = H .
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>
// *>          It is assumed that A is already upper triangular in rows
// *>          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
// *>          set by a previous call to DGEBAL; otherwise they should be
// *>          set to 1 and N respectively. See Further Details.
// *>          1 <= ILO <= IHI <= max(1,N).
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the n by n general matrix to be reduced.
// *>          On exit, the upper triangle and the first subdiagonal of A
// *>          are overwritten with the upper Hessenberg matrix H, and the

// *>          elements below the first subdiagonal, with the array TAU,
// *>          represent the orthogonal matrix Q as a product of elementary
// *>          reflectors. See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          The scalar factors of the elementary reflectors (see Further
// *>          Details).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit.
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrix Q is represented as a product of (ihi-ilo) elementary
// *>  reflectors
// *>
// *>     Q = H(ilo) H(ilo+1) . . . H(ihi-1).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i) = 0, v(i+1) = 1 and v(ihi+1:n) = 0; v(i+2:ihi) is stored on
// *>  exit in A(i+2:ihi,i), and tau in TAU(i).
// *>
// *>  The contents of A are illustrated by the following example, with
// *>  n = 7, ilo = 2 and ihi = 6:
// *>
// *>  on entry,                        on exit,
// *>
// *>  ( a   a   a   a   a   a   a )    (  a   a   h   h   h   h   a )
// *>  (     a   a   a   a   a   a )    (      a   h   h   h   h   a )
// *>  (     a   a   a   a   a   a )    (      h   h   h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  h   h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  v3  h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  v3  v4  h   h   h )
// *>  (                         a )    (                          a )
// *>
// *>  where a denotes an element of the original matrix A, h denotes a
// *>  modified element of the upper Hessenberg matrix H, and vi denotes an
// *>  element of the vector defining H(i).
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dgehd2 (int n,
int ilo,
int ihi,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
intW info)  {

int i= 0;
double aii= 0.0d;
info.val = 0;
if ((n < 0))  {
    info.val = -1;
}
else if (((ilo < 1) || (ilo > Math.max(1, n))))  {
    info.val = -2;
}              // Close else if()
else if (((ihi < Math.min(ilo, n)) || (ihi > n)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -5;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGEHD2",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dgehd2",999999);
// *
}
/***
*
*        Compute elementary reflector H(i) to annihilate A(i+2:ihi,i)
*
***/
{
for (i = ilo; i <= (ihi-1); i++) {
dlarfg_adapter((ihi-i),a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,a,(Math.min((i+2), n)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,(i-(1))+ _tau_offset);
aii = a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset];
/***
*
*        Apply H(i) to A(1:ihi,i+1:ihi) from the right
*
***/
a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
/***
*
*        Apply H(i) to A(i+1:ihi,i+1:n) from the left
*
***/
com.polytechnik.lapack.Dlarf.dlarf('R',ihi,(ihi-i),a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau[(i-(1))+ _tau_offset],a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,work,_work_offset);
// *
com.polytechnik.lapack.Dlarf.dlarf('L',(ihi-i),(n-i),a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau[(i-(1))+ _tau_offset],a,((i+1)-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,work,_work_offset);
a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] = aii;
// *
//Dummy.label("com/polytechnik/lapack/Dgehd2",10);
}              //  Close for() loop. 
}
/***
*
*     End of DGEHD2
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgehd2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dgehd2",999999);
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

com.polytechnik.lapack.Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg1[arg1_offset] = _f2j_tmp1.val;
arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
