/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgemm.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgemm.java,v 1.4 2012/12/20 18:20:05 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dgemm {

// *> \brief \b DGEMM
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEMM(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LD
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA,BETA
// *       INTEGER K,LDA,LDB,LDC,M,N
// *       CHARACTER TRANSA,TRANSB
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),B(LDB,*),C(LDC,*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEMM  performs one of the matrix-matrix operations
// *>
// *>    C := alpha*op( A )*op( B ) + beta*C,
// *>
// *> where  op( X ) is one of
// *>
// *>    op( X ) = X   or   op( X ) = X**T,
// *>
// *> alpha and beta are scalars, and A, B and C are matrices, with op( A )
// *> an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] TRANSA
// *> \verbatim
// *>          TRANSA is CHARACTER*1
// *>           On entry, TRANSA specifies the form of op( A ) to be used i
// *>           the matrix multiplication as follows:
// *>
// *>              TRANSA = 'N' or 'n',  op( A ) = A.
// *>
// *>              TRANSA = 'T' or 't',  op( A ) = A**T.
// *>
// *>              TRANSA = 'C' or 'c',  op( A ) = A**T.
// *> \endverbatim
// *>
// *> \param[in] TRANSB
// *> \verbatim
// *>          TRANSB is CHARACTER*1
// *>           On entry, TRANSB specifies the form of op( B ) to be used i
// *>           the matrix multiplication as follows:
// *>
// *>              TRANSB = 'N' or 'n',  op( B ) = B.
// *>
// *>              TRANSB = 'T' or 't',  op( B ) = B**T.
// *>
// *>              TRANSB = 'C' or 'c',  op( B ) = B**T.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>           On entry,  M  specifies  the number  of rows  of the  matri
// *>           op( A )  and of the  matrix  C.  M  must  be at least  zero
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry,  N  specifies the number  of columns of the matri
// *>           op( B ) and the number of columns of the matrix C. N must b
// *>           at least zero.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>           On entry,  K  specifies  the number of columns of the matri
// *>           op( A ) and the number of rows of the matrix op( B ). K mus
// *>           be at least  zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where 
// *>           k  when  TRANSA = 'N' or 'n',  and is  m  otherwise.
// *>           Before entry with  TRANSA = 'N' or 'n',  the leading  m by 
// *>           part of the array  A  must contain the matrix  A,  otherwis
// *>           the leading  k by m  part of the array  A  must contain  th
// *>           matrix A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program. When  TRANSA = 'N' or 'n' the
// *>           LDA must be at least  max( 1, m ), otherwise  LDA must be a
// *>           least  max( 1, k ).
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where 
// *>           n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
// *>           Before entry with  TRANSB = 'N' or 'n',  the leading  k by 
// *>           part of the array  B  must contain the matrix  B,  otherwis
// *>           the leading  n by k  part of the array  B  must contain  th
// *>           matrix B.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>           On entry, LDB specifies the first dimension of B as declare
// *>           in the calling (sub) program. When  TRANSB = 'N' or 'n' the
// *>           LDB must be at least  max( 1, k ), otherwise  LDB must be a
// *>           least  max( 1, n ).
// *> \endverbatim
// *>
// *> \param[in] BETA
// *> \verbatim
// *>          BETA is DOUBLE PRECISION.
// *>           On entry,  BETA  specifies the scalar  beta.  When  BETA  i
// *>           supplied as zero then C need not be set on input.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array of DIMENSION ( LDC, n ).
// *>           Before entry, the leading  m by n  part of the array  C mus
// *>           contain the matrix  C,  except when  beta  is zero, in whic
// *>           case C need not be set on entry.
// *>           On exit, the array  C  is overwritten by the  m by n  matri
// *>           ( alpha*op( A )*op( B ) + beta*C ).
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>           On entry, LDC specifies the first dimension of C as declare
// *>           in  the  calling  (sub)  program.   LDC  must  be  at  leas
// *>           max( 1, m ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level3
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 3 Blas routine.
// *>
// *>  -- Written on 8-February-1989.
// *>     Jack Dongarra, Argonne National Laboratory.
// *>     Iain Duff, AERE Harwell.
// *>     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *>     Sven Hammarling, Numerical Algorithms Group Ltd.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Parameters ..
***/
/***
*     ..
*
*     Set  NOTA  and  NOTB  as  true if  A  and  B  respectively are not
*     transposed and set  NROWA, NCOLA and  NROWB  as the number of rows
*     and  columns of  A  and the  number of  rows  of  B  respectively.
*
***/

public static void dgemm (final char transa,
final char transb,
int m,
int n,
int k,
double alpha,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double beta,
double [] c, int _c_offset,
int ldc)  {

double temp= 0.0d;
int i= 0;
int info= 0;
int j= 0;
int l= 0;
int ncola= 0;
int nrowa= 0;
int nrowb= 0;
boolean nota= false;
boolean notb= false;
//transa = transa.substring((1)-1,1);
//transb = transb.substring((1)-1,1);
nota = com.polytechnik.lapack.Lsame.lsame(transa,'N');
notb = com.polytechnik.lapack.Lsame.lsame(transb,'N');
if (nota)  {
    nrowa = m;
ncola = k;
}
else  {
  nrowa = k;
ncola = m;
}              //  Close else.
if (notb)  {
    nrowb = k;
}
else  {
  nrowb = n;
/***
*
*     Test the input parameters.
*
***/
}              //  Close else.
info = 0;
if (((((!nota)) && ((!com.polytechnik.lapack.Lsame.lsame(transa,'C')))) && ((!com.polytechnik.lapack.Lsame.lsame(transa,'T')))))  {
    info = 1;
}
else if (((((!notb)) && ((!com.polytechnik.lapack.Lsame.lsame(transb,'C')))) && ((!com.polytechnik.lapack.Lsame.lsame(transb,'T')))))  {
    info = 2;
}              // Close else if()
else if ((m < 0))  {
    info = 3;
}              // Close else if()
else if ((n < 0))  {
    info = 4;
}              // Close else if()
else if ((k < 0))  {
    info = 5;
}              // Close else if()
else if ((lda < Math.max(1, nrowa)))  {
    info = 8;
}              // Close else if()
else if ((ldb < Math.max(1, nrowb)))  {
    info = 10;
}              // Close else if()
else if ((ldc < Math.max(1, m)))  {
    info = 13;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGEMM ",info);
    return;//Dummy.go_to("com/polytechnik/lapack/Dgemm",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     And if  alpha.eq.zero.
*
***/
if (((((m == 0)) || ((n == 0))) || ((((((alpha == 0.0)) || ((k == 0)))) && ((beta == 1.0)))))) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dgemm",999999);
}
    if ((alpha == 0.0))  {
    if ((beta == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dgemm",10);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dgemm",20);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dgemm",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dgemm",40);
}              //  Close for() loop. 
}
}              //  Close else.
return;//Dummy.go_to("com/polytechnik/lapack/Dgemm",999999);
/***
*
*     Start the operations.
*
***/
}
if (notb)  {
    /***
*
*           Form  C := alpha*A*B + beta*C.
*
***/
if (nota)  {
    {
for (j = 1; j <= n; j++) {
if ((beta == 0.0))  {
    {
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dgemm",50);
}              //  Close for() loop. 
}
}
else if ((beta != 1.0))  {
    {
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dgemm",60);
}              //  Close for() loop. 
}
}              // Close else if()
{
for (l = 1; l <= k; l++) {
if ((b[(l-(1))+(j-(1)) * (ldb)+ _b_offset] != 0.0))  {
    temp = (alpha*b[(l-(1))+(j-(1)) * (ldb)+ _b_offset]);
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp*a[(i-(1))+(l-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dgemm",70);
}              //  Close for() loop. 
}
}
//Dummy.label("com/polytechnik/lapack/Dgemm",80);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dgemm",90);
}              //  Close for() loop. 
}
/***
*
*           Form  C := alpha*A**T*B + beta*C
*
***/
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
temp = 0.0;
{
for (l = 1; l <= k; l++) {
temp = (temp+(a[(l-(1))+(i-(1)) * (lda)+ _a_offset]*b[(l-(1))+(j-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack/Dgemm",100);
}              //  Close for() loop. 
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (alpha*temp);
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((alpha*temp)+(beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dgemm",110);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dgemm",120);
}              //  Close for() loop. 
}
}              //  Close else.
}
else  {
  /***
*
*           Form  C := alpha*A*B**T + beta*C
*
***/
if (nota)  {
    {
for (j = 1; j <= n; j++) {
if ((beta == 0.0))  {
    {
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dgemm",130);
}              //  Close for() loop. 
}
}
else if ((beta != 1.0))  {
    {
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dgemm",140);
}              //  Close for() loop. 
}
}              // Close else if()
{
for (l = 1; l <= k; l++) {
if ((b[(j-(1))+(l-(1)) * (ldb)+ _b_offset] != 0.0))  {
    temp = (alpha*b[(j-(1))+(l-(1)) * (ldb)+ _b_offset]);
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp*a[(i-(1))+(l-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dgemm",150);
}              //  Close for() loop. 
}
}
//Dummy.label("com/polytechnik/lapack/Dgemm",160);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dgemm",170);
}              //  Close for() loop. 
}
/***
*
*           Form  C := alpha*A**T*B**T + beta*C
*
***/
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
temp = 0.0;
{
for (l = 1; l <= k; l++) {
temp = (temp+(a[(l-(1))+(i-(1)) * (lda)+ _a_offset]*b[(j-(1))+(l-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack/Dgemm",180);
}              //  Close for() loop. 
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (alpha*temp);
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((alpha*temp)+(beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dgemm",190);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dgemm",200);
}              //  Close for() loop. 
}
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DGEMM .
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgemm",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dgemm",999999);
return;
   }
} // End class.
