/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dger.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dger.java,v 1.4 2012/12/20 18:24:01 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dger {

// *> \brief \b DGER
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGER(M,N,ALPHA,X,INCX,Y,INCY,A,LDA)
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA
// *       INTEGER INCX,INCY,LDA,M,N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),X(*),Y(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGER   performs the rank 1 operation
// *>
// *>    A := alpha*x*y**T + A,
// *>
// *> where alpha is a scalar, x is an m element vector, y is an n element

// *> vector and A is an m by n matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>           On entry, M specifies the number of rows of the matrix A.
// *>           M must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry, N specifies the number of columns of the matrix A
// *>           N must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array of dimension at least
// *>           ( 1 + ( m - 1 )*abs( INCX ) ).
// *>           Before entry, the incremented array X must contain the m
// *>           element vector x.
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>           On entry, INCX specifies the increment for the elements of

// *>           X. INCX must not be zero.
// *> \endverbatim
// *>
// *> \param[in] Y
// *> \verbatim
// *>          Y is DOUBLE PRECISION array of dimension at least
// *>           ( 1 + ( n - 1 )*abs( INCY ) ).
// *>           Before entry, the incremented array Y must contain the n
// *>           element vector y.
// *> \endverbatim
// *>
// *> \param[in] INCY
// *> \verbatim
// *>          INCY is INTEGER
// *>           On entry, INCY specifies the increment for the elements of

// *>           Y. INCY must not be zero.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *>           Before entry, the leading m by n part of the array A must
// *>           contain the matrix of coefficients. On exit, A is
// *>           overwritten by the updated matrix.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program. LDA must be at least
// *>           max( 1, m ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level2
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 2 Blas routine.
// *>
// *>  -- Written on 22-October-1986.
// *>     Jack Dongarra, Argonne National Lab.
// *>     Jeremy Du Croz, Nag Central Office.
// *>     Sven Hammarling, Nag Central Office.
// *>     Richard Hanson, Sandia National Labs.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dger (int m,
int n,
double alpha,
double [] x, int _x_offset,
int incx,
double [] y, int _y_offset,
int incy,
double [] a, int _a_offset,
int lda)  {

double temp= 0.0d;
int i= 0;
int info= 0;
int ix= 0;
int j= 0;
int jy= 0;
int kx= 0;
info = 0;
if ((m < 0))  {
    info = 1;
}
else if ((n < 0))  {
    info = 2;
}              // Close else if()
else if ((incx == 0))  {
    info = 5;
}              // Close else if()
else if ((incy == 0))  {
    info = 7;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info = 9;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGER  ",info);
    return;//Dummy.go_to("com/polytechnik/lapack/Dger",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
***/
if (((((m == 0)) || ((n == 0))) || ((alpha == 0.0)))) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dger",999999);
}
    if ((incy > 0))  {
    jy = 1;
}
else  {
  jy = (1-(((n-1))*incy));
}              //  Close else.
if ((incx == 1))  {
    {
for (j = 1; j <= n; j++) {
if ((y[(jy-(1))+ _y_offset] != 0.0))  {
    temp = (alpha*y[(jy-(1))+ _y_offset]);
{
for (i = 1; i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]+(x[(i-(1))+ _x_offset]*temp));
//Dummy.label("com/polytechnik/lapack/Dger",10);
}              //  Close for() loop. 
}
}
jy = (jy+incy);
//Dummy.label("com/polytechnik/lapack/Dger",20);
}              //  Close for() loop. 
}
}
else  {
  if ((incx > 0))  {
    kx = 1;
}
else  {
  kx = (1-(((m-1))*incx));
}              //  Close else.
{
for (j = 1; j <= n; j++) {
if ((y[(jy-(1))+ _y_offset] != 0.0))  {
    temp = (alpha*y[(jy-(1))+ _y_offset]);
ix = kx;
{
for (i = 1; i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]+(x[(ix-(1))+ _x_offset]*temp));
ix = (ix+incx);
//Dummy.label("com/polytechnik/lapack/Dger",30);
}              //  Close for() loop. 
}
}
jy = (jy+incy);
//Dummy.label("com/polytechnik/lapack/Dger",40);
}              //  Close for() loop. 
}
// *
}              //  Close else.
/***
*
*     End of DGER  .
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dger",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dger",999999);
return;
   }
} // End class.
