/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlabad.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlabad.java,v 1.2 2012/12/22 10:29:25 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlabad {

// *> \brief \b DLABAD
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLABAD + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLABAD( SMALL, LARGE )
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   LARGE, SMALL
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLABAD takes as input the values computed by DLAMCH for underflow and
// *> overflow, and returns the square root of each of these values if the

// *> log of LARGE is sufficiently large.  This subroutine is intended to
// *> identify machines with a large exponent range, such as the Crays, and
// *> redefine the underflow and overflow limits to be the square roots of

// *> the values computed by DLAMCH.  This subroutine is needed because
// *> DLAMCH does not compensate for poor arithmetic in the upper half of
// *> the exponent range, as is found on a Cray.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in,out] SMALL
// *> \verbatim
// *>          SMALL is DOUBLE PRECISION
// *>          On entry, the underflow threshold as computed by DLAMCH.
// *>          On exit, if LOG10(LARGE) is sufficiently large, the square
// *>          root of SMALL, otherwise unchanged.
// *> \endverbatim
// *>
// *> \param[in,out] LARGE
// *> \verbatim
// *>          LARGE is DOUBLE PRECISION
// *>          On entry, the overflow threshold as computed by DLAMCH.
// *>          On exit, if LOG10(LARGE) is sufficiently large, the square
// *>          root of LARGE, otherwise unchanged.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*
*  =====================================================================
*
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     If it looks like we're on a Cray, take the square root of
*     SMALL and LARGE to avoid overflow and underflow problems.
*
***/

public static void dlabad (doubleW small,
doubleW large)  {

if ((Math.log10(large.val) > 2000.e0))  {
    small.val = Math.sqrt(small.val);
large.val = Math.sqrt(large.val);
// *
}
/***
*
*     End of DLABAD
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlabad",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlabad",999999);
return;
   }
} // End class.
