/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlacpy.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlacpy.java,v 1.6 2012/12/20 19:17:24 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlacpy {

// *> \brief \b DLACPY copies all or part of one two-dimensional array to a
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLACPY + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLACPY( UPLO, M, N, A, LDA, B, LDB )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            LDA, LDB, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLACPY copies all or part of a two-dimensional matrix A to another
// *> matrix B.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies the part of the matrix A to be copied to B.
// *>          = 'U':      Upper triangular part
// *>          = 'L':      Lower triangular part
// *>          Otherwise:  All of the matrix A
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          The m by n matrix A.  If UPLO = 'U', only the upper triangle
// *>          or trapezoid is accessed; if UPLO = 'L', only the lower
// *>          triangle or trapezoid is accessed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (LDB,N)
// *>          On exit, B = A in the locations specified by UPLO.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>          The leading dimension of the array B.  LDB >= max(1,M).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlacpy (final char uplo,
int m,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb)  {

int i= 0;
int j= 0;
//uplo = uplo.substring((1)-1,1);
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    {
for (j = 1; j <= n; j++) {
{
final int ie=Math.min(j, m);
for (i = 1; i <= ie; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack/Dlacpy",10);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlacpy",20);
}              //  Close for() loop. 
}
}
else if (com.polytechnik.lapack.Lsame.lsame(uplo,'L'))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack/Dlacpy",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlacpy",40);
}              //  Close for() loop. 
}
}              // Close else if()
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack/Dlacpy",50);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlacpy",60);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*     End of DLACPY
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlacpy",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlacpy",999999);
return;
   }
} // End class.
