/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlagts.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlagts.java,v 1.5 2012/12/19 22:00:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlagts {

// *> \brief \b DLAGTS solves the system of equations (T-λI)x = y or (T-λ
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAGTS + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAGTS( JOB, N, A, B, C, D, IN, Y, TOL, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, JOB, N
// *       DOUBLE PRECISION   TOL
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IN( * )
// *       DOUBLE PRECISION   A( * ), B( * ), C( * ), D( * ), Y( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAGTS may be used to solve one of the systems of equations
// *>
// *>    (T - lambda*I)*x = y   or   (T - lambda*I)**T*x = y,
// *>
// *> where T is an n by n tridiagonal matrix, for x, following the
// *> factorization of (T - lambda*I) as
// *>
// *>    (T - lambda*I) = P*L*U ,
// *>
// *> by routine DLAGTF. The choice of equation to be solved is
// *> controlled by the argument JOB, and in each case there is an option
// *> to perturb zero or very small diagonal elements of U, this option
// *> being intended for use in applications such as inverse iteration.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOB
// *> \verbatim
// *>          JOB is INTEGER
// *>          Specifies the job to be performed by DLAGTS as follows:
// *>          =  1: The equations  (T - lambda*I)x = y  are to be solved,

// *>                but diagonal elements of U are not to be perturbed.
// *>          = -1: The equations  (T - lambda*I)x = y  are to be solved
// *>                and, if overflow would otherwise occur, the diagonal
// *>                elements of U are to be perturbed. See argument TOL
// *>                below.
// *>          =  2: The equations  (T - lambda*I)**Tx = y  are to be solve
// *>                but diagonal elements of U are not to be perturbed.
// *>          = -2: The equations  (T - lambda*I)**Tx = y  are to be solve
// *>                and, if overflow would otherwise occur, the diagonal
// *>                elements of U are to be perturbed. See argument TOL
// *>                below.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix T.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (N)
// *>          On entry, A must contain the diagonal elements of U as
// *>          returned from DLAGTF.
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, B must contain the first super-diagonal elements o
// *>          U as returned from DLAGTF.
// *> \endverbatim
// *>
// *> \param[in] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, C must contain the sub-diagonal elements of L as
// *>          returned from DLAGTF.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N-2)
// *>          On entry, D must contain the second super-diagonal elements

// *>          of U as returned from DLAGTF.
// *> \endverbatim
// *>
// *> \param[in] IN
// *> \verbatim
// *>          IN is INTEGER array, dimension (N)
// *>          On entry, IN must contain details of the matrix P as returne
// *>          from DLAGTF.
// *> \endverbatim
// *>
// *> \param[in,out] Y
// *> \verbatim
// *>          Y is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the right hand side vector y.
// *>          On exit, Y is overwritten by the solution vector x.
// *> \endverbatim
// *>
// *> \param[in,out] TOL
// *> \verbatim
// *>          TOL is DOUBLE PRECISION
// *>          On entry, with  JOB .lt. 0, TOL should be the minimum
// *>          perturbation to be made to very small diagonal elements of U
// *>          TOL should normally be chosen as about eps*norm(U), where ep
// *>          is the relative machine precision, but if TOL is supplied as
// *>          non-positive, then it is reset to eps*max( abs( u(i,j) ) ).

// *>          If  JOB .gt. 0  then TOL is not referenced.
// *>
// *>          On exit, TOL is changed as described above, only if TOL is
// *>          non-positive on entry. Otherwise TOL is unchanged.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0   : successful exit
// *>          .lt. 0: if INFO = -i, the i-th argument had an illegal value
// *>          .gt. 0: overflow would occur when computing the INFO(th)
// *>                  element of the solution vector x. This can only occu
// *>                  when JOB is supplied as positive and either means
// *>                  that a diagonal element of U is very small, or that

// *>                  the elements of the right-hand side vector y are ver
// *>                  large.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlagts (int job,
int n,
double [] a, int _a_offset,
double [] b, int _b_offset,
double [] c, int _c_offset,
double [] d, int _d_offset,
int [] in, int _in_offset,
double [] y, int _y_offset,
doubleW tol,
intW info)  {

final int NLOOPMAX=2*n+Util.ITMAX;
int k= 0;
double absak= 0.0d;
double ak= 0.0d;
double bignum= 0.0d;
double eps= 0.0d;
double pert= 0.0d;
double sfmin= 0.0d;
double temp= 0.0d;
info.val = 0;
if ((((Math.abs(job) > 2)) || ((job == 0))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DLAGTS",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagts",999999);
// *
}
// *
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagts",999999);
}
    eps = com.polytechnik.lapack.Dlamch.dlamch('E');
sfmin = com.polytechnik.lapack.Dlamch.dlamch('S');
// *
bignum = (1.0/sfmin);
if ((job < 0))  {
    if ((tol.val <= 0.0))  {
    tol.val = Math.abs(a[(1-(1))+ _a_offset]);
if ((n > 1)) {
    tol.val = Util.max(tol.val, Math.abs(a[(2-(1))+ _a_offset]), Math.abs(b[(1-(1))+ _b_offset]));
}
    {
for (k = 3; k <= n; k++) {
tol.val = Math.max(Util.max(tol.val, Math.abs(a[(k-(1))+ _a_offset]), Math.abs(b[((k-1)-(1))+ _b_offset])), Math.abs(d[((k-2)-(1))+ _d_offset])) ;
//Dummy.label("com/polytechnik/lapack/Dlagts",10);
}              //  Close for() loop. 
}
tol.val = (tol.val*eps);
if ((tol.val == 0.0)) {
    tol.val = eps;
}
    }
// *
}
if ((Math.abs(job) == 1))  {
    {
for (k = 2; k <= n; k++) {
if ((in[((k-1)-(1))+ _in_offset] == 0))  {
    y[(k-(1))+ _y_offset] = (y[(k-(1))+ _y_offset]-(c[((k-1)-(1))+ _c_offset]*y[((k-1)-(1))+ _y_offset]));
}
else  {
  temp = y[((k-1)-(1))+ _y_offset];
y[((k-1)-(1))+ _y_offset] = y[(k-(1))+ _y_offset];
y[(k-(1))+ _y_offset] = (temp-(c[((k-1)-(1))+ _c_offset]*y[(k-(1))+ _y_offset]));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlagts",20);
}              //  Close for() loop. 
}
if ((job == 1))  {
    {
int _k_inc = -1;
for (k = n; k >= 1; k += _k_inc) {
if ((k <= (n-2)))  {
    temp = ((y[(k-(1))+ _y_offset]-(b[(k-(1))+ _b_offset]*y[((k+1)-(1))+ _y_offset]))-(d[(k-(1))+ _d_offset]*y[((k+2)-(1))+ _y_offset]));
}
else if ((k == (n-1)))  {
    temp = (y[(k-(1))+ _y_offset]-(b[(k-(1))+ _b_offset]*y[((k+1)-(1))+ _y_offset]));
}              // Close else if()
else  {
  temp = y[(k-(1))+ _y_offset];
}              //  Close else.
ak = a[(k-(1))+ _a_offset];
absak = Math.abs(ak);
if ((absak < 1.0))  {
    if ((absak < sfmin))  {
    if (((absak == 0.0) || ((Math.abs(temp)*sfmin) > absak)))  {
    info.val = k;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagts",999999);
}
else  {
  temp = (temp*bignum);
ak = (ak*bignum);
}              //  Close else.
}
else if ((Math.abs(temp) > (absak*bignum)))  {
    info.val = k;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagts",999999);
}              // Close else if()
}
y[(k-(1))+ _y_offset] = (temp/ak);
//Dummy.label("com/polytechnik/lapack/Dlagts",30);
}              //  Close for() loop. 
}
}
else  {
  {
int _k_inc = -1;
for (k = n; k >= 1; k += _k_inc) {
if ((k <= (n-2)))  {
    temp = ((y[(k-(1))+ _y_offset]-(b[(k-(1))+ _b_offset]*y[((k+1)-(1))+ _y_offset]))-(d[(k-(1))+ _d_offset]*y[((k+2)-(1))+ _y_offset]));
}
else if ((k == (n-1)))  {
    temp = (y[(k-(1))+ _y_offset]-(b[(k-(1))+ _b_offset]*y[((k+1)-(1))+ _y_offset]));
}              // Close else if()
else  {
  temp = y[(k-(1))+ _y_offset];
}              //  Close else.
ak = a[(k-(1))+ _a_offset];
pert = Util.dsign(tol.val,ak);
label40:for(int jj40=0;jj40<NLOOPMAX;jj40++){ // Dummy.label("com/polytechnik/lapack/Dlagts",40);
absak = Math.abs(ak);
if ((absak < 1.0))  {
    if ((absak < sfmin))  {
    if (((absak == 0.0) || ((Math.abs(temp)*sfmin) > absak)))  {
    ak = (ak+pert);
pert = (2*pert);
continue label40;//Dummy.go_to("com/polytechnik/lapack/Dlagts",40);
}
else  {
  temp = (temp*bignum);
ak = (ak*bignum);
}              //  Close else.
}
else if ((Math.abs(temp) > (absak*bignum)))  {
    ak = (ak+pert);
pert = (2*pert);
continue label40;//Dummy.go_to("com/polytechnik/lapack/Dlagts",40);
}              // Close else if()
}
break label40;}
y[(k-(1))+ _y_offset] = (temp/ak);
//Dummy.label("com/polytechnik/lapack/Dlagts",50);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Come to here if  JOB = 2 or -2
*
***/
}
else  {
  if ((job == 2))  {
    {
for (k = 1; k <= n; k++) {
if ((k >= 3))  {
    temp = ((y[(k-(1))+ _y_offset]-(b[((k-1)-(1))+ _b_offset]*y[((k-1)-(1))+ _y_offset]))-(d[((k-2)-(1))+ _d_offset]*y[((k-2)-(1))+ _y_offset]));
}
else if ((k == 2))  {
    temp = (y[(k-(1))+ _y_offset]-(b[((k-1)-(1))+ _b_offset]*y[((k-1)-(1))+ _y_offset]));
}              // Close else if()
else  {
  temp = y[(k-(1))+ _y_offset];
}              //  Close else.
ak = a[(k-(1))+ _a_offset];
absak = Math.abs(ak);
if ((absak < 1.0))  {
    if ((absak < sfmin))  {
    if (((absak == 0.0) || ((Math.abs(temp)*sfmin) > absak)))  {
    info.val = k;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagts",999999);
}
else  {
  temp = (temp*bignum);
ak = (ak*bignum);
}              //  Close else.
}
else if ((Math.abs(temp) > (absak*bignum)))  {
    info.val = k;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagts",999999);
}              // Close else if()
}
y[(k-(1))+ _y_offset] = (temp/ak);
//Dummy.label("com/polytechnik/lapack/Dlagts",60);
}              //  Close for() loop. 
}
}
else  {
  {
for (k = 1; k <= n; k++) {
if ((k >= 3))  {
    temp = ((y[(k-(1))+ _y_offset]-(b[((k-1)-(1))+ _b_offset]*y[((k-1)-(1))+ _y_offset]))-(d[((k-2)-(1))+ _d_offset]*y[((k-2)-(1))+ _y_offset]));
}
else if ((k == 2))  {
    temp = (y[(k-(1))+ _y_offset]-(b[((k-1)-(1))+ _b_offset]*y[((k-1)-(1))+ _y_offset]));
}              // Close else if()
else  {
  temp = y[(k-(1))+ _y_offset];
}              //  Close else.
ak = a[(k-(1))+ _a_offset];
pert = Util.dsign(tol.val,ak);
label70:for(int jj70=0;jj70<NLOOPMAX;jj70++){ //Dummy.label("com/polytechnik/lapack/Dlagts",70);
absak = Math.abs(ak);
if ((absak < 1.0))  {
    if ((absak < sfmin))  {
    if (((absak == 0.0) || ((Math.abs(temp)*sfmin) > absak)))  {
    ak = (ak+pert);
pert = (2*pert);
continue label70;//Dummy.go_to("com/polytechnik/lapack/Dlagts",70);
}
else  {
  temp = (temp*bignum);
ak = (ak*bignum);
}              //  Close else.
}
else if ((Math.abs(temp) > (absak*bignum)))  {
    ak = (ak+pert);
pert = (2*pert);
continue label70;//Dummy.go_to("com/polytechnik/lapack/Dlagts",70);
}              // Close else if()
}
break label70;}
y[(k-(1))+ _y_offset] = (temp/ak);
//Dummy.label("com/polytechnik/lapack/Dlagts",80);
}              //  Close for() loop. 
}
// *
}              //  Close else.
{
int _k_inc = -1;
for (k = n; k >= 2; k += _k_inc) {
if ((in[((k-1)-(1))+ _in_offset] == 0))  {
    y[((k-1)-(1))+ _y_offset] = (y[((k-1)-(1))+ _y_offset]-(c[((k-1)-(1))+ _c_offset]*y[(k-(1))+ _y_offset]));
}
else  {
  temp = y[((k-1)-(1))+ _y_offset];
y[((k-1)-(1))+ _y_offset] = y[(k-(1))+ _y_offset];
y[(k-(1))+ _y_offset] = (temp-(c[((k-1)-(1))+ _c_offset]*y[(k-(1))+ _y_offset]));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlagts",90);
}              //  Close for() loop. 
}
/***
*
*     End of DLAGTS
*
***/
}              //  Close else.
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlagts",999999);
return;
   }
} // End class.
