/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlahqr.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlahqr.java,v 1.6 2013/01/27 16:28:34 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlahqr {

// *> \brief \b DLAHQR computes the eigenvalues and Schur factorization of 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAHQR + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAHQR( WANTT, WANTZ, N, ILO, IHI, H, LDH, WR, WI,
// *                          ILOZ, IHIZ, Z, LDZ, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IHI, IHIZ, ILO, ILOZ, INFO, LDH, LDZ, N
// *       LOGICAL            WANTT, WANTZ
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   H( LDH, * ), WI( * ), WR( * ), Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DLAHQR is an auxiliary routine called by DHSEQR to update the
// *>    eigenvalues and Schur decomposition already computed by DHSEQR, by
// *>    dealing with the Hessenberg submatrix in rows and columns ILO to
// *>    IHI.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] WANTT
// *> \verbatim
// *>          WANTT is LOGICAL
// *>          = .TRUE. : the full Schur form T is required;
// *>          = .FALSE.: only eigenvalues are required.
// *> \endverbatim
// *>
// *> \param[in] WANTZ
// *> \verbatim
// *>          WANTZ is LOGICAL
// *>          = .TRUE. : the matrix of Schur vectors Z is required;
// *>          = .FALSE.: Schur vectors are not required.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix H.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>          It is assumed that H is already upper quasi-triangular in
// *>          rows and columns IHI+1:N, and that H(ILO,ILO-1) = 0 (unless

// *>          ILO = 1). DLAHQR works primarily with the Hessenberg
// *>          submatrix in rows and columns ILO to IHI, but applies
// *>          transformations to all of H if WANTT is .TRUE..
// *>          1 <= ILO <= max(1,IHI); IHI <= N.
// *> \endverbatim
// *>
// *> \param[in,out] H
// *> \verbatim
// *>          H is DOUBLE PRECISION array, dimension (LDH,N)
// *>          On entry, the upper Hessenberg matrix H.
// *>          On exit, if INFO is zero and if WANTT is .TRUE., H is upper

// *>          quasi-triangular in rows and columns ILO:IHI, with any
// *>          2-by-2 diagonal blocks in standard form. If INFO is zero
// *>          and WANTT is .FALSE., the contents of H are unspecified on
// *>          exit.  The output state of H if INFO is nonzero is given
// *>          below under the description of INFO.
// *> \endverbatim
// *>
// *> \param[in] LDH
// *> \verbatim
// *>          LDH is INTEGER
// *>          The leading dimension of the array H. LDH >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WR
// *> \verbatim
// *>          WR is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] WI
// *> \verbatim
// *>          WI is DOUBLE PRECISION array, dimension (N)
// *>          The real and imaginary parts, respectively, of the computed

// *>          eigenvalues ILO to IHI are stored in the corresponding
// *>          elements of WR and WI. If two eigenvalues are computed as a

// *>          complex conjugate pair, they are stored in consecutive
// *>          elements of WR and WI, say the i-th and (i+1)th, with
// *>          WI(i) > 0 and WI(i+1) < 0. If WANTT is .TRUE., the
// *>          eigenvalues are stored in the same order as on the diagonal

// *>          of the Schur form returned in H, with WR(i) = H(i,i), and, i
// *>          H(i:i+1,i:i+1) is a 2-by-2 diagonal block,
// *>          WI(i) = sqrt(H(i+1,i)*H(i,i+1)) and WI(i+1) = -WI(i).
// *> \endverbatim
// *>
// *> \param[in] ILOZ
// *> \verbatim
// *>          ILOZ is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHIZ
// *> \verbatim
// *>          IHIZ is INTEGER
// *>          Specify the rows of Z to which transformations must be
// *>          applied if WANTZ is .TRUE..
// *>          1 <= ILOZ <= ILO; IHI <= IHIZ <= N.
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ,N)
// *>          If WANTZ is .TRUE., on entry Z must contain the current
// *>          matrix Z of transformations accumulated by DHSEQR, and on
// *>          exit Z has been updated; transformations are applied only to
// *>          the submatrix Z(ILOZ:IHIZ,ILO:IHI).
// *>          If WANTZ is .FALSE., Z is not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z. LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>           =   0: successful exit
// *>          .GT. 0: If INFO = i, DLAHQR failed to compute all the
// *>                  eigenvalues ILO to IHI in a total of 30 iterations
// *>                  per eigenvalue; elements i+1:ihi of WR and WI
// *>                  contain those eigenvalues which have been
// *>                  successfully computed.
// *>
// *>                  If INFO .GT. 0 and WANTT is .FALSE., then on exit,
// *>                  the remaining unconverged eigenvalues are the
// *>                  eigenvalues of the upper Hessenberg matrix rows
// *>                  and columns ILO thorugh INFO of the final, output
// *>                  value of H.
// *>
// *>                  If INFO .GT. 0 and WANTT is .TRUE., then on exit
// *>          (*)       (initial value of H)*U  = U*(final value of H)
// *>                  where U is an orthognal matrix.    The final
// *>                  value of H is upper Hessenberg and triangular in
// *>                  rows and columns INFO+1 through IHI.
// *>
// *>                  If INFO .GT. 0 and WANTZ is .TRUE., then on exit
// *>                      (final value of Z)  = (initial value of Z)*U
// *>                  where U is the orthogonal matrix in (*)
// *>                  (regardless of the value of WANTT.)
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>     02-96 Based on modifications by
// *>     David Day, Sandia National Laboratory, USA
// *>
// *>     12-04 Further modifications by
// *>     Ralph Byers, University of Kansas, USA
// *>     This is a modified version of DLAHQR from LAPACK version 3.0.
// *>     It is (1) more robust against overflow and underflow and
// *>     (2) adopts the more conservative Ahues & Tisseur stopping
// *>     criterion (LAWN 122, 1997).
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =========================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
/***
*
*     Quick return if possible
*
***/

public static void dlahqr (boolean wantt,
boolean wantz,
int n,
int ilo,
int ihi,
double [] h, int _h_offset,
int ldh,
double [] wr, int _wr_offset,
double [] wi, int _wi_offset,
int iloz,
int ihiz,
double [] z, int _z_offset,
int ldz,
intW info)  {

double aa= 0.0d;
double ab= 0.0d;
double ba= 0.0d;
double bb= 0.0d;
doubleW cs= new doubleW(0.0d);
double det= 0.0d;
double h11= 0.0d;
double h12= 0.0d;
double h21= 0.0d;
double h21s= 0.0d;
double h22= 0.0d;
double rt1i= 0.0d;
double rt1r= 0.0d;
double rt2i= 0.0d;
double rt2r= 0.0d;
double rtdisc= 0.0d;
double s= 0.0d;
doubleW safmax= new doubleW(0.0d);
doubleW safmin= new doubleW(0.0d);
double smlnum= 0.0d;
doubleW sn= new doubleW(0.0d);
double sum= 0.0d;
doubleW t1= new doubleW(0.0d);
double t2= 0.0d;
double t3= 0.0d;
double tr= 0.0d;
double tst= 0.0d;
double ulp= 0.0d;
double v2= 0.0d;
double v3= 0.0d;
int i= 0;
int i1= 0;
int i2= 0;
int its= 0;
int j= 0;
int k= 0;
int l= 0;
int m= 0;
int nh= 0;
int nr= 0;
int nz= 0;
double [] v= new double[(3)];
final int ITMAX=150,NLOOPMAX=2*n+Util.ITMAX;
info.val = 0;
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",999999);
}
    if ((ilo == ihi))  {
    wr[(ilo-(1))+ _wr_offset] = h[(ilo-(1))+(ilo-(1)) * (ldh)+ _h_offset];
wi[(ilo-(1))+ _wi_offset] = 0.0;
return;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",999999);
/***
*
*     ==== clear out the trash ====
***/
}
{
for (j = ilo; j <= (ihi-3); j++) {
h[((j+2)-(1))+(j-(1)) * (ldh)+ _h_offset] = 0.0;
h[((j+3)-(1))+(j-(1)) * (ldh)+ _h_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dlahqr",10);
}              //  Close for() loop. 
}
// *
if ((ilo <= (ihi-2))) {
    h[(ihi-(1))+((ihi-2)-(1)) * (ldh)+ _h_offset] = 0.0;
}
    nh = ((ihi-ilo)+1);
/***
*
*     Set machine-dependent constants for the stopping criterion.
*
***/
nz = ((ihiz-iloz)+1);
safmin.val = com.polytechnik.lapack.Dlamch.dlamch('S');
safmax.val = (1.0/safmin.val);
com.polytechnik.lapack.Dlabad.dlabad(safmin,safmax);
ulp = com.polytechnik.lapack.Dlamch.dlamch('P');
/***
*
*     I1 and I2 are the indices of the first row and last column of H
*     to which transformations must be applied. If eigenvalues only are
*     being computed, I1 and I2 are set inside the main loop.
*
***/
smlnum = (safmin.val*(((double)(nh)/ulp)));
if (wantt)  {
    i1 = 1;
i2 = n;
/***
*
*     The main loop begins here. I is the loop index and decreases from
*     IHI to ILO in steps of 1 or 2. Each iteration of the loop works
*     with the active submatrix in rows and columns L to I.
*     Eigenvalues I+1 to IHI have already converged. Either L = ILO or
*     H(L,L-1) is negligible so that the matrix splits.
*
***/
}
i = ihi;
label20:for(int jj20=0;jj20<NLOOPMAX;jj20++){//Dummy.label("com/polytechnik/lapack/Dlahqr",20);

l = ilo;
/***
*
*     Perform QR iterations on rows and columns ILO to I until a
*     submatrix of order 1 or 2 splits off at the bottom because a
*     subdiagonal element has become negligible.
*
***/
if ((i < ilo)) {
    break label20;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",160);
}
    /***
*
*        Look for a single small subdiagonal element.
*
***/
label150:{
{
label140:for (its = 0; its <= ITMAX; its++) {
{
int _k_inc = -1;
label30:for (k = i; k >= (l+1); k += _k_inc) {
if ((Math.abs(h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]) <= smlnum)) {
    break label30;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",40);
}
    tst = (Math.abs(h[((k-1)-(1))+((k-1)-(1)) * (ldh)+ _h_offset])+Math.abs(h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]));
if ((tst == 0.0))  {
    if (((k-2) >= ilo)) {
    tst = (tst+Math.abs(h[((k-1)-(1))+((k-2)-(1)) * (ldh)+ _h_offset]));
}
    if (((k+1) <= ihi)) {
    tst = (tst+Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]));
}
    /***
*           ==== The following is a conservative small subdiagonal
*           .    deflation  criterion due to Ahues & Tisseur (LAWN 122,
*           .    1997). It has better mathematical foundation and
*           .    improves accuracy in some cases.  ====
***/
}
if ((Math.abs(h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]) <= (ulp*tst)))  {
    ab = Math.max(Math.abs(h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]), Math.abs(h[((k-1)-(1))+(k-(1)) * (ldh)+ _h_offset]));
ba = Math.min(Math.abs(h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]), Math.abs(h[((k-1)-(1))+(k-(1)) * (ldh)+ _h_offset]));
aa = Math.max(Math.abs(h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]), Math.abs((h[((k-1)-(1))+((k-1)-(1)) * (ldh)+ _h_offset]-h[(k-(1))+(k-(1)) * (ldh)+ _h_offset])));
bb = Math.min(Math.abs(h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]), Math.abs((h[((k-1)-(1))+((k-1)-(1)) * (ldh)+ _h_offset]-h[(k-(1))+(k-(1)) * (ldh)+ _h_offset])));
s = (aa+ab);
if (((ba*((ab/s))) <= Math.max(smlnum, (ulp*((bb*((aa/s)))))))) {
    break label30;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",40);
}
    }
//Dummy.label("com/polytechnik/lapack/Dlahqr",30);
}              //  Close for() loop. 
}
//label40:
//   Dummy.label("com/polytechnik/lapack/Dlahqr",40);
l = k;
/***
*
*           H(L,L-1) is negligible
*
***/
if ((l > ilo))  {
    h[(l-(1))+((l-1)-(1)) * (ldh)+ _h_offset] = 0.0;
/***
*
*        Exit from loop if a submatrix of order 1 or 2 has split off.
*
***/
}
/***
*
*        Now the active submatrix is in rows and columns L to I. If
*        eigenvalues only are being computed, only the active submatrix
*        need be transformed.
*
***/
if ((l >= (i-1))) {
    break label150;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",150);
}
    if ((!wantt))  {
    i1 = l;
i2 = i;
// *
}
/***
*
*           Exceptional shift.
*
***/
if ((its == 10))  {
    s = (Math.abs(h[((l+1)-(1))+(l-(1)) * (ldh)+ _h_offset])+Math.abs(h[((l+2)-(1))+((l+1)-(1)) * (ldh)+ _h_offset]));
h11 = ((0.75*s)+h[(l-(1))+(l-(1)) * (ldh)+ _h_offset]);
h12 = (-0.4375*s);
h21 = s;
h22 = h11;
/***
*
*           Exceptional shift.
*
***/
}
else if ((its == 20))  {
    s = (Math.abs(h[(i-(1))+((i-1)-(1)) * (ldh)+ _h_offset])+Math.abs(h[((i-1)-(1))+((i-2)-(1)) * (ldh)+ _h_offset]));
h11 = ((0.75*s)+h[(i-(1))+(i-(1)) * (ldh)+ _h_offset]);
h12 = (-0.4375*s);
h21 = s;
h22 = h11;
/***
*
*           Prepare to use Francis' double shift
*           (i.e. 2nd degree generalized Rayleigh quotient)
*
***/
}              // Close else if()
else  {
  h11 = h[((i-1)-(1))+((i-1)-(1)) * (ldh)+ _h_offset];
h21 = h[(i-(1))+((i-1)-(1)) * (ldh)+ _h_offset];
h12 = h[((i-1)-(1))+(i-(1)) * (ldh)+ _h_offset];
h22 = h[(i-(1))+(i-(1)) * (ldh)+ _h_offset];
}              //  Close else.
s = (((Math.abs(h11)+Math.abs(h12))+Math.abs(h21))+Math.abs(h22));
if ((s == 0.0))  {
    rt1r = 0.0;
rt1i = 0.0;
rt2r = 0.0;
rt2i = 0.0;
}
else  {
  h11 = (h11/s);
h21 = (h21/s);
h12 = (h12/s);
h22 = (h22/s);
tr = (((h11+h22))/2.0);
det = ((((h11-tr))*((h22-tr)))-(h12*h21));
rtdisc = Math.sqrt(Math.abs(det));
/***
*
*              ==== complex conjugate shifts ====
*
***/
if ((det >= 0.0))  {
    rt1r = (tr*s);
rt2r = rt1r;
rt1i = (rtdisc*s);
rt2i = (-(rt1i));
/***
*
*              ==== real shifts (use only one of them)  ====
*
***/
}
else  {
  rt1r = (tr+rtdisc);
rt2r = (tr-rtdisc);
if ((Math.abs((rt1r-h22)) <= Math.abs((rt2r-h22))))  {
    rt1r = (rt1r*s);
rt2r = rt1r;
}
else  {
  rt2r = (rt2r*s);
rt1r = rt2r;
}              //  Close else.
rt1i = 0.0;
rt2i = 0.0;
}              //  Close else.
/***
*
*        Look for two consecutive small subdiagonal elements.
*
***/
}              //  Close else.
/***
*           Determine the effect of starting the double-shift QR
*           iteration at row M, and see if this would make H(M,M-1)
*           negligible.  (The following uses scaling to avoid
*           overflows and most underflows.)
*
***/
{
int _m_inc = -1;
label50:for (m = (i-2); m >= l; m += _m_inc) {
h21s = h[((m+1)-(1))+(m-(1)) * (ldh)+ _h_offset];
s = ((Math.abs((h[(m-(1))+(m-(1)) * (ldh)+ _h_offset]-rt2r))+Math.abs(rt2i))+Math.abs(h21s));
h21s = (h[((m+1)-(1))+(m-(1)) * (ldh)+ _h_offset]/s);
v[(1-(1))] = (((h21s*h[(m-(1))+((m+1)-(1)) * (ldh)+ _h_offset])+(((h[(m-(1))+(m-(1)) * (ldh)+ _h_offset]-rt1r))*((((h[(m-(1))+(m-(1)) * (ldh)+ _h_offset]-rt2r))/s))))-(rt1i*((rt2i/s))));
v[(2-(1))] = (h21s*((((h[(m-(1))+(m-(1)) * (ldh)+ _h_offset]+h[((m+1)-(1))+((m+1)-(1)) * (ldh)+ _h_offset])-rt1r)-rt2r)));
v[(3-(1))] = (h21s*h[((m+2)-(1))+((m+1)-(1)) * (ldh)+ _h_offset]);
s = ((Math.abs(v[(1-(1))])+Math.abs(v[(2-(1))]))+Math.abs(v[(3-(1))]));
v[(1-(1))] = (v[(1-(1))]/s);
v[(2-(1))] = (v[(2-(1))]/s);
v[(3-(1))] = (v[(3-(1))]/s);
if ((m == l)) {
    break label50;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",60);
}
    if (((Math.abs(h[(m-(1))+((m-1)-(1)) * (ldh)+ _h_offset])*((Math.abs(v[(2-(1))])+Math.abs(v[(3-(1))])))) <= ((ulp*Math.abs(v[(1-(1))]))*(((Math.abs(h[((m-1)-(1))+((m-1)-(1)) * (ldh)+ _h_offset])+Math.abs(h[(m-(1))+(m-(1)) * (ldh)+ _h_offset]))+Math.abs(h[((m+1)-(1))+((m+1)-(1)) * (ldh)+ _h_offset])))))) {
	break label50;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",60);
}
    //Dummy.label("com/polytechnik/lapack/Dlahqr",50);
}              //  Close for() loop. 
}
/***
*
*        Double-shift QR step
*
***/
//label60:
//   Dummy.label("com/polytechnik/lapack/Dlahqr",60);
/***
*
*           The first iteration of this loop determines a reflection G
*           from the vector V and applies it from left and right to H,
*           thus creating a nonzero bulge below the subdiagonal.
*
*           Each subsequent iteration determines a reflection G to
*           restore the Hessenberg form in the (K-1)th column, and thus
*           chases the bulge one step toward the bottom of the active
*           submatrix. NR is the order of G.
*
***/
{
for (k = m; k <= (i-1); k++) {
nr = Math.min(3, ((i-k)+1));
if ((k > m)) {
    com.polytechnik.lapack.Dcopy.dcopy(nr,h,(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset,1,v,0,1);
}
    dlarfg_adapter(nr,v,(1-(1)),v,(2-(1)),1,t1);
if ((k > m))  {
    h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset] = v[(1-(1))];
h[((k+1)-(1))+((k-1)-(1)) * (ldh)+ _h_offset] = 0.0;
if ((k < (i-1))) {
    h[((k+2)-(1))+((k-1)-(1)) * (ldh)+ _h_offset] = 0.0;
}
    /***
*               ==== Use the following instead of
*               .    H( K, K-1 ) = -H( K, K-1 ) to
*               .    avoid a bug when v(2) and v(3)
*               .    underflow. ====
***/
}
else if ((m > l))  {
    h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset] = (h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]*((1.0-t1.val)));
}              // Close else if()
v2 = v[(2-(1))];
t2 = (t1.val*v2);
if ((nr == 3))  {
    v3 = v[(3-(1))];
/***
*
*              Apply G from the left to transform the rows of the matrix
*              in columns K to I2.
*
***/
t3 = (t1.val*v3);
{
for (j = k; j <= i2; j++) {
sum = ((h[(k-(1))+(j-(1)) * (ldh)+ _h_offset]+(v2*h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]))+(v3*h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset]));
h[(k-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[(k-(1))+(j-(1)) * (ldh)+ _h_offset]-(sum*t1.val));
h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]-(sum*t2));
h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset]-(sum*t3));
/***
*
*              Apply G from the right to transform the columns of the
*              matrix in rows I1 to min(K+3,I).
*
***/
//Dummy.label("com/polytechnik/lapack/Dlahqr",70);
}              //  Close for() loop. 
}
{
for (j = i1; j <= Math.min((k+3), i); j++) {
sum = ((h[(j-(1))+(k-(1)) * (ldh)+ _h_offset]+(v2*h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]))+(v3*h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset]));
h[(j-(1))+(k-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+(k-(1)) * (ldh)+ _h_offset]-(sum*t1.val));
h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-(sum*t2));
h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset]-(sum*t3));
// *
//Dummy.label("com/polytechnik/lapack/Dlahqr",80);
}              //  Close for() loop. 
}
/***
*
*                 Accumulate transformations in the matrix Z
*
***/
if (wantz)  {
    {
for (j = iloz; j <= ihiz; j++) {
sum = ((z[(j-(1))+(k-(1)) * (ldz)+ _z_offset]+(v2*z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]))+(v3*z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset]));
z[(j-(1))+(k-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+(k-(1)) * (ldz)+ _z_offset]-(sum*t1.val));
z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]-(sum*t2));
z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset]-(sum*t3));
//Dummy.label("com/polytechnik/lapack/Dlahqr",90);
}              //  Close for() loop. 
}
}
/***
*
*              Apply G from the left to transform the rows of the matrix
*              in columns K to I2.
*
***/
}
else if ((nr == 2))  {
    {
for (j = k; j <= i2; j++) {
sum = (h[(k-(1))+(j-(1)) * (ldh)+ _h_offset]+(v2*h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]));
h[(k-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[(k-(1))+(j-(1)) * (ldh)+ _h_offset]-(sum*t1.val));
h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]-(sum*t2));
/***
*
*              Apply G from the right to transform the columns of the
*              matrix in rows I1 to min(K+3,I).
*
***/
//Dummy.label("com/polytechnik/lapack/Dlahqr",100);
}              //  Close for() loop. 
}
{
for (j = i1; j <= i; j++) {
sum = (h[(j-(1))+(k-(1)) * (ldh)+ _h_offset]+(v2*h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
h[(j-(1))+(k-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+(k-(1)) * (ldh)+ _h_offset]-(sum*t1.val));
h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-(sum*t2));
// *
//Dummy.label("com/polytechnik/lapack/Dlahqr",110);
}              //  Close for() loop. 
}
/***
*
*                 Accumulate transformations in the matrix Z
*
***/
if (wantz)  {
    {
for (j = iloz; j <= ihiz; j++) {
sum = (z[(j-(1))+(k-(1)) * (ldz)+ _z_offset]+(v2*z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]));
z[(j-(1))+(k-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+(k-(1)) * (ldz)+ _z_offset]-(sum*t1.val));
z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]-(sum*t2));
//Dummy.label("com/polytechnik/lapack/Dlahqr",120);
}              //  Close for() loop. 
}
}
}              // Close else if()
// *
//Dummy.label("com/polytechnik/lapack/Dlahqr",130);
}              //  Close for() loop. 
}
/***
*
*     Failure to converge in remaining number of iterations
*
***/
//Dummy.label("com/polytechnik/lapack/Dlahqr",140);
}              //  Close for() loop. 
}
info.val = i;
// *
return;//Dummy.go_to("com/polytechnik/lapack/Dlahqr",999999);
// *
} //label150: Dummy.label("com/polytechnik/lapack/Dlahqr",150);
/***
*
*        H(I,I-1) is negligible: one eigenvalue has converged.
*
***/
if ((l == i))  {
    wr[(i-(1))+ _wr_offset] = h[(i-(1))+(i-(1)) * (ldh)+ _h_offset];
wi[(i-(1))+ _wi_offset] = 0.0;
/***
*
*        H(I-1,I-2) is negligible: a pair of eigenvalues have converged.
*
*        Transform the 2-by-2 submatrix to standard Schur form,
*        and compute and store the eigenvalues.
*
***/
}
else if ((l == (i-1)))  {
    // *
dlanv2_adapter(h,((i-1)-(1))+((i-1)-(1)) * (ldh)+ _h_offset,h,((i-1)-(1))+(i-(1)) * (ldh)+ _h_offset,h,(i-(1))+((i-1)-(1)) * (ldh)+ _h_offset,h,(i-(1))+(i-(1)) * (ldh)+ _h_offset,wr,((i-1)-(1))+ _wr_offset,wi,((i-1)-(1))+ _wi_offset,wr,(i-(1))+ _wr_offset,wi,(i-(1))+ _wi_offset,cs,sn);
/***
*
*           Apply the transformation to the rest of H.
*
***/
if (wantt)  {
    if ((i2 > i)) {
    com.polytechnik.lapack.Drot.drot((i2-i),h,((i-1)-(1))+((i+1)-(1)) * (ldh)+ _h_offset,ldh,h,(i-(1))+((i+1)-(1)) * (ldh)+ _h_offset,ldh,cs.val,sn.val);
}
    com.polytechnik.lapack.Drot.drot(((i-i1)-1),h,(i1-(1))+((i-1)-(1)) * (ldh)+ _h_offset,1,h,(i1-(1))+(i-(1)) * (ldh)+ _h_offset,1,cs.val,sn.val);
}
/***
*
*           Apply the transformation to Z.
*
***/
if (wantz)  {
    com.polytechnik.lapack.Drot.drot(nz,z,(iloz-(1))+((i-1)-(1)) * (ldz)+ _z_offset,1,z,(iloz-(1))+(i-(1)) * (ldz)+ _z_offset,1,cs.val,sn.val);
}
/***
*
*     return to start of the main loop with new value of I.
*
***/
}              // Close else if()
i = (l-1);
// *
// no break label20
} //Dummy.go_to("com/polytechnik/lapack/Dlahqr",20);
//label160:
//   Dummy.label("com/polytechnik/lapack/Dlahqr",160);
/***
*
*     End of DLAHQR
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlahqr",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlahqr",999999);
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,doubleW arg4 )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);

com.polytechnik.lapack.Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,arg4);

arg1[arg1_offset] = _f2j_tmp1.val;
}

// adapter for dlanv2(Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;)V
private static void dlanv2_adapter(double [] arg0 , int arg0_offset ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,double [] arg3 , int arg3_offset ,double [] arg4 , int arg4_offset ,double [] arg5 , int arg5_offset ,double [] arg6 , int arg6_offset ,double [] arg7 , int arg7_offset ,doubleW arg8 ,doubleW arg9 )
{
doubleW _f2j_tmp0 = new doubleW(arg0[arg0_offset]);
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp2 = new doubleW(arg2[arg2_offset]);
doubleW _f2j_tmp3 = new doubleW(arg3[arg3_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);
doubleW _f2j_tmp5 = new doubleW(arg5[arg5_offset]);
doubleW _f2j_tmp6 = new doubleW(arg6[arg6_offset]);
doubleW _f2j_tmp7 = new doubleW(arg7[arg7_offset]);

com.polytechnik.lapack.Dlanv2.dlanv2(_f2j_tmp0,_f2j_tmp1,_f2j_tmp2,_f2j_tmp3,_f2j_tmp4,_f2j_tmp5,_f2j_tmp6,_f2j_tmp7,arg8,arg9);

arg0[arg0_offset] = _f2j_tmp0.val;
arg1[arg1_offset] = _f2j_tmp1.val;
arg2[arg2_offset] = _f2j_tmp2.val;
arg3[arg3_offset] = _f2j_tmp3.val;
arg4[arg4_offset] = _f2j_tmp4.val;
arg5[arg5_offset] = _f2j_tmp5.val;
arg6[arg6_offset] = _f2j_tmp6.val;
arg7[arg7_offset] = _f2j_tmp7.val;
}

} // End class.
