/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaneg.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlaneg.java,v 1.2 2012/12/15 18:16:27 mal Exp $
 *
 *
 */

package com.polytechnik.lapack;



public class Dlaneg {

// *> \brief \b DLANEG computes the Sturm count.
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLANEG + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       INTEGER FUNCTION DLANEG( N, D, LLD, SIGMA, PIVMIN, R )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            N, R
// *       DOUBLE PRECISION   PIVMIN, SIGMA
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), LLD( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLANEG computes the Sturm count, the number of negative pivots
// *> encountered while factoring tridiagonal T - sigma I = L D L^T.
// *> This implementation works directly on the factors without forming
// *> the tridiagonal matrix T.  The Sturm count is also the number of
// *> eigenvalues of T less than sigma.
// *>
// *> This routine is called from DLARRB.
// *>
// *> The current routine does not use the PIVMIN parameter but rather
// *> requires IEEE-754 propagation of Infinities and NaNs.  This
// *> routine also has no input range restrictions but does require
// *> default exception handling such that x/0 produces Inf when x is
// *> non-zero, and Inf/Inf produces NaN.  For more information, see:
// *>
// *>   Marques, Riedy, and Voemel, "Benefits of IEEE-754 Features in
// *>   Modern Symmetric Tridiagonal Eigensolvers," SIAM Journal on
// *>   Scientific Computing, v28, n5, 2006.  DOI 10.1137/050641624
// *>   (Tech report version in LAWN 172 with the same title.)
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The N diagonal elements of the diagonal matrix D.
// *> \endverbatim
// *>
// *> \param[in] LLD
// *> \verbatim
// *>          LLD is DOUBLE PRECISION array, dimension (N-1)
// *>          The (N-1) elements L(i)*L(i)*D(i).
// *> \endverbatim
// *>
// *> \param[in] SIGMA
// *> \verbatim
// *>          SIGMA is DOUBLE PRECISION
// *>          Shift amount in T - sigma I = L D L^T.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot in the Sturm sequence.  May be used
// *>          when zero pivots are encountered on non-IEEE-754
// *>          architectures.
// *> \endverbatim
// *>
// *> \param[in] R
// *> \verbatim
// *>          R is INTEGER
// *>          The twist index for the twisted factorization that is used
// *>          for the negcount.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *>     Osni Marques, LBNL/NERSC, USA \n
// *>     Christof Voemel, University of California, Berkeley, USA \n
// *>     Jason Riedy, University of California, Berkeley, USA \n
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     Some architectures propagate Infinities and NaNs very slowly, so
*     the code computes counts in BLKLEN chunks.  Then a NaN can
*     propagate at most BLKLEN columns before being detected.  This is
*     not a general tuning parameter; it needs only to be just large
*     enough that the overhead is tiny in common cases.
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..

***/
/***

*     I) upper part: L D L^T - SIGMA I = L+ D+ L+^T
***/

public static int dlaneg (int n,
double [] d, int _d_offset,
double [] lld, int _lld_offset,
double sigma,
double pivmin,
int r)  {

int dlaneg= 0;
int bj= 0;
int j= 0;
int neg1= 0;
int neg2= 0;
int negcnt= 0;
double bsav= 0.0d;
double dminus= 0.0d;
double dplus= 0.0d;
double gamma= 0.0d;
double p= 0.0d;
double t= 0.0d;
double tmp= 0.0d;
boolean sawnan= false;
negcnt = 0;
t = (-(sigma));
{
int _bj_inc = 128;
for (bj = 1; bj <= (r-1); bj += _bj_inc) {
neg1 = 0;
bsav = t;
{
for (j = bj; j <= Math.min(((bj+128)-1), (r-1)); j++) {
dplus = (d[(j-(1))+ _d_offset]+t);
if ((dplus < 0.0)) {
    neg1 = (neg1+1);
}
    tmp = (t/dplus);
t = ((tmp*lld[(j-(1))+ _lld_offset])-sigma);
//Dummy.label("com/polytechnik/lapack/Dlaneg",21);
}              //  Close for() loop. 
}
/***
*     Run a slower version of the above loop if a NaN is detected.
*     A NaN should occur only with a zero pivot after an infinite
*     pivot.  In that case, substituting 1 for T/DPLUS is the
*     correct limit.
***/
sawnan = com.polytechnik.lapack.Disnan.disnan(t);
if (sawnan)  {
    neg1 = 0;
t = bsav;
{
for (j = bj; j <= Math.min(((bj+128)-1), (r-1)); j++) {
dplus = (d[(j-(1))+ _d_offset]+t);
if ((dplus < 0.0)) {
    neg1 = (neg1+1);
}
    tmp = (t/dplus);
if (com.polytechnik.lapack.Disnan.disnan(tmp)) {
    tmp = 1.0;
}
    t = ((tmp*lld[(j-(1))+ _lld_offset])-sigma);
//Dummy.label("com/polytechnik/lapack/Dlaneg",22);
}              //  Close for() loop. 
}
}
negcnt = (negcnt+neg1);
/***
*
*     II) lower part: L D L^T - SIGMA I = U- D- U-^T
***/
//Dummy.label("com/polytechnik/lapack/Dlaneg",210);
}              //  Close for() loop. 
}
p = (d[(n-(1))+ _d_offset]-sigma);
{
int _bj_inc = -128;
for (bj = (n-1); bj >= r; bj += _bj_inc) {
neg2 = 0;
bsav = p;
{
int _j_inc = -1;
for (j = bj; j >= Math.max(((bj-128)+1), r); j += _j_inc) {
dminus = (lld[(j-(1))+ _lld_offset]+p);
if ((dminus < 0.0)) {
    neg2 = (neg2+1);
}
    tmp = (p/dminus);
p = ((tmp*d[(j-(1))+ _d_offset])-sigma);
//Dummy.label("com/polytechnik/lapack/Dlaneg",23);
}              //  Close for() loop. 
}
/***
*     As above, run a slower version that substitutes 1 for Inf/Inf.
*
***/
sawnan = com.polytechnik.lapack.Disnan.disnan(p);
if (sawnan)  {
    neg2 = 0;
p = bsav;
{
int _j_inc = -1;
for (j = bj; j >= Math.max(((bj-128)+1), r); j += _j_inc) {
dminus = (lld[(j-(1))+ _lld_offset]+p);
if ((dminus < 0.0)) {
    neg2 = (neg2+1);
}
    tmp = (p/dminus);
if (com.polytechnik.lapack.Disnan.disnan(tmp)) {
    tmp = 1.0;
}
    p = ((tmp*d[(j-(1))+ _d_offset])-sigma);
//Dummy.label("com/polytechnik/lapack/Dlaneg",24);
}              //  Close for() loop. 
}
}
negcnt = (negcnt+neg2);
/***
*
*     III) Twist index
*       T was shifted by SIGMA initially.
***/
//Dummy.label("com/polytechnik/lapack/Dlaneg",230);
}              //  Close for() loop. 
}
gamma = (((t+sigma))+p);
// 
if ((gamma < 0.0)) {
    negcnt = (negcnt+1);
}
    dlaneg = negcnt;
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlaneg",999999);
return dlaneg;
   }
} // End class.
