/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlange.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlange.java,v 1.2 2012/12/22 14:18:49 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlange {

// *> \brief \b DLANGE returns the value of the 1-norm, Frobenius norm, inf
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLANGE + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       DOUBLE PRECISION FUNCTION DLANGE( NORM, M, N, A, LDA, WORK )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          NORM
// *       INTEGER            LDA, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLANGE  returns the value of the one norm,  or the Frobenius norm, or
// *> the  infinity norm,  or the  element of  largest absolute value  of a
// *> real matrix A.
// *> \endverbatim
// *>
// *> \return DLANGE
// *> \verbatim
// *>
// *>    DLANGE = ( max(abs(A(i,j))), NORM = 'M' or 'm'
// *>             (
// *>             ( norm1(A),         NORM = '1', 'O' or 'o'
// *>             (
// *>             ( normI(A),         NORM = 'I' or 'i'
// *>             (
// *>             ( normF(A),         NORM = 'F', 'f', 'E' or 'e'
// *>
// *> where  norm1  denotes the  one norm of a matrix (maximum column sum),
// *> normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
// *> normF  denotes the  Frobenius norm of a matrix (square root of sum of
// *> squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix no
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] NORM
// *> \verbatim
// *>          NORM is CHARACTER*1
// *>          Specifies the value to be returned in DLANGE as described
// *>          above.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  M >= 0.  When M = 0,
// *>          DLANGE is set to zero.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  N >= 0.  When N = 0,
// *>          DLANGE is set to zero.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          The m by n matrix A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(M,1).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK)),
// *>          where LWORK >= M when NORM = 'I'; otherwise, WORK is not
// *>          referenced.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleGEauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static double dlange (final char norm,
int m,
int n,
double [] a, int _a_offset,
int lda,
double [] work, int _work_offset)  {

double dlange= 0.0d;
int i= 0;
int j= 0;
doubleW scale= new doubleW(0.0d);
doubleW sum= new doubleW(0.0d);
double value= 0.0d;
double temp= 0.0d;
//norm = norm.substring((1)-1,1);
if ((Math.min(m, n) == 0))  {
    value = 0.0;
/***
*
*        Find max(abs(A(i,j))).
*
***/
}
else if (com.polytechnik.lapack.Lsame.lsame(norm,'M'))  {
    value = 0.0;
{
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
temp = Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]);
if (((value < temp) || com.polytechnik.lapack.Disnan.disnan(temp))) {
    value = temp;
}
//Dummy.label("com/polytechnik/lapack/Dlange",10);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlange",20);
}              //  Close for() loop. 
}
/***
*
*        Find norm1(A).
*
***/
}              // Close else if()
else if (((com.polytechnik.lapack.Lsame.lsame(norm,'O')) || ((com.polytechnik.lapack.Lsame.lsame(norm, '1')))))  {
    value = 0.0;
{
for (j = 1; j <= n; j++) {
sum.val = 0.0;
{
for (i = 1; i <= m; i++) {
sum.val = (sum.val+Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dlange",30);
}              //  Close for() loop. 
}
if (((value < sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    value = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlange",40);
}              //  Close for() loop. 
}
/***
*
*        Find normI(A).
*
***/
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(norm,'I'))  {
    {
for (i = 1; i <= m; i++) {
work[(i-(1))+ _work_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dlange",50);
}              //  Close for() loop. 
}
{
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
work[(i-(1))+ _work_offset] = (work[(i-(1))+ _work_offset]+Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dlange",60);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlange",70);
}              //  Close for() loop. 
}
value = 0.0;
{
for (i = 1; i <= m; i++) {
temp = work[(i-(1))+ _work_offset];
if (((value < temp) || com.polytechnik.lapack.Disnan.disnan(temp))) {
    value = temp;
}
//Dummy.label("com/polytechnik/lapack/Dlange",80);
}              //  Close for() loop. 
}
/***
*
*        Find normF(A).
*
***/
}              // Close else if()
else if (((com.polytechnik.lapack.Lsame.lsame(norm,'F')) || (com.polytechnik.lapack.Lsame.lsame(norm,'E'))))  {
    scale.val = 0.0;
sum.val = 1.0;
{
for (j = 1; j <= n; j++) {
com.polytechnik.lapack.Dlassq.dlassq(m,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1,scale,sum);
//Dummy.label("com/polytechnik/lapack/Dlange",90);
}              //  Close for() loop. 
}
value = (scale.val*Math.sqrt(sum.val));
// *
}              // Close else if()
dlange = value;
/***
*
*     End of DLANGE
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlange",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlange",999999);
return dlange;
   }
} // End class.
