/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlanst.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlanst.java,v 1.4 2012/12/20 04:48:58 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlanst {

// *> \brief \b DLANST returns the value of the 1-norm, or the Frobenius no
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLANST + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       DOUBLE PRECISION FUNCTION DLANST( NORM, N, D, E )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          NORM
// *       INTEGER            N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), E( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLANST  returns the value of the one norm,  or the Frobenius norm, or
// *> the  infinity norm,  or the  element of  largest absolute value  of a
// *> real symmetric tridiagonal matrix A.
// *> \endverbatim
// *>
// *> \return DLANST
// *> \verbatim
// *>
// *>    DLANST = ( max(abs(A(i,j))), NORM = 'M' or 'm'
// *>             (
// *>             ( norm1(A),         NORM = '1', 'O' or 'o'
// *>             (
// *>             ( normI(A),         NORM = 'I' or 'i'
// *>             (
// *>             ( normF(A),         NORM = 'F', 'f', 'E' or 'e'
// *>
// *> where  norm1  denotes the  one norm of a matrix (maximum column sum),
// *> normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
// *> normF  denotes the  Frobenius norm of a matrix (square root of sum of
// *> squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix no
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] NORM
// *> \verbatim
// *>          NORM is CHARACTER*1
// *>          Specifies the value to be returned in DLANST as described
// *>          above.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.  When N = 0, DLANST is
// *>          set to zero.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The diagonal elements of A.
// *> \endverbatim
// *>
// *> \param[in] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          The (n-1) sub-diagonal or super-diagonal elements of A.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static double dlanst (final char norm,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset)  {

double dlanst= 0.0d;
int i= 0;
double anorm= 0.0d;
doubleW scale= new doubleW(0.0d);
doubleW sum= new doubleW(0.0d);
//norm = norm.substring((1)-1,1);
if ((n <= 0))  {
    anorm = 0.0;
/***
*
*        Find max(abs(A(i,j))).
*
***/
}
else if (com.polytechnik.lapack.Lsame.lsame(norm,'M'))  {
    anorm = Math.abs(d[(n-(1))+ _d_offset]);
{
for (i = 1; i <= (n-1); i++) {
sum.val = Math.abs(d[(i-(1))+ _d_offset]);
if ((!(anorm >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    anorm = sum.val;
}
    sum.val = Math.abs(e[(i-(1))+ _e_offset]);
if ((!(anorm >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    anorm = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlanst",10);
}              //  Close for() loop. 
}
/***
*
*        Find norm1(A).
*
***/
}              // Close else if()
else if (((com.polytechnik.lapack.Lsame.lsame(norm,'O') || (com.polytechnik.lapack.Lsame.lsame(norm, '1'))) || com.polytechnik.lapack.Lsame.lsame(norm,'I')))  {
    if ((n == 1))  {
    anorm = Math.abs(d[(1-(1))+ _d_offset]);
}
else  {
  anorm = (Math.abs(d[(1-(1))+ _d_offset])+Math.abs(e[(1-(1))+ _e_offset]));
sum.val = (Math.abs(e[((n-1)-(1))+ _e_offset])+Math.abs(d[(n-(1))+ _d_offset]));
if ((!(anorm >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    anorm = sum.val;
}
    {
for (i = 2; i <= (n-1); i++) {
sum.val = ((Math.abs(d[(i-(1))+ _d_offset])+Math.abs(e[(i-(1))+ _e_offset]))+Math.abs(e[((i-1)-(1))+ _e_offset]));
if ((!(anorm >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    anorm = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlanst",20);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Find normF(A).
*
***/
}              // Close else if()
else if (((com.polytechnik.lapack.Lsame.lsame(norm,'F')) || (com.polytechnik.lapack.Lsame.lsame(norm,'E'))))  {
    scale.val = 0.0;
sum.val = 1.0;
if ((n > 1))  {
    com.polytechnik.lapack.Dlassq.dlassq((n-1),e,_e_offset,1,scale,sum);
sum.val = (2*sum.val);
}
com.polytechnik.lapack.Dlassq.dlassq(n,d,_d_offset,1,scale,sum);
anorm = (scale.val*Math.sqrt(sum.val));
// *
}              // Close else if()
dlanst = anorm;
/***
*
*     End of DLANST
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlanst",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlanst",999999);
return dlanst;
   }
} // End class.
