/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarnv.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarnv.java,v 1.2 2012/12/16 09:07:28 mal Exp $
 *
 */

package com.polytechnik.lapack;


public class Dlarnv {

// *> \brief \b DLARNV returns a vector of random numbers from a uniform or
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARNV + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARNV( IDIST, ISEED, N, X )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IDIST, N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISEED( 4 )
// *       DOUBLE PRECISION   X( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARNV returns a vector of n random real numbers from a uniform or
// *> normal distribution.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] IDIST
// *> \verbatim
// *>          IDIST is INTEGER
// *>          Specifies the distribution of the random numbers:
// *>          = 1:  uniform (0,1)
// *>          = 2:  uniform (-1,1)
// *>          = 3:  normal (0,1)
// *> \endverbatim
// *>
// *> \param[in,out] ISEED
// *> \verbatim
// *>          ISEED is INTEGER array, dimension (4)
// *>          On entry, the seed of the random number generator; the array
// *>          elements must be between 0 and 4095, and ISEED(4) must be
// *>          odd.
// *>          On exit, the seed is updated.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of random numbers to be generated.
// *> \endverbatim
// *>
// *> \param[out] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array, dimension (N)
// *>          The generated random numbers.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  This routine calls the auxiliary routine DLARUV to generate random
// *>  real numbers from a uniform (0,1) distribution, in batches of up to

// *>  128 using vectorisable code. The Box-Muller method is used to
// *>  transform numbers from a uniform to a normal distribution.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarnv (int idist,
final java.util.Random iseed, int _iseed_offset,
int n,
double [] x, int _x_offset)  {

if ((idist == 1))  {
    {
for (int i = 1; i <= n; i++) {
    x[((i)-(1))+ _x_offset] = iseed.nextDouble();
//Dummy.label("com/polytechnik/lapack/Dlarnv",10);
}              //  Close for() loop. 
}
}
else if ((idist == 2))  {
    {
for (int i = 1; i <= n; i++) {
    x[((i)-(1))+ _x_offset] = 2*iseed.nextDouble()-1;
//Dummy.label("com/polytechnik/lapack/Dlarnv",20);
}              //  Close for() loop. 
}
}
else
{
    throw new RuntimeException("idist="+idist);
}

}
} // End class.
