/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarra.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarra.java,v 1.2 2012/12/15 18:28:23 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarra {

// *> \brief \b DLARRA computes the splitting points with the specified thr
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRA + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRA( N, D, E, E2, SPLTOL, TNRM,
// *                           NSPLIT, ISPLIT, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, N, NSPLIT
// *       DOUBLE PRECISION    SPLTOL, TNRM
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISPLIT( * )
// *       DOUBLE PRECISION   D( * ), E( * ), E2( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Compute the splitting points with threshold SPLTOL.
// *> DLARRA sets any "small" off-diagonal elements to zero.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix. N > 0.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the N diagonal elements of the tridiagonal
// *>          matrix T.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the first (N-1) entries contain the subdiagonal
// *>          elements of the tridiagonal matrix T; E(N) need not be set.

// *>          On exit, the entries E( ISPLIT( I ) ), 1 <= I <= NSPLIT,
// *>          are set to zero, the other entries of E are untouched.
// *> \endverbatim
// *>
// *> \param[in,out] E2
// *> \verbatim
// *>          E2 is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the first (N-1) entries contain the SQUARES of the
// *>          subdiagonal elements of the tridiagonal matrix T;
// *>          E2(N) need not be set.
// *>          On exit, the entries E2( ISPLIT( I ) ),
// *>          1 <= I <= NSPLIT, have been set to zero
// *> \endverbatim
// *>
// *> \param[in] SPLTOL
// *> \verbatim
// *>          SPLTOL is DOUBLE PRECISION
// *>          The threshold for splitting. Two criteria can be used:
// *>          SPLTOL<0 : criterion based on absolute off-diagonal value
// *>          SPLTOL>0 : criterion that preserves relative accuracy
// *> \endverbatim
// *>
// *> \param[in] TNRM
// *> \verbatim
// *>          TNRM is DOUBLE PRECISION
// *>          The norm of the matrix.
// *> \endverbatim
// *>
// *> \param[out] NSPLIT
// *> \verbatim
// *>          NSPLIT is INTEGER
// *>          The number of blocks T splits into. 1 <= NSPLIT <= N.
// *> \endverbatim
// *>
// *> \param[out] ISPLIT
// *> \verbatim
// *>          ISPLIT is INTEGER array, dimension (N)
// *>          The splitting points, at which T breaks up into blocks.
// *>          The first block consists of rows/columns 1 to ISPLIT(1),
// *>          the second of rows/columns ISPLIT(1)+1 through ISPLIT(2),
// *>          etc., and the NSPLIT-th consists of rows/columns
// *>          ISPLIT(NSPLIT-1)+1 through ISPLIT(NSPLIT)=N.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***

*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
/***

*     Compute splitting points
***/

public static void dlarra (int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] e2, int _e2_offset,
double spltol,
double tnrm,
intW nsplit,
int [] isplit, int _isplit_offset,
intW info)  {

int i= 0;
double eabs= 0.0d;
double tmp1= 0.0d;
info.val = 0;
nsplit.val = 1;
// *        Criterion based on absolute off-diagonal value
if ((spltol < 0.0))  {
    tmp1 = (Math.abs(spltol)*tnrm);
{
for (i = 1; i <= (n-1); i++) {
eabs = Math.abs(e[(i-(1))+ _e_offset]);
if ((eabs <= tmp1))  {
    e[(i-(1))+ _e_offset] = 0.0;
e2[(i-(1))+ _e2_offset] = 0.0;
isplit[(nsplit.val-(1))+ _isplit_offset] = i;
nsplit.val = (nsplit.val+1);
}
//Dummy.label("com/polytechnik/lapack/Dlarra",9);
}              //  Close for() loop. 
}
// *        Criterion that guarantees relative accuracy
}
else  {
  {
for (i = 1; i <= (n-1); i++) {
eabs = Math.abs(e[(i-(1))+ _e_offset]);
if ((eabs <= ((spltol*Math.sqrt(Math.abs(d[(i-(1))+ _d_offset])))*Math.sqrt(Math.abs(d[((i+1)-(1))+ _d_offset])))))  {
    e[(i-(1))+ _e_offset] = 0.0;
e2[(i-(1))+ _e2_offset] = 0.0;
isplit[(nsplit.val-(1))+ _isplit_offset] = i;
nsplit.val = (nsplit.val+1);
}
//Dummy.label("com/polytechnik/lapack/Dlarra",10);
}              //  Close for() loop. 
}
}              //  Close else.
// 
isplit[(nsplit.val-(1))+ _isplit_offset] = n;
/***
*
*     End of DLARRA
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarra",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarra",999999);
return;
   }
} // End class.
