/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarre.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarre.java,v 1.11 2012/12/16 09:07:28 mal Exp $
 *
 */

package com.polytechnik.lapack;


public class Dlarre {

// *> \brief \b DLARRE given the tridiagonal matrix T, sets small off-diago
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRE + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRE( RANGE, N, VL, VU, IL, IU, D, E, E2,
// *                           RTOL1, RTOL2, SPLTOL, NSPLIT, ISPLIT, M,
// *                           W, WERR, WGAP, IBLOCK, INDEXW, GERS, PIVMIN,
// *                           WORK, IWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          RANGE
// *       INTEGER            IL, INFO, IU, M, N, NSPLIT
// *       DOUBLE PRECISION  PIVMIN, RTOL1, RTOL2, SPLTOL, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IBLOCK( * ), ISPLIT( * ), IWORK( * ),
// *      $                   INDEXW( * )
// *       DOUBLE PRECISION   D( * ), E( * ), E2( * ), GERS( * ),
// *      $                   W( * ),WERR( * ), WGAP( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> To find the desired eigenvalues of a given real symmetric
// *> tridiagonal matrix T, DLARRE sets any "small" off-diagonal
// *> elements to zero, and for each unreduced block T_i, it finds
// *> (a) a suitable shift at one end of the block's spectrum,
// *> (b) the base representation, T_i - sigma_i I = L_i D_i L_i^T, and
// *> (c) eigenvalues of each L_i D_i L_i^T.
// *> The representations and eigenvalues found are then used by
// *> DSTEMR to compute the eigenvectors of T.
// *> The accuracy varies depending on whether bisection is used to
// *> find a few eigenvalues or the dqds algorithm (subroutine DLASQ2) to
// *> conpute all and then discard any unwanted one.
// *> As an added benefit, DLARRE also outputs the n
// *> Gerschgorin intervals for the matrices L_i D_i L_i^T.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': ("All")   all eigenvalues will be found.
// *>          = 'V': ("Value") all eigenvalues in the half-open interval
// *>                           (VL, VU] will be found.
// *>          = 'I': ("Index") the IL-th through IU-th eigenvalues (of the
// *>                           entire matrix) will be found.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix. N > 0.
// *> \endverbatim
// *>
// *> \param[in,out] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          If RANGE='V', the lower and upper bounds for the eigenvalues
// *>          Eigenvalues less than or equal to VL, or greater than VU,
// *>          will not be returned.  VL < VU.
// *>          If RANGE='I' or ='A', DLARRE computes bounds on the desired

// *>          part of the spectrum.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the N diagonal elements of the tridiagonal
// *>          matrix T.
// *>          On exit, the N diagonal elements of the diagonal
// *>          matrices D_i.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the first (N-1) entries contain the subdiagonal
// *>          elements of the tridiagonal matrix T; E(N) need not be set.

// *>          On exit, E contains the subdiagonal elements of the unit
// *>          bidiagonal matrices L_i. The entries E( ISPLIT( I ) ),
// *>          1 <= I <= NSPLIT, contain the base points sigma_i on output.
// *> \endverbatim
// *>
// *> \param[in,out] E2
// *> \verbatim
// *>          E2 is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the first (N-1) entries contain the SQUARES of the
// *>          subdiagonal elements of the tridiagonal matrix T;
// *>          E2(N) need not be set.
// *>          On exit, the entries E2( ISPLIT( I ) ),
// *>          1 <= I <= NSPLIT, have been set to zero
// *> \endverbatim
// *>
// *> \param[in] RTOL1
// *> \verbatim
// *>          RTOL1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] RTOL2
// *> \verbatim
// *>          RTOL2 is DOUBLE PRECISION
// *>           Parameters for bisection.
// *>           An interval [LEFT,RIGHT] has converged if
// *>           RIGHT-LEFT.LT.MAX( RTOL1*GAP, RTOL2*MAX(|LEFT|,|RIGHT|) )
// *> \endverbatim
// *>
// *> \param[in] SPLTOL
// *> \verbatim
// *>          SPLTOL is DOUBLE PRECISION
// *>          The threshold for splitting.
// *> \endverbatim
// *>
// *> \param[out] NSPLIT
// *> \verbatim
// *>          NSPLIT is INTEGER
// *>          The number of blocks T splits into. 1 <= NSPLIT <= N.
// *> \endverbatim
// *>
// *> \param[out] ISPLIT
// *> \verbatim
// *>          ISPLIT is INTEGER array, dimension (N)
// *>          The splitting points, at which T breaks up into blocks.
// *>          The first block consists of rows/columns 1 to ISPLIT(1),
// *>          the second of rows/columns ISPLIT(1)+1 through ISPLIT(2),
// *>          etc., and the NSPLIT-th consists of rows/columns
// *>          ISPLIT(NSPLIT-1)+1 through ISPLIT(NSPLIT)=N.
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The total number of eigenvalues (of all L_i D_i L_i^T)
// *>          found.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          The first M elements contain the eigenvalues. The
// *>          eigenvalues of each of the blocks, L_i D_i L_i^T, are
// *>          sorted in ascending order ( DLARRE may use the
// *>          remaining N-M elements as workspace).
// *> \endverbatim
// *>
// *> \param[out] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION array, dimension (N)
// *>          The error bound on the corresponding eigenvalue in W.
// *> \endverbatim
// *>
// *> \param[out] WGAP
// *> \verbatim
// *>          WGAP is DOUBLE PRECISION array, dimension (N)
// *>          The separation from the right neighbor eigenvalue in W.
// *>          The gap is only with respect to the eigenvalues of the same 
// *>          as each block has its own representation tree.
// *>          Exception: at the right end of a block we store the left gap
// *> \endverbatim
// *>
// *> \param[out] IBLOCK
// *> \verbatim
// *>          IBLOCK is INTEGER array, dimension (N)
// *>          The indices of the blocks (submatrices) associated with the

// *>          corresponding eigenvalues in W; IBLOCK(i)=1 if eigenvalue
// *>          W(i) belongs to the first block from the top, =2 if W(i)
// *>          belongs to the second block, etc.
// *> \endverbatim
// *>
// *> \param[out] INDEXW
// *> \verbatim
// *>          INDEXW is INTEGER array, dimension (N)
// *>          The indices of the eigenvalues within each block (submatrix)
// *>          for example, INDEXW(i)= 10 and IBLOCK(i)=2 imply that the
// *>          i-th eigenvalue W(i) is the 10-th eigenvalue in block 2
// *> \endverbatim
// *>
// *> \param[out] GERS
// *> \verbatim
// *>          GERS is DOUBLE PRECISION array, dimension (2*N)
// *>          The N Gerschgorin intervals (the i-th Gerschgorin interval
// *>          is (GERS(2*i-1), GERS(2*i)).
// *> \endverbatim
// *>
// *> \param[out] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot in the Sturm sequence for T.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (6*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (5*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          > 0:  A problem occured in DLARRE.
// *>          < 0:  One of the called subroutines signaled an internal pro
// *>                Needs inspection of the corresponding parameter IINFO

// *>                for further information.
// *>
// *>          =-1:  Problem in DLARRD.
// *>          = 2:  No base representation could be found in MAXTRY iterat
// *>                Increasing MAXTRY and recompilation might be a remedy.
// *>          =-3:  Problem in DLARRB when computing the refined root
// *>                representation for DLASQ2.
// *>          =-4:  Problem in DLARRB when preforming bisection on the
// *>                desired part of the spectrum.
// *>          =-5:  Problem in DLASQ2.
// *>          =-6:  Problem in DLASQ2.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The base representations are required to suffer very little
// *>  element growth and consequently define all their eigenvalues to
// *>  high relative accuracy.
// *> \endverbatim
// *
// *> \par Contributors:
// *  ==================
// *>
// *>     Beresford Parlett, University of California, Berkeley, USA \n
// *>     Jim Demmel, University of California, Berkeley, USA \n
// *>     Inderjit Dhillon, University of Texas, Austin, USA \n
// *>     Osni Marques, LBNL/NERSC, USA \n
// *>     Christof Voemel, University of California, Berkeley, USA \n
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***


*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***

*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***

*     ..
*     .. Executable Statements ..
*

***/
/***

*
*     Decode RANGE
*
***/

public static void dlarre (final char range,
int n,
doubleW vl,
doubleW vu,
int il,
int iu,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] e2, int _e2_offset,
double rtol1,
double rtol2,
double spltol,
intW nsplit,
int [] isplit, int _isplit_offset,
intW m,
double [] w, int _w_offset,
double [] werr, int _werr_offset,
double [] wgap, int _wgap_offset,
int [] iblock, int _iblock_offset,
int [] indexw, int _indexw_offset,
double [] gers, int _gers_offset,
doubleW pivmin,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

boolean forceb= false;
boolean norep= false;
boolean usedqd= false;
intW cnt= new intW(0);
intW cnt1= new intW(0);
intW cnt2= new intW(0);
int i= 0;
int ibegin= 0;
int idum= 0;
int iend= 0;
intW iinfo= new intW(0);
int in= 0;
int indl= 0;
int indu= 0;
int irange= 0;
int j= 0;
int jblk= 0;
int mb= 0;
intW mm= new intW(0);
int wbegin= 0;
int wend= 0;
double avgap= 0.0d;
double bsrtol= 0.0d;
double clwdth= 0.0d;
double dmax= 0.0d;
double dpivot= 0.0d;
double eabs= 0.0d;
double emax= 0.0d;
double eold= 0.0d;
double eps= 0.0d;
double gl= 0.0d;
double gu= 0.0d;
double isleft= 0.0d;
double isrght= 0.0d;
double rtl= 0.0d;
double rtol= 0.0d;
double s1= 0.0d;
double s2= 0.0d;
double safmin= 0.0d;
double sgndef= 0.0d;
double sigma= 0.0d;
double spdiam= 0.0d;
double tau= 0.0d;
doubleW tmp= new doubleW(0.0d);
doubleW tmp1= new doubleW(0.0d);
final java.util.Random iseed= new java.util.Random(1111);
//range = range.substring((1)-1,1);
info.val = 0;
if (com.polytechnik.lapack.Lsame.lsame(range,'A'))  {
    irange = 1;
}
else if (com.polytechnik.lapack.Lsame.lsame(range,'V'))  {
    irange = 3;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(range,'I'))  {
    irange = 2;
// 
}              // Close else if()
/***

*     Get machine constants
***/
m.val = 0;
safmin = com.polytechnik.lapack.Dlamch.dlamch('S');
/***

*     Set parameters
***/
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
rtl = Math.sqrt(eps);
/***

*     Treat case of 1x1 matrix for quick return
***/
bsrtol = Math.sqrt(eps);
if ((n == 1))  {
    if (((((irange == 1)) || (((((irange == 3)) && ((d[(1-(1))+ _d_offset] > vl.val))) && ((d[(1-(1))+ _d_offset] <= vu.val))))) || (((((irange == 2)) && ((il == 1))) && ((iu == 1))))))  {
    m.val = 1;
// *           The computation error of the eigenvalue is zero
w[(1-(1))+ _w_offset] = d[(1-(1))+ _d_offset];
werr[(1-(1))+ _werr_offset] = 0.0;
wgap[(1-(1))+ _wgap_offset] = 0.0;
iblock[(1-(1))+ _iblock_offset] = 1;
indexw[(1-(1))+ _indexw_offset] = 1;
gers[(1-(1))+ _gers_offset] = d[(1-(1))+ _d_offset];
gers[(2-(1))+ _gers_offset] = d[(1-(1))+ _d_offset];
// *        store the shift for the initial RRR, which is zero in this case
}
e[(1-(1))+ _e_offset] = 0.0;
return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
/***

*     General case: tridiagonal matrix of order > 1
*
*     Init WERR, WGAP. Compute Gerschgorin intervals and spectral diameter.
*     Compute maximum off-diagonal entry and pivmin.
***/
}
gl = d[(1-(1))+ _d_offset];
gu = d[(1-(1))+ _d_offset];
eold = 0.0;
emax = 0.0;
e[(n-(1))+ _e_offset] = 0.0;
{
for (i = 1; i <= n; i++) {
werr[(i-(1))+ _werr_offset] = 0.0;
wgap[(i-(1))+ _wgap_offset] = 0.0;
eabs = Math.abs(e[(i-(1))+ _e_offset]);
if ((eabs >= emax))  {
    emax = eabs;
}
tmp1.val = (eabs+eold);
gers[(((2*i)-1)-(1))+ _gers_offset] = (d[(i-(1))+ _d_offset]-tmp1.val);
gl = Math.min(gl, gers[(((2*i)-1)-(1))+ _gers_offset]);
gers[((2*i)-(1))+ _gers_offset] = (d[(i-(1))+ _d_offset]+tmp1.val);
gu = Math.max(gu, gers[((2*i)-(1))+ _gers_offset]);
eold = eabs;
// *     The minimum pivot allowed in the Sturm sequence for T
//Dummy.label("com/polytechnik/lapack/Dlarre",5);
}              //  Close for() loop. 
}
/***
*     Compute spectral diameter. The Gerschgorin bounds give an
*     estimate that is wrong by at most a factor of SQRT(2)
***/
pivmin.val = (safmin*Math.max(1.0, ( Math.pow(emax, 2))));
/***

*     Compute splitting points
***/
spdiam = (gu-gl);
/***

*     Can force use of bisection instead of faster DQDS.
*     Option left in the code for future multisection work.
***/
com.polytechnik.lapack.Dlarra.dlarra(n,d,_d_offset,e,_e_offset,e2,_e2_offset,spltol,spdiam,nsplit,isplit,_isplit_offset,iinfo);
/***

*     Initialize USEDQD, DQDS should be used for ALLRNG unless someone
*     explicitly wants bisection.
***/
forceb = true;//false;
// 
usedqd = ((((irange == 1)) && ((!forceb))));
// *        Set interval [VL,VU] that contains all eigenvalues
if ((((irange == 1)) && ((!forceb))))  {
    vl.val = gl;
vu.val = gu;
/***
*        We call DLARRD to find crude approximations to the eigenvalues
*        in the desired range. In case IRANGE = INDRNG, we also obtain the
*        interval (VL,VU] that contains all the wanted eigenvalues.
*        An interval [LEFT,RIGHT] has converged if
*        RIGHT-LEFT.LT.RTOL*MAX(ABS(LEFT),ABS(RIGHT))
*        DLARRD needs a WORK of size 4*N, IWORK of size 3*N
***/
}
else  {
  com.polytechnik.lapack.Dlarrd.dlarrd(range,'B',n,vl.val,vu.val,il,iu,gers,_gers_offset,bsrtol,d,_d_offset,e,_e_offset,e2,_e2_offset,pivmin.val,nsplit.val,isplit,_isplit_offset,mm,w,_w_offset,werr,_werr_offset,vl,vu,iblock,_iblock_offset,indexw,_indexw_offset,work,_work_offset,iwork,_iwork_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = -1;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
// *        Make sure that the entries M+1 to N in W, WERR, IBLOCK, INDEXW are 0
}
{
for (i = (mm.val+1); i <= n; i++) {
w[(i-(1))+ _w_offset] = 0.0;
werr[(i-(1))+ _werr_offset] = 0.0;
iblock[(i-(1))+ _iblock_offset] = 0;
indexw[(i-(1))+ _indexw_offset] = 0;
//Dummy.label("com/polytechnik/lapack/Dlarre",14);
}              //  Close for() loop. 
}
/***


***
*     Loop over unreduced blocks
***/
}              //  Close else.
ibegin = 1;
wbegin = 1;
{
label170:for (jblk = 1; jblk <= nsplit.val; jblk++) {
iend = isplit[(jblk-(1))+ _isplit_offset];
/***

*        1 X 1 block
***/
in = ((iend-ibegin)+1);
if ((in == 1))  {
    if (((((irange == 1)) || (((((irange == 3)) && ((d[(ibegin-(1))+ _d_offset] > vl.val))) && ((d[(ibegin-(1))+ _d_offset] <= vu.val))))) || ((((irange == 2)) && ((iblock[(wbegin-(1))+ _iblock_offset] == jblk))))))  {
    m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = d[(ibegin-(1))+ _d_offset];
/***
*              The gap for a single block doesn't matter for the later
*              algorithm and is assigned an arbitrary large value
***/
werr[(m.val-(1))+ _werr_offset] = 0.0;
wgap[(m.val-(1))+ _wgap_offset] = 0.0;
iblock[(m.val-(1))+ _iblock_offset] = jblk;
indexw[(m.val-(1))+ _indexw_offset] = 1;
wbegin = (wbegin+1);
// *           E( IEND ) holds the shift for the initial RRR
}
e[(iend-(1))+ _e_offset] = 0.0;
ibegin = (iend+1);
continue label170;//Dummy.go_to("com/polytechnik/lapack/Dlarre",170);
/***
*
*        Blocks of size larger than 1x1
*
*        E( IEND ) will hold the shift for the initial RRR, for now set it =0
***/
}
/***
*
*        Find local outer bounds GL,GU for the block
***/
e[(iend-(1))+ _e_offset] = 0.0;
gl = d[(ibegin-(1))+ _d_offset];
gu = d[(ibegin-(1))+ _d_offset];
{
for (i = ibegin; i <= iend; i++) {
gl = Math.min(gers[(((2*i)-1)-(1))+ _gers_offset], gl);
gu = Math.max(gers[((2*i)-(1))+ _gers_offset], gu);
//Dummy.label("com/polytechnik/lapack/Dlarre",15);
}              //  Close for() loop. 
}
// 
spdiam = (gu-gl);
// *           Count the number of eigenvalues in the current block.
if ((!((((irange == 1)) && ((!forceb))))))  {
    mb = 0;
{
label20:for (i = wbegin; i <= mm.val; i++) {
if ((iblock[(i-(1))+ _iblock_offset] == jblk))  {
    mb = (mb+1);
}
else  {
    break label20;//Dummy.go_to("com/polytechnik/lapack/Dlarre",21);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlarre",20);
}              //  Close for() loop. 
}
// 
//label21:
//   Dummy.label("com/polytechnik/lapack/Dlarre",21);
/***
*              No eigenvalue in the current block lies in the desired range
*              E( IEND ) holds the shift for the initial RRR
***/
if ((mb == 0))  {
    e[(iend-(1))+ _e_offset] = 0.0;
ibegin = (iend+1);
continue label170;//Dummy.go_to("com/polytechnik/lapack/Dlarre",170);
/***

*              Decide whether dqds or bisection is more efficient
***/
}
else  {
  usedqd = ((((mb > (0.5*in))) && ((!forceb))));
/***
*              Calculate gaps for the current block
*              In later stages, when representations for individual
*              eigenvalues are different, we use SIGMA = E( IEND ).
***/
wend = ((wbegin+mb)-1);
sigma = 0.0;
{
for (i = wbegin; i <= (wend-1); i++) {
wgap[(i-(1))+ _wgap_offset] = Math.max(0.0, ((w[((i+1)-(1))+ _w_offset]-werr[((i+1)-(1))+ _werr_offset])-((w[(i-(1))+ _w_offset]+werr[(i-(1))+ _werr_offset]))));
//Dummy.label("com/polytechnik/lapack/Dlarre",30);
}              //  Close for() loop. 
}
// *              Find local index of the first and last desired evalue.
wgap[(wend-(1))+ _wgap_offset] = Math.max(0.0, ((vu.val-sigma)-((w[(wend-(1))+ _w_offset]+werr[(wend-(1))+ _werr_offset]))));
indl = indexw[(wbegin-(1))+ _indexw_offset];
indu = indexw[(wend-(1))+ _indexw_offset];
}              //  Close else.
}
/***
*           Case of DQDS
*           Find approximations to the extremal eigenvalues of the block
***/
if ((((((irange == 1)) && ((!forceb)))) || usedqd))  {
    com.polytechnik.lapack.Dlarrk.dlarrk(in,1,gl,gu,d,(ibegin-(1))+ _d_offset,e2,(ibegin-(1))+ _e2_offset,pivmin.val,rtl,tmp,tmp1,iinfo);
if ((iinfo.val != 0))  {
    info.val = -1;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
}
// 
isleft = Math.max(gl, ((tmp.val-tmp1.val)-((100.0*eps)*Math.abs((tmp.val-tmp1.val)))));
com.polytechnik.lapack.Dlarrk.dlarrk(in,in,gl,gu,d,(ibegin-(1))+ _d_offset,e2,(ibegin-(1))+ _e2_offset,pivmin.val,rtl,tmp,tmp1,iinfo);
if ((iinfo.val != 0))  {
    info.val = -1;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
}
// *           Improve the estimate of the spectral diameter
isrght = Math.min(gu, ((tmp.val+tmp1.val)+((100.0*eps)*Math.abs((tmp.val+tmp1.val)))));
spdiam = (isrght-isleft);
/***
*           Case of bisection
*           Find approximations to the wanted extremal eigenvalues
***/
}
else  {
  isleft = Math.max(gl, ((w[(wbegin-(1))+ _w_offset]-werr[(wbegin-(1))+ _werr_offset])-((100.0*eps)*Math.abs((w[(wbegin-(1))+ _w_offset]-werr[(wbegin-(1))+ _werr_offset])))));
isrght = Math.min(gu, ((w[(wend-(1))+ _w_offset]+werr[(wend-(1))+ _werr_offset])+((100.0*eps)*Math.abs((w[(wend-(1))+ _w_offset]+werr[(wend-(1))+ _werr_offset])))));
/***


*        Decide whether the base representation for the current block
*        L_JBLK D_JBLK L_JBLK^T = T_JBLK - sigma_JBLK I
*        should be on the left or the right end of the current block.
*        The strategy is to shift to the end which is "more populated"
*        Furthermore, decide whether to use DQDS for the computation of
*        the eigenvalue approximations at the end of DLARRE or bisection.
*        dqds is chosen if all eigenvalues are desired or the number of
*        eigenvalues to be computed is large compared to the blocksize.
***/
}              //  Close else.
// *           If all the eigenvalues have to be computed, we use dqd
if ((((irange == 1)) && ((!forceb))))  {
    // *           INDL is the local index of the first eigenvalue to compute
usedqd = true;
indl = 1;
// *           MB =  number of eigenvalues to compute
indu = in;
mb = in;
// *           Define 1/4 and 3/4 points of the spectrum
wend = ((wbegin+mb)-1);
s1 = (isleft+(0.25*spdiam));
s2 = (isrght-(0.25*spdiam));
/***
*           DLARRD has computed IBLOCK and INDEXW for each eigenvalue
*           approximation.
*           choose sigma
***/
}
else  {
  if (usedqd)  {
    s1 = (isleft+(0.25*spdiam));
s2 = (isrght-(0.25*spdiam));
}
else  {
  tmp.val = (Math.min(isrght, vu.val)-Math.max(isleft, vl.val));
s1 = (Math.max(isleft, vl.val)+(0.25*tmp.val));
s2 = (Math.min(isrght, vu.val)-(0.25*tmp.val));
}              //  Close else.
/***

*        Compute the negcount at the 1/4 and 3/4 points
***/
}              //  Close else.
if ((mb > 1))  {
    com.polytechnik.lapack.Dlarrc.dlarrc('T',in,s1,s2,d,(ibegin-(1))+ _d_offset,e,(ibegin-(1))+ _e_offset,pivmin.val,cnt,cnt1,cnt2,iinfo);
// 
}
if ((mb == 1))  {
    sigma = gl;
sgndef = 1.0;
}
else if (((cnt1.val-indl) >= (indu-cnt2.val)))  {
    if ((((irange == 1)) && ((!forceb))))  {
    sigma = Math.max(isleft, gl);
/***
*              use Gerschgorin bound as shift to get pos def matrix
*              for dqds
***/
}
else if (usedqd)  {
    sigma = isleft;
/***
*              use approximation of the first desired eigenvalue of the
*              block as shift
***/
}              // Close else if()
else  {
  sigma = Math.max(isleft, vl.val);
}              //  Close else.
sgndef = 1.0;
}              // Close else if()
else  {
  if ((((irange == 1)) && ((!forceb))))  {
    sigma = Math.min(isrght, gu);
/***
*              use Gerschgorin bound as shift to get neg def matrix
*              for dqds
***/
}
else if (usedqd)  {
    sigma = isrght;
/***
*              use approximation of the first desired eigenvalue of the
*              block as shift
***/
}              // Close else if()
else  {
  sigma = Math.min(isrght, vu.val);
}              //  Close else.
sgndef = -1.0;
/***


*        An initial SIGMA has been chosen that will be used for computing
*        T - SIGMA I = L D L^T
*        Define the increment TAU of the shift in case the initial shift
*        needs to be refined to obtain a factorization with not too much
*        element growth.
***/
}              //  Close else.
/***
*           The initial SIGMA was to the outer end of the spectrum
*           the matrix is definite and we need not retreat.
***/
if (usedqd)  {
    tau = (((spdiam*eps)*n)+(2.0*pivmin.val));
tau = Math.max(tau, ((2.0*eps)*Math.abs(sigma)));
}
else  {
  if ((mb > 1))  {
    clwdth = (((w[(wend-(1))+ _w_offset]+werr[(wend-(1))+ _werr_offset])-w[(wbegin-(1))+ _w_offset])-werr[(wbegin-(1))+ _werr_offset]);
avgap = Math.abs((clwdth/(double)((wend-wbegin))));
if ((sgndef == 1.0))  {
    tau = (0.5*Math.max(wgap[(wbegin-(1))+ _wgap_offset], avgap));
tau = Math.max(tau, werr[(wbegin-(1))+ _werr_offset]);
}
else  {
  tau = (0.5*Math.max(wgap[((wend-1)-(1))+ _wgap_offset], avgap));
tau = Math.max(tau, werr[(wend-(1))+ _werr_offset]);
}              //  Close else.
}
else  {
  tau = werr[(wbegin-(1))+ _werr_offset];
}              //  Close else.
// *
}              //  Close else.
/***
*           Compute L D L^T factorization of tridiagonal matrix T - sigma I.
*           Store D in WORK(1:IN), L in WORK(IN+1:2*IN), and reciprocals of
*           pivots in WORK(2*IN+1:3*IN)
***/
label83:{
{
for (idum = 1; idum <= 6; idum++) {
dpivot = (d[(ibegin-(1))+ _d_offset]-sigma);
work[(1-(1))+ _work_offset] = dpivot;
dmax = Math.abs(work[(1-(1))+ _work_offset]);
j = ibegin;
{
for (i = 1; i <= (in-1); i++) {
work[(((2*in)+i)-(1))+ _work_offset] = (1.0/work[(i-(1))+ _work_offset]);
tmp.val = (e[(j-(1))+ _e_offset]*work[(((2*in)+i)-(1))+ _work_offset]);
work[((in+i)-(1))+ _work_offset] = tmp.val;
dpivot = (((d[((j+1)-(1))+ _d_offset]-sigma))-(tmp.val*e[(j-(1))+ _e_offset]));
work[((i+1)-(1))+ _work_offset] = dpivot;
dmax = Math.max(dmax, Math.abs(dpivot));
j = (j+1);
// *           check for element growth
//Dummy.label("com/polytechnik/lapack/Dlarre",70);
}              //  Close for() loop. 
}
if ((dmax > (64.0*spdiam)))  {
    norep = true;
}
else  {
  norep = false;
}              //  Close else.
/***
*              Ensure the definiteness of the representation
*              All entries of D (of L D L^T) must have the same sign
***/
if ((usedqd && (!norep)))  {
    {
for (i = 1; i <= in; i++) {
tmp.val = (sgndef*work[(i-(1))+ _work_offset]);
if ((tmp.val < 0.0)) {
    norep = true;
}
//Dummy.label("com/polytechnik/lapack/Dlarre",71);
}              //  Close for() loop. 
}
}
/***
*              Note that in the case of IRANGE=ALLRNG, we use the Gerschgorin
*              shift which makes the matrix definite. So we should end up
*              here really only in the case of IRANGE = VALRNG or INDRNG.
***/
if (norep)  {
    if ((idum == (6-1)))  {
    // *                    The fudged Gerschgorin shift should succeed
if ((sgndef == 1.0))  {
    sigma = ((gl-(((2.0*spdiam)*eps)*n))-((2.0*2.0)*pivmin.val));
}
else  {
  sigma = ((gu+(((2.0*spdiam)*eps)*n))+((2.0*2.0)*pivmin.val));
}              //  Close else.
}
else  {
  sigma = (sigma-(sgndef*tau));
tau = (2.0*tau);
}              //  Close else.
// *              an initial RRR is found
}
else  {
    break label83;//Dummy.go_to("com/polytechnik/lapack/Dlarre",83);
}              //  Close else.
/***
*        if the program reaches this point, no base representation could be
*        found in MAXTRY iterations.
***/
//Dummy.label("com/polytechnik/lapack/Dlarre",80);
}              //  Close for() loop. 
}
info.val = 2;
// 
return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
/***
*        At this point, we have found an initial base representation
*        T - SIGMA I = L D L^T with not too much element growth.
*        Store the shift.
***/
}//label83: Dummy.label("com/polytechnik/lapack/Dlarre",83);
// *        Store D and L.
e[(iend-(1))+ _e_offset] = sigma;
com.polytechnik.lapack.Dcopy.dcopy(in,work,_work_offset,1,d,(ibegin-(1))+ _d_offset,1);
/***


***/
com.polytechnik.lapack.Dcopy.dcopy((in-1),work,((in+1)-(1))+ _work_offset,1,e,(ibegin-(1))+ _e_offset,1);
/***
*
*           Perturb each entry of the base representation by a small
*           (but random) relative amount to overcome difficulties with
*           glued matrices.
*
***/
if ((mb > 1))  {
    {
/*for (i = 1; i <= 4; i++) {
iseed[(i-(1))] = 1;
// 
//Dummy.label("com/polytechnik/lapack/Dlarre",122);
}*/              //  Close for() loop. 
}
com.polytechnik.lapack.Dlarnv.dlarnv(2,iseed,0,((2*in)-1),work,(1-(1))+ _work_offset);
{
for (i = 1; i <= (in-1); i++) {
d[(((ibegin+i)-1)-(1))+ _d_offset] = (d[(((ibegin+i)-1)-(1))+ _d_offset]*((1.0+((eps*8.0)*work[(i-(1))+ _work_offset]))));
e[(((ibegin+i)-1)-(1))+ _e_offset] = (e[(((ibegin+i)-1)-(1))+ _e_offset]*((1.0+((eps*8.0)*work[((in+i)-(1))+ _work_offset]))));
//Dummy.label("com/polytechnik/lapack/Dlarre",125);
}              //  Close for() loop. 
}
// *
d[(iend-(1))+ _d_offset] = (d[(iend-(1))+ _d_offset]*((1.0+((eps*4.0)*work[(in-(1))+ _work_offset]))));
/***
*
*        Don't update the Gerschgorin intervals because keeping track
*        of the updates would be too much work in DLARRV.
*        We update W instead and use it to locate the proper Gerschgorin
*        intervals.

*        Compute the required eigenvalues of L D L' by bisection or dqds
***/
}
/***
*           If DLARRD has been used, shift the eigenvalue approximations
*           according to their representation. This is necessary for
*           a uniform DLARRV since dqds computes eigenvalues of the
*           shifted representation. In DLARRV, W will always hold the
*           UNshifted eigenvalue approximation.
***/
if ((!usedqd))  {
    {
for (j = wbegin; j <= wend; j++) {
w[(j-(1))+ _w_offset] = (w[(j-(1))+ _w_offset]-sigma);
werr[(j-(1))+ _werr_offset] = (werr[(j-(1))+ _werr_offset]+(Math.abs(w[(j-(1))+ _w_offset])*eps));
/***
*           call DLARRB to reduce eigenvalue error of the approximations
*           from DLARRD
***/
//Dummy.label("com/polytechnik/lapack/Dlarre",134);
}              //  Close for() loop. 
}
{
for (i = ibegin; i <= (iend-1); i++) {
work[(i-(1))+ _work_offset] = (d[(i-(1))+ _d_offset]*( Math.pow(e[(i-(1))+ _e_offset], 2)));
// *           use bisection to find EV from INDL to INDU
//Dummy.label("com/polytechnik/lapack/Dlarre",135);
}              //  Close for() loop. 
}
com.polytechnik.lapack.Dlarrb.dlarrb(in,d,(ibegin-(1))+ _d_offset,work,(ibegin-(1))+ _work_offset,indl,indu,rtol1,rtol2,(indl-1),w,(wbegin-(1))+ _w_offset,wgap,(wbegin-(1))+ _wgap_offset,werr,(wbegin-(1))+ _werr_offset,work,(((2*n)+1)-(1))+ _work_offset,iwork,_iwork_offset,pivmin.val,spdiam,in,iinfo);
if ((iinfo.val != 0))  {
    info.val = -4;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
/***
*           DLARRB computes all gaps correctly except for the last one
*           Record distance to VU/GU
***/
}
wgap[(wend-(1))+ _wgap_offset] = Math.max(0.0, (((vu.val-sigma))-((w[(wend-(1))+ _w_offset]+werr[(wend-(1))+ _werr_offset]))));
{
for (i = indl; i <= indu; i++) {
m.val = (m.val+1);
iblock[(m.val-(1))+ _iblock_offset] = jblk;
indexw[(m.val-(1))+ _indexw_offset] = i;
//Dummy.label("com/polytechnik/lapack/Dlarre",138);
}              //  Close for() loop. 
}
/***
*           Call dqds to get all eigs (and then possibly delete unwanted
*           eigenvalues).
*           Note that dqds finds the eigenvalues of the L D L^T representation
*           of T to high relative accuracy. High relative accuracy
*           might be lost when the shift of the RRR is subtracted to obtain
*           the eigenvalues of T. However, T is not guaranteed to define its
*           eigenvalues to high relative accuracy anyway.
*           Set RTOL to the order of the tolerance used in DLASQ2
*           This is an ESTIMATED error, the worst case bound is 4*N*EPS
*           which is usually too large and requires unnecessary work to be
*           done by bisection when computing the eigenvectors
***/
}
else  {
  rtol = ((Math.log((double)(in))*4.0)*eps);
j = ibegin;
{
for (i = 1; i <= (in-1); i++) {
work[(((2*i)-1)-(1))+ _work_offset] = Math.abs(d[(j-(1))+ _d_offset]);
work[((2*i)-(1))+ _work_offset] = ((e[(j-(1))+ _e_offset]*e[(j-(1))+ _e_offset])*work[(((2*i)-1)-(1))+ _work_offset]);
j = (j+1);
//Dummy.label("com/polytechnik/lapack/Dlarre",140);
}              //  Close for() loop. 
}
work[(((2*in)-1)-(1))+ _work_offset] = Math.abs(d[(iend-(1))+ _d_offset]);
work[((2*in)-(1))+ _work_offset] = 0.0;
//com.polytechnik.lapack.Dlasq2.dlasq2(in,work,_work_offset,iinfo.val);
if(true) throw new RuntimeException("Dlasq2.dlasq2 should be disabled");
/***
*              If IINFO = -5 then an index is part of a tight cluster
*              and should be changed. The index is in IWORK(1) and the
*              gap is in WORK(N+1)
***/
if ((iinfo.val != 0))  {
    info.val = -5;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
// *              Test that all eigenvalues are positive as expected
}
else  {
  {
for (i = 1; i <= in; i++) {
if ((work[(i-(1))+ _work_offset] < 0.0))  {
    info.val = -6;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
}
//Dummy.label("com/polytechnik/lapack/Dlarre",149);
}              //  Close for() loop. 
}
}              //  Close else.
if ((sgndef > 0.0))  {
    {
for (i = indl; i <= indu; i++) {
m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = work[(((in-i)+1)-(1))+ _work_offset];
iblock[(m.val-(1))+ _iblock_offset] = jblk;
indexw[(m.val-(1))+ _indexw_offset] = i;
//Dummy.label("com/polytechnik/lapack/Dlarre",150);
}              //  Close for() loop. 
}
}
else  {
  {
for (i = indl; i <= indu; i++) {
m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = (-(work[(i-(1))+ _work_offset]));
iblock[(m.val-(1))+ _iblock_offset] = jblk;
indexw[(m.val-(1))+ _indexw_offset] = i;
//Dummy.label("com/polytechnik/lapack/Dlarre",160);
}              //  Close for() loop. 
}
// 
}              //  Close else.
// *              the value of RTOL below should be the tolerance in DLASQ2
{
for (i = ((m.val-mb)+1); i <= m.val; i++) {
werr[(i-(1))+ _werr_offset] = (rtol*Math.abs(w[(i-(1))+ _w_offset]));
//Dummy.label("com/polytechnik/lapack/Dlarre",165);
}              //  Close for() loop. 
}
// *              compute the right gap between the intervals
{
for (i = ((m.val-mb)+1); i <= (m.val-1); i++) {
wgap[(i-(1))+ _wgap_offset] = Math.max(0.0, ((w[((i+1)-(1))+ _w_offset]-werr[((i+1)-(1))+ _werr_offset])-((w[(i-(1))+ _w_offset]+werr[(i-(1))+ _werr_offset]))));
//Dummy.label("com/polytechnik/lapack/Dlarre",166);
}              //  Close for() loop. 
}
wgap[(m.val-(1))+ _wgap_offset] = Math.max(0.0, (((vu.val-sigma))-((w[(m.val-(1))+ _w_offset]+werr[(m.val-(1))+ _werr_offset]))));
// *        proceed with next block
}              //  Close else.
ibegin = (iend+1);
wbegin = (wend+1);
/***
*

***/
//Dummy.label("com/polytechnik/lapack/Dlarre",170);
}              //  Close for() loop. 
}
/***
*
*     end of DLARRE
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarre",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarre",999999);
return;
   }
} // End class.
