/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrf.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrf.java,v 1.4 2012/12/21 18:16:24 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrf {

// *> \brief \b DLARRF finds a new relatively robust representation such th
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRF + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRF( N, D, L, LD, CLSTRT, CLEND,
// *                          W, WGAP, WERR,
// *                          SPDIAM, CLGAPL, CLGAPR, PIVMIN, SIGMA,
// *                          DPLUS, LPLUS, WORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            CLSTRT, CLEND, INFO, N
// *       DOUBLE PRECISION   CLGAPL, CLGAPR, PIVMIN, SIGMA, SPDIAM
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), DPLUS( * ), L( * ), LD( * ),
// *      $          LPLUS( * ), W( * ), WGAP( * ), WERR( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Given the initial representation L D L^T and its cluster of close
// *> eigenvalues (in a relative measure), W( CLSTRT ), W( CLSTRT+1 ), ...

// *> W( CLEND ), DLARRF finds a new relatively robust representation
// *> L D L^T - SIGMA I = L(+) D(+) L(+)^T such that at least one of the
// *> eigenvalues of L(+) D(+) L(+)^T is relatively isolated.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix (subblock, if the matrix splitted).

// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The N diagonal elements of the diagonal matrix D.
// *> \endverbatim
// *>
// *> \param[in] L
// *> \verbatim
// *>          L is DOUBLE PRECISION array, dimension (N-1)
// *>          The (N-1) subdiagonal elements of the unit bidiagonal
// *>          matrix L.
// *> \endverbatim
// *>
// *> \param[in] LD
// *> \verbatim
// *>          LD is DOUBLE PRECISION array, dimension (N-1)
// *>          The (N-1) elements L(i)*D(i).
// *> \endverbatim
// *>
// *> \param[in] CLSTRT
// *> \verbatim
// *>          CLSTRT is INTEGER
// *>          The index of the first eigenvalue in the cluster.
// *> \endverbatim
// *>
// *> \param[in] CLEND
// *> \verbatim
// *>          CLEND is INTEGER
// *>          The index of the last eigenvalue in the cluster.
// *> \endverbatim
// *>
// *> \param[in] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension
// *>          dimension is >=  (CLEND-CLSTRT+1)
// *>          The eigenvalue APPROXIMATIONS of L D L^T in ascending order.
// *>          W( CLSTRT ) through W( CLEND ) form the cluster of relativel
// *>          close eigenalues.
// *> \endverbatim
// *>
// *> \param[in,out] WGAP
// *> \verbatim
// *>          WGAP is DOUBLE PRECISION array, dimension
// *>          dimension is >=  (CLEND-CLSTRT+1)
// *>          The separation from the right neighbor eigenvalue in W.
// *> \endverbatim
// *>
// *> \param[in] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION array, dimension
// *>          dimension is  >=  (CLEND-CLSTRT+1)
// *>          WERR contain the semiwidth of the uncertainty
// *>          interval of the corresponding eigenvalue APPROXIMATION in W

// *> \endverbatim
// *>
// *> \param[in] SPDIAM
// *> \verbatim
// *>          SPDIAM is DOUBLE PRECISION
// *>          estimate of the spectral diameter obtained from the
// *>          Gerschgorin intervals
// *> \endverbatim
// *>
// *> \param[in] CLGAPL
// *> \verbatim
// *>          CLGAPL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] CLGAPR
// *> \verbatim
// *>          CLGAPR is DOUBLE PRECISION
// *>          absolute gap on each end of the cluster.
// *>          Set by the calling routine to protect against shifts too clo
// *>          to eigenvalues outside the cluster.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot allowed in the Sturm sequence.
// *> \endverbatim
// *>
// *> \param[out] SIGMA
// *> \verbatim
// *>          SIGMA is DOUBLE PRECISION
// *>          The shift used to form L(+) D(+) L(+)^T.
// *> \endverbatim
// *>
// *> \param[out] DPLUS
// *> \verbatim
// *>          DPLUS is DOUBLE PRECISION array, dimension (N)
// *>          The N diagonal elements of the diagonal matrix D(+).
// *> \endverbatim
// *>
// *> \param[out] LPLUS
// *> \verbatim
// *>          LPLUS is DOUBLE PRECISION array, dimension (N-1)
// *>          The first (N-1) elements of LPLUS contain the subdiagonal
// *>          elements of the unit bidiagonal matrix L(+).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (2*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          Signals processing OK (=0) or failure (=1)
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarrf (int n,
double [] d, int _d_offset,
double [] l, int _l_offset,
double [] ld, int _ld_offset,
int clstrt,
int clend,
double [] w, int _w_offset,
double [] wgap, int _wgap_offset,
double [] werr, int _werr_offset,
double spdiam,
double clgapl,
double clgapr,
double pivmin,
doubleW sigma,
double [] dplus, int _dplus_offset,
double [] lplus, int _lplus_offset,
double [] work, int _work_offset,
intW info)  {

boolean dorrr1= false;
boolean forcer= false;
boolean nofail= false;
boolean sawnan1= false;
boolean sawnan2= false;
boolean tryrrr1= false;
int i= 0;
int indx= 0;
int ktry= 0;
int shift= 0;
double avgap= 0.0d;
double bestshift= 0.0d;
double clwdth= 0.0d;
double eps= 0.0d;
double fact= 0.0d;
double fail= 0.0d;
double fail2= 0.0d;
double growthbound= 0.0d;
double ldelta= 0.0d;
double ldmax= 0.0d;
double lsigma= 0.0d;
double max1= 0.0d;
double max2= 0.0d;
double mingap= 0.0d;
double oldp= 0.0d;
double prod= 0.0d;
double rdelta= 0.0d;
double rdmax= 0.0d;
double rrr1= 0.0d;
double rrr2= 0.0d;
double rsigma= 0.0d;
double s= 0.0d;
double smlgrowth= 0.0d;
double tmp= 0.0d;
double znm2= 0.0d;
info.val = 0;
fact = (double)(((int)Math.pow(2, 1)));
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
shift = 0;
/***


*     Note that we cannot guarantee that for any of the shifts tried,
*     the factorization has a small or even moderate element growth.
*     There could be Ritz values at both ends of the cluster and despite
*     backing off, there are examples where all factorizations tried
*     (in IEEE mode, allowing zero pivots & infinities) have INFINITE
*     element growth.
*     For this reason, we should use PIVMIN in this subroutine so that at
*     least the L D L^T factorization exists. It can be checked afterwards
*     whether the element growth caused bad residuals/orthogonality.

*     Decide whether the code should accept the best among all
*     representations despite large element growth or signal INFO=1
***/
forcer = false;
/***
*

*     Compute the average gap length of the cluster
***/
nofail = true;
clwdth = ((Math.abs((w[(clend-(1))+ _w_offset]-w[(clstrt-(1))+ _w_offset]))+werr[(clend-(1))+ _werr_offset])+werr[(clstrt-(1))+ _werr_offset]);
avgap = (clwdth/(double)((clend-clstrt)));
// *     Initial values for shifts to both ends of cluster
mingap = Math.min(clgapl, clgapr);
lsigma = (Math.min(w[(clstrt-(1))+ _w_offset], w[(clend-(1))+ _w_offset])-werr[(clstrt-(1))+ _werr_offset]);
/***

*     Use a small fudge to make sure that we really shift to the outside
***/
rsigma = (Math.max(w[(clstrt-(1))+ _w_offset], w[(clend-(1))+ _w_offset])+werr[(clend-(1))+ _werr_offset]);
lsigma = (lsigma-((Math.abs(lsigma)*4.0)*eps));
/***

*     Compute upper bounds for how much to back off the initial shifts
***/
rsigma = (rsigma+((Math.abs(rsigma)*4.0)*eps));
ldmax = ((0.25*mingap)+(2.0*pivmin));
// 
rdmax = ((0.25*mingap)+(2.0*pivmin));
ldelta = (Math.max(avgap, wgap[(clstrt-(1))+ _wgap_offset])/fact);
/***
*
*     Initialize the record of the best representation found
*
***/
rdelta = (Math.max(avgap, wgap[((clend-1)-(1))+ _wgap_offset])/fact);
s = com.polytechnik.lapack.Dlamch.dlamch('S');
smlgrowth = (1.0/s);
fail = (((double)((n-1))*mingap)/((spdiam*eps)));
fail2 = (((double)((n-1))*mingap)/((spdiam*Math.sqrt(eps))));
/***
*
*     while (KTRY <= KTRYMAX)
***/
bestshift = lsigma;
ktry = 0;
// 
growthbound = (8.0*spdiam);
label5_100:for(;;){
    //Dummy.label("com/polytechnik/lapack/Dlarrf",5);
sawnan1 = false;
// *     Ensure that we do not back off too much of the initial shifts
sawnan2 = false;
ldelta = Math.min(ldmax, ldelta);
/***

*     Compute the element growth when shifting to both ends of the cluster
*     accept the shift if there is no element growth at one of the two ends

*     Left end
***/
rdelta = Math.min(rdmax, rdelta);
s = (-(lsigma));
dplus[(1-(1))+ _dplus_offset] = (d[(1-(1))+ _d_offset]+s);
if ((Math.abs(dplus[(1-(1))+ _dplus_offset]) < pivmin))  {
    /***
*        Need to set SAWNAN1 because refined RRR test should not be used
*        in this case
***/
dplus[(1-(1))+ _dplus_offset] = (-(pivmin));
sawnan1 = true;
}
max1 = Math.abs(dplus[(1-(1))+ _dplus_offset]);
{
for (i = 1; i <= (n-1); i++) {
lplus[(i-(1))+ _lplus_offset] = (ld[(i-(1))+ _ld_offset]/dplus[(i-(1))+ _dplus_offset]);
s = (((s*lplus[(i-(1))+ _lplus_offset])*l[(i-(1))+ _l_offset])-lsigma);
dplus[((i+1)-(1))+ _dplus_offset] = (d[((i+1)-(1))+ _d_offset]+s);
if ((Math.abs(dplus[((i+1)-(1))+ _dplus_offset]) < pivmin))  {
    /***
*           Need to set SAWNAN1 because refined RRR test should not be used
*           in this case
***/
dplus[((i+1)-(1))+ _dplus_offset] = (-(pivmin));
sawnan1 = true;
}
max1 = Math.max(max1, Math.abs(dplus[((i+1)-(1))+ _dplus_offset]));
//Dummy.label("com/polytechnik/lapack/Dlarrf",6);
}              //  Close for() loop. 
}
// 
sawnan1 = (sawnan1 || com.polytechnik.lapack.Disnan.disnan(max1));
if ((forcer || (((max1 <= growthbound) && (!sawnan1)))))  {
    sigma.val = lsigma;
shift = 1;
break label5_100;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",100);
/***

*     Right end
***/
}
s = (-(rsigma));
work[(1-(1))+ _work_offset] = (d[(1-(1))+ _d_offset]+s);
if ((Math.abs(work[(1-(1))+ _work_offset]) < pivmin))  {
    /***
*        Need to set SAWNAN2 because refined RRR test should not be used
*        in this case
***/
work[(1-(1))+ _work_offset] = (-(pivmin));
sawnan2 = true;
}
max2 = Math.abs(work[(1-(1))+ _work_offset]);
{
for (i = 1; i <= (n-1); i++) {
work[((n+i)-(1))+ _work_offset] = (ld[(i-(1))+ _ld_offset]/work[(i-(1))+ _work_offset]);
s = (((s*work[((n+i)-(1))+ _work_offset])*l[(i-(1))+ _l_offset])-rsigma);
work[((i+1)-(1))+ _work_offset] = (d[((i+1)-(1))+ _d_offset]+s);
if ((Math.abs(work[((i+1)-(1))+ _work_offset]) < pivmin))  {
    /***
*           Need to set SAWNAN2 because refined RRR test should not be used
*           in this case
***/
work[((i+1)-(1))+ _work_offset] = (-(pivmin));
sawnan2 = true;
}
max2 = Math.max(max2, Math.abs(work[((i+1)-(1))+ _work_offset]));
//Dummy.label("com/polytechnik/lapack/Dlarrf",7);
}              //  Close for() loop. 
}
// 
sawnan2 = (sawnan2 || com.polytechnik.lapack.Disnan.disnan(max2));
if ((forcer || (((max2 <= growthbound) && (!sawnan2)))))  {
    sigma.val = rsigma;
shift = 2;
break label5_100;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",100);
/***
*     If we are at this point, both shifts led to too much element growth

*     Record the better of the two shifts (provided it didn't lead to NaN)
***/
}
// *        both MAX1 and MAX2 are NaN
label50:{
if ((sawnan1 && sawnan2))  {
    break label50;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",50);
}
else  {
  if ((!sawnan1))  {
    indx = 1;
if ((max1 <= smlgrowth))  {
    smlgrowth = max1;
bestshift = lsigma;
}
}
if ((!sawnan2))  {
    if ((sawnan1 || (max2 <= max1))) {
    indx = 2;
}
    if ((max2 <= smlgrowth))  {
    smlgrowth = max2;
bestshift = rsigma;
}
}
/***

*     If we are here, both the left and the right shift led to
*     element growth. If the element growth is moderate, then
*     we may still accept the representation, if it passes a
*     refined test for RRR. This test supposes that no NaN occurred.
*     Moreover, we use the refined RRR test only for isolated clusters.
***/
}              //  Close else.
if ((((((clwdth < (mingap/(double)(128)))) && ((Math.min(max1, max2) < fail2))) && ((!sawnan1))) && ((!sawnan2))))  {
    dorrr1 = true;
}
else  {
  dorrr1 = false;
}              //  Close else.
tryrrr1 = true;
if ((tryrrr1 && dorrr1))  {
    if ((indx == 1))  {
    tmp = Math.abs(dplus[(n-(1))+ _dplus_offset]);
znm2 = 1.0;
prod = 1.0;
oldp = 1.0;
{
int _i_inc = -1;
for (i = (n-1); i >= 1; i += _i_inc) {
if ((prod <= eps))  {
    prod = (((((dplus[((i+1)-(1))+ _dplus_offset]*work[(((n+i)+1)-(1))+ _work_offset]))/((dplus[(i-(1))+ _dplus_offset]*work[((n+i)-(1))+ _work_offset]))))*oldp);
}
else  {
  prod = (prod*Math.abs(work[((n+i)-(1))+ _work_offset]));
}              //  Close else.
oldp = prod;
znm2 = (znm2+( Math.pow(prod, 2)));
tmp = Math.max(tmp, Math.abs((dplus[(i-(1))+ _dplus_offset]*prod)));
//Dummy.label("com/polytechnik/lapack/Dlarrf",15);
}              //  Close for() loop. 
}
rrr1 = (tmp/((spdiam*Math.sqrt(znm2))));
if ((rrr1 <= 8.0))  {
    sigma.val = lsigma;
shift = 1;
break label5_100;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",100);
}
}
else if ((indx == 2))  {
    tmp = Math.abs(work[(n-(1))+ _work_offset]);
znm2 = 1.0;
prod = 1.0;
oldp = 1.0;
{
int _i_inc = -1;
for (i = (n-1); i >= 1; i += _i_inc) {
if ((prod <= eps))  {
    prod = (((((work[((i+1)-(1))+ _work_offset]*lplus[((i+1)-(1))+ _lplus_offset]))/((work[(i-(1))+ _work_offset]*lplus[(i-(1))+ _lplus_offset]))))*oldp);
}
else  {
  prod = (prod*Math.abs(lplus[(i-(1))+ _lplus_offset]));
}              //  Close else.
oldp = prod;
znm2 = (znm2+( Math.pow(prod, 2)));
tmp = Math.max(tmp, Math.abs((work[(i-(1))+ _work_offset]*prod)));
//Dummy.label("com/polytechnik/lapack/Dlarrf",16);
}              //  Close for() loop. 
}
rrr2 = (tmp/((spdiam*Math.sqrt(znm2))));
if ((rrr2 <= 8.0))  {
    sigma.val = rsigma;
shift = 2;
break label5_100;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",100);
}
}              // Close else if()
// 
}
// 
}//label50: Dummy.label("com/polytechnik/lapack/Dlarrf",50);
/***
*        If we are here, both shifts failed also the RRR test.
*        Back off to the outside
***/
if ((ktry < 1))  {
    lsigma = Math.max((lsigma-ldelta), (lsigma-ldmax));
rsigma = Math.min((rsigma+rdelta), (rsigma+rdmax));
ldelta = (2.0*ldelta);
rdelta = (2.0*rdelta);
ktry = (ktry+1);
continue label5_100;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",5);
/***
*        None of the representations investigated satisfied our
*        criteria. Take the best one we found.
***/
}
else  {
  if ((((smlgrowth < fail)) || nofail))  {
    lsigma = bestshift;
rsigma = bestshift;
forcer = true;
continue label5_100;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",5);
}
else  {
  info.val = 1;
  return;//Dummy.go_to("com/polytechnik/lapack/Dlarrf",999999);
}              //  Close else.
// 
}              //  Close else.

/* break label5_100; // unreacheable anyway */ 
}// END label5_100;
//label100:
//Dummy.label("com/polytechnik/lapack/Dlarrf",100);
if ((shift == 1))  {
    // *        store new L and D back into DPLUS, LPLUS
}
else if ((shift == 2))  {
    com.polytechnik.lapack.Dcopy.dcopy(n,work,_work_offset,1,dplus,_dplus_offset,1);
com.polytechnik.lapack.Dcopy.dcopy((n-1),work,((n+1)-(1))+ _work_offset,1,lplus,_lplus_offset,1);
// 
}              // Close else if()
/***
*
*     End of DLARRF
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrf",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrf",999999);
return;
   }
} // End class.
