/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrr.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrr.java,v 1.3 2012/12/15 18:38:32 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrr {

// *> \brief \b DLARRR performs tests to decide whether the symmetric tridi
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRR + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRR( N, D, E, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            N, INFO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), E( * )
// *       ..
// *  
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Perform tests to decide whether the symmetric tridiagonal matrix T
// *> warrants expensive computations which guarantee high relative accurac
// *> in the eigenvalues.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix. N > 0.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The N diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the first (N-1) entries contain the subdiagonal
// *>          elements of the tridiagonal matrix T; E(N) is set to ZERO.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          INFO = 0(default) : the matrix warrants computations preserv
// *>                              relative accuracy.
// *>          INFO = 1          : the matrix warrants computations guarant
// *>                              only absolute accuracy.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***

*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     As a default, do NOT go for relative-accuracy preserving computations.
***/
// 

public static void dlarrr (int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
intW info)  {

final double RELCOND = 0.999;
int i= 0;
boolean yesrel= false;
double eps= 0.0d;
double safmin= 0.0d;
double smlnum= 0.0d;
double rmin= 0.0d;
double tmp= 0.0d;
double tmp2= 0.0d;
double offdig= 0.0d;
double offdig2= 0.0d;
info.val = 1;
safmin = com.polytechnik.lapack.Dlamch.dlamch('S');
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
smlnum = (safmin/eps);
/***

*     Tests for relative accuracy
*
*     Test for scaled diagonal dominance
*     Scale the diagonal entries to one and check whether the sum of the
*     off-diagonals is less than one
*
*     The sdd relative error bounds have a 1/(1- 2*x) factor in them,
*     x = max(OFFDIG + OFFDIG2), so when x is close to 1/2, no relative
*     accuracy is promised.  In the notation of the code fragment below,
*     1/(1 - (OFFDIG + OFFDIG2)) is the condition number.
*     We don't think it is worth going into "sdd mode" unless the relative
*     condition number is reasonable, not 1/macheps.
*     The threshold should be compatible with other thresholds used in the
*     code. We set  OFFDIG + OFFDIG2 <= .999 =: RELCOND, it corresponds
*     to losing at most 3 decimal digits: 1 / (1 - (OFFDIG + OFFDIG2)) <= 1000
*     instead of the current OFFDIG + OFFDIG2 < 1
*
***/
rmin = Math.sqrt(smlnum);
yesrel = true;
offdig = 0.0;
tmp = Math.sqrt(Math.abs(d[(1-(1))+ _d_offset]));
if ((tmp < rmin)) {
    yesrel = false;
}
label11:{
    if ((!yesrel)) {
	break label11;//Dummy.go_to("com/polytechnik/lapack/Dlarrr",11);
    }
    {
for (i = 2; i <= n; i++) {
tmp2 = Math.sqrt(Math.abs(d[(i-(1))+ _d_offset]));
if ((tmp2 < rmin)) {
    yesrel = false;
}
    if ((!yesrel)) {
	break label11;//Dummy.go_to("com/polytechnik/lapack/Dlarrr",11);
}
    offdig2 = (Math.abs(e[((i-1)-(1))+ _e_offset])/((tmp*tmp2)));
if (((offdig+offdig2) >= RELCOND)) {
    yesrel = false;
}
    if ((!yesrel)) {
	break label11;//Dummy.go_to("com/polytechnik/lapack/Dlarrr",11);
}
    tmp = tmp2;
offdig = offdig2;
//Dummy.label("com/polytechnik/lapack/Dlarrr",10);
}              //  Close for() loop. 
}
// 
} // END label11, //Dummy.label("com/polytechnik/lapack/Dlarrr",11);

if (yesrel)  {
    info.val = 0;
//Dummy.go_to("com/polytechnik/lapack/Dlarrr",999999);
}
else  {
  /***
*

*
*     *** MORE TO BE IMPLEMENTED ***
*

*
*     Test if the lower bidiagonal matrix L from T = L D L^T
*     (zero shift facto) is well conditioned
*

*
*     Test if the upper bidiagonal matrix U from T = U D U^T
*     (zero shift facto) is well conditioned.
*     In this case, the matrix needs to be flipped and, at the end
*     of the eigenvector computation, the flip needs to be applied
*     to the computed eigenvectors (and the support)
*

*
***/
}              //  Close else.
/***
*
*     END OF DLARRR
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrr",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrr",999999);
return;
   }
} // End class.
