/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrv.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrv.java,v 1.7 2012/12/16 07:04:43 mal Exp $
 *
 */

package com.polytechnik.lapack;


public class Dlarrv {

// *> \brief \b DLARRV computes the eigenvectors of the tridiagonal matrix 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRV + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRV( N, VL, VU, D, L, PIVMIN,
// *                          ISPLIT, M, DOL, DOU, MINRGP,
// *                          RTOL1, RTOL2, W, WERR, WGAP,
// *                          IBLOCK, INDEXW, GERS, Z, LDZ, ISUPPZ,
// *                          WORK, IWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            DOL, DOU, INFO, LDZ, M, N
// *       DOUBLE PRECISION   MINRGP, PIVMIN, RTOL1, RTOL2, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IBLOCK( * ), INDEXW( * ), ISPLIT( * ),
// *      $                   ISUPPZ( * ), IWORK( * )
// *       DOUBLE PRECISION   D( * ), GERS( * ), L( * ), W( * ), WERR( * ),
// *      $                   WGAP( * ), WORK( * )
// *       DOUBLE PRECISION  Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARRV computes the eigenvectors of the tridiagonal matrix
// *> T = L D L**T given L, D and APPROXIMATIONS to the eigenvalues of L D 
// *> The input eigenvalues should have been computed by DLARRE.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          Lower and upper bounds of the interval that contains the des
// *>          eigenvalues. VL < VU. Needed to compute gaps on the left or 
// *>          end of the extremal eigenvalues in the desired RANGE.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the N diagonal elements of the diagonal matrix D.

// *>          On exit, D may be overwritten.
// *> \endverbatim
// *>
// *> \param[in,out] L
// *> \verbatim
// *>          L is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the (N-1) subdiagonal elements of the unit
// *>          bidiagonal matrix L are in elements 1 to N-1 of L
// *>          (if the matrix is not splitted.) At the end of each block
// *>          is stored the corresponding shift as given by DLARRE.
// *>          On exit, L is overwritten.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot allowed in the Sturm sequence.
// *> \endverbatim
// *>
// *> \param[in] ISPLIT
// *> \verbatim
// *>          ISPLIT is INTEGER array, dimension (N)
// *>          The splitting points, at which T breaks up into blocks.
// *>          The first block consists of rows/columns 1 to
// *>          ISPLIT( 1 ), the second of rows/columns ISPLIT( 1 )+1
// *>          through ISPLIT( 2 ), etc.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The total number of input eigenvalues.  0 <= M <= N.
// *> \endverbatim
// *>
// *> \param[in] DOL
// *> \verbatim
// *>          DOL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] DOU
// *> \verbatim
// *>          DOU is INTEGER
// *>          If the user wants to compute only selected eigenvectors from
// *>          the eigenvalues supplied, he can specify an index range DOL:
// *>          Or else the setting DOL=1, DOU=M should be applied.
// *>          Note that DOL and DOU refer to the order in which the eigenv
// *>          are stored in W.
// *>          If the user wants to compute only selected eigenpairs, then

// *>          the columns DOL-1 to DOU+1 of the eigenvector space Z contai
// *>          computed eigenvectors. All other columns of Z are set to zer
// *> \endverbatim
// *>
// *> \param[in] MINRGP
// *> \verbatim
// *>          MINRGP is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] RTOL1
// *> \verbatim
// *>          RTOL1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] RTOL2
// *> \verbatim
// *>          RTOL2 is DOUBLE PRECISION
// *>           Parameters for bisection.
// *>           An interval [LEFT,RIGHT] has converged if
// *>           RIGHT-LEFT.LT.MAX( RTOL1*GAP, RTOL2*MAX(|LEFT|,|RIGHT|) )
// *> \endverbatim
// *>
// *> \param[in,out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          The first M elements of W contain the APPROXIMATE eigenvalue
// *>          which eigenvectors are to be computed.  The eigenvalues
// *>          should be grouped by split-off block and ordered from
// *>          smallest to largest within the block ( The output array
// *>          W from DLARRE is expected here ). Furthermore, they are with
// *>          respect to the shift of the corresponding root representatio
// *>          for their block. On exit, W holds the eigenvalues of the
// *>          UNshifted matrix.
// *> \endverbatim
// *>
// *> \param[in,out] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION array, dimension (N)
// *>          The first M elements contain the semiwidth of the uncertaint
// *>          interval of the corresponding eigenvalue in W
// *> \endverbatim
// *>
// *> \param[in,out] WGAP
// *> \verbatim
// *>          WGAP is DOUBLE PRECISION array, dimension (N)
// *>          The separation from the right neighbor eigenvalue in W.
// *> \endverbatim
// *>
// *> \param[in] IBLOCK
// *> \verbatim
// *>          IBLOCK is INTEGER array, dimension (N)
// *>          The indices of the blocks (submatrices) associated with the

// *>          corresponding eigenvalues in W; IBLOCK(i)=1 if eigenvalue
// *>          W(i) belongs to the first block from the top, =2 if W(i)
// *>          belongs to the second block, etc.
// *> \endverbatim
// *>
// *> \param[in] INDEXW
// *> \verbatim
// *>          INDEXW is INTEGER array, dimension (N)
// *>          The indices of the eigenvalues within each block (submatrix)
// *>          for example, INDEXW(i)= 10 and IBLOCK(i)=2 imply that the
// *>          i-th eigenvalue W(i) is the 10-th eigenvalue in the second b
// *> \endverbatim
// *>
// *> \param[in] GERS
// *> \verbatim
// *>          GERS is DOUBLE PRECISION array, dimension (2*N)
// *>          The N Gerschgorin intervals (the i-th Gerschgorin interval
// *>          is (GERS(2*i-1), GERS(2*i)). The Gerschgorin intervals shoul
// *>          be computed from the original UNshifted matrix.
// *> \endverbatim
// *>
// *> \param[out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, max(1,M) )
// *>          If INFO = 0, the first M columns of Z contain the
// *>          orthonormal eigenvectors of the matrix T
// *>          corresponding to the input eigenvalues, with the i-th
// *>          column of Z holding the eigenvector associated with W(i).
// *>          Note: the user must ensure that at least max(1,M) columns ar
// *>          supplied in the array Z.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1, and if
// *>          JOBZ = 'V', LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] ISUPPZ
// *> \verbatim
// *>          ISUPPZ is INTEGER array, dimension ( 2*max(1,M) )
// *>          The support of the eigenvectors in Z, i.e., the indices
// *>          indicating the nonzero elements in Z. The I-th eigenvector
// *>          is nonzero only in elements ISUPPZ( 2*I-1 ) through
// *>          ISUPPZ( 2*I ).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (12*N)
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (7*N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>
// *>          > 0:  A problem occured in DLARRV.
// *>          < 0:  One of the called subroutines signaled an internal pro
// *>                Needs inspection of the corresponding parameter IINFO

// *>                for further information.
// *>
// *>          =-1:  Problem in DLARRB when refining a child's eigenvalues.
// *>          =-2:  Problem in DLARRF when computing the RRR of a child.
// *>                When a child is inside a tight cluster, it can be diff
// *>                to find an RRR. A partial remedy from the user's point
// *>                view is to make the parameter MINRGP smaller and recom
// *>                However, as the orthogonality of the computed vectors 
// *>                proportional to 1/MINRGP, the user should be aware tha
// *>                he might be trading in precision when he decreases MIN
// *>          =-3:  Problem in DLARRB when refining a single eigenvalue
// *>                after the Rayleigh correction was rejected.
// *>          = 5:  The Rayleigh Quotient Iteration failed to converge to

// *>                full accuracy in MAXITR steps.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*     ..

*     The first N entries of WORK are reserved for the eigenvalues
***/

public static void dlarrv (int n,
double vl,
double vu,
double [] d, int _d_offset,
double [] l, int _l_offset,
double pivmin,
int [] isplit, int _isplit_offset,
int m,
int dol,
int dou,
double minrgp,
doubleW rtol1,
doubleW rtol2,
double [] w, int _w_offset,
double [] werr, int _werr_offset,
double [] wgap, int _wgap_offset,
int [] iblock, int _iblock_offset,
int [] indexw, int _indexw_offset,
double [] gers, int _gers_offset,
double [] z, int _z_offset,
int ldz,
int [] isuppz, int _isuppz_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

boolean eskip= false;
boolean needbs= false;
boolean stp2ii= false;
boolean tryrqc= false;
boolean usedbs= false;
boolean usedrq= false;
int done= 0;
int i= 0;
int ibegin= 0;
int idone= 0;
int iend= 0;
int ii= 0;
int iindc1= 0;
int iindc2= 0;
int iindr= 0;
int iindwk= 0;
intW iinfo= new intW(0);
int im= 0;
int in= 0;
int indeig= 0;
int indld= 0;
int indlld= 0;
int indwrk= 0;
int isupmn= 0;
int isupmx= 0;
int iter= 0;
int itmp1= 0;
int j= 0;
int jblk= 0;
int k= 0;
int miniwsize= 0;
int minwsize= 0;
int nclus= 0;
int ndepth= 0;
intW negcnt= new intW(0);
int newcls= 0;
int newfst= 0;
int newftt= 0;
int newlst= 0;
int newsiz= 0;
int offset= 0;
int oldcls= 0;
int oldfst= 0;
int oldien= 0;
int oldlst= 0;
int oldncl= 0;
int p= 0;
int parity= 0;
int q= 0;
int wbegin= 0;
int wend= 0;
int windex= 0;
int windmn= 0;
int windpl= 0;
int zfrom= 0;
int zto= 0;
int zusedl= 0;
int zusedu= 0;
int zusedw= 0;
double bstres= 0.0d;
double bstw= 0.0d;
double eps= 0.0d;
double fudge= 0.0d;
double gap= 0.0d;
double gaptol= 0.0d;
double gl= 0.0d;
double gu= 0.0d;
double lambda= 0.0d;
double left= 0.0d;
double lgap= 0.0d;
doubleW mingma= new doubleW(0.0d);
doubleW nrminv= new doubleW(0.0d);
doubleW resid= new doubleW(0.0d);
double rgap= 0.0d;
double right= 0.0d;
doubleW rqcorr= new doubleW(0.0d);
double rqtol= 0.0d;
double savgap= 0.0d;
double sgndef= 0.0d;
double sigma= 0.0d;
double spdiam= 0.0d;
double ssigma= 0.0d;
doubleW tau= new doubleW(0.0d);
double tmp= 0.0d;
double tol= 0.0d;
doubleW ztz= new doubleW(0.0d);
indld = (n+1);
indlld = ((2*n)+1);
indwrk = ((3*n)+1);
// 
minwsize = (12*n);
{
for (i = 1; i <= minwsize; i++) {
work[(i-(1))+ _work_offset] = 0.0;
/***

*     IWORK(IINDR+1:IINDR+N) hold the twist indices R for the
*     factorization used to compute the FP vector
***/
//Dummy.label("com/polytechnik/lapack/Dlarrv",5);
}              //  Close for() loop. 
}
/***
*     IWORK(IINDC1+1:IINC2+N) are used to store the clusters of the current
*     layer and the one above.
***/
iindr = 0;
iindc1 = n;
iindc2 = (2*n);
// 
iindwk = ((3*n)+1);
miniwsize = (7*n);
{
for (i = 1; i <= miniwsize; i++) {
iwork[(i-(1))+ _iwork_offset] = 0;
// 
//Dummy.label("com/polytechnik/lapack/Dlarrv",10);
}              //  Close for() loop. 
}
zusedl = 1;
// *        Set lower bound for use of Z
if ((dol > 1))  {
    zusedl = (dol-1);
}
zusedu = m;
// *        Set lower bound for use of Z
if ((dou < m))  {
    zusedu = (dou+1);
// *     The width of the part of Z that is used
}
/***


***/
zusedw = ((zusedu-zusedl)+1);
// 
com.polytechnik.lapack.Dlaset.dlaset('F',n,zusedw,0.0,0.0,z,(1-(1))+(zusedl-(1)) * (ldz)+ _z_offset,ldz);
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
/***
*
*     Set expert flags for standard code.
***/
rqtol = (2.0*eps);
// 
tryrqc = true;
if ((((dol == 1)) && ((dou == m))))  {
    /***
*        Only selected eigenpairs are computed. Since the other evalues
*        are not refined by RQ iteration, bisection has to compute to full
*        accuracy.
***/
}
else  {
  rtol1.val = (4.0*eps);
rtol2.val = (4.0*eps);
/***

*     The entries WBEGIN:WEND in W, WERR, WGAP correspond to the
*     desired eigenvalues. The support of the nonzero eigenvector
*     entries is contained in the interval IBEGIN:IEND.
*     Remark that if k eigenpairs are desired, then the eigenvectors
*     are stored in k contiguous columns of Z.

*     DONE is the number of eigenvectors already computed
***/
}              //  Close else.
done = 0;
ibegin = 1;
wbegin = 1;
{
label170:for (jblk = 1; jblk <= iblock[(m-(1))+ _iblock_offset]; jblk++) {
iend = isplit[(jblk-(1))+ _isplit_offset];
/***
*        Find the eigenvectors of the submatrix indexed IBEGIN
*        through IEND.
***/
sigma = l[(iend-(1))+ _l_offset];
wend = (wbegin-1);
label15:for(;;) {
//Dummy.label("com/polytechnik/lapack/Dlarrv",15);
if ((wend < m))  {
    if ((iblock[((wend+1)-(1))+ _iblock_offset] == jblk))  {
    wend = (wend+1);
    continue label15;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",15);
}
}
break label15;}
if ((wend < wbegin))  {
    ibegin = (iend+1);
    continue label170;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",170);
}
else if ((((wend < dol)) || ((wbegin > dou))))  {
    ibegin = (iend+1);
wbegin = (wend+1);
continue label170;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",170);
/***

*        Find local spectral diameter of the block
***/
}              // Close else if()
gl = gers[(((2*ibegin)-1)-(1))+ _gers_offset];
gu = gers[((2*ibegin)-(1))+ _gers_offset];
{
for (i = (ibegin+1); i <= iend; i++) {
gl = Math.min(gers[(((2*i)-1)-(1))+ _gers_offset], gl);
gu = Math.max(gers[((2*i)-(1))+ _gers_offset], gu);
//Dummy.label("com/polytechnik/lapack/Dlarrv",20);
}              //  Close for() loop. 
}
/***

*        OLDIEN is the last index of the previous block
***/
spdiam = (gu-gl);
// *        Calculate the size of the current block
oldien = (ibegin-1);
// *        The number of eigenvalues in the current block
in = ((iend-ibegin)+1);
/***

*        This is for a 1x1 block
***/
im = ((wend-wbegin)+1);
if ((ibegin == iend))  {
    done = (done+1);
z[(ibegin-(1))+(wbegin-(1)) * (ldz)+ _z_offset] = 1.0;
isuppz[(((2*wbegin)-1)-(1))+ _isuppz_offset] = ibegin;
isuppz[((2*wbegin)-(1))+ _isuppz_offset] = ibegin;
w[(wbegin-(1))+ _w_offset] = (w[(wbegin-(1))+ _w_offset]+sigma);
work[(wbegin-(1))+ _work_offset] = w[(wbegin-(1))+ _w_offset];
ibegin = (iend+1);
wbegin = (wbegin+1);
continue label170;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",170);
/***

*        The desired (shifted) eigenvalues are stored in W(WBEGIN:WEND)
*        Note that these can be approximations, in this case, the corresp.
*        entries of WERR give the size of the uncertainty interval.
*        The eigenvalue approximations will be refined when necessary as
*        high relative accuracy is required for the computation of the
*        corresponding eigenvectors.
***/
}
/***

*        We store in W the eigenvalue approximations w.r.t. the original
*        matrix T.
***/
com.polytechnik.lapack.Dcopy.dcopy(im,w,(wbegin-(1))+ _w_offset,1,work,(wbegin-(1))+ _work_offset,1);
{
for (i = 1; i <= im; i++) {
w[(((wbegin+i)-1)-(1))+ _w_offset] = (w[(((wbegin+i)-1)-(1))+ _w_offset]+sigma);
/***


*        NDEPTH is the current depth of the representation tree
***/
//Dummy.label("com/polytechnik/lapack/Dlarrv",30);
}              //  Close for() loop. 
}
// *        PARITY is either 1 or 0
ndepth = 0;
/***
*        NCLUS is the number of clusters for the next level of the
*        representation tree, we start with NCLUS = 1 for the root
***/
parity = 1;
nclus = 1;
iwork[((iindc1+1)-(1))+ _iwork_offset] = 1;
/***

*        IDONE is the number of eigenvectors already computed in the current
*        block
***/
iwork[((iindc1+2)-(1))+ _iwork_offset] = im;
/***
*        loop while( IDONE.LT.IM )
*        generate the representation tree for the current block and
*        compute the eigenvectors
***/
idone = 0;
label40:for(;;){
    //Dummy.label("com/polytechnik/lapack/Dlarrv",40);
// *           This is a crude protection against infinitely deep trees
if ((idone < im))  {
    if ((ndepth > m))  {
    info.val = -2;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",999999);
/***
*           breadth first processing of the current level of the representation
*           tree: OLDNCL = number of clusters on current level
***/
}
// *           reset NCLUS to count the number of child clusters
oldncl = nclus;
// *
nclus = 0;
parity = (1-parity);
if ((parity == 0))  {
    oldcls = iindc1;
newcls = iindc2;
}
else  {
  oldcls = iindc2;
newcls = iindc1;
// *           Process the clusters on the current level
}              //  Close else.
{
for (i = 1; i <= oldncl; i++) {
/***
*              OLDFST, OLDLST = first, last index of current cluster.
*                               cluster indices start with 1 and are relative
*                               to WBEGIN when accessing W, WGAP, WERR, Z
***/
j = (oldcls+(2*i));
oldfst = iwork[((j-1)-(1))+ _iwork_offset];
oldlst = iwork[(j-(1))+ _iwork_offset];
/***
*                 Retrieve relatively robust representation (RRR) of cluster
*                 that has been computed at the previous level
*                 The RRR is stored in Z and overwritten once the eigenvectors
*                 have been computed or when the cluster is refined

***/
if ((ndepth > 0))  {
    /***
*                    Get representation from location of the leftmost evalue
*                    of the cluster
***/
if ((((dol == 1)) && ((dou == m))))  {
    j = ((wbegin+oldfst)-1);
}
else  {
  // *                       Get representation from the left end of Z array
if ((((wbegin+oldfst)-1) < dol))  {
    j = (dol-1);
// *                       Get representation from the right end of Z array
}
else if ((((wbegin+oldfst)-1) > dou))  {
    j = dou;
}              // Close else if()
else  {
  j = ((wbegin+oldfst)-1);
}              //  Close else.
}              //  Close else.
com.polytechnik.lapack.Dcopy.dcopy(in,z,(ibegin-(1))+(j-(1)) * (ldz)+ _z_offset,1,d,(ibegin-(1))+ _d_offset,1);
com.polytechnik.lapack.Dcopy.dcopy((in-1),z,(ibegin-(1))+((j+1)-(1)) * (ldz)+ _z_offset,1,l,(ibegin-(1))+ _l_offset,1);
/***

*                 Set the corresponding entries in Z to zero
***/
sigma = z[(iend-(1))+((j+1)-(1)) * (ldz)+ _z_offset];
com.polytechnik.lapack.Dlaset.dlaset('F',in,2,0.0,0.0,z,(ibegin-(1))+(j-(1)) * (ldz)+ _z_offset,ldz);
/***

*              Compute DL and DLL of current RRR
***/
}
{
for (j = ibegin; j <= (iend-1); j++) {
tmp = (d[(j-(1))+ _d_offset]*l[(j-(1))+ _l_offset]);
work[(((indld-1)+j)-(1))+ _work_offset] = tmp;
work[(((indlld-1)+j)-(1))+ _work_offset] = (tmp*l[(j-(1))+ _l_offset]);
// 
//Dummy.label("com/polytechnik/lapack/Dlarrv",50);
}              //  Close for() loop. 
}
/***
*                 P and Q are index of the first and last eigenvalue to compute
*                 within the current block
***/
if ((ndepth > 0))  {
    p = indexw[(((wbegin-1)+oldfst)-(1))+ _indexw_offset];
/***
*                 Offset for the arrays WORK, WGAP and WERR, i.e., the P-OFFSET
*                 through the Q-OFFSET elements of these arrays are to be used.
*                  OFFSET = P-OLDFST
***/
q = indexw[(((wbegin-1)+oldlst)-(1))+ _indexw_offset];
/***
*                 perform limited bisection (if necessary) to get approximate
*                 eigenvalues to the precision needed.
***/
offset = (indexw[(wbegin-(1))+ _indexw_offset]-1);
com.polytechnik.lapack.Dlarrb.dlarrb(in,d,(ibegin-(1))+ _d_offset,work,(((indlld+ibegin)-1)-(1))+ _work_offset,p,q,rtol1.val,rtol2.val,offset,work,(wbegin-(1))+ _work_offset,wgap,(wbegin-(1))+ _wgap_offset,werr,(wbegin-(1))+ _werr_offset,work,(indwrk-(1))+ _work_offset,iwork,(iindwk-(1))+ _iwork_offset,pivmin,spdiam,in,iinfo);
if ((iinfo.val != 0))  {
    info.val = -1;
    return ;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",999999);
/***
*                 We also recompute the extremal gaps. W holds all eigenvalues
*                 of the unshifted matrix and must be used for computation
*                 of WGAP, the entries of WORK might stem from RRRs with
*                 different shifts. The gaps from WBEGIN-1+OLDFST to
*                 WBEGIN-1+OLDLST are correctly computed in DLARRB.
*                 However, we only allow the gaps to become greater since
*                 this is what should happen when we decrease WERR
***/
}
if ((oldfst > 1))  {
    wgap[(((wbegin+oldfst)-2)-(1))+ _wgap_offset] = Math.max(wgap[(((wbegin+oldfst)-2)-(1))+ _wgap_offset], (((w[(((wbegin+oldfst)-1)-(1))+ _w_offset]-werr[(((wbegin+oldfst)-1)-(1))+ _werr_offset])-w[(((wbegin+oldfst)-2)-(1))+ _w_offset])-werr[(((wbegin+oldfst)-2)-(1))+ _werr_offset]));
}
if ((((wbegin+oldlst)-1) < wend))  {
    wgap[(((wbegin+oldlst)-1)-(1))+ _wgap_offset] = Math.max(wgap[(((wbegin+oldlst)-1)-(1))+ _wgap_offset], (((w[((wbegin+oldlst)-(1))+ _w_offset]-werr[((wbegin+oldlst)-(1))+ _werr_offset])-w[(((wbegin+oldlst)-1)-(1))+ _w_offset])-werr[(((wbegin+oldlst)-1)-(1))+ _werr_offset]));
/***
*                 Each time the eigenvalues in WORK get refined, we store
*                 the newly found approximation with all shifts applied in W
***/
}
{
for (j = oldfst; j <= oldlst; j++) {
w[(((wbegin+j)-1)-(1))+ _w_offset] = (work[(((wbegin+j)-1)-(1))+ _work_offset]+sigma);
//Dummy.label("com/polytechnik/lapack/Dlarrv",53);
}              //  Close for() loop. 
}
/***

*              Process the current node.
***/
}
newfst = oldfst;
{
label140:for (j = oldfst; j <= oldlst; j++) {
label139:{
/***
*                    we are at the right end of the cluster, this is also the
*                    boundary of the child cluster
***/
if ((j == oldlst))  {
    newlst = j;
/***
*                    the right relative gap is big enough, the child cluster
*                    (NEWFST,..,NEWLST) is well separated from the following
***/
}
else if ((wgap[(((wbegin+j)-1)-(1))+ _wgap_offset] >= (minrgp*Math.abs(work[(((wbegin+j)-1)-(1))+ _work_offset]))))  {
    newlst = j;
/***
*                    inside a child cluster, the relative gap is not
*                    big enough.
***/
}              // Close else if()
else  {
    continue label140;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",140);
/***

*                 Compute size of child cluster found
***/
}              //  Close else.
/***

*                 NEWFTT is the place in Z where the new RRR or the computed
*                 eigenvector is to be stored
***/
newsiz = ((newlst-newfst)+1);
/***
*                    Store representation at location of the leftmost evalue
*                    of the cluster
***/
if ((((dol == 1)) && ((dou == m))))  {
    newftt = ((wbegin+newfst)-1);
}
else  {
  // *                       Store representation at the left end of Z array
if ((((wbegin+newfst)-1) < dol))  {
    newftt = (dol-1);
// *                       Store representation at the right end of Z array
}
else if ((((wbegin+newfst)-1) > dou))  {
    newftt = dou;
}              // Close else if()
else  {
  newftt = ((wbegin+newfst)-1);
}              //  Close else.
// 
}              //  Close else.
/***
*
*                    Current child is not a singleton but a cluster.
*                    Compute and store new representation of child.
*
*
*                    Compute left and right cluster gap.
*
*                    LGAP and RGAP are not computed from WORK because
*                    the eigenvalue approximations may stem from RRRs
*                    different shifts. However, W hold all eigenvalues
*                    of the unshifted matrix. Still, the entries in WGAP
*                    have to be computed from WORK since the entries
*                    in W might be of the same order so that gaps are not
*                    exhibited correctly for very close eigenvalues.
***/
if ((newsiz > 1))  {
    if ((newfst == 1))  {
    lgap = Math.max(0.0, ((w[(wbegin-(1))+ _w_offset]-werr[(wbegin-(1))+ _werr_offset])-vl));
}
else  {
  lgap = wgap[(((wbegin+newfst)-2)-(1))+ _wgap_offset];
}              //  Close else.
/***
*
*                    Compute left- and rightmost eigenvalue of child
*                    to high precision in order to shift as close
*                    as possible and obtain as large relative gaps
*                    as possible
*
***/
rgap = wgap[(((wbegin+newlst)-1)-(1))+ _wgap_offset];
{
for (k = 1; k <= 2; k++) {
if ((k == 1))  {
    p = indexw[(((wbegin-1)+newfst)-(1))+ _indexw_offset];
}
else  {
  p = indexw[(((wbegin-1)+newlst)-(1))+ _indexw_offset];
}              //  Close else.
offset = (indexw[(wbegin-(1))+ _indexw_offset]-1);
com.polytechnik.lapack.Dlarrb.dlarrb(in,d,(ibegin-(1))+ _d_offset,work,(((indlld+ibegin)-1)-(1))+ _work_offset,p,p,rqtol,rqtol,offset,work,(wbegin-(1))+ _work_offset,wgap,(wbegin-(1))+ _wgap_offset,werr,(wbegin-(1))+ _werr_offset,work,(indwrk-(1))+ _work_offset,iwork,(iindwk-(1))+ _iwork_offset,pivmin,spdiam,in,iinfo);
// *
//Dummy.label("com/polytechnik/lapack/Dlarrv",55);
}              //  Close for() loop. 
}
/***
*                       if the cluster contains no desired eigenvalues
*                       skip the computation of that branch of the rep. tree
*
*                       We could skip before the refinement of the extremal
*                       eigenvalues of the child, but then the representation
*                       tree could be different from the one when nothing is
*                       skipped. For this reason we skip at this place.
***/
if ((((((wbegin+newlst)-1) < dol)) || ((((wbegin+newfst)-1) > dou))))  {
    idone = (((idone+newlst)-newfst)+1);
    break label139;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",139);
/***
*
*                    Compute RRR of child cluster.
*                    Note that the new RRR is stored in Z
*
*                    DLARRF needs LWORK = 2*N
***/
}
com.polytechnik.lapack.Dlarrf.dlarrf(in,d,(ibegin-(1))+ _d_offset,l,(ibegin-(1))+ _l_offset,work,(((indld+ibegin)-1)-(1))+ _work_offset,newfst,newlst,work,(wbegin-(1))+ _work_offset,wgap,(wbegin-(1))+ _wgap_offset,werr,(wbegin-(1))+ _werr_offset,spdiam,lgap,rgap,pivmin,tau,z,(ibegin-(1))+(newftt-(1)) * (ldz)+ _z_offset,z,(ibegin-(1))+((newftt+1)-(1)) * (ldz)+ _z_offset,work,(indwrk-(1))+ _work_offset,iinfo);
/***
*                       a new RRR for the cluster was found by DLARRF
*                       update shift and store it
***/
if ((iinfo.val == 0))  {
    ssigma = (sigma+tau.val);
/***
*                       WORK() are the midpoints and WERR() the semi-width
*                       Note that the entries in W are unchanged.
***/
z[(iend-(1))+((newftt+1)-(1)) * (ldz)+ _z_offset] = ssigma;
{
for (k = newfst; k <= newlst; k++) {
fudge = ((3.0*eps)*Math.abs(work[(((wbegin+k)-1)-(1))+ _work_offset]));
work[(((wbegin+k)-1)-(1))+ _work_offset] = (work[(((wbegin+k)-1)-(1))+ _work_offset]-tau.val);
// *                          Fudge errors
fudge = (fudge+((4.0*eps)*Math.abs(work[(((wbegin+k)-1)-(1))+ _work_offset])));
/***
*                          Gaps are not fudged. Provided that WERR is small
*                          when eigenvalues are close, a zero gap indicates
*                          that a new representation is needed for resolving
*                          the cluster. A fudge could lead to a wrong decision
*                          of judging eigenvalues 'separated' which in
*                          reality are not. This could have a negative impact
*                          on the orthogonality of the computed eigenvectors.
***/
werr[(((wbegin+k)-1)-(1))+ _werr_offset] = (werr[(((wbegin+k)-1)-(1))+ _werr_offset]+fudge);
// 
//Dummy.label("com/polytechnik/lapack/Dlarrv",116);
}              //  Close for() loop. 
}
nclus = (nclus+1);
k = (newcls+(2*nclus));
iwork[((k-1)-(1))+ _iwork_offset] = newfst;
iwork[(k-(1))+ _iwork_offset] = newlst;
}
else  {
  info.val = -2;
  return;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",999999);
}              //  Close else.
/***
*
*                    Compute eigenvector of singleton
*
***/
}
else  {
  // *
iter = 0;
// *
tol = ((4.0*Math.log((double)(in)))*eps);
k = newfst;
windex = ((wbegin+k)-1);
windmn = Math.max((windex-1), 1);
windpl = Math.min((windex+1), m);
lambda = work[(windex-(1))+ _work_offset];
// *                    Check if eigenvector computation is to be skipped
done = (done+1);
if ((((windex < dol)) || ((windex > dou))))  {
    eskip = true;
//Dummy.go_to("com/polytechnik/lapack/Dlarrv",125);
}
else  {
  eskip = false;
}              //  Close else.
if(!eskip){ //label125
left = (work[(windex-(1))+ _work_offset]-werr[(windex-(1))+ _werr_offset]);
right = (work[(windex-(1))+ _work_offset]+werr[(windex-(1))+ _werr_offset]);
/***
*                    Note that since we compute the eigenpairs for a child,
*                    all eigenvalue approximations are w.r.t the same shift.
*                    In this case, the entries in WORK should be used for
*                    computing the gaps since they exhibit even very small
*                    differences in the eigenvalues, as opposed to the
*                    entries in W which might "look" the same.

***/
indeig = indexw[(windex-(1))+ _indexw_offset];
/***
*                       In the case RANGE='I' and with not much initial
*                       accuracy in LAMBDA and VL, the formula
*                       LGAP = MAX( ZERO, (SIGMA - VL) + LAMBDA )
*                       can lead to an overestimation of the left gap and
*                       thus to inadequately early RQI 'convergence'.
*                       Prevent this by forcing a small left gap.
***/
if ((k == 1))  {
    lgap = (eps*Math.max(Math.abs(left), Math.abs(right)));
}
else  {
  lgap = wgap[(windmn-(1))+ _wgap_offset];
}              //  Close else.
/***
*                       In the case RANGE='I' and with not much initial
*                       accuracy in LAMBDA and VU, the formula
*                       can lead to an overestimation of the right gap and
*                       thus to inadequately early RQI 'convergence'.
*                       Prevent this by forcing a small right gap.
***/
if ((k == im))  {
    rgap = (eps*Math.max(Math.abs(left), Math.abs(right)));
}
else  {
  rgap = wgap[(windex-(1))+ _wgap_offset];
}              //  Close else.
gap = Math.min(lgap, rgap);
/***
*                       The eigenvector support can become wrong
*                       because significant entries could be cut off due to a
*                       large GAPTOL parameter in LAR1V. Prevent this.
***/
if ((((k == 1)) || ((k == im))))  {
    gaptol = 0.0;
}
else  {
  gaptol = (gap*eps);
}              //  Close else.
isupmn = in;
/***
*                    Update WGAP so that it holds the minimum gap
*                    to the left or the right. This is crucial in the
*                    case where bisection is used to ensure that the
*                    eigenvalue is refined up to the required precision.
*                    The correct value is restored afterwards.
***/
isupmx = 1;
savgap = wgap[(windex-(1))+ _wgap_offset];
/***
*                    We want to use the Rayleigh Quotient Correction
*                    as often as possible since it converges quadratically
*                    when we are close enough to the desired eigenvalue.
*                    However, the Rayleigh Quotient can have the wrong sign
*                    and lead us away from the desired eigenvalue. In this
*                    case, the best we can do is to use bisection.
***/
wgap[(windex-(1))+ _wgap_offset] = gap;
usedbs = false;
// *                    Bisection is initially turned off unless it is forced
usedrq = false;
needbs = (!tryrqc);
// *                    Check if bisection should be used to refine eigenvalue
label120:for(;;){
    //Dummy.label("com/polytechnik/lapack/Dlarrv",120);
// *                       Take the bisection as new iterate
if (needbs)  {
    usedbs = true;
itmp1 = iwork[((iindr+windex)-(1))+ _iwork_offset];
offset = (indexw[(wbegin-(1))+ _indexw_offset]-1);
com.polytechnik.lapack.Dlarrb.dlarrb(in,d,(ibegin-(1))+ _d_offset,work,(((indlld+ibegin)-1)-(1))+ _work_offset,indeig,indeig,0.0,(2.0*eps),offset,work,(wbegin-(1))+ _work_offset,wgap,(wbegin-(1))+ _wgap_offset,werr,(wbegin-(1))+ _werr_offset,work,(indwrk-(1))+ _work_offset,iwork,(iindwk-(1))+ _iwork_offset,pivmin,spdiam,itmp1,iinfo);
if ((iinfo.val != 0))  {
    info.val = -3;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",999999);
}
/***
*                       Reset twist index from inaccurate LAMBDA to
*                       force computation of true MINGMA
***/
lambda = work[(windex-(1))+ _work_offset];
iwork[((iindr+windex)-(1))+ _iwork_offset] = 0;
// *                    Given LAMBDA, compute the eigenvector.
}
dlar1v_adapter(in,1,in,lambda,d,(ibegin-(1))+ _d_offset,l,(ibegin-(1))+ _l_offset,work,(((indld+ibegin)-1)-(1))+ _work_offset,work,(((indlld+ibegin)-1)-(1))+ _work_offset,pivmin,gaptol,z,(ibegin-(1))+(windex-(1)) * (ldz)+ _z_offset,(!usedbs),negcnt,ztz,mingma,iwork,((iindr+windex)-(1))+ _iwork_offset,isuppz,(((2*windex)-1)-(1))+ _isuppz_offset,nrminv,resid,rqcorr,work,(indwrk-(1))+ _work_offset);
if ((iter == 0))  {
    bstres = resid.val;
bstw = lambda;
}
else if ((resid.val < bstres))  {
    bstres = resid.val;
bstw = lambda;
}              // Close else if()
isupmn = Math.min(isupmn, isuppz[(((2*windex)-1)-(1))+ _isuppz_offset]);
isupmx = Math.max(isupmx, isuppz[((2*windex)-(1))+ _isuppz_offset]);
/***

*                    sin alpha <= |resid|/gap
*                    Note that both the residual and the gap are
*                    proportional to the matrix, so ||T|| doesn't play
*                    a role in the quotient

*
*                    Convergence test for Rayleigh-Quotient iteration
*                    (omitted when Bisection has been used)
*
***/
iter = (iter+1);
/***
*                       We need to check that the RQCORR update doesn't
*                       move the eigenvalue away from the desired one and
*                       towards a neighbor. -> protection with bisection
***/
if ((((resid.val > (tol*gap)) && (Math.abs(rqcorr.val) > (rqtol*Math.abs(lambda)))) && (!usedbs)))  {
    // *                          The wanted eigenvalue lies to the left
if ((indeig <= negcnt.val))  {
    sgndef = -1.0;
// *                          The wanted eigenvalue lies to the right
}
else  {
  sgndef = 1.0;
/***
*                       We only use the RQCORR if it improves the
*                       the iterate reasonably.
***/
}              //  Close else.
if ((((((rqcorr.val*sgndef) >= 0.0)) && (((lambda+rqcorr.val) <= right))) && (((lambda+rqcorr.val) >= left))))  {
    // *                          Store new midpoint of bisection interval in WORK
usedrq = true;
/***
*                             The current LAMBDA is on the left of the true
*                             eigenvalue
***/
if ((sgndef == 1.0))  {
    /***
*                             We prefer to assume that the error estimate
*                             is correct. We could make the interval not
*                             as a bracket but to be modified if the RQCORR
*                             chooses to. In this case, the RIGHT side should
*                             be modified as follows:
*                              RIGHT = MAX(RIGHT, LAMBDA + RQCORR)
***/
left = lambda;
/***
*                             The current LAMBDA is on the right of the true
*                             eigenvalue
***/
}
else  {
  /***
*                             See comment about assuming the error estimate is
*                             correct above.
*                              LEFT = MIN(LEFT, LAMBDA + RQCORR)
***/
right = lambda;
}              //  Close else.
/***
*                          Take RQCORR since it has the correct sign and
*                          improves the iterate reasonably
***/
work[(windex-(1))+ _work_offset] = (0.5*((right+left)));
// *                          Update width of error interval
lambda = (lambda+rqcorr.val);
werr[(windex-(1))+ _werr_offset] = (0.5*((right-left)));
}
else  {
  needbs = true;
}              //  Close else.
/***
*                             The eigenvalue is computed to bisection accuracy
*                             compute eigenvector and stop
***/
if (((right-left) < (rqtol*Math.abs(lambda))))  {
    usedbs = true;
    continue label120;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",120);
}
else if ((iter < 10))  {
    continue label120;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",120);
}              // Close else if()
else if ((iter == 10))  {
    needbs = true;
    continue label120;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",120);
}              // Close else if()
else  {
  info.val = 5;
  return;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",999999);
}              //  Close else.
}
else  {
  stp2ii = false;
if (((usedrq && usedbs) && (bstres <= resid.val)))  {
    lambda = bstw;
stp2ii = true;
}
// *                          improve error angle by second step
if (stp2ii)  {
    dlar1v_adapter(in,1,in,lambda,d,(ibegin-(1))+ _d_offset,l,(ibegin-(1))+ _l_offset,work,(((indld+ibegin)-1)-(1))+ _work_offset,work,(((indlld+ibegin)-1)-(1))+ _work_offset,pivmin,gaptol,z,(ibegin-(1))+(windex-(1)) * (ldz)+ _z_offset,(!usedbs),negcnt,ztz,mingma,iwork,((iindr+windex)-(1))+ _iwork_offset,isuppz,(((2*windex)-1)-(1))+ _isuppz_offset,nrminv,resid,rqcorr,work,(indwrk-(1))+ _work_offset);
}
work[(windex-(1))+ _work_offset] = lambda;
/***
*
*                    Compute FP-vector support w.r.t. whole matrix
*
***/
}              //  Close else.
break label120;}
isuppz[(((2*windex)-1)-(1))+ _isuppz_offset] = (isuppz[(((2*windex)-1)-(1))+ _isuppz_offset]+oldien);
isuppz[((2*windex)-(1))+ _isuppz_offset] = (isuppz[((2*windex)-(1))+ _isuppz_offset]+oldien);
zfrom = isuppz[(((2*windex)-1)-(1))+ _isuppz_offset];
zto = isuppz[((2*windex)-(1))+ _isuppz_offset];
isupmn = (isupmn+oldien);
// *                    Ensure vector is ok if support in the RQI has changed
isupmx = (isupmx+oldien);
if ((isupmn < zfrom))  {
    {
for (ii = isupmn; ii <= (zfrom-1); ii++) {
z[(ii-(1))+(windex-(1)) * (ldz)+ _z_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dlarrv",122);
}              //  Close for() loop. 
}
}
if ((isupmx > zto))  {
    {
for (ii = (zto+1); ii <= isupmx; ii++) {
z[(ii-(1))+(windex-(1)) * (ldz)+ _z_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dlarrv",123);
}              //  Close for() loop. 
}
}
com.polytechnik.lapack.Dscal.dscal(((zto-zfrom)+1),nrminv.val,z,(zfrom-(1))+(windex-(1)) * (ldz)+ _z_offset,1);
// *                    Update W

} //label125:  Dummy.label("com/polytechnik/lapack/Dlarrv",125);
/***
*                    Recompute the gaps on the left and right
*                    But only allow them to become larger and not
*                    smaller (which can only happen through "bad"
*                    cancellation and doesn't reflect the theory
*                    where the initial gaps are underestimated due
*                    to WERR being too crude.)
***/
w[(windex-(1))+ _w_offset] = (lambda+sigma);
if ((!eskip))  {
    if ((k > 1))  {
    wgap[(windmn-(1))+ _wgap_offset] = Math.max(wgap[(windmn-(1))+ _wgap_offset], (((w[(windex-(1))+ _w_offset]-werr[(windex-(1))+ _werr_offset])-w[(windmn-(1))+ _w_offset])-werr[(windmn-(1))+ _werr_offset]));
}
if ((windex < wend))  {
    wgap[(windex-(1))+ _wgap_offset] = Math.max(savgap, (((w[(windpl-(1))+ _w_offset]-werr[(windpl-(1))+ _werr_offset])-w[(windex-(1))+ _w_offset])-werr[(windex-(1))+ _werr_offset]));
}
}
idone = (idone+1);
/***
*                 here ends the code for the current child
*
***/
}              //  Close else.
// *                 Proceed to any remaining child nodes
} //label139:
//   Dummy.label("com/polytechnik/lapack/Dlarrv",139);
newfst = (j+1);
//Dummy.label("com/polytechnik/lapack/Dlarrv",140);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlarrv",150);
}              //  Close for() loop. 
}
ndepth = (ndepth+1);
continue label40;//Dummy.go_to("com/polytechnik/lapack/Dlarrv",40);
}
break label40;}
ibegin = (iend+1);
wbegin = (wend+1);
/***
*

***/
//Dummy.label("com/polytechnik/lapack/Dlarrv",170);
}              //  Close for() loop. 
}
/***
*
*     End of DLARRV
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrv",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrv",999999);
return;
   }
// adapter for dlar1v(IIID[DI[DI[DI[DIDD[DIZLorg/netlib/util/intW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/intW;[IILorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;[DI)V
private static void dlar1v_adapter(int arg0 ,int arg1 ,int arg2 ,double arg3 ,double [] arg4 , int arg4_offset ,double [] arg5 , int arg5_offset ,double [] arg6 , int arg6_offset ,double [] arg7 , int arg7_offset ,double arg8 ,double arg9 ,double [] arg10 , int arg10_offset ,boolean arg11 ,intW arg12 ,doubleW arg13 ,doubleW arg14 ,int [] arg15 , int arg15_offset ,int [] arg16 , int arg16_offset ,doubleW arg17 ,doubleW arg18 ,doubleW arg19 ,double [] arg20 , int arg20_offset )
{
intW _f2j_tmp15 = new intW(arg15[arg15_offset]);

com.polytechnik.lapack.Dlar1v.dlar1v(arg0,arg1,arg2,arg3,arg4, arg4_offset,arg5, arg5_offset,arg6, arg6_offset,arg7, arg7_offset,arg8,arg9,arg10, arg10_offset,arg11,arg12,arg13,arg14,_f2j_tmp15,arg16, arg16_offset,arg17,arg18,arg19,arg20, arg20_offset);

arg15[arg15_offset] = _f2j_tmp15.val;
}

} // End class.
