/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaset.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlaset.java,v 1.4 2012/12/15 18:26:01 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlaset {

// *> \brief \b DLASET initializes the off-diagonal elements and the diagon
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLASET + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASET( UPLO, M, N, ALPHA, BETA, A, LDA )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            LDA, M, N
// *       DOUBLE PRECISION   ALPHA, BETA
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASET initializes an m-by-n matrix A to BETA on the diagonal and
// *> ALPHA on the offdiagonals.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies the part of the matrix A to be set.
// *>          = 'U':      Upper triangular part is set; the strictly lower
// *>                      triangular part of A is not changed.
// *>          = 'L':      Lower triangular part is set; the strictly upper
// *>                      triangular part of A is not changed.
// *>          Otherwise:  All of the matrix A is set.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION
// *>          The constant to which the offdiagonal elements are to be set
// *> \endverbatim
// *>
// *> \param[in] BETA
// *> \verbatim
// *>          BETA is DOUBLE PRECISION
// *>          The constant to which the diagonal elements are to be set.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On exit, the leading m-by-n submatrix of A is set as follows
// *>
// *>          if UPLO = 'U', A(i,j) = ALPHA, 1<=i<=j-1, 1<=j<=n,
// *>          if UPLO = 'L', A(i,j) = ALPHA, j+1<=i<=m, 1<=j<=n,
// *>          otherwise,     A(i,j) = ALPHA, 1<=i<=m, 1<=j<=n, i.ne.j,
// *>
// *>          and, for all UPLO, A(i,i) = BETA, 1<=i<=min(m,n).
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
/***
*
*        Set the strictly upper triangular or trapezoidal part of the
*        array to ALPHA.
*
***/

public static void dlaset (final char uplo,
int m,
int n,
double alpha,
double beta,
double [] a, int _a_offset,
int lda)  {

int i= 0;
int j= 0;
//uplo = uplo.substring((1)-1,1);
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    {
for (j = 2; j <= n; j++) {
{
for (i = 1; i <= Math.min((j-1), m); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = alpha;
//Dummy.label("com/polytechnik/lapack/Dlaset",10);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlaset",20);
}              //  Close for() loop. 
}
/***
*
*        Set the strictly lower triangular or trapezoidal part of the
*        array to ALPHA.
*
***/
}
else if (com.polytechnik.lapack.Lsame.lsame(uplo,'L'))  {
    {
for (j = 1; j <= Math.min(m, n); j++) {
{
for (i = (j+1); i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = alpha;
//Dummy.label("com/polytechnik/lapack/Dlaset",30);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlaset",40);
}              //  Close for() loop. 
}
/***
*
*        Set the leading m-by-n submatrix to ALPHA.
*
***/
}              // Close else if()
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = alpha;
//Dummy.label("com/polytechnik/lapack/Dlaset",50);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlaset",60);
}              //  Close for() loop. 
}
/***
*
*     Set the first min(M,N) diagonal elements to BETA.
*
***/
}              //  Close else.
{
for (i = 1; i <= Math.min(m, n); i++) {
a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = beta;
// *
//Dummy.label("com/polytechnik/lapack/Dlaset",70);
}              //  Close for() loop. 
}
/***
*
*     End of DLASET
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlaset",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlaset",999999);
return;
   }
} // End class.
