/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlassq.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlassq.java,v 1.3 2012/12/20 04:53:56 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlassq {

// *> \brief \b DLASSQ updates a sum of squares represented in scaled form.
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLASSQ + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASSQ( N, X, INCX, SCALE, SUMSQ )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INCX, N
// *       DOUBLE PRECISION   SCALE, SUMSQ
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   X( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASSQ  returns the values  scl  and  smsq  such that
// *>
// *>    ( scl**2 )*smsq = x( 1 )**2 +...+ x( n )**2 + ( scale**2 )*sumsq,

// *>
// *> where  x( i ) = X( 1 + ( i - 1 )*INCX ). The value of  sumsq  is
// *> assumed to be non-negative and  scl  returns the value
// *>
// *>    scl = max( scale, abs( x( i ) ) ).
// *>
// *> scale and sumsq must be supplied in SCALE and SUMSQ and
// *> scl and smsq are overwritten on SCALE and SUMSQ respectively.
// *>
// *> The routine makes only one pass through the vector x.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of elements to be used from the vector X.
// *> \endverbatim
// *>
// *> \param[in] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array, dimension (N)
// *>          The vector for which a scaled sum of squares is computed.
// *>             x( i )  = X( 1 + ( i - 1 )*INCX ), 1 <= i <= n.
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>          The increment between successive values of the vector X.
// *>          INCX > 0.
// *> \endverbatim
// *>
// *> \param[in,out] SCALE
// *> \verbatim
// *>          SCALE is DOUBLE PRECISION
// *>          On entry, the value  scale  in the equation above.
// *>          On exit, SCALE is overwritten with  scl , the scaling factor
// *>          for the sum of squares.
// *> \endverbatim
// *>
// *> \param[in,out] SUMSQ
// *> \verbatim
// *>          SUMSQ is DOUBLE PRECISION
// *>          On entry, the value  sumsq  in the equation above.
// *>          On exit, SUMSQ is overwritten with  smsq , the basic sum of

// *>          squares from which  scl  has been factored out.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlassq (int n,
double [] x, int _x_offset,
int incx,
doubleW scale,
doubleW sumsq)  {

int ix= 0;
double absxi= 0.0d;
if ((n > 0))  {
    {
int _ix_inc = incx;
for (ix = 1; (_ix_inc < 0) ? ix >= (1+(((n-1))*incx)) : ix <= (1+(((n-1))*incx)); ix += _ix_inc) {
absxi = Math.abs(x[(ix-(1))+ _x_offset]);
if (((absxi > 0.0) || com.polytechnik.lapack.Disnan.disnan(absxi)))  {
    if ((scale.val < absxi))  {
    sumsq.val = (1+(sumsq.val*( Math.pow(((scale.val/absxi)), 2))));
scale.val = absxi;
}
else  {
  sumsq.val = (sumsq.val+( Math.pow(((absxi/scale.val)), 2)));
}              //  Close else.
}
//Dummy.label("com/polytechnik/lapack/Dlassq",10);
}              //  Close for() loop. 
}
}
/***
*
*     End of DLASSQ
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlassq",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlassq",999999);
return;
   }
} // End class.
