/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorg2l.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dorg2l.java,v 1.2 2012/12/22 06:55:13 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dorg2l {

// *> \brief \b DORG2L generates all or part of the orthogonal matrix Q fro
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DORG2L + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORG2L( M, N, K, A, LDA, TAU, WORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, K, LDA, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORG2L generates an m by n real matrix Q with orthonormal columns,
// *> which is defined as the last n columns of a product of k elementary
// *> reflectors of order m
// *>
// *>       Q  =  H(k) . . . H(2) H(1)
// *>
// *> as returned by DGEQLF.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix Q. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix Q. M >= N >= 0.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The number of elementary reflectors whose product defines th
// *>          matrix Q. N >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the (n-k+i)-th column must contain the vector whic
// *>          defines the elementary reflector H(i), for i = 1,2,...,k, as
// *>          returned by DGEQLF in the last k columns of its array
// *>          argument A.
// *>          On exit, the m by n matrix Q.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The first dimension of the array A. LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGEQLF.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -i, the i-th argument has an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorg2l (int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
intW info)  {

int i= 0;
int ii= 0;
int j= 0;
int l= 0;
info.val = 0;
if ((m < 0))  {
    info.val = -1;
}
else if (((n < 0) || (n > m)))  {
    info.val = -2;
}              // Close else if()
else if (((k < 0) || (k > n)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -5;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DORG2L",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dorg2l",999999);
/***
*
*     Quick return if possible
*
***/
}
/***
*
*     Initialise columns 1:n-k to columns of the unit matrix
*
***/
if ((n <= 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dorg2l",999999);
}
    {
for (j = 1; j <= (n-k); j++) {
{
for (l = 1; l <= m; l++) {
a[(l-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorg2l",10);
}              //  Close for() loop. 
}
a[(((m-n)+j)-(1))+(j-(1)) * (lda)+ _a_offset] = 1.0;
// *
//Dummy.label("com/polytechnik/lapack/Dorg2l",20);
}              //  Close for() loop. 
}
{
for (i = 1; i <= k; i++) {
/***
*
*        Apply H(i) to A(1:m-k+i,1:n-k+i) from the left
*
***/
ii = ((n-k)+i);
a[(((m-n)+ii)-(1))+(ii-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack.Dlarf.dlarf('L',((m-n)+ii),(ii-1),a,(1-(1))+(ii-(1)) * (lda)+ _a_offset,1,tau[(i-(1))+ _tau_offset],a,_a_offset,lda,work,_work_offset);
com.polytechnik.lapack.Dscal.dscal((((m-n)+ii)-1),(-(tau[(i-(1))+ _tau_offset])),a,(1-(1))+(ii-(1)) * (lda)+ _a_offset,1);
/***
*
*        Set A(m-k+i+1:m,n-k+i) to zero
*
***/
a[(((m-n)+ii)-(1))+(ii-(1)) * (lda)+ _a_offset] = (1.0-tau[(i-(1))+ _tau_offset]);
{
for (l = (((m-n)+ii)+1); l <= m; l++) {
a[(l-(1))+(ii-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorg2l",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dorg2l",40);
}              //  Close for() loop. 
}
/***
*
*     End of DORG2L
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dorg2l",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dorg2l",999999);
return;
   }
} // End class.
