/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorg2r.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dorg2r.java,v 1.2 2012/12/22 06:53:07 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dorg2r {

// *> \brief \b DORG2R generates all or part of the orthogonal matrix Q fro
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DORG2R + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORG2R( M, N, K, A, LDA, TAU, WORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, K, LDA, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORG2R generates an m by n real matrix Q with orthonormal columns,
// *> which is defined as the first n columns of a product of k elementary

// *> reflectors of order m
// *>
// *>       Q  =  H(1) H(2) . . . H(k)
// *>
// *> as returned by DGEQRF.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix Q. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix Q. M >= N >= 0.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The number of elementary reflectors whose product defines th
// *>          matrix Q. N >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the i-th column must contain the vector which
// *>          defines the elementary reflector H(i), for i = 1,2,...,k, as
// *>          returned by DGEQRF in the first k columns of its array
// *>          argument A.
// *>          On exit, the m-by-n matrix Q.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The first dimension of the array A. LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGEQRF.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -i, the i-th argument has an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorg2r (int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
intW info)  {

int i= 0;
int j= 0;
int l= 0;
info.val = 0;
if ((m < 0))  {
    info.val = -1;
}
else if (((n < 0) || (n > m)))  {
    info.val = -2;
}              // Close else if()
else if (((k < 0) || (k > n)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -5;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DORG2R",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dorg2r",999999);
/***
*
*     Quick return if possible
*
***/
}
/***
*
*     Initialise columns k+1:n to columns of the unit matrix
*
***/
if ((n <= 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dorg2r",999999);
}
    {
for (j = (k+1); j <= n; j++) {
{
for (l = 1; l <= m; l++) {
a[(l-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorg2r",10);
}              //  Close for() loop. 
}
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = 1.0;
// *
//Dummy.label("com/polytechnik/lapack/Dorg2r",20);
}              //  Close for() loop. 
}
/***
*
*        Apply H(i) to A(i:m,i:n) from the left
*
***/
{
int _i_inc = -1;
for (i = k; i >= 1; i += _i_inc) {
if ((i < n))  {
    a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack.Dlarf.dlarf('L',((m-i)+1),(n-i),a,(i-(1))+(i-(1)) * (lda)+ _a_offset,1,tau[(i-(1))+ _tau_offset],a,(i-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,work,_work_offset);
}
if ((i < m)) {
    com.polytechnik.lapack.Dscal.dscal((m-i),(-(tau[(i-(1))+ _tau_offset])),a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1);
}
    /***
*
*        Set A(1:i-1,i) to zero
*
***/
a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = (1.0-tau[(i-(1))+ _tau_offset]);
{
for (l = 1; l <= (i-1); l++) {
a[(l-(1))+(i-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorg2r",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dorg2r",40);
}              //  Close for() loop. 
}
/***
*
*     End of DORG2R
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dorg2r",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dorg2r",999999);
return;
   }
} // End class.
