/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorghr.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dorghr.java,v 1.2 2012/12/22 14:13:05 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dorghr {

// *> \brief \b DORGHR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DORGHR + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORGHR( N, ILO, IHI, A, LDA, TAU, WORK, LWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IHI, ILO, INFO, LDA, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORGHR generates a real orthogonal matrix Q which is defined as the
// *> product of IHI-ILO elementary reflectors of order N, as returned by
// *> DGEHRD:
// *>
// *> Q = H(ilo) H(ilo+1) . . . H(ihi-1).
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix Q. N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>
// *>          ILO and IHI must have the same values as in the previous cal
// *>          of DGEHRD. Q is equal to the unit matrix except in the
// *>          submatrix Q(ilo+1:ihi,ilo+1:ihi).
// *>          1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the vectors which define the elementary reflectors
// *>          as returned by DGEHRD.
// *>          On exit, the N-by-N orthogonal matrix Q.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A. LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGEHRD.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK. LWORK >= IHI-ILO.
// *>          For optimum performance LWORK >= (IHI-ILO)*NB, where NB is
// *>          the optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorghr (int n,
int ilo,
int ihi,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

boolean lquery= false;
int i= 0;
int j= 0;
int lwkopt= 0;
int nb= 0;
int nh= 0;
info.val = 0;
nh = (ihi-ilo);
lquery = ((lwork == -1));
if ((n < 0))  {
    info.val = -1;
}
else if (((ilo < 1) || (ilo > Math.max(1, n))))  {
    info.val = -2;
}              // Close else if()
else if (((ihi < Math.min(ilo, n)) || (ihi > n)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -5;
}              // Close else if()
else if (((lwork < Math.max(1, nh)) && (!lquery)))  {
    info.val = -8;
// *
}              // Close else if()
if ((info.val == 0))  {
    nb = 1;//com.polytechnik.lapack.Ilaenv.ilaenv(1,"DORGQR"," ",nh,nh,nh,-1);
lwkopt = (Math.max(1, nh)*nb);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DORGHR",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dorghr",999999);
}
else if (lquery)  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dorghr",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((n == 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
    return;//Dummy.go_to("com/polytechnik/lapack/Dorghr",999999);
/***
*
*     Shift the vectors which define the elementary reflectors one
*     column to the right, and set the first ilo and the last n-ihi
*     rows and columns to those of the unit matrix
*
***/
}
{
int _j_inc = -1;
for (j = ihi; j >= (ilo+1); j += _j_inc) {
{
for (i = 1; i <= (j-1); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorghr",10);
}              //  Close for() loop. 
}
{
for (i = (j+1); i <= ihi; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = a[(i-(1))+((j-1)-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack/Dorghr",20);
}              //  Close for() loop. 
}
{
for (i = (ihi+1); i <= n; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorghr",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dorghr",40);
}              //  Close for() loop. 
}
{
for (j = 1; j <= ilo; j++) {
{
for (i = 1; i <= n; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorghr",50);
}              //  Close for() loop. 
}
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = 1.0;
//Dummy.label("com/polytechnik/lapack/Dorghr",60);
}              //  Close for() loop. 
}
{
for (j = (ihi+1); j <= n; j++) {
{
for (i = 1; i <= n; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dorghr",70);
}              //  Close for() loop. 
}
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = 1.0;
// *
//Dummy.label("com/polytechnik/lapack/Dorghr",80);
}              //  Close for() loop. 
}
/***
*
*        Generate Q(ilo+1:ihi,ilo+1:ihi)
*
***/
if ((nh > 0))  {
    com.polytechnik.lapack.Dorg2r.dorg2r(nh,nh,nh,a,((ilo+1)-(1))+((ilo+1)-(1)) * (lda)+ _a_offset,lda,tau,(ilo-(1))+ _tau_offset,work,_work_offset,info);
}
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DORGHR
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dorghr",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dorghr",999999);
return;
   }
} // End class.
