/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dpotrf.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dpotrf.java,v 1.2 2012/06/18 23:09:17 mal Exp $
 */

package com.polytechnik.lapack;


public class Dpotrf {

// *> \brief \b DPOTRF
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DPOTRF + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DPOTRF( UPLO, N, A, LDA, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DPOTRF computes the Cholesky factorization of a real symmetric
// *> positive definite matrix A.
// *>
// *> The factorization has the form
// *>    A = U**T * U,  if UPLO = 'U', or
// *>    A = L  * L**T,  if UPLO = 'L',
// *> where U is an upper triangular matrix and L is lower triangular.
// *>
// *> This is the block version of the algorithm, calling Level 3 BLAS.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored;
// *>          = 'L':  Lower triangle of A is stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          N-by-N upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading N-by-N lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>
// *>          On exit, if INFO = 0, the factor U or L from the Cholesky
// *>          factorization A = U**T*U or A = L*L**T.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  if INFO = i, the leading minor of order i is not
// *>                positive definite, and the factorization could not be

// *>                completed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doublePOcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dpotrf (final char uplo,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {
    com.polytechnik.lapack.Dpotf2.dpotf2(uplo,n,a,_a_offset,lda,info);
}

} // End class.
