/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: drot.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Drot.java,v 1.2 2012/12/22 10:54:49 mal Exp $
 */

package com.polytechnik.lapack;



public class Drot {

// *> \brief \b DROT
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DROT(N,DX,INCX,DY,INCY,C,S)
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION C,S
// *       INTEGER INCX,INCY,N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION DX(*),DY(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DROT applies a plane rotation.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level1
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>     jack dongarra, linpack, 3/11/78.
// *>     modified 12/3/93, array(1) declarations changed to array(*)
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
// *     ..

public static void drot (int n,
double [] dx, int _dx_offset,
int incx,
double [] dy, int _dy_offset,
int incy,
double c,
double s)  {

double dtemp= 0.0d;
int i= 0;
int ix= 0;
int iy= 0;
if ((n <= 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Drot",999999);
}
    /***
*
*       code for both increments equal to 1
*
***/
if (((incx == 1) && (incy == 1)))  {
    {
for (i = 1; i <= n; i++) {
dtemp = ((c*dx[(i-(1))+ _dx_offset])+(s*dy[(i-(1))+ _dy_offset]));
dy[(i-(1))+ _dy_offset] = ((c*dy[(i-(1))+ _dy_offset])-(s*dx[(i-(1))+ _dx_offset]));
dx[(i-(1))+ _dx_offset] = dtemp;
//Dummy.label("com/polytechnik/lapack/Drot",1000000);
}              //  Close for() loop. 
}
/***
*
*       code for unequal increments or equal increments not equal
*         to 1
*
***/
}
else  {
  ix = 1;
iy = 1;
if ((incx < 0)) {
    ix = (((((-(n))+1))*incx)+1);
}
    if ((incy < 0)) {
    iy = (((((-(n))+1))*incy)+1);
}
    {
for (i = 1; i <= n; i++) {
dtemp = ((c*dx[(ix-(1))+ _dx_offset])+(s*dy[(iy-(1))+ _dy_offset]));
dy[(iy-(1))+ _dy_offset] = ((c*dy[(iy-(1))+ _dy_offset])-(s*dx[(ix-(1))+ _dx_offset]));
dx[(ix-(1))+ _dx_offset] = dtemp;
ix = (ix+incx);
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack/Drot",1000001);
}              //  Close for() loop. 
}
}              //  Close else.
//Dummy.go_to("com/polytechnik/lapack/Drot",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Drot",999999);
return;
   }
} // End class.
