/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dstein.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dstein.java,v 1.11 2015/06/13 08:17:40 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dstein {

// *> \brief \b DSTEIN
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSTEIN + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSTEIN( N, D, E, M, W, IBLOCK, ISPLIT, Z, LDZ, WORK,

// *                          IWORK, IFAIL, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, LDZ, M, N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IBLOCK( * ), IFAIL( * ), ISPLIT( * ),
// *      $                   IWORK( * )
// *       DOUBLE PRECISION   D( * ), E( * ), W( * ), WORK( * ), Z( LDZ, * 
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSTEIN computes the eigenvectors of a real symmetric tridiagonal
// *> matrix T corresponding to specified eigenvalues, using inverse
// *> iteration.
// *>
// *> The maximum number of iterations allowed for each eigenvector is
// *> specified by an internal parameter MAXITS (currently set to 5).
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The n diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          The (n-1) subdiagonal elements of the tridiagonal matrix
// *>          T, in elements 1 to N-1.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of eigenvectors to be found.  0 <= M <= N.
// *> \endverbatim
// *>
// *> \param[in] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          The first M elements of W contain the eigenvalues for
// *>          which eigenvectors are to be computed.  The eigenvalues
// *>          should be grouped by split-off block and ordered from
// *>          smallest to largest within the block.  ( The output array
// *>          W from DSTEBZ with ORDER = 'B' is expected here. )
// *> \endverbatim
// *>
// *> \param[in] IBLOCK
// *> \verbatim
// *>          IBLOCK is INTEGER array, dimension (N)
// *>          The submatrix indices associated with the corresponding
// *>          eigenvalues in W; IBLOCK(i)=1 if eigenvalue W(i) belongs to

// *>          the first submatrix from the top, =2 if W(i) belongs to
// *>          the second submatrix, etc.  ( The output array IBLOCK
// *>          from DSTEBZ is expected here. )
// *> \endverbatim
// *>
// *> \param[in] ISPLIT
// *> \verbatim
// *>          ISPLIT is INTEGER array, dimension (N)
// *>          The splitting points, at which T breaks up into submatrices.
// *>          The first submatrix consists of rows/columns 1 to
// *>          ISPLIT( 1 ), the second of rows/columns ISPLIT( 1 )+1
// *>          through ISPLIT( 2 ), etc.
// *>          ( The output array ISPLIT from DSTEBZ is expected here. )
// *> \endverbatim
// *>
// *> \param[out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, M)
// *>          The computed eigenvectors.  The eigenvector associated
// *>          with the eigenvalue W(i) is stored in the i-th column of
// *>          Z.  Any vector which fails to converge is set to its current
// *>          iterate after MAXITS iterations.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (5*N)
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] IFAIL
// *> \verbatim
// *>          IFAIL is INTEGER array, dimension (M)
// *>          On normal exit, all elements of IFAIL are zero.
// *>          If one or more eigenvectors fail to converge after
// *>          MAXITS iterations, then their indices are stored in
// *>          array IFAIL.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit.
// *>          < 0: if INFO = -i, the i-th argument had an illegal value
// *>          > 0: if INFO = i, then i eigenvectors failed to converge
// *>               in MAXITS iterations.  Their indices are stored in
// *>               array IFAIL.
// *> \endverbatim
// *
// *> \par Internal Parameters:
// *  =========================
// *>
// *> \verbatim
// *>  MAXITS  INTEGER, default = 5
// *>          The maximum number of iterations performed.
// *>
// *>  EXTRA   INTEGER, default = 2
// *>          The number of iterations performed after norm growth
// *>          criterion is satisfied, should be at least 1.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dstein (int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
int m,
double [] w, int _w_offset,
int [] iblock, int _iblock_offset,
int [] isplit, int _isplit_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
int [] ifail, int _ifail_offset,
intW info)  {

final double ODM3 = 1.0e-3, ODM1 = 1.0e-1;
int b1= 0;
int blksiz= 0;
int bn= 0;
int gpind= 0;
int i= 0;
intW iinfo= new intW(0);
int indrv1= 0;
int indrv2= 0;
int indrv3= 0;
int indrv4= 0;
int indrv5= 0;
int its= 0;
int j= 0;
int j1= 0;
int jblk= 0;
int jmax= 0;
int nblk= 0;
int nrmchk= 0;
double dtpcrt= 0.0d;
double eps= 0.0d;
double eps1= 0.0d;
double nrm= 0.0d;
double onenrm= 0.0d;
double ortol= 0.0d;
double pertol= 0.0d;
double scl= 0.0d;
double sep= 0.0d;
doubleW tol= new doubleW(0.0d);
double xj= 0.0d;
double xjm= 0.0d;
double ztr= 0.0d;
final int MAXITS = 5, EXTRA = 2;
java.util.Random iseed= new java.util.Random (11111);
info.val = 0;
{
for (i = 1; i <= m; i++) {
ifail[(i-(1))+ _ifail_offset] = 0;
// *
//Dummy.label("com/polytechnik/lapack/Dstein",10);
}              //  Close for() loop. 
}
if ((n < 0))  {
    info.val = -1;
}
else if (((m < 0) || (m > n)))  {
    info.val = -4;
}              // Close else if()
else if ((ldz < Math.max(1, n)))  {
    info.val = -9;
}              // Close else if()
else  {
  {
  label20:for (j = 2; j <= m; j++) {
if ((iblock[(j-(1))+ _iblock_offset] < iblock[((j-1)-(1))+ _iblock_offset]))  {
    info.val = -6;
    break label20;//Dummy.go_to("com/polytechnik/lapack/Dstein",30);
}
if (((iblock[(j-(1))+ _iblock_offset] == iblock[((j-1)-(1))+ _iblock_offset]) && (w[(j-(1))+ _w_offset] < w[((j-1)-(1))+ _w_offset])))  {
    info.val = -5;
    break label20;//Dummy.go_to("com/polytechnik/lapack/Dstein",30);
}
//Dummy.label("com/polytechnik/lapack/Dstein",20);
} // label20              //  Close for() loop. 
}
//label30: Dummy.label("com/polytechnik/lapack/Dstein",30);
// *
}              //  Close else.
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSTEIN",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dstein",999999);
/***
*
*     Quick return if possible
*
***/
}
if (((n == 0) || (m == 0)))  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dstein",999999);
}
else if ((n == 1))  {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = 1.0;
    return;//Dummy.go_to("com/polytechnik/lapack/Dstein",999999);
/***
*
*     Get machine constants.
*
***/
}              // Close else if()
/****
*     Initialize seed for random number generator DLARNV.
*
***/
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
/*{
for (i = 1; i <= 4; i++) {
iseed[(i-(1))] = 1;
//Dummy.label("com/polytechnik/lapack/Dstein",40);
}              //  Close for() loop. 
}*/

/***
*
*     Initialize pointers.
*
***/
indrv1 = 0;
indrv2 = (indrv1+n);
indrv3 = (indrv2+n);
indrv4 = (indrv3+n);
/***
*
*     Compute eigenvectors of matrix blocks.
*
***/
indrv5 = (indrv4+n);
j1 = 1;
/***
*
*        Find starting and ending indices of block nblk.
*
***/
{
label160:for (nblk = 1; nblk <= iblock[(m-(1))+ _iblock_offset]; nblk++) {
if ((nblk == 1))  {
    b1 = 1;
}
else  {
  b1 = (isplit[((nblk-1)-(1))+ _isplit_offset]+1);
}              //  Close else.
bn = isplit[(nblk-(1))+ _isplit_offset];
blksiz = ((bn-b1)+1);
if ((blksiz == 1)) {
    //Dummy.go_to("com/polytechnik/lapack/Dstein",60);
}
/* label60 */ else { // label60
    /***
*
*        Compute reorthogonalization criterion and stopping criterion.
*
***/
gpind = b1;
onenrm = (Math.abs(d[(b1-(1))+ _d_offset])+Math.abs(e[(b1-(1))+ _e_offset]));
onenrm = Math.max(onenrm, (Math.abs(d[(bn-(1))+ _d_offset])+Math.abs(e[((bn-1)-(1))+ _e_offset])));
{
for (i = (b1+1); i <= (bn-1); i++) {
onenrm = Math.max(onenrm, ((Math.abs(d[(i-(1))+ _d_offset])+Math.abs(e[((i-1)-(1))+ _e_offset]))+Math.abs(e[(i-(1))+ _e_offset])));
//Dummy.label("com/polytechnik/lapack/Dstein",50);
}              //  Close for() loop. 
}
// *
ortol = (ODM3*onenrm);
/***
*
*        Loop through eigenvalues of block nblk.
*
***/
dtpcrt = Math.sqrt((ODM1/blksiz));
}
//label60: Dummy.label("com/polytechnik/lapack/Dstein",60);
jblk = 0;
{
for (j = j1; j <= m; j++) {
if ((iblock[(j-(1))+ _iblock_offset] != nblk))  {
    j1 = j;
    continue label160;//Dummy.go_to("com/polytechnik/lapack/Dstein",160);
}
jblk = (jblk+1);
/***
*
*           Skip all the work if the block size is one.
*
***/
xj = w[(j-(1))+ _w_offset];
if ((blksiz == 1))  {
    work[((indrv1+1)-(1))+ _work_offset] = 1.0;
//Dummy.go_to("com/polytechnik/lapack/Dstein",120);
/***
*
*           If eigenvalues j and j-1 are too close, add a relatively
*           small perturbation.
*
***/
} /* Not label120 */ else {
if ((jblk > 1))  {
    eps1 = Math.abs((eps*xj));
pertol = (10.0*eps1);
sep = (xj-xjm);
if ((sep < pertol)) {
    xj = (xjm+pertol);
}
    // *
}
its = 0;
/***
*
*           Get random starting vector.
*
***/
nrmchk = 0;
/***
*
*           Copy the matrix T so it won't be destroyed in factorization.
*
***/
com.polytechnik.lapack.Dlarnv.dlarnv(2,iseed,0,blksiz,work,((indrv1+1)-(1))+ _work_offset);
com.polytechnik.lapack.Dcopy.dcopy(blksiz,d,(b1-(1))+ _d_offset,1,work,((indrv4+1)-(1))+ _work_offset,1);
com.polytechnik.lapack.Dcopy.dcopy((blksiz-1),e,(b1-(1))+ _e_offset,1,work,((indrv2+2)-(1))+ _work_offset,1);
/***
*
*           Compute LU factors with partial pivoting  ( PT = LU )
*
***/
com.polytechnik.lapack.Dcopy.dcopy((blksiz-1),e,(b1-(1))+ _e_offset,1,work,((indrv3+1)-(1))+ _work_offset,1);
tol.val = 0.0;
/***
*
*           Update iteration count.
*
***/
com.polytechnik.lapack.Dlagtf.dlagtf(blksiz,work,((indrv4+1)-(1))+ _work_offset,xj,work,((indrv2+2)-(1))+ _work_offset,work,((indrv3+1)-(1))+ _work_offset,tol.val,work,((indrv5+1)-(1))+ _work_offset,iwork,_iwork_offset,iinfo);
label70:for(;;){ // Dummy.label("com/polytechnik/lapack/Dstein",70);
its = (its+1);
/***
*
*           Normalize and scale the righthand side vector Pb.
*
***/
if ((its > MAXITS)) {
    //Dummy.go_to("com/polytechnik/lapack/Dstein",100);
    //System.err.println("dstein: its="+its);
    info.val = (info.val+1);
    ifail[(info.val-(1))+ _ifail_offset] = j;
    break label70;
}
    scl = (((blksiz*onenrm)*Math.max(eps, Math.abs(work[((indrv4+blksiz)-(1))+ _work_offset])))/com.polytechnik.lapack.Dasum.dasum(blksiz,work,((indrv1+1)-(1))+ _work_offset,1));
/***
*
*           Solve the system LU = Pb.
*
***/
com.polytechnik.lapack.Dscal.dscal(blksiz,scl,work,((indrv1+1)-(1))+ _work_offset,1);
/***
*
*           Reorthogonalize by modified Gram-Schmidt if eigenvalues are
*           close enough.
*
***/
com.polytechnik.lapack.Dlagts.dlagts(-1,blksiz,work,((indrv4+1)-(1))+ _work_offset,work,((indrv2+2)-(1))+ _work_offset,work,((indrv3+1)-(1))+ _work_offset,work,((indrv5+1)-(1))+ _work_offset,iwork,_iwork_offset,work,((indrv1+1)-(1))+ _work_offset,tol,iinfo);
if ((jblk == 1)) {
    //Dummy.go_to("com/polytechnik/lapack/Dstein",90);
}
else /* notlabel 90 */ {
    if ((Math.abs((xj-xjm)) > ortol)) {
    gpind = j;
}
    if ((gpind != j))  {
    {
for (i = gpind; i <= (j-1); i++) {
ztr = (-(com.polytechnik.lapack.Ddot.ddot(blksiz,work,((indrv1+1)-(1))+ _work_offset,1,z,(b1-(1))+(i-(1)) * (ldz)+ _z_offset,1)));
com.polytechnik.lapack.Daxpy.daxpy(blksiz,ztr,z,(b1-(1))+(i-(1)) * (ldz)+ _z_offset,1,work,((indrv1+1)-(1))+ _work_offset,1);
//Dummy.label("com/polytechnik/lapack/Dstein",80);
}              //  Close for() loop. 
}
/***
*
*           Check the infinity norm of the iterate.
*
***/
}

} //label90: Dummy.label("com/polytechnik/lapack/Dstein",90);
jmax = com.polytechnik.lapack.Idamax.idamax(blksiz,work,((indrv1+1)-(1))+ _work_offset,1);
/***
*
*           Continue for additional iterations after norm reaches
*           stopping criterion.
*
***/
nrm = Math.abs(work[((indrv1+jmax)-(1))+ _work_offset]);
if ((nrm < dtpcrt)) {
    continue label70; //Dummy.go_to("com/polytechnik/lapack/Dstein",70);
}
    nrmchk = (nrmchk+1);
// *
if ((nrmchk < (EXTRA+1))) {
    continue label70; //Dummy.go_to("com/polytechnik/lapack/Dstein",70);
}

break label70;}
    /***
*
*           If stopping criterion was not satisfied, update info and
*           store eigenvector number in array ifail.
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dstein",110);
//label100:
//   Dummy.label("com/polytechnik/lapack/Dstein",100);

/***
*
*           Accept iterate as jth eigenvector.
*
***/
//label110: Dummy.label("com/polytechnik/lapack/Dstein",110);
scl = (1.0/com.polytechnik.lapack.Dnrm2.dnrm2(blksiz,work,((indrv1+1)-(1))+ _work_offset,1));
jmax = com.polytechnik.lapack.Idamax.idamax(blksiz,work,((indrv1+1)-(1))+ _work_offset,1);
if ((work[((indrv1+jmax)-(1))+ _work_offset] < 0.0)) {
    scl = (-(scl));
}
    com.polytechnik.lapack.Dscal.dscal(blksiz,scl,work,((indrv1+1)-(1))+ _work_offset,1);
}// end label120:// Dummy.label("com/polytechnik/lapack/Dstein",120);

{
for (i = 1; i <= n; i++) {
z[(i-(1))+(j-(1)) * (ldz)+ _z_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dstein",130);
}              //  Close for() loop. 
}
{
for (i = 1; i <= blksiz; i++) {
z[(((b1+i)-1)-(1))+(j-(1)) * (ldz)+ _z_offset] = work[((indrv1+i)-(1))+ _work_offset];
/***
*
*           Save the shift to check eigenvalue spacing at next
*           iteration.
*
***/
//Dummy.label("com/polytechnik/lapack/Dstein",140);
}              //  Close for() loop. 
}
// *
xjm = xj;
//Dummy.label("com/polytechnik/lapack/Dstein",150);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dstein",160);
}              //  Close for() loop. 
}
/***
*
*     End of DSTEIN
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dstein",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dstein",999999);
return;
   }
} // End class.
