/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dstemr.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dstemr.java,v 1.11 2012/12/25 19:10:53 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dstemr {

// *> \brief \b DSTEMR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSTEMR + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSTEMR( JOBZ, RANGE, N, D, E, VL, VU, IL, IU,
// *                          M, W, Z, LDZ, NZC, ISUPPZ, TRYRAC, WORK, LWOR
// *                          IWORK, LIWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBZ, RANGE
// *       LOGICAL            TRYRAC
// *       INTEGER            IL, INFO, IU, LDZ, NZC, LIWORK, LWORK, M, N
// *       DOUBLE PRECISION VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISUPPZ( * ), IWORK( * )
// *       DOUBLE PRECISION   D( * ), E( * ), W( * ), WORK( * )
// *       DOUBLE PRECISION   Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSTEMR computes selected eigenvalues and, optionally, eigenvectors
// *> of a real symmetric tridiagonal matrix T. Any such unreduced matrix h
// *> a well defined set of pairwise different real eigenvalues, the corres
// *> real eigenvectors are pairwise orthogonal.
// *>
// *> The spectrum may be computed either completely or partially by specif
// *> either an interval (VL,VU] or a range of indices IL:IU for the desire
// *> eigenvalues.
// *>
// *> Depending on the number of desired eigenvalues, these are computed ei
// *> by bisection or the dqds algorithm. Numerically orthogonal eigenvecto
// *> computed by the use of various suitable L D L^T factorizations near c
// *> of close eigenvalues (referred to as RRRs, Relatively Robust
// *> Representations). An informal sketch of the algorithm follows.
// *>
// *> For each unreduced block (submatrix) of T,
// *>    (a) Compute T - sigma I  = L D L^T, so that L and D
// *>        define all the wanted eigenvalues to high relative accuracy.
// *>        This means that small relative changes in the entries of D and
// *>        cause only small relative changes in the eigenvalues and
// *>        eigenvectors. The standard (unfactored) representation of the

// *>        tridiagonal matrix T does not have this property in general.
// *>    (b) Compute the eigenvalues to suitable accuracy.
// *>        If the eigenvectors are desired, the algorithm attains full
// *>        accuracy of the computed eigenvalues only right before
// *>        the corresponding vectors have to be computed, see steps c) an
// *>    (c) For each cluster of close eigenvalues, select a new
// *>        shift close to the cluster, find a new factorization, and refi
// *>        the shifted eigenvalues to suitable accuracy.
// *>    (d) For each eigenvalue with a large enough relative separation co
// *>        the corresponding eigenvector by forming a rank revealing twis
// *>        factorization. Go back to (c) for any clusters that remain.
// *>
// *> For more details, see:
// *> - Inderjit S. Dhillon and Beresford N. Parlett: "Multiple representat
// *>   to compute orthogonal eigenvectors of symmetric tridiagonal matrice
// *>   Linear Algebra and its Applications, 387(1), pp. 1-28, August 2004.
// *> - Inderjit Dhillon and Beresford Parlett: "Orthogonal Eigenvectors an
// *>   Relative Gaps," SIAM Journal on Matrix Analysis and Applications, V
// *>   2004.  Also LAPACK Working Note 154.
// *> - Inderjit Dhillon: "A new O(n^2) algorithm for the symmetric
// *>   tridiagonal eigenvalue/eigenvector problem",
// *>   Computer Science Division Technical Report No. UCB/CSD-97-971,
// *>   UC Berkeley, May 1997.
// *>
// *> Further Details
// *> 1.DSTEMR works only on machines which follow IEEE-754
// *> floating-point standard in their handling of infinities and NaNs.
// *> This permits the use of efficient inner loops avoiding a check for
// *> zero divisors.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBZ
// *> \verbatim
// *>          JOBZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only;
// *>          = 'V':  Compute eigenvalues and eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': all eigenvalues will be found.
// *>          = 'V': all eigenvalues in the half-open interval (VL,VU]
// *>                 will be found.
// *>          = 'I': the IL-th through IU-th eigenvalues will be found.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the N diagonal elements of the tridiagonal matrix

// *>          T. On exit, D is overwritten.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the (N-1) subdiagonal elements of the tridiagonal

// *>          matrix T in elements 1 to N-1 of E. E(N) need not be set on

// *>          input, but is used internally as workspace.
// *>          On exit, E is overwritten.
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>
// *>          If RANGE='V', the lower and upper bounds of the interval to

// *>          be searched for eigenvalues. VL < VU.
// *>          Not referenced if RANGE = 'A' or 'I'.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N, if N > 0.
// *>          Not referenced if RANGE = 'A' or 'V'.
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The total number of eigenvalues found.  0 <= M <= N.
// *>          If RANGE = 'A', M = N, and if RANGE = 'I', M = IU-IL+1.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          The first M elements contain the selected eigenvalues in
// *>          ascending order.
// *> \endverbatim
// *>
// *> \param[out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, max(1,M) )
// *>          If JOBZ = 'V', and if INFO = 0, then the first M columns of 
// *>          contain the orthonormal eigenvectors of the matrix T
// *>          corresponding to the selected eigenvalues, with the i-th
// *>          column of Z holding the eigenvector associated with W(i).
// *>          If JOBZ = 'N', then Z is not referenced.
// *>          Note: the user must ensure that at least max(1,M) columns ar
// *>          supplied in the array Z; if RANGE = 'V', the exact value of 
// *>          is not known in advance and can be computed with a workspace
// *>          query by setting NZC = -1, see below.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1, and if
// *>          JOBZ = 'V', then LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] NZC
// *> \verbatim
// *>          NZC is INTEGER
// *>          The number of eigenvectors to be held in the array Z.
// *>          If RANGE = 'A', then NZC >= max(1,N).
// *>          If RANGE = 'V', then NZC >= the number of eigenvalues in (VL
// *>          If RANGE = 'I', then NZC >= IU-IL+1.
// *>          If NZC = -1, then a workspace query is assumed; the
// *>          routine calculates the number of columns of the array Z that
// *>          are needed to hold the eigenvectors.
// *>          This value is returned as the first entry of the Z array, an
// *>          no error message related to NZC is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] ISUPPZ
// *> \verbatim
// *>          ISUPPZ is INTEGER ARRAY, dimension ( 2*max(1,M) )
// *>          The support of the eigenvectors in Z, i.e., the indices
// *>          indicating the nonzero elements in Z. The i-th computed eige
// *>          is nonzero only in elements ISUPPZ( 2*i-1 ) through
// *>          ISUPPZ( 2*i ). This is relevant in the case when the matrix

// *>          is split. ISUPPZ is only accessed when JOBZ is 'V' and N > 0
// *> \endverbatim
// *>
// *> \param[in,out] TRYRAC
// *> \verbatim
// *>          TRYRAC is LOGICAL
// *>          If TRYRAC.EQ..TRUE., indicates that the code should check wh
// *>          the tridiagonal matrix defines its eigenvalues to high relat
// *>          accuracy.  If so, the code uses relative-accuracy preserving
// *>          algorithms that might be (a bit) slower depending on the mat
// *>          If the matrix does not define its eigenvalues to high relati
// *>          accuracy, the code can uses possibly faster algorithms.
// *>          If TRYRAC.EQ..FALSE., the code is not required to guarantee

// *>          relatively accurate eigenvalues and can use the fastest poss
// *>          techniques.
// *>          On exit, a .TRUE. TRYRAC will be set to .FALSE. if the matri
// *>          does not define its eigenvalues to high relative accuracy.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (LWORK)
// *>          On exit, if INFO = 0, WORK(1) returns the optimal
// *>          (and minimal) LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK. LWORK >= max(1,18*N)
// *>          if JOBZ = 'V', and LWORK >= max(1,12*N) if JOBZ = 'N'.
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (LIWORK)
// *>          On exit, if INFO = 0, IWORK(1) returns the optimal LIWORK.
// *> \endverbatim
// *>
// *> \param[in] LIWORK
// *> \verbatim
// *>          LIWORK is INTEGER
// *>          The dimension of the array IWORK.  LIWORK >= max(1,10*N)
// *>          if the eigenvectors are desired, and LIWORK >= max(1,8*N)
// *>          if only the eigenvalues are to be computed.
// *>          If LIWORK = -1, then a workspace query is assumed; the
// *>          routine only calculates the optimal size of the IWORK array,
// *>          returns this value as the first entry of the IWORK array, an
// *>          no error message related to LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          On exit, INFO
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  if INFO = 1X, internal error in DLARRE,
// *>                if INFO = 2X, internal error in DLARRV.
// *>                Here, the digit X = ABS( IINFO ) < 10, where IINFO is

// *>                the nonzero error code returned by DLARRE or
// *>                DLARRV, respectively.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERcomputational
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***


*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dstemr (final char jobz,
final char range,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double vl,
double vu,
int il,
int iu,
intW m,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
int nzc,
int [] isuppz, int _isuppz_offset,
booleanW tryrac,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

boolean alleig= false;
boolean indeig= false;
boolean lquery= false;
boolean valeig= false;
boolean wantz= false;
boolean zquery= false;
int i= 0;
int ibegin= 0;
int iend= 0;
int ifirst= 0;
int iil= 0;
int iindbl= 0;
int iindw= 0;
int iindwk= 0;
intW iinfo= new intW(0);
int iinspl= 0;
int iiu= 0;
int ilast= 0;
int in= 0;
int indd= 0;
int inde2= 0;
int inderr= 0;
int indgp= 0;
int indgrs= 0;
int indwrk= 0;
intW itmp= new intW(0);
intW itmp2= new intW(0);
int j= 0;
int jblk= 0;
int jj= 0;
int liwmin= 0;
int lwmin= 0;
intW nsplit= new intW(0);
intW nzcmin= new intW(0);
int offset= 0;
int wbegin= 0;
int wend= 0;
double bignum= 0.0d;
doubleW cs= new doubleW(0.0d);
double eps= 0.0d;
doubleW pivmin= new doubleW(0.0d);
doubleW r1= new doubleW(0.0d);
doubleW r2= new doubleW(0.0d);
double rmax= 0.0d;
double rmin= 0.0d;
doubleW rtol1= new doubleW(0.0d);
doubleW rtol2= new doubleW(0.0d);
double safmin= 0.0d;
double scale= 0.0d;
double smlnum= 0.0d;
doubleW sn= new doubleW(0.0d);
double thresh= 0.0d;
double tmp= 0.0d;
double tnrm= 0.0d;
doubleW wl= new doubleW(0.0d);
doubleW wu= new doubleW(0.0d);
final double MINRGP = 1.0e-3;
//jobz = jobz.substring((1)-1,1);
//range = range.substring((1)-1,1);
wantz = com.polytechnik.lapack.Lsame.lsame(jobz,'V');
alleig = com.polytechnik.lapack.Lsame.lsame(range,'A');
valeig = com.polytechnik.lapack.Lsame.lsame(range,'V');
// *
indeig = com.polytechnik.lapack.Lsame.lsame(range,'I');
lquery = ((((lwork == -1)) || ((liwork == -1))));
/***

*     DSTEMR needs WORK of size 6*N, IWORK of size 3*N.
*     In addition, DLARRE needs WORK of size 6*N, IWORK of size 5*N.
*     Furthermore, DLARRV needs WORK of size 12*N, IWORK of size 7*N.
***/
zquery = ((nzc == -1));
if (wantz)  {
    lwmin = (18*n);
liwmin = (10*n);
// *        need less workspace if only the eigenvalues are wanted
}
else  {
  lwmin = (12*n);
liwmin = (8*n);
// 
}              //  Close else.
wl.val = 0.0;
wu.val = 0.0;
iil = 0;
// 
iiu = 0;
/***
*        We do not reference VL, VU in the cases RANGE = 'I','A'
*        The interval (WL, WU] contains all the wanted eigenvalues.
*        It is either given by the user or computed in DLARRE.
***/
if (valeig)  {
    wl.val = vl;
wu.val = vu;
// *        We do not reference IL, IU in the cases RANGE = 'V','A'
}
else if (indeig)  {
    iil = il;
iiu = iu;
// *
}              // Close else if()
info.val = 0;
if ((!((wantz || com.polytechnik.lapack.Lsame.lsame(jobz,'N')))))  {
    info.val = -1;
}
else if ((!(((alleig || valeig) || indeig))))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if (((valeig && (n > 0)) && (wu.val <= wl.val)))  {
    info.val = -7;
}              // Close else if()
else if ((indeig && (((iil < 1) || (iil > n)))))  {
    info.val = -8;
}              // Close else if()
else if ((indeig && (((iiu < iil) || (iiu > n)))))  {
    info.val = -9;
}              // Close else if()
else if (((ldz < 1) || ((wantz && (ldz < n)))))  {
    info.val = -13;
}              // Close else if()
else if (((lwork < lwmin) && (!lquery)))  {
    info.val = -17;
}              // Close else if()
else if (((liwork < liwmin) && (!lquery)))  {
    info.val = -19;
/***
*
*     Get machine constants.
*
***/
}              // Close else if()
safmin = com.polytechnik.lapack.Dlamch.dlamch('S');
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
smlnum = (safmin/eps);
bignum = (1.0/smlnum);
rmin = Math.sqrt(smlnum);
// *
rmax = Math.min(Math.sqrt(bignum), (1.0/Math.sqrt(Math.sqrt(safmin))));
if ((info.val == 0))  {
    work[(1-(1))+ _work_offset] = (double)(lwmin);
// *
iwork[(1-(1))+ _iwork_offset] = liwmin;
if ((wantz && alleig))  {
    nzcmin.val = n;
}
else if ((wantz && valeig))  {
    com.polytechnik.lapack.Dlarrc.dlarrc('T',n,vl,vu,d,_d_offset,e,_e_offset,safmin,nzcmin,itmp,itmp2,info);
}              // Close else if()
else if ((wantz && indeig))  {
    nzcmin.val = ((iiu-iil)+1);
// *           WANTZ .EQ. FALSE.
}              // Close else if()
else  {
  nzcmin.val = 0;
}              //  Close else.
if ((zquery && (info.val == 0)))  {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = (double)(nzcmin.val);
}
else if (((nzc < nzcmin.val) && (!zquery)))  {
    info.val = -14;
}              // Close else if()
// 
}
// *
if ((info.val != 0))  {
    // *
com.polytechnik.lapack.Xerbla.xerbla("DSTEMR",(-(info.val)));
return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
}
else if ((lquery || zquery))  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
/***
*
*     Handle N = 0, 1, and 2 cases immediately
*
***/
}              // Close else if()
m.val = 0;
// *
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
}
    if ((n == 1))  {
    if ((alleig || indeig))  {
    m.val = 1;
w[(1-(1))+ _w_offset] = d[(1-(1))+ _d_offset];
}
else  {
  if (((wl.val < d[(1-(1))+ _d_offset]) && (wu.val >= d[(1-(1))+ _d_offset])))  {
    m.val = 1;
w[(1-(1))+ _w_offset] = d[(1-(1))+ _d_offset];
}
}              //  Close else.
if ((wantz && ((!zquery))))  {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = 1.0;
isuppz[(1-(1))+ _isuppz_offset] = 1;
isuppz[(2-(1))+ _isuppz_offset] = 1;
}
return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
// *
}
if ((n == 2))  {
    if ((!wantz))  {
    com.polytechnik.lapack.Dlae2.dlae2(d[(1-(1))+ _d_offset],e[(1-(1))+ _e_offset],d[(2-(1))+ _d_offset],r1,r2);
}
else if ((wantz && ((!zquery))))  {
    com.polytechnik.lapack.Dlaev2.dlaev2(d[(1-(1))+ _d_offset],e[(1-(1))+ _e_offset],d[(2-(1))+ _d_offset],r1,r2,cs,sn);
}              // Close else if()
if (((alleig || (((valeig && ((r2.val > wl.val))) && ((r2.val <= wu.val))))) || ((indeig && ((iil == 1))))))  {
    m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = r2.val;
if ((wantz && ((!zquery))))  {
    z[(1-(1))+(m.val-(1)) * (ldz)+ _z_offset] = (-(sn.val));
// *              Note: At most one of SN and CS can be zero.
z[(2-(1))+(m.val-(1)) * (ldz)+ _z_offset] = cs.val;
if ((sn.val != 0.0))  {
    if ((cs.val != 0.0))  {
    isuppz[(((2*m.val)-1)-(1))+ _isuppz_offset] = 1;
isuppz[((2*m.val)-(1))+ _isuppz_offset] = 2;
}
else  {
  isuppz[(((2*m.val)-1)-(1))+ _isuppz_offset] = 1;
isuppz[((2*m.val)-(1))+ _isuppz_offset] = 1;
}              //  Close else.
}
else  {
  isuppz[(((2*m.val)-1)-(1))+ _isuppz_offset] = 2;
isuppz[((2*m.val)-(1))+ _isuppz_offset] = 2;
}              //  Close else.
}
}
if (((alleig || (((valeig && ((r1.val > wl.val))) && ((r1.val <= wu.val))))) || ((indeig && ((iiu == 2))))))  {
    m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = r1.val;
if ((wantz && ((!zquery))))  {
    z[(1-(1))+(m.val-(1)) * (ldz)+ _z_offset] = cs.val;
// *              Note: At most one of SN and CS can be zero.
z[(2-(1))+(m.val-(1)) * (ldz)+ _z_offset] = sn.val;
if ((sn.val != 0.0))  {
    if ((cs.val != 0.0))  {
    isuppz[(((2*m.val)-1)-(1))+ _isuppz_offset] = 1;
isuppz[((2*m.val)-(1))+ _isuppz_offset] = 2;
}
else  {
  isuppz[(((2*m.val)-1)-(1))+ _isuppz_offset] = 1;
isuppz[((2*m.val)-(1))+ _isuppz_offset] = 1;
}              //  Close else.
}
else  {
  isuppz[(((2*m.val)-1)-(1))+ _isuppz_offset] = 2;
isuppz[((2*m.val)-(1))+ _isuppz_offset] = 2;
}              //  Close else.
}
// 
}
/***

*     Continue with general N

***/
}
else  {
  indgrs = 1;
inderr = ((2*n)+1);
indgp = ((3*n)+1);
indd = ((4*n)+1);
inde2 = ((5*n)+1);
// *
indwrk = ((6*n)+1);
iinspl = 1;
iindbl = (n+1);
iindw = ((2*n)+1);
/***
*
*        Scale matrix to allowable range, if necessary.
*        The allowable range is related to the PIVMIN parameter; see the
*        comments in DLARRD.  The preference for scaling small values
*        up is heuristic; we expect users' matrices not to be close to the
*        RMAX threshold.
*
***/
iindwk = ((3*n)+1);
scale = 1.0;
tnrm = com.polytechnik.lapack.Dlanst.dlanst('M',n,d,_d_offset,e,_e_offset);
if (((tnrm > 0.0) && (tnrm < rmin)))  {
    scale = (rmin/tnrm);
}
else if ((tnrm > rmax))  {
    scale = (rmax/tnrm);
}              // Close else if()
if ((scale != 1.0))  {
    com.polytechnik.lapack.Dscal.dscal(n,scale,d,_d_offset,1);
com.polytechnik.lapack.Dscal.dscal((n-1),scale,e,_e_offset,1);
tnrm = (tnrm*scale);
/***
*              If eigenvalues in interval have to be found,
*              scale (WL, WU] accordingly
***/
if (valeig)  {
    wl.val = (wl.val*scale);
wu.val = (wu.val*scale);
}
/***
*
*        Compute the desired eigenvalues of the tridiagonal after splitting
*        into smaller subblocks if the corresponding off-diagonal elements
*        are small
*        THRESH is the splitting parameter for DLARRE
*        A negative THRESH forces the old splitting criterion based on the
*        size of the off-diagonal. A positive THRESH switches to splitting
*        which preserves relative accuracy.
*
***/
}
// *           Test whether the matrix warrants the more expensive relative approach.
if (tryrac.val)  {
    com.polytechnik.lapack.Dlarrr.dlarrr(n,d,_d_offset,e,_e_offset,iinfo);
// *           The user does not care about relative accurately eigenvalues
}
else  {
  iinfo.val = -1;
// *        Set the splitting criterion
}              //  Close else.
if ((iinfo.val == 0))  {
    thresh = eps;
}
else  {
  // *           relative accuracy is desired but T does not guarantee it
thresh = (-(eps));
tryrac.val = false;
// *
}              //  Close else.
// *           Copy original diagonal, needed to guarantee relative accuracy
if (tryrac.val)  {
    com.polytechnik.lapack.Dcopy.dcopy(n,d,_d_offset,1,work,(indd-(1))+ _work_offset,1);
// *        Store the squares of the offdiagonal values of T
}
{
for (j = 1; j <= (n-1); j++) {
work[(((inde2+j)-1)-(1))+ _work_offset] = ( Math.pow(e[(j-(1))+ _e_offset], 2));
/***

*        Set the tolerance parameters for bisection
***/
//Dummy.label("com/polytechnik/lapack/Dstemr",5);
}              //  Close for() loop. 
}
// *           DLARRE computes the eigenvalues to full precision.
if ((!wantz))  {
    rtol1.val = (4.0*eps);
rtol2.val = (4.0*eps);
/***
*           DLARRE computes the eigenvalues to less than full precision.
*           DLARRV will refine the eigenvalue approximations, and we can
*           need less accurate initial bisection in DLARRE.
*           Note: these settings do only affect the subset case and DLARRE
***/
}
else  {
  rtol1.val = Math.sqrt(eps);
rtol2.val = Math.max((Math.sqrt(eps)*5.0e-3), (4.0*eps));
}              //  Close else.
com.polytechnik.lapack.Dlarre.dlarre(range,n,wl,wu,iil,iiu,d,_d_offset,e,_e_offset,work,(inde2-(1))+ _work_offset,rtol1.val,rtol2.val,thresh,nsplit,iwork,(iinspl-(1))+ _iwork_offset,m,w,_w_offset,work,(inderr-(1))+ _work_offset,work,(indgp-(1))+ _work_offset,iwork,(iindbl-(1))+ _iwork_offset,iwork,(iindw-(1))+ _iwork_offset,work,(indgrs-(1))+ _work_offset,pivmin,work,(indwrk-(1))+ _work_offset,iwork,(iindwk-(1))+ _iwork_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = (10+Math.abs(iinfo.val)+Math.abs(info.val));
    return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
/***
*        Note that if RANGE .NE. 'V', DLARRE computes bounds on the desired
*        part of the spectrum. All desired eigenvalues are contained in
*        (WL,WU]


***/
}
/***
*
*           Compute the desired eigenvectors corresponding to the computed
*           eigenvalues
*
***/
if (wantz)  {
    com.polytechnik.lapack.Dlarrv.dlarrv(n,wl.val,wu.val,d,_d_offset,e,_e_offset,pivmin.val,iwork,(iinspl-(1))+ _iwork_offset,m.val,1,m.val,MINRGP,rtol1,rtol2,w,_w_offset,work,(inderr-(1))+ _work_offset,work,(indgp-(1))+ _work_offset,iwork,(iindbl-(1))+ _iwork_offset,iwork,(iindw-(1))+ _iwork_offset,work,(indgrs-(1))+ _work_offset,z,_z_offset,ldz,isuppz,_isuppz_offset,work,(indwrk-(1))+ _work_offset,iwork,(iindwk-(1))+ _iwork_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = (20+Math.abs(iinfo.val)+Math.abs(info.val));
    return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
}
/***
*           DLARRE computes eigenvalues of the (shifted) root representation
*           DLARRV returns the eigenvalues of the unshifted matrix.
*           However, if the eigenvectors are not desired by the user, we need
*           to apply the corresponding shifts from DLARRE to obtain the
*           eigenvalues of the original matrix.
***/
}
else  {
  {
for (j = 1; j <= m.val; j++) {
itmp.val = iwork[(((iindbl+j)-1)-(1))+ _iwork_offset];
w[(j-(1))+ _w_offset] = (w[(j-(1))+ _w_offset]+e[(iwork[(((iinspl+itmp.val)-1)-(1))+ _iwork_offset]-(1))+ _e_offset]);
//Dummy.label("com/polytechnik/lapack/Dstemr",20);
}              //  Close for() loop. 
}
/***
*

***/
}              //  Close else.
/***
*           Refine computed eigenvalues so that they are relatively accurate
*           with respect to the original matrix T.
***/
if (tryrac.val)  {
    ibegin = 1;
wbegin = 1;
{
label39:for (jblk = 1; jblk <= iwork[(((iindbl+m.val)-1)-(1))+ _iwork_offset]; jblk++) {
iend = iwork[(((iinspl+jblk)-1)-(1))+ _iwork_offset];
in = ((iend-ibegin)+1);
// *              check if any eigenvalues have to be refined in this block
wend = (wbegin-1);
label36:for(;;){
    //Dummy.label("com/polytechnik/lapack/Dstemr",36);
if ((wend < m.val))  {
    if ((iwork[((iindbl+wend)-(1))+ _iwork_offset] == jblk))  {
    wend = (wend+1);
    continue label36;//Dummy.go_to("com/polytechnik/lapack/Dstemr",36);
}
}
break label36;}
if ((wend < wbegin))  {
    ibegin = (iend+1);
    continue label39;//Dummy.go_to("com/polytechnik/lapack/Dstemr",39);
// 
}
offset = (iwork[(((iindw+wbegin)-1)-(1))+ _iwork_offset]-1);
ifirst = iwork[(((iindw+wbegin)-1)-(1))+ _iwork_offset];
ilast = iwork[(((iindw+wend)-1)-(1))+ _iwork_offset];
rtol2.val = (4.0*eps);
com.polytechnik.lapack.Dlarrj.dlarrj(in,work,(((indd+ibegin)-1)-(1))+ _work_offset,work,(((inde2+ibegin)-1)-(1))+ _work_offset,ifirst,ilast,rtol2.val,offset,w,(wbegin-(1))+ _w_offset,work,(((inderr+wbegin)-1)-(1))+ _work_offset,work,(indwrk-(1))+ _work_offset,iwork,(iindwk-(1))+ _iwork_offset,pivmin.val,tnrm,iinfo);
ibegin = (iend+1);
wbegin = (wend+1);
//Dummy.label("com/polytechnik/lapack/Dstemr",39);
}              //  Close for() loop. 
}
/***
*
*        If matrix was scaled, then rescale eigenvalues appropriately.
*
***/
}
if ((scale != 1.0))  {
    com.polytechnik.lapack.Dscal.dscal(m.val,(1.0/scale),w,_w_offset,1);
}
//   
}              //  Close else.
//     
// *
// *     If eigenvalues are not in increasing order, then sort them,
// *     possibly along with eigenvectors.
// *
if (((nsplit.val > 1) || (n == 2)))  {
    if (false)  {
/*    com.polytechnik.lapack.Dlasrt.dlasrt('I',m.val,w,_w_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = 3;
    return;//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
}
*/
}
else  {
  {
for (j = 1; j <= (m.val-1); j++) {
i = 0;
tmp = w[(j-(1))+ _w_offset];
{
for (jj = (j+1); jj <= m.val; jj++) {
if ((w[(jj-(1))+ _w_offset] < tmp))  {
    i = jj;
tmp = w[(jj-(1))+ _w_offset];
}
//Dummy.label("com/polytechnik/lapack/Dstemr",50);
}              //  Close for() loop. 
}
if ((i != 0))  {
    w[(i-(1))+ _w_offset] = w[(j-(1))+ _w_offset];
w[(j-(1))+ _w_offset] = tmp;
if (wantz)  {
    com.polytechnik.lapack.Dswap.dswap(n,z,(1-(1))+(i-(1)) * (ldz)+ _z_offset,1,z,(1-(1))+(j-(1)) * (ldz)+ _z_offset,1);
itmp.val = isuppz[(((2*i)-1)-(1))+ _isuppz_offset];
isuppz[(((2*i)-1)-(1))+ _isuppz_offset] = isuppz[(((2*j)-1)-(1))+ _isuppz_offset];
isuppz[(((2*j)-1)-(1))+ _isuppz_offset] = itmp.val;
itmp.val = isuppz[((2*i)-(1))+ _isuppz_offset];
isuppz[((2*i)-(1))+ _isuppz_offset] = isuppz[((2*j)-(1))+ _isuppz_offset];
isuppz[((2*j)-(1))+ _isuppz_offset] = itmp.val;
}
}
//Dummy.label("com/polytechnik/lapack/Dstemr",60);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*
***/
}
work[(1-(1))+ _work_offset] = (double)(lwmin);
iwork[(1-(1))+ _iwork_offset] = liwmin;
/***
*
*     End of DSTEMR
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dstemr",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dstemr",999999);
return;
   }
} // End class.
