/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsygs2.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsygs2.java,v 1.5 2012/12/20 18:09:13 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dsygs2 {

// *> \brief \b DSYGS2 reduces a symmetric definite generalized eigenproble
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSYGS2 + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYGS2( ITYPE, UPLO, N, A, LDA, B, LDB, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, ITYPE, LDA, LDB, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYGS2 reduces a real symmetric-definite generalized eigenproblem
// *> to standard form.
// *>
// *> If ITYPE = 1, the problem is A*x = lambda*B*x,
// *> and A is overwritten by inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T)
// *>
// *> If ITYPE = 2 or 3, the problem is A*B*x = lambda*x or
// *> B*A*x = lambda*x, and A is overwritten by U*A*U**T or L**T *A*L.
// *>
// *> B must have been previously factorized as U**T *U or L*L**T by DPOTRF
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] ITYPE
// *> \verbatim
// *>          ITYPE is INTEGER
// *>          = 1: compute inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T);
// *>          = 2 or 3: compute U*A*U**T or L**T *A*L.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies whether the upper or lower triangular part of the

// *>          symmetric matrix A is stored, and how B has been factorized.
// *>          = 'U':  Upper triangular
// *>          = 'L':  Lower triangular
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrices A and B.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          n by n upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading n by n lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>
// *>          On exit, if INFO = 0, the transformed matrix, stored in the

// *>          same format as A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (LDB,N)
// *>          The triangular factor from the Cholesky factorization of B,

// *>          as returned by DPOTRF.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>          The leading dimension of the array B.  LDB >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit.
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleSYcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dsygs2 (int itype,
final char uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
intW info)  {

boolean upper= false;
int k= 0;
double akk= 0.0d;
double bkk= 0.0d;
double ct= 0.0d;
//uplo = uplo.substring((1)-1,1);
info.val = 0;
upper = com.polytechnik.lapack.Lsame.lsame(uplo,'U');
if (((itype < 1) || (itype > 3)))  {
    info.val = -1;
}
else if (((!upper) && (!com.polytechnik.lapack.Lsame.lsame(uplo,'L'))))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -5;
}              // Close else if()
else if ((ldb < Math.max(1, n)))  {
    info.val = -7;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYGS2",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dsygs2",999999);
// *
}
if ((itype == 1))  {
    /***
*
*           Compute inv(U**T)*A*inv(U)
*
***/
if (upper)  {
    /***
*
*              Update the upper triangle of A(k:n,k:n)
*
***/
{
for (k = 1; k <= n; k++) {
akk = a[(k-(1))+(k-(1)) * (lda)+ _a_offset];
bkk = b[(k-(1))+(k-(1)) * (ldb)+ _b_offset];
akk = (akk/( Math.pow(bkk, 2)));
a[(k-(1))+(k-(1)) * (lda)+ _a_offset] = akk;
if ((k < n))  {
    com.polytechnik.lapack.Dscal.dscal((n-k),(1.0/bkk),a,(k-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda);
ct = (-((0.5*akk)));
com.polytechnik.lapack.Daxpy.daxpy((n-k),ct,b,(k-(1))+((k+1)-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Dsyr2.dsyr2(uplo,(n-k),-1.0,a,(k-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+((k+1)-(1)) * (ldb)+ _b_offset,ldb,a,((k+1)-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Daxpy.daxpy((n-k),ct,b,(k-(1))+((k+1)-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Dtrsv.dtrsv(uplo,'T','N',(n-k),b,((k+1)-(1))+((k+1)-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda);
}
//Dummy.label("com/polytechnik/lapack/Dsygs2",10);
}              //  Close for() loop. 
}
/***
*
*           Compute inv(L)*A*inv(L**T)
*
***/
}
else  {
  /***
*
*              Update the lower triangle of A(k:n,k:n)
*
***/
{
for (k = 1; k <= n; k++) {
akk = a[(k-(1))+(k-(1)) * (lda)+ _a_offset];
bkk = b[(k-(1))+(k-(1)) * (ldb)+ _b_offset];
akk = (akk/( Math.pow(bkk, 2)));
a[(k-(1))+(k-(1)) * (lda)+ _a_offset] = akk;
if ((k < n))  {
    com.polytechnik.lapack.Dscal.dscal((n-k),(1.0/bkk),a,((k+1)-(1))+(k-(1)) * (lda)+ _a_offset,1);
ct = (-((0.5*akk)));
com.polytechnik.lapack.Daxpy.daxpy((n-k),ct,b,((k+1)-(1))+(k-(1)) * (ldb)+ _b_offset,1,a,((k+1)-(1))+(k-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dsyr2.dsyr2(uplo,(n-k),-1.0,a,((k+1)-(1))+(k-(1)) * (lda)+ _a_offset,1,b,((k+1)-(1))+(k-(1)) * (ldb)+ _b_offset,1,a,((k+1)-(1))+((k+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Daxpy.daxpy((n-k),ct,b,((k+1)-(1))+(k-(1)) * (ldb)+ _b_offset,1,a,((k+1)-(1))+(k-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dtrsv.dtrsv(uplo,'N','N',(n-k),b,((k+1)-(1))+((k+1)-(1)) * (ldb)+ _b_offset,ldb,a,((k+1)-(1))+(k-(1)) * (lda)+ _a_offset,1);
}
//Dummy.label("com/polytechnik/lapack/Dsygs2",20);
}              //  Close for() loop. 
}
}              //  Close else.
}
else  {
  /***
*
*           Compute U*A*U**T
*
***/
if (upper)  {
    /***
*
*              Update the upper triangle of A(1:k,1:k)
*
***/
{
for (k = 1; k <= n; k++) {
akk = a[(k-(1))+(k-(1)) * (lda)+ _a_offset];
bkk = b[(k-(1))+(k-(1)) * (ldb)+ _b_offset];
com.polytechnik.lapack.Dtrmv.dtrmv(uplo,'N','N',(k-1),b,_b_offset,ldb,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,1);
ct = (0.5*akk);
com.polytechnik.lapack.Daxpy.daxpy((k-1),ct,b,(1-(1))+(k-(1)) * (ldb)+ _b_offset,1,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dsyr2.dsyr2(uplo,(k-1),1.0,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,1,b,(1-(1))+(k-(1)) * (ldb)+ _b_offset,1,a,_a_offset,lda);
com.polytechnik.lapack.Daxpy.daxpy((k-1),ct,b,(1-(1))+(k-(1)) * (ldb)+ _b_offset,1,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dscal.dscal((k-1),bkk,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,1);
a[(k-(1))+(k-(1)) * (lda)+ _a_offset] = (akk*( Math.pow(bkk, 2)));
//Dummy.label("com/polytechnik/lapack/Dsygs2",30);
}              //  Close for() loop. 
}
/***
*
*           Compute L**T *A*L
*
***/
}
else  {
  /***
*
*              Update the lower triangle of A(1:k,1:k)
*
***/
{
for (k = 1; k <= n; k++) {
akk = a[(k-(1))+(k-(1)) * (lda)+ _a_offset];
bkk = b[(k-(1))+(k-(1)) * (ldb)+ _b_offset];
com.polytechnik.lapack.Dtrmv.dtrmv(uplo,'T','N',(k-1),b,_b_offset,ldb,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
ct = (0.5*akk);
com.polytechnik.lapack.Daxpy.daxpy((k-1),ct,b,(k-(1))+(1-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Dsyr2.dsyr2(uplo,(k-1),1.0,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(1-(1)) * (ldb)+ _b_offset,ldb,a,_a_offset,lda);
com.polytechnik.lapack.Daxpy.daxpy((k-1),ct,b,(k-(1))+(1-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Dscal.dscal((k-1),bkk,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
a[(k-(1))+(k-(1)) * (lda)+ _a_offset] = (akk*( Math.pow(bkk, 2)));
//Dummy.label("com/polytechnik/lapack/Dsygs2",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
/***
*
*     End of DSYGS2
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsygs2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsygs2",999999);
return;
   }
} // End class.
