/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsyr2k.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsyr2k.java,v 1.4 2012/12/20 17:40:29 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dsyr2k {

// *> \brief \b DSYR2K
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYR2K(UPLO,TRANS,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA,BETA
// *       INTEGER K,LDA,LDB,LDC,N
// *       CHARACTER TRANS,UPLO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),B(LDB,*),C(LDC,*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYR2K  performs one of the symmetric rank 2k operations
// *>
// *>    C := alpha*A*B**T + alpha*B*A**T + beta*C,
// *>
// *> or
// *>
// *>    C := alpha*A**T*B + alpha*B**T*A + beta*C,
// *>
// *> where  alpha and beta  are scalars, C is an  n by n  symmetric matrix
// *> and  A and B  are  n by k  matrices  in the  first  case  and  k by n
// *> matrices in the second case.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>           On  entry,   UPLO  specifies  whether  the  upper  or  lowe
// *>           triangular  part  of the  array  C  is to be  referenced  a
// *>           follows:
// *>
// *>              UPLO = 'U' or 'u'   Only the  upper triangular part of  
// *>                                  is to be referenced.
// *>
// *>              UPLO = 'L' or 'l'   Only the  lower triangular part of  
// *>                                  is to be referenced.
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>           On entry,  TRANS  specifies the operation to be performed a
// *>           follows:
// *>
// *>              TRANS = 'N' or 'n'   C := alpha*A*B**T + alpha*B*A**T +

// *>                                        beta*C.
// *>
// *>              TRANS = 'T' or 't'   C := alpha*A**T*B + alpha*B**T*A +

// *>                                        beta*C.
// *>
// *>              TRANS = 'C' or 'c'   C := alpha*A**T*B + alpha*B**T*A +

// *>                                        beta*C.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry,  N specifies the order of the matrix C.  N must b
// *>           at least zero.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>           On entry with  TRANS = 'N' or 'n',  K  specifies  the numbe
// *>           of  columns  of the  matrices  A and B,  and on  entry  wit
// *>           TRANS = 'T' or 't' or 'C' or 'c',  K  specifies  the  numbe
// *>           of rows of the matrices  A and B.  K must be at least  zero
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where 
// *>           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
// *>           Before entry with  TRANS = 'N' or 'n',  the  leading  n by 
// *>           part of the array  A  must contain the matrix  A,  otherwis
// *>           the leading  k by n  part of the array  A  must contain  th
// *>           matrix A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n
// *>           then  LDA must be at least  max( 1, n ), otherwise  LDA mus
// *>           be at least  max( 1, k ).
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where 
// *>           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
// *>           Before entry with  TRANS = 'N' or 'n',  the  leading  n by 
// *>           part of the array  B  must contain the matrix  B,  otherwis
// *>           the leading  k by n  part of the array  B  must contain  th
// *>           matrix B.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>           On entry, LDB specifies the first dimension of B as declare
// *>           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n
// *>           then  LDB must be at least  max( 1, n ), otherwise  LDB mus
// *>           be at least  max( 1, k ).
// *> \endverbatim
// *>
// *> \param[in] BETA
// *> \verbatim
// *>          BETA is DOUBLE PRECISION.
// *>           On entry, BETA specifies the scalar beta.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array of DIMENSION ( LDC, n ).
// *>           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
// *>           upper triangular part of the array C must contain the upper
// *>           triangular part  of the  symmetric matrix  and the strictly
// *>           lower triangular part of C is not referenced.  On exit, the
// *>           upper triangular part of the array  C is overwritten by the
// *>           upper triangular part of the updated matrix.
// *>           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
// *>           lower triangular part of the array C must contain the lower
// *>           triangular part  of the  symmetric matrix  and the strictly
// *>           upper triangular part of C is not referenced.  On exit, the
// *>           lower triangular part of the array  C is overwritten by the
// *>           lower triangular part of the updated matrix.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>           On entry, LDC specifies the first dimension of C as declare
// *>           in  the  calling  (sub)  program.   LDC  must  be  at  leas
// *>           max( 1, n ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level3
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 3 Blas routine.
// *>
// *>
// *>  -- Written on 8-February-1989.
// *>     Jack Dongarra, Argonne National Laboratory.
// *>     Iain Duff, AERE Harwell.
// *>     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *>     Sven Hammarling, Numerical Algorithms Group Ltd.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Parameters ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dsyr2k (final char uplo,
final char trans,
int n,
int k,
double alpha,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double beta,
double [] c, int _c_offset,
int ldc)  {

double temp1= 0.0d;
double temp2= 0.0d;
int i= 0;
int info= 0;
int j= 0;
int l= 0;
int nrowa= 0;
boolean upper= false;
//uplo = uplo.substring((1)-1,1);
//trans = trans.substring((1)-1,1);
if (com.polytechnik.lapack.Lsame.lsame(trans,'N'))  {
    nrowa = n;
}
else  {
  nrowa = k;
}              //  Close else.
// *
upper = com.polytechnik.lapack.Lsame.lsame(uplo,'U');
info = 0;
if ((((!upper)) && ((!com.polytechnik.lapack.Lsame.lsame(uplo,'L')))))  {
    info = 1;
}
else if (((((!com.polytechnik.lapack.Lsame.lsame(trans,'N'))) && ((!com.polytechnik.lapack.Lsame.lsame(trans,'T')))) && ((!com.polytechnik.lapack.Lsame.lsame(trans,'C')))))  {
    info = 2;
}              // Close else if()
else if ((n < 0))  {
    info = 3;
}              // Close else if()
else if ((k < 0))  {
    info = 4;
}              // Close else if()
else if ((lda < Math.max(1, nrowa)))  {
    info = 7;
}              // Close else if()
else if ((ldb < Math.max(1, nrowa)))  {
    info = 9;
}              // Close else if()
else if ((ldc < Math.max(1, n)))  {
    info = 12;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYR2K",info);
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyr2k",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     And when  alpha.eq.zero.
*
***/
if ((((n == 0)) || ((((((alpha == 0.0)) || ((k == 0)))) && ((beta == 1.0)))))) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyr2k",999999);
}
    if ((alpha == 0.0))  {
    if (upper)  {
    if ((beta == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dsyr2k",10);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",20);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dsyr2k",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",40);
}              //  Close for() loop. 
}
}              //  Close else.
}
else  {
  if ((beta == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dsyr2k",50);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",60);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dsyr2k",70);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
return;//Dummy.go_to("com/polytechnik/lapack/Dsyr2k",999999);
/***
*
*     Start the operations.
*
***/
}
/***
*
*        Form  C := alpha*A*B**T + alpha*B*A**T + C.
*
***/
if (com.polytechnik.lapack.Lsame.lsame(trans,'N'))  {
    if (upper)  {
    {
for (j = 1; j <= n; j++) {
if ((beta == 0.0))  {
    {
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dsyr2k",90);
}              //  Close for() loop. 
}
}
else if ((beta != 1.0))  {
    {
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dsyr2k",100);
}              //  Close for() loop. 
}
}              // Close else if()
{
for (l = 1; l <= k; l++) {
if ((((a[(j-(1))+(l-(1)) * (lda)+ _a_offset] != 0.0)) || ((b[(j-(1))+(l-(1)) * (ldb)+ _b_offset] != 0.0))))  {
    temp1 = (alpha*b[(j-(1))+(l-(1)) * (ldb)+ _b_offset]);
temp2 = (alpha*a[(j-(1))+(l-(1)) * (lda)+ _a_offset]);
{
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(a[(i-(1))+(l-(1)) * (lda)+ _a_offset]*temp1))+(b[(i-(1))+(l-(1)) * (ldb)+ _b_offset]*temp2));
//Dummy.label("com/polytechnik/lapack/Dsyr2k",110);
}              //  Close for() loop. 
}
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",120);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",130);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
if ((beta == 0.0))  {
    {
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dsyr2k",140);
}              //  Close for() loop. 
}
}
else if ((beta != 1.0))  {
    {
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack/Dsyr2k",150);
}              //  Close for() loop. 
}
}              // Close else if()
{
for (l = 1; l <= k; l++) {
if ((((a[(j-(1))+(l-(1)) * (lda)+ _a_offset] != 0.0)) || ((b[(j-(1))+(l-(1)) * (ldb)+ _b_offset] != 0.0))))  {
    temp1 = (alpha*b[(j-(1))+(l-(1)) * (ldb)+ _b_offset]);
temp2 = (alpha*a[(j-(1))+(l-(1)) * (lda)+ _a_offset]);
{
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(a[(i-(1))+(l-(1)) * (lda)+ _a_offset]*temp1))+(b[(i-(1))+(l-(1)) * (ldb)+ _b_offset]*temp2));
//Dummy.label("com/polytechnik/lapack/Dsyr2k",160);
}              //  Close for() loop. 
}
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",170);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",180);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Form  C := alpha*A**T*B + alpha*B**T*A + C.
*
***/
}
else  {
  if (upper)  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
temp1 = 0.0;
temp2 = 0.0;
{
for (l = 1; l <= k; l++) {
temp1 = (temp1+(a[(l-(1))+(i-(1)) * (lda)+ _a_offset]*b[(l-(1))+(j-(1)) * (ldb)+ _b_offset]));
temp2 = (temp2+(b[(l-(1))+(i-(1)) * (ldb)+ _b_offset]*a[(l-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dsyr2k",190);
}              //  Close for() loop. 
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((alpha*temp1)+(alpha*temp2));
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (((beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset])+(alpha*temp1))+(alpha*temp2));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dsyr2k",200);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",210);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
temp1 = 0.0;
temp2 = 0.0;
{
for (l = 1; l <= k; l++) {
temp1 = (temp1+(a[(l-(1))+(i-(1)) * (lda)+ _a_offset]*b[(l-(1))+(j-(1)) * (ldb)+ _b_offset]));
temp2 = (temp2+(b[(l-(1))+(i-(1)) * (ldb)+ _b_offset]*a[(l-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dsyr2k",220);
}              //  Close for() loop. 
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((alpha*temp1)+(alpha*temp2));
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (((beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset])+(alpha*temp1))+(alpha*temp2));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dsyr2k",230);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dsyr2k",240);
}              //  Close for() loop. 
}
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DSYR2K.
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsyr2k",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsyr2k",999999);
return;
   }
} // End class.
