package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Book builder.
 *  @version $Id: BookBuilder.java,v 1.62 2010/04/13 21:00:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class BookBuilder {

    // set load factor pretty low, when we do not iterate over.
    static final float LOADFACTOR=0.27f;
    static final int LARGETABLE=500501;
    static final int SMALLTABLE=5501;

    /** A map of order ID -> Book of orders.
     */
    private final com.polytechnik.utils.LongHashMap<RealtimeBook> globalordermapping=
	new com.polytechnik.utils.LongHashMap<RealtimeBook>(BookBuilder.LARGETABLE,BookBuilder.LOADFACTOR);

    public RealtimeBook idLookupBook(final long itemid)
    {
	return this.globalordermapping.get(itemid);
    }


    public BookItem removeItem(final long itemid,final RealtimeBook tickerbook_ext)
    {
	final RealtimeBook tickerbook=this.globalordermapping.remove(itemid);
	if(tickerbook==null)
	{
	    throw new IllegalStateException("No book found for item="+itemid);
	}
	if(tickerbook_ext!=tickerbook)
	{
	    throw new RuntimeException("Tickerbook mismatch for item="+itemid+
				       " ticker1="+tickerbook.ref.ticker+
				       " and ticker2="+tickerbook_ext.ref.ticker);
	}

	final BookItem bookitems=tickerbook.remove(itemid);
	if(bookitems==null)
	{
	    throw new IllegalStateException("No bookitems found for item="+
					    itemid+" ticker="+tickerbook.ref.ticker);
	}
	return bookitems;
    }



    public RealtimeBook addNewItem(final long itemid,
				   final RealtimeBook tickerbook,
				   final BookItem item)
    {

	final RealtimeBook test_book=this.globalordermapping.put(itemid,tickerbook);
	if(test_book!=null)
	{
	    throw new IllegalStateException(
		"The item="+itemid+" ticker="+tickerbook.ref.ticker+
		" is already in the globalordermapping"+
		" cannot add a new.");
	}

	// add an item to the book
	final BookItem olditem=tickerbook.put(itemid,item);
	if(olditem!=null)
	{
	    throw new IllegalStateException(
		"The item="+itemid+" ticker="+tickerbook.ref.ticker+
		" is already in the ticker book."+
		" cannot add a new.");
	}
	addMapping(itemid,item);
	return tickerbook;
    }


    int getNOrders()
    {
	return this.globalordermapping.size();
    }



    private static final boolean DEBUG_FLAG_CHECK_ORIGTIME=false;
    private final com.polytechnik.utils.LongHashMap<BookItem>
	check_origtime=DEBUG_FLAG_CHECK_ORIGTIME?
	new com.polytechnik.utils.LongHashMap<BookItem>():null;

    public static final long TIME_KEEP_ORDER_INFO=1000*TraderInterface.TIME_SCALE;
    private static final int INITIALLSIZE=20;
    private long []
	oid_remembered=new long [INITIALLSIZE],
	time_remembered=new long [INITIALLSIZE];
    private int ptr_start=0,ptr_end=0;
    private long prev_oid=-1,prev_time=-1;
    private void addMapping(final long oid,final BookItem o)
    {
	if(!OrdersAndTrades.USE_ORIGTIME)
	{
	    return;
	}

	if(/* oid<this.prev_oid|| */o.time<this.prev_time)
	{
	    // order ID may actually go backward
	    throw new IllegalArgumentException(
		"Order ID or time is not increasing:"+
		" oid="+oid+
		" prev_oid="+this.prev_oid+
		" o.time="+o.time+
		" prev_time="+this.prev_time);
	}
	this.prev_oid=oid;
	this.prev_time=o.time;


	final int capacity=oid_remembered.length,n=ptr_end-ptr_start;
	if(n>=capacity)
	{
	    if(DEBUG_FLAG_CHECK_ORIGTIME)
	    {
		System.err.println("CLEAR");
		long xxprevoid=-1;
		for(int i=ptr_start;i<ptr_end;i++)
		{
		    if(oid_remembered[i]<=xxprevoid||oid_remembered[i]<=0||time_remembered[i]<=0)
		    {
			throw new RuntimeException("Unordered data i="+i+
						   " oid="+oid_remembered[i]+
						   " t="+time_remembered[i]);
		    }
		    xxprevoid=oid_remembered[i];
		}
	    }
	    clearTimeGuessingInfoBefore(o.time-TIME_KEEP_ORDER_INFO);
	    final int n1=ptr_end-ptr_start;
	    if(n1>=(capacity-INITIALLSIZE))
	    {
		shiftArrays(1+2*capacity);
	    }
	    else
	    {
		shiftArrays(-1);
	    }
	}
	else
	{
	    if(ptr_end>=capacity)
	    {
		shiftArrays(-1);
	    }
	}
	if(ptr_end<=0||oid>=oid_remembered[ptr_end-1])
	{
	    oid_remembered[ptr_end]=oid;
	    time_remembered[ptr_end]=o.time;
	    ptr_end++;
	    if(DEBUG_FLAG_CHECK_ORIGTIME)
	    {
		check_origtime.put(oid,o);
	    }
	}
	else if(false) // "false" - for now ignore out of order orderid, they often screw interpolation
	{
	    final int ptr=java.util.Arrays.binarySearch(oid_remembered,
							ptr_start,
							ptr_end,
							oid);
	    if(ptr>=0)
	    {
		throw new RuntimeException("Duplicated order ID="+oid+" currenttime="+o.time);
	    }
	    final int ptr_i=-1-ptr,ntomove=n-(ptr_i-ptr_start);
	    System.arraycopy(oid_remembered,ptr_i,oid_remembered,ptr_i+1,ntomove);
	    System.arraycopy(time_remembered,ptr_i,time_remembered,ptr_i+1,ntomove);
	    oid_remembered[ptr_i]=oid;
	    time_remembered[ptr_i]=o.time;
	    if(DEBUG_FLAG_CHECK_ORIGTIME)
	    {
		System.err.println("added "+oid+" to ptr_i="+ptr_i+" ptr_start="+ptr_start+
				   " ptr_end="+ptr_end+" ntomove="+ntomove);
		long xxprevoid=-1;
		for(int i=ptr_start;i<ptr_end;i++)
		{
		    if(oid_remembered[i]<=xxprevoid||oid_remembered[i]<=0||time_remembered[i]<=0)
		    {
			throw new RuntimeException("Unordered data at i="+i+
						   " oid="+oid_remembered[i]+
						   " t="+time_remembered[i]);
		    }
		    xxprevoid=oid_remembered[i];
		}
	    }
	    ptr_end++;
	    if(DEBUG_FLAG_CHECK_ORIGTIME)
	    {
		check_origtime.put(oid,o);
	    }
	}
    }

    private void shiftArrays(int newsize)
    {
	final long [] t,o;
	if(newsize>0)
	{
	    o=new long [newsize];
	    t=new long [newsize];
	}
	else
	{
	    o=oid_remembered;
	    t=time_remembered;
	}
	final int n=ptr_end-ptr_start;
	System.arraycopy(oid_remembered,ptr_start,o,0,n);
	System.arraycopy(time_remembered,ptr_start,t,0,n);
	if(newsize>0)
	{
	    oid_remembered=o;
	    time_remembered=t;
	}
	ptr_start=0;
	ptr_end=n;
    }


    public long guessTimeFromID(final long oid,final long exe_time)
    {
	if(!OrdersAndTrades.USE_ORIGTIME)
	{
	    return -1;
	}
	final int ptr=java.util.Arrays.binarySearch(oid_remembered,
						    ptr_start,
						    ptr_end,
						    oid);
	if(ptr>=0)
	{
	    throw new RuntimeException("Exact match of order ID="+oid+" currenttime="+exe_time);
	}
	final long torig;
	final int ptr_l=-2-ptr;
	if(DEBUG_FLAG_CHECK_ORIGTIME)
	{
	    int ptr_lt=ptr_start-1;
	    for(int i=ptr_end-1;i>=ptr_start;i--)
	    {
		if(oid_remembered[i]<=oid)
		{
		    ptr_lt=i;
		    break;
		}
	    }
	    if(ptr_lt!=ptr_l)
	    {
		throw new RuntimeException("Pointers mismatch:"+
					   " ptr_lt="+ptr_lt+
					   " ptr_l="+ptr_l+
					   " ptr_start="+ptr_start+
					   " ptr_end="+ptr_end+
					   " oid="+oid+
					   " currenttime="+exe_time);
	    }
	}
	if(ptr_l<ptr_start)
	{
	    //System.err.println(" ptr="+ptr+" n="+(ptr_end-ptr_start)+" oid="+oid+" ptr_start="+ptr_start+" ptr_end="+ptr_end);
	    torig=exe_time-TIME_KEEP_ORDER_INFO;
	}
	else
	{
	    final long t_cut=exe_time-TIME_KEEP_ORDER_INFO;
	    final long t=time_remembered[ptr_l];
	    torig=t<t_cut?t_cut:t;
	}
	if(DEBUG_FLAG_CHECK_ORIGTIME)
	{
	    BookItem b=null;
	    for(long l=oid;l-->0;)
	    {
		if(null!=(b=check_origtime.get(l)))
		{
		    break;
		}
	    }
	    final long t_cut=exe_time-TIME_KEEP_ORDER_INFO;
	    final long t_test=b==null?t_cut:(b.time<t_cut?t_cut:b.time);
	    if(torig!=t_test)
	    {
		for(int i=ptr_end-1;i>=ptr_start;i--)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       oid_remembered[i]+"\t"+
				       time_remembered[i]);
		}
		throw new RuntimeException("Time interpolation failure:"+
					   " torig="+torig+
					   " t_test="+t_test+
					   " b.time="+(b!=null?b.time:-1)+
					   " b.orderreferencenumber="+(b!=null?b.orderreferencenumber:-1)+
					   " ptr_l="+ptr_l+
					   " reforderreferencenumber ="+(ptr_l>=0?oid_remembered[ptr_l]:-1)+
					   " oid="+oid+
					   " currenttime="+exe_time);
	    }
	}
	return torig<=exe_time?torig:exe_time;
    }

    private long prev_clear_id=-1;
    private int clearTimeGuessingInfoBefore(final long t0)
    {
	int nc=ptr_start;
	for(int pe=ptr_end;nc<pe;)
	{
	    if(time_remembered[nc]<t0)
	    {
		nc++;
	    }
	    else
	    {
		break;
	    }
	}
	if(prev_clear_id>=0 &&(ptr_start<ptr_end && prev_clear_id!=oid_remembered[ptr_start]))
	{
	    throw new RuntimeException("Incorrect clear");
	}
	final int n_cleared=nc-ptr_start;
	ptr_start=nc;
	if(nc<ptr_end)
	{
	    prev_clear_id=oid_remembered[nc];
	}
	else
	{
	    prev_clear_id=-1;
	}
	return n_cleared;
    }

}
