package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Book item.
 *  @version $Id: BookItem.java,v 1.28 2012/08/21 23:25:56 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class BookItem implements com.polytechnik.utils.CircularList.TimeAccess {
    public static final byte
	BOOK_BUY=21,
	BOOK_SELL=22,
	BOOK_UNKNOWN=-1;

    public final long time;
    /** Possibly to remove to save memory? */
    public final long orderreferencenumber;
    public final double price;
    public final int shares;
    public final String attribution;
    public final byte buy_sell_indicator;
    public final BookItem order;


    public BookItem(final double price,
		    final int shares,
		    final String attribution,
		    final byte buy_sell_indicator,
		    final long time,
		    final long orderreferencenumber,
		    final BookItem order)
    {
	this.time=time;
	this.price=price;
	this.shares=shares;
	this.attribution=attribution;
	this.buy_sell_indicator=buy_sell_indicator;
	this.orderreferencenumber=orderreferencenumber;
	this.order=order;
    }

    public long getTime()
    {
	return this.time;
    }

    public static final BookItem adjustShares(final BookItem bi,
					      final int newshares,
					      final long time)
    {
	if(time!=bi.time)
	{
	    throw new IllegalArgumentException("Changing order time is temporarily disabled new time="+time+" old time="+bi.time);
	}
	return new BookItem(bi.price,
			    newshares,
			    bi.attribution,
			    bi.buy_sell_indicator,
			    time,
			    bi.orderreferencenumber,
			    bi.order);
    }


    public static int cmpByTime(final BookItem o1,final BookItem o2)
    {
	if(o1.time<o2.time)
	{
	    return -1;
	}
	if(o1.time>o2.time)
	{
	    return 1;
	}
	if(o1.orderreferencenumber<o2.orderreferencenumber)
	{
	    return -1;
	}
	if(o1.orderreferencenumber>o2.orderreferencenumber)
	{
	    return 1;
	}
	return 0;
    }


    public static String getOrderDescription(final BookItem order)
    {
	return (order==null?"null":
		"[ORDER: price="+order.price+
		" time="+BookItem.getTimeDiag(order.time)+
		" shares="+order.shares+
		" buy_sell_indicator="+order.buy_sell_indicator+
		" orderreferencenumber="+order.orderreferencenumber
		+"]");
    }

    public static String getTimeDiag(final long t_internalscale)
    {
	return getTimeHHMMSS(t_internalscale)+
	    " ("+t_internalscale+")";
    }

    public static String getTimeHHMMSS(final long t_nanoseconds)
    {
	final long t_seconds=t_nanoseconds/TraderInterface.TIME_SCALE,s=t_seconds%60L,m=(t_seconds%3600L)/60L,h=t_seconds/(3600L);
	return
	    (h<10L?"0":"")+h+":"+
	    (m<10L?"0":"")+m+":"+
	    (s<10L?"0":"")+s;
    }

    public static long getSystemTimeMidnight00Milliseconds()
    {
	final java.util.GregorianCalendar c=new java.util.GregorianCalendar();
	c.set(java.util.Calendar.HOUR_OF_DAY,0);
	c.set(java.util.Calendar.MINUTE,0);
	c.set(java.util.Calendar.SECOND,0);
	c.set(java.util.Calendar.MILLISECOND,0);
	// midnight time in milliseconds
	return c.getTimeInMillis();
    }
}
