package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A class to store array buffer of variable length.
 *  @version $Id: Distribution.java,v 1.10 2012/01/13 17:32:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class Distribution<T> {
    private T [] buffer=null;
    private int n_elements=0;


    public void clearAll()
    {
	this.n_elements=0;
	if(getCapacity()>0) java.util.Arrays.fill(this.buffer,null);
    }

    /** Used to catch incorrect usage.
     */
    public void setInvalidState()
    {
	clearAll();
	this.n_elements=-1;
    }

    public int getSize()
    {
	if(this.n_elements<0)
	{
	    throw new IllegalStateException("To catch incorrect usage; this.n_elements="+this.n_elements);
	}
	return this.n_elements;
    }

    /** A kludge for direct manipulation with size.
     */
    public void setSize(final int newsize)
    {
	final int c=getCapacity();
	if(c<=0||newsize>c||this.n_elements<0)
	{
	    throw new IllegalStateException();
	}
	if(newsize>0 && this.buffer[newsize-1]==null)
	{
	    throw new IllegalStateException();
	}
	if(newsize<this.buffer.length && this.buffer[newsize]!=null)
	{
	    throw new IllegalStateException();
	}
	this.n_elements=newsize;
    }

    public int getCapacity()
    {
	if(this.buffer==null) return 0;
	return this.buffer.length;
    }

    public T [] getBuffer()
    {
	if(this.n_elements<0)
	{
	    throw new IllegalStateException("To catch incorrect usage; this.n_elements="+this.n_elements);
	}
	return this.buffer;
    }

    /** Because we cannot create a T[] array without reflect -
     *  create one outside of this class.
     */
    public void replaceBuffer(final T [] newbuffer)
    {
	if(this.n_elements>0)
	{
	    throw new IllegalStateException();
	}
	if(getCapacity()>=newbuffer.length)
	{
	    throw new IllegalArgumentException("A check to catch possible getSize()/getCapacity() misusage");
	}
	this.buffer=newbuffer;
    }


    /** Implementation of interface to add data.
     *  Used for testing only.
     */
    public void add(final T b)
    {
	this.buffer[this.n_elements++]=b;
    }

    public static final java.util.Comparator<BookItem>
	cmp_by_price_lowest_up=new java.util.Comparator<BookItem>()
	{
	    public int compare(final BookItem o1,final BookItem o2)
	    {
		return o1.price<o2.price?-1:(o1.price>o2.price?1:BookItem.cmpByTime(o1,o2));
	    }
	},
	cmp_by_price_lowest_down=new java.util.Comparator<BookItem>()
	{
	    public int compare(final BookItem o1,final BookItem o2)
	    {
		return o1.price<o2.price?1:(o1.price>o2.price?-1:BookItem.cmpByTime(o1,o2));
	    }
	},
	cmp_orders_by_price_and_type=new java.util.Comparator<BookItem>()
	{
	    public int compare(final BookItem o1,final BookItem o2)
	    {
		final boolean ne=o1.buy_sell_indicator!=o2.buy_sell_indicator;
		if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(ne)
		    {
			// buy on top
			return -1;
		    }
		    // max on top
		    return o1.price<o2.price?1:(o1.price>o2.price?-1:BookItem.cmpByTime(o1,o2));
		}
		if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(ne)
		    {
			// sell on bottom
			return 1;
		    }
		    // min on bottom (max on top)
		    // the order will be reversed when copying to actual storage array.
		    return o1.price<o2.price?1:(o1.price>o2.price?-1:-BookItem.cmpByTime(o1,o2));
		}
		throw new RuntimeException("Unknown buy_sell_indicator="+
					   o1.buy_sell_indicator+
					   " time="+o1.time);
	    }
	},
	cmp_orders_by_price_and_type_latest_at_00=new java.util.Comparator<BookItem>()
	{
	    public int compare(final BookItem o1,final BookItem o2)
	    {
		final boolean ne=o1.buy_sell_indicator!=o2.buy_sell_indicator;
		if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(ne)
		    {
			// buy on top
			return -1;
		    }
		    // max on top
		    return o1.price<o2.price?1:(o1.price>o2.price?-1:-BookItem.cmpByTime(o1,o2));
		}
		if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(ne)
		    {
			// sell on bottom
			return 1;
		    }
		    // min on bottom (max on top)
		    // the order will be reversed when copying to actual storage array.
		    return o1.price<o2.price?1:(o1.price>o2.price?-1:BookItem.cmpByTime(o1,o2));
		}
		throw new RuntimeException("Unknown buy_sell_indicator="+
					   o1.buy_sell_indicator+
					   " time="+o1.time);
	    }
	};


/*
    void sortByPrices(final boolean lowest_up)
    {

	// a sorting without creation of array copy.
	com.polytechnik.utils.TMPQSortAlgorithm.objectQuickSortPointers(this.buffer,0,this.n_elements-1,lowest_up?cmp_lowest_up:cmp_lowest_down);

	// // the java.util.Arrays.sort makes internal copy of the array.
	// // do we want this?
	// java.util.Arrays.sort(this.buffer,0,this.n_elements,lowest_up?cmp_lowest_up:cmp_lowest_down);
    }
*/


    /*
    public boolean forEachEntry(final com.polytechnik.utils.TObjectProcedure<T> procedure)
    {
	final T [] buf=this.buffer;
	for(int i=0,n=getSize();i<n;i++)
	{
	    if(!procedure.execute(buf[i]))
	    {
		return false;
	    }
	}
	return true;
    }
    */

 public static <V> Distribution<V> [] getDistributionArray(final int dim)
 {
     @SuppressWarnings("unchecked")
     final Distribution<V> [] ii=new Distribution [dim];
     return ii;
 }

}
