package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Match two set of byu/sell orders.
 *  @version $Id: MatchOrdersNoShift.java,v 1.4 2009/07/08 23:42:28 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MatchOrdersNoShift
{

    public static class MatchResults {
	public final double p_buy,p_sell;
	public final double p_match_buy,p_match_sell;
	public final long v_executed,v_buy,v_sell,vbuy0,vsell0;
	public final int nmatched_pricelevels_buy,nmatched_pricelevels_sell;
	MatchResults(final double p_buy,
		     final double p_sell,
		     final double p_match_buy,
		     final double p_match_sell,
		     final long v_executed,
		     final long v_buy,
		     final long v_sell,
		     final long vbuy0,
		     final long vsell0,
		     final int nmatched_pricelevels_buy,
		     final int nmatched_pricelevels_sell)
	{
	    this.p_buy=p_buy;
	    this.p_sell=p_sell;
	    this.p_match_buy=p_match_buy;
	    this.p_match_sell=p_match_sell;
	    this.v_executed=v_executed;
	    this.v_buy=v_buy;
	    this.v_sell=v_sell;
	    this.vbuy0=vbuy0;
	    this.vsell0=vsell0;
	    this.nmatched_pricelevels_buy=nmatched_pricelevels_buy;
	    this.nmatched_pricelevels_sell=nmatched_pricelevels_sell;
	}
    }


    private static void checkIdentical(final MatchResults m,final MatchResults mt)
    {
	if(Double.isNaN(mt.p_buy)!=Double.isNaN(m.p_buy) ||
	   (!Double.isNaN(mt.p_buy) && mt.p_buy!=m.p_buy) ||
	   Double.isNaN(mt.p_sell)!=Double.isNaN(m.p_sell) ||
	   (!Double.isNaN(mt.p_sell) && mt.p_sell!=m.p_sell) ||
	   Double.isNaN(mt.p_match_buy)!=Double.isNaN(m.p_match_buy) ||
	   (!Double.isNaN(mt.p_match_buy) && mt.p_match_buy!=m.p_match_buy) ||
	   Double.isNaN(mt.p_match_sell)!=Double.isNaN(m.p_match_sell) ||
	   (!Double.isNaN(mt.p_match_sell) && mt.p_match_sell!=m.p_match_sell) ||
	   mt.v_executed!=m.v_executed||
	   mt.v_buy!=m.v_buy||
	   mt.v_sell!=m.v_sell||
	   mt.nmatched_pricelevels_buy!=m.nmatched_pricelevels_buy||
	   mt.nmatched_pricelevels_sell!=m.nmatched_pricelevels_sell)
	{
	    throw new RuntimeException(
		"Results mismatch:"+
		" mt.p_buy="+mt.p_buy+" m.p_buy="+m.p_buy+
		" mt.p_sell="+mt.p_sell+" m.p_sell="+m.p_sell+
		" mt.p_match_buy="+mt.p_match_buy+" m.p_match_buy="+m.p_match_buy+
		" mt.p_match_sell="+mt.p_match_sell+" m.p_match_sell="+m.p_match_sell+
		" mt.v_executed="+mt.v_executed+" m.v_executed="+m.v_executed+
		" mt.v_buy="+mt.v_buy+" m.v_buy="+m.v_buy+
		" mt.v_sell="+mt.v_sell+ " m.v_sell="+m.v_sell+
		" mt.nmatched_pricelevels_buy="+mt.nmatched_pricelevels_buy+" m.nmatched_pricelevels_buy="+m.nmatched_pricelevels_buy+
		" mt.nmatched_pricelevels_sell="+mt.nmatched_pricelevels_sell+" m.nmatched_pricelevels_sell="+m.nmatched_pricelevels_sell);
	}
    }


    public MatchResults match(final OrdersFlow buy,final OrdersFlow sell)
    {
	final MatchResults m=match(buy,sell,true);
	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    checkIdentical(m,match(buy,sell,false));

	}
	return m;
    }


    /** The object and method argument must have
     *  mergeIdenticalPrices() called before this method usage.
     */
    private MatchResults match(final OrdersFlow buy,
			       final OrdersFlow sell,
			       final boolean linear_algorithm)
    {
	if(!buy.flag_descending||sell.flag_descending)
	{
	    throw new IllegalArgumentException("match method can be called for buy match sell");
	}

	final long vb=buy.getTotalVolume(),vs=sell.getTotalVolume();
	final int n_buy=buy.getSize(),n_sell=sell.getSize();
	if(n_buy>0 && n_sell<=0)
	{
	    return new MatchResults(buy.price[0],
				    Double.NaN,
				    Double.NaN,
				    Double.NaN,
				    0,
				    vb,
				    0,
				    0,
				    0,
				    0,
				    0);
	}
	if(n_buy<=0 && n_sell>0)
	{
	  return new MatchResults(Double.NaN,
				  sell.price[0],
				  Double.NaN,
				  Double.NaN,
				  0,
				  0,
				  vs,
				  0,
				  0,
				  0,
				  0);
	}
	if(n_buy<=0 && n_sell<=0)
	{
	    return new MatchResults(Double.NaN,
				    Double.NaN,
				    Double.NaN,
				    Double.NaN,
				    0,
				    0,
				    0,
				    0,
				    0,
				    0,
				    0);
	}

	// buy & sell volumes
	final long [] v_buy=buy.vol_total,v_sell=sell.vol_total;


	final boolean PRINT_DEBUG=false;
	if(PRINT_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============match N_BUY=\t"+n_buy+
			       "\t N_SELL=\t"+n_sell);

	    System.err.println("IND\t\tP_BUY\tV_TOT_BUY\tO_BUY\t\tP_SELL\tV_TOT_SELL\tO_SELL");
	    final int n=Math.max(n_buy,n_sell);
	    for(int i=0;i<n;i++)
	    {
		System.err.print(String.valueOf(i)+"\t\t");
		if(i<n_buy)
		{
		    System.err.print(String.valueOf(buy.price[i])+"\t"+v_buy[i]+"\t"+buy.vol_total[i]+"\t");
		}
		else
		{
		    System.err.print("\t\t\t");
		}
		System.err.print("\t");
		if(i<n_sell)
		{
		    System.err.print(String.valueOf(sell.price[i])+"\t"+v_sell[i]+"\t"+sell.vol_total[i]+"\t");
		}
		else
		{
		    System.err.print("\t\t\t");
		}
		System.err.println();
	    }
	}


	long v=-1;
	int ind_p_buy=-1,ind_p_sell=-1;
	if(linear_algorithm)
	{
	    // Optimized version (N).
	    // it is possible to do in log(N) but we have other calculations in N
	    // so there is no much point in further optimization.
	    int ips=n_sell-1;
	    for(int ipb=0;ipb<n_buy;ipb++)
	    {
		final double priceipb=buy.price[ipb]+OrdersFlow.EPS;
		for(;ips>=0;ips--)
		{
		    if(sell.price[ips]<=priceipb)
		    {
			// found
			break;
		    }
		}
		if(ips<0)
		{
		    // no sell point found
		    break;
		}
		final long vx=Math.min(v_buy[ipb],v_sell[ips]);
		if(vx>=v)
		{
		    v=vx;
		    ind_p_buy=ipb;
		    ind_p_sell=ips;
		}
	    }

	}
	else
	{
	    // I have no time right now to write an efficient implementation.
	    // do it in N^2 time for now.
	    for(int ipb=0;ipb<n_buy;ipb++)
	    {
		final double priceipb=buy.price[ipb]+OrdersFlow.EPS;
		int ips=0;
		for(;ips<n_sell;ips++)
		{
		    if(sell.price[ips]>priceipb)
		    {
			break;
		    }
		    final long vx=Math.min(v_buy[ipb],v_sell[ips]);
		    if(vx>=v)
		    {
			v=vx;
			ind_p_buy=ipb;
			ind_p_sell=ips;
		    }
		}
		if(ips<=0)
		{
		    break;
		}
	    }
	}

	if(v<=0)
	{
	    // no single order match
	    return new MatchResults(buy.price[0],
				    sell.price[0],
				    Double.NaN,
				    Double.NaN,
				    0,
				    v_buy[n_buy-1],
				    v_sell[n_sell-1],
				    0,
				    0,
				    0,
				    0);
	}


	final double p_buy;
	if(v_buy[ind_p_buy]>v)
	{
	    p_buy=buy.price[ind_p_buy];
	}
	else if((ind_p_buy+1)<n_buy)
	{
	    p_buy=buy.price[ind_p_buy+1];
	}
	else
	{
	    p_buy=Double.NaN;
	}

	final double p_sell;
	if(v_sell[ind_p_sell]>v)
	{
	    p_sell=sell.price[ind_p_sell];
	}
	else if((ind_p_sell+1)<n_sell)
	{
	    p_sell=sell.price[ind_p_sell+1];
	}
	else
	{
	    p_sell=Double.NaN;
	}


	if(PRINT_DEBUG)
	{
	    System.err.println();
	    System.err.println("p_buy\t"+p_buy+"\tp_buy_match\t"+buy.price[ind_p_buy]+"\tind\t"+ind_p_buy);
	    System.err.println("p_sell\t"+p_sell+"\tp_sell_match\t"+sell.price[ind_p_sell]+"\tind\t"+ind_p_sell);
	    System.err.println("executed\t"+v);
	    System.err.println("ord_buy\t"+(v_buy[n_buy-1]-v));
	    System.err.println("ord_sell\t"+(v_sell[n_sell-1]-v));
	}
	return new MatchResults(p_buy,
				p_sell,
				buy.price[ind_p_buy],
				sell.price[ind_p_sell],
				v,
				v_buy[n_buy-1],
				v_sell[n_sell-1],
				v_buy[ind_p_buy],
				v_sell[ind_p_sell],
				ind_p_buy+1,
				ind_p_sell+1);
    }



    private static OrdersFlow gettestflowsetdown(java.util.Random r,
						 final double p0,
						 final int n_below,
						 final long v_below,
						 final int n_above)
    {
	final OrdersFlow of=new OrdersFlow(false);
	final int vb=(int)(2.0*v_below/n_below);
	long v=0;
	double pp=p0;
	for(int i=0;i<n_below;i++)
	{
	    final int vi=1+(int)((0.1+r.nextDouble())*vb);
	    if(v+vi>=v_below||i>=(n_below-1))
	    {
		if(v>=v_below)
		{
		    throw new RuntimeException();
		}
		of.addOrder(p0-(0.00001+r.nextDouble())*10.0,(int)(v_below-v));
		break;
	    }
	    if(r.nextDouble()<0.1)
	    {
		of.addOrder(pp,vi);
	    }
	    else
	    {
		of.addOrder(pp=(i>0?p0-(0.00001+r.nextDouble())*10.0:p0),vi);
	    }
	    v+=vi;
	}
	of.mergeIdenticalPrices();
	if(of.getTotalVolume()!=v_below)
	{
	    throw new RuntimeException("Volume mismatch");
	}
	for(int i=0;i<n_above;i++)
	{
	    of.addOrder(p0+(0.001+r.nextDouble())*10.0,1+(int)(r.nextDouble()*vb));
	}
	return of;
    }
    private static OrdersFlow gettestflowsetup(java.util.Random r,
					       final double p0,
					       final int n_above,
					       final long v_above,
					       final int n_below)
    {
	final OrdersFlow of=new OrdersFlow(true);
	final int vb=(int)(2.0*v_above/n_above);
	long v=0;
	double pp=p0;
	for(int i=0;i<n_above;i++)
	{
	    final int vi=1+(int)((0.1+r.nextDouble())*vb);
	    if(v+vi>=v_above||i>=(n_above-1))
	    {
		if(v>=v_above)
		{
		    throw new RuntimeException();
		}
		of.addOrder(p0+(0.00001+r.nextDouble())*10.0,(int)(v_above-v));
		break;
	    }
	    if(r.nextDouble()<0.1)
	    {
		of.addOrder(pp,vi);
	    }
	    else
	    {
		of.addOrder(pp=(i>0?p0+(0.00001+r.nextDouble())*10.0:p0),vi);
	    }
	    v+=vi;
	}
	of.mergeIdenticalPrices();
	if(of.getTotalVolume()!=v_above)
	{
	    throw new RuntimeException("Volume mismatch");
	}
	for(int i=0;i<n_below;i++)
	{
	    of.addOrder(p0-(0.001+r.nextDouble())*10.0,1+(int)(r.nextDouble()*vb));
	}
	return of;
    }
    // Unit test.
    public static void main(String [] args)
    {
	if(!OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    throw new IllegalArgumentException("the ModelParameters.CONSTRAINS_EXPENSIVE_CHECKS should be set to true");
	}

     final MatchOrdersNoShift matcher=new MatchOrdersNoShift();
     for(boolean f: new boolean[]{true,false})
     {
	final long t0=System.currentTimeMillis();
	System.err.println("Started at "+new java.util.Date(t0));
	final java.util.Random r=new java.util.Random(-991L);
	final int n_tests=1000000;
	for(int i=0;i<n_tests;i++)
	{
	    final long vtotal=5000000;
	    final double p0=r.nextDouble()*100.0;
	    //System.err.println("P0="+p0+" vtotal="+vtotal);
	    long o1=0,o2=0;
	    switch(i%3) {
	    case 0:
		o1=0;
		o2=r.nextInt(104);
		break;
	    case 1:
		o1=r.nextInt(154);
		o2=0;
		break;
	    case 2:
		o1=o2=0;
		break;
	    }

	    final OrdersFlow obuy=gettestflowsetup(r,
						   p0,
						   1000,
						   vtotal+o1,
						   4000);
	    final OrdersFlow osell=gettestflowsetdown(r,
						      p0,
						      1342,
						      vtotal+o2,
						      2129);
	    obuy.mergeIdenticalPrices();
	    osell.mergeIdenticalPrices();
	    final MatchResults m;
	    if(f)
	    {
		m=matcher.match(obuy,osell,true);

		final OrdersFlow xbuy=new OrdersFlow(true);
		xbuy.addOrdersFrom(obuy);
		final OrdersFlow xsell=new OrdersFlow(false);
		xsell.addOrdersFrom(osell);
		checkIdentical(m,matcher.match(xbuy,xsell,true));
	    }
	    else
	    {
		m=matcher.match(obuy,osell);
	    }
	    if(m.p_match_buy!=p0||m.p_match_sell!=p0||m.v_executed!=vtotal)
	    {
		throw new RuntimeException("matching failed p0="+p0+
					   " m.p_match_buy="+m.p_match_buy+
					   " m.p_match_sell="+m.p_match_sell+
					   " m.v_executed="+m.v_executed+
					   " vtotal="+vtotal);
	    }
	    if((i/1000)*1000==i)
	    {
		System.err.println("I="+i+" tests OK");
	    }
	}
	System.err.println("Tests OK="+n_tests);
	final long t1=System.currentTimeMillis();
	System.err.println("Finished "+n_tests+" tests at "+new java.util.Date(t1)+
			   " diff="+((t1-t0)/1000L)+
			   " match type="+f);
     }
    }
}
