package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Try matching orders using various shifts.
 *  @version $Id: MatchOrdersVariousShifts.java,v 1.7 2012/08/21 23:25:56 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MatchOrdersVariousShifts {

    public static class MatchWithShift {
	public static final int MT_BUY=88,MT_SELL=89;
	public final double test_pmatch,test_dp;
	public final long vmatch;
	public final double delta,delta0,shift,shift0;
	public final double pobuy,posell;
	public final long vbuy0,vsell0;
	public final int type;
	MatchWithShift(final double test_pmatch,
		       final double test_dp,
		       final long vmatch,
		       final double delta,
		       final double delta0,
		       final double shift,
		       final double shift0,
		       final double pobuy,
		       final double posell,
		       final long vbuy0,
		       final long vsell0,
		       final int type)
	{
	    this.test_pmatch=test_pmatch;
	    this.test_dp=test_dp;
	    this.vmatch=vmatch;
	    this.delta=delta;
	    this.delta0=delta0;
	    this.shift=shift;
	    this.shift0=shift0;
	    this.pobuy=pobuy;
	    this.posell=posell;
	    this.vbuy0=vbuy0;
	    this.vsell0=vsell0;
	    this.type=type;
	}
    }

    /** Sort the results. 0-th element corresponds to lowest shift.
     */
    private static final java.util.Comparator<MatchWithShift>
	CMP_BY_SHIFT=new java.util.Comparator<MatchWithShift>()
	{
	    public int compare(final MatchWithShift o1,final MatchWithShift o2)
	    {
		return o1.shift<o2.shift?-1:(o1.shift>o2.shift?1:0);
	    }
	};
    private static final boolean
	FORCE_DPADJUSTMENT_TO_ZERO=false,
    /** FIXME, test not working yet. */
	FORCE_FIRST_DPADJUSTMENT_TO_ZERO=false;

    public void copyOrderedOrdersToFlow(final Distribution<?extends BookItem> orders,
					final OrdersFlow oflow)
    {
	if(oflow.getSize()>0)
	{
	    throw new IllegalArgumentException();
	}
	final BookItem [] buy=orders.getBuffer();
	for(int i=0,n=orders.getSize();i<n;i++)
	{
	    final BookItem t=buy[i];
	    oflow.addOrderCheckPrev(t.price,t.shares);
	}
    }


    public void matchWithVariousShifts(final OrdersFlow obuy,
				       final OrdersFlow osell,
				       final Distribution<MatchWithShift> mr)
    {
	final boolean PRINT_DEBUG=false;
	if(mr.getSize()>0)
	{
	    throw new IllegalArgumentException("Input buffer mr is not emty size="+mr.getSize());
	}

	// calculate total volume & make sure the data is ordered
	obuy.getTotalVolume();
	osell.getTotalVolume();

	final int n_buy=obuy.getSize(),n_sell=osell.getSize();
	// ensure capacity
	if(n_buy+n_sell>=mr.getCapacity())
	{
	    mr.replaceBuffer(new MatchWithShift [1+2*(n_buy+n_sell)]);
	}
	if(n_buy<=0 || n_sell<=0)
	{
	    // empty
	    return;
	}

	// oftenly used
	final double [] p_buy=obuy.price,p_sell=osell.price;
	final long [] v_buy=obuy.vol_total,v_sell=osell.vol_total;
	final double pcurrent_buy=p_buy[0],pcurrent_sell=p_sell[0];

	if(PRINT_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============BOOK VSHIFT "+
			       "\tp_buy\t"+pcurrent_buy+"\tp_sell\t"+pcurrent_sell+"\t"+
			       "n_buy\t"+n_buy+"\t"+n_sell);
	    final int n=Math.max(n_buy,n_sell);
	    System.err.println("N\tbuy\tV_BUY\t\tsell\tV_SELL\t\t");
	    for(int i=0;i<n;i++)
	    {
		System.err.print(String.valueOf(i)+"\t");
		if(i<n_buy)
		{
		    System.err.print(String.valueOf(p_buy[i])+"\t"+v_buy[i]+"\t\t");
		}
		else
		{
		    System.err.print("\t\t\t");
		}
		if(i<n_sell)
		{
		    System.err.print(String.valueOf(p_sell[i])+"\t"+v_sell[i]+"\t\t");
		}
		else
		{
		    System.err.print("\t\t\t");
		}
		System.err.println();
	    }
	    System.err.println("END");
	}

	if(PRINT_DEBUG)
	{
	    System.err.println("BUY MATCHES\t"+MatchWithShift.MT_BUY);
	    System.err.println("BUY N\tPBUY\tVBUY\t\tSELL N\tPSELL\tVSELL\t\tpobuy\tposell\tSHIFT0\tSHIFT\tdp\tvol");
	}
	// scan all buy prices
	for(int ibuy=0,isell=0;ibuy<n_buy;ibuy++)
	{
	    final long vol=v_buy[ibuy];
	    for(;isell<n_sell;isell++)
	    {
		if(v_sell[isell]>=vol)
		{
		    break;
		}
	    }
	    if(isell>=n_sell)
	    {
		// nothing found
		break;
	    }
	    final double pobuy=pcurrent_buy-p_buy[ibuy],posell=p_sell[isell]-pcurrent_sell;
	    // assymetry weighted adjustment
	    final double dp;
	    if(FORCE_DPADJUSTMENT_TO_ZERO)
	    {
		dp=0;
	    }
	    else
	    {
		if(FORCE_FIRST_DPADJUSTMENT_TO_ZERO && isell<=0)
		{
		    dp=0;
		}
		else
		{
		    // adjustment to equilibrium sell price
		    dp=((p_sell[isell]-(isell>0?p_sell[isell-1]:pcurrent_buy))*(v_sell[isell]-vol))/(v_sell[isell]-(isell>0?v_sell[isell-1]:0));
		}
	    }
	    if(PRINT_DEBUG)
	    {
		System.err.println(String.valueOf(ibuy)+"\t"+
				   p_buy[ibuy]+"\t"+
				   v_buy[ibuy]+"\t\t"+
				   String.valueOf(isell)+"\t"+
				   p_sell[isell]+"\t"+
				   v_sell[isell]+"\t\t"+
				   pobuy+"\t"+
				   posell+"\t"+
				   (pobuy+posell)+"\t"+
				   (pobuy+posell-dp)+"\t"+
				   dp+"\t"+
				   vol);
	    }
	    if(dp<0)
	    {
		throw new IllegalArgumentException();
	    }

	    final double pa=posell-dp;
	    // save point
	    mr.add(new MatchWithShift(p_buy[ibuy], // for testing
				      dp,
				      vol, // matched volume
				      pa-pobuy, // delta
				      posell-pobuy, //delta0
				      pa+pobuy, // shift
				      posell+pobuy, // shift0
				      pobuy,
				      posell,
				      vol,
				      v_sell[isell],
				      MatchWithShift.MT_BUY));
	}

	if(PRINT_DEBUG)
	{
	    System.err.println("SELL MATCHES\t"+MatchWithShift.MT_SELL);
	    System.err.println("SELL N\tPSELL\tVSELL\t\tBUY N\tPBUY\tVBUY\t\tpobuy\tposell\tSHIFT0\tSHIFT\tdp\tvol");
	}
	// scan all sell prices, the result is almost identical to buy results
	for(int isell=0,ibuy=0;isell<n_sell;isell++)
	{
	    final long vol=v_sell[isell];
	    for(;ibuy<n_buy;ibuy++)
	    {
		if(v_buy[ibuy]>=vol)
		{
		    break;
		}
	    }
	    if(ibuy>=n_buy)
	    {
		// nothing found
		break;
	    }
	    final double pobuy=pcurrent_buy-p_buy[ibuy],posell=p_sell[isell]-pcurrent_sell;
	    // assymetry weighted adjustment
	    final double dp;
	    if(FORCE_DPADJUSTMENT_TO_ZERO)
	    {
		dp=0;
	    }
	    else
	    {
		if(FORCE_FIRST_DPADJUSTMENT_TO_ZERO && ibuy<=0)
		{
		    dp=0;
		}
		else
		{
		    // adjustment to equilibrium buy price
		    dp=-((p_buy[ibuy]-(ibuy>0?p_buy[ibuy-1]:pcurrent_sell))*(v_buy[ibuy]-vol))/
		    (v_buy[ibuy]-(ibuy>0?v_buy[ibuy-1]:0));
		}
	    }
	    if(PRINT_DEBUG)
	    {
		System.err.println(String.valueOf(isell)+"\t"+
				   p_sell[isell]+"\t"+
				   v_sell[isell]+"\t\t"+
				   String.valueOf(ibuy)+"\t"+
				   p_buy[ibuy]+"\t"+
				   v_buy[ibuy]+"\t\t"+
				   pobuy+"\t"+
				   posell+"\t"+
				   (pobuy+posell)+"\t"+
				   (pobuy+posell-dp)+"\t"+
				   dp+"\t"+
				   vol);
	    }
	    if(dp<0)
	    {
		throw new IllegalArgumentException();
	    }
	    final double pa=pobuy-dp;
	    // save point
	    mr.add(new MatchWithShift(p_sell[isell], // for testing
				      dp,
				      vol, // matched volume
				      posell-pa, // delta
				      posell-pobuy, // delta0
				      posell+pa, // shift
				      posell+pobuy, // shift0
				      pobuy,
				      posell,
				      v_buy[ibuy],
				      vol,
				      MatchWithShift.MT_SELL));
	}

	com.polytechnik.utils.TMPQSortAlgorithm.objectQuickSortPointers(mr.getBuffer(),0,mr.getSize()-1,CMP_BY_SHIFT);

	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    final MatchOrdersNoShift matcher=new MatchOrdersNoShift();
	    // very expensive constrains.
	    final OrdersFlow
		tst_buy=new OrdersFlow(n_buy,true),
		tmp_buy=new OrdersFlow(n_buy,true),
		tst_sell=new OrdersFlow(n_sell,false),
		tmp_sell=new OrdersFlow(n_sell,false);
	    final double spread=pcurrent_sell-pcurrent_buy;
	    for(int i=0;i<n_buy;i++)
	    {
		tst_buy.addOrder(p_buy[i],(int)(i>0?v_buy[i]-v_buy[i-1]:v_buy[i]));
		if(false && PRINT_DEBUG)
		{
		    System.err.println("BUY\t"+p_buy[i]+"\t"+v_buy[i]);
		}
	    }
	    for(int i=0;i<n_sell;i++)
	    {
		tst_sell.addOrder(p_sell[i],(int)(i>0?v_sell[i]-v_sell[i-1]:v_sell[i]));
		if(false && PRINT_DEBUG)
		{
		    System.err.println("SELL\t"+p_sell[i]+"\t"+v_sell[i]);
		}
	    }
	    final MatchWithShift [] tb=mr.getBuffer();
	    final int size=mr.getSize();
	    for(int i=0;i<size;i++)
	    {
		final MatchWithShift t=tb[i];
		if(t.type==MatchWithShift.MT_BUY)
		{
		    tmp_sell.clear();
		    tmp_sell.addOrdersFrom(tst_sell);
		    tmp_sell.shiftPrices(-(t.shift0+spread));
		    // already ordered, no need to call mergeIdenticalPrices()
		    final MatchOrdersNoShift.MatchResults mt=matcher.match(tst_buy,tmp_sell);
		    if(mt.v_executed!=t.vmatch ||
		       Double.isNaN(mt.p_match_buy)!=Double.isNaN(t.test_pmatch) ||
		       (!Double.isNaN(mt.p_match_buy) && mt.p_match_buy!=t.test_pmatch))
		    {
			throw new RuntimeException(
			    "Results do not match with OrdersFlow:"+
			    " mt.v_executed="+mt.v_executed+
			    " t.vmatch="+t.vmatch+
			    " mt.p_match_buy="+mt.p_match_buy+
			    " t.test_pmatch="+t.test_pmatch+
			    " t.shift0="+t.shift0+
			    " spread="+spread+
			    " t.type="+t.type);
		    }
		}
		else
		{
		    tmp_buy.clear();
		    tmp_buy.addOrdersFrom(tst_buy);
		    tmp_buy.shiftPrices(t.shift0+spread);
		    // already ordered, no need to call mergeIdenticalPrices()
		    final MatchOrdersNoShift.MatchResults mt=matcher.match(tmp_buy,tst_sell);
		    if(mt.v_executed!=t.vmatch ||
		       Double.isNaN(mt.p_match_sell)!=Double.isNaN(t.test_pmatch) ||
		       (!Double.isNaN(mt.p_match_sell) && mt.p_match_sell!=t.test_pmatch))
		    {
			throw new RuntimeException(
			    "Results do not match with OrdersFlow:"+
			    " mt.v_executed="+mt.v_executed+
			    " t.vmatch="+t.vmatch+
			    " mt.p_match_sell="+mt.p_match_sell+
			    " t.test_pmatch="+t.test_pmatch+
			    " t.shift0="+t.shift0+
			    " spread="+spread+
			    " t.type="+t.type);
		    }
		}
	    }
	}
    }



    private static void checkDP(final MatchWithShift m,
				final OrdersFlow obuy,
				final OrdersFlow osell)
    {
	final boolean PRINT_DEBUG=false;
	final double s_sell,s_buy;
	if(m.type==MatchWithShift.MT_BUY)
	{
	    s_buy=0.5*(m.shift-m.delta);
	    s_sell=0.5*(m.shift+m.delta); // adjusted by dp
	}
	else if(m.type==MatchWithShift.MT_SELL)
	{
	    s_buy=0.5*(m.shift-m.delta); // adjusted by dp
	    s_sell=0.5*(m.shift+m.delta);
	}
	else
	{
	    throw new IllegalArgumentException();
	}

	/*
	if(s_buy<0||s_sell<0)
	{
	    throw new IllegalArgumentException("s_buy="+s_buy+" s_sell="+s_sell);
	    }
	*/

	final int n_buy=obuy.getSize(),n_sell=osell.getSize();
	final double pcurrent_buy=obuy.price[0],pcurrent_sell=osell.price[0];
	if(PRINT_DEBUG)
	{
	    System.err.println("PNOBUY="+m.pobuy+
			       " PNOSELL="+m.posell);
	}
	int ibuy=-1;
	for(int i=0;i<n_buy;i++)
	{
	    if(PRINT_DEBUG)
	    {
		System.err.println("BUY:PR["+i+"]="+obuy.price[i]+
				   " v="+obuy.vol_total[i]+" CUT="+(pcurrent_buy-s_buy-1e-9));
	    }
	    if(obuy.price[i]<=pcurrent_buy-s_buy-1e-9)
	    {
		break;
	    }
	    ibuy=i;
	}

	int isell=-1;
	for(int i=0;i<n_sell;i++)
	{
	    if(PRINT_DEBUG)
	    {
		System.err.println("SELL:PR["+i+"]="+osell.price[i]+
				   " v="+osell.vol_total[i]+" CUT="+(pcurrent_sell+s_sell+1e-9));
	    }
	    if(osell.price[i]>=pcurrent_sell+s_sell+1e-9)
	    {
		break;
	    }
	    isell=i;
	}


	final double dp,assym;
	// find equilinrium price
	if(ibuy>=0 && obuy.vol_total[ibuy]==m.vmatch)
	{ // vsel>vbuy at level
	    // sell price adjustment
	    if(FORCE_DPADJUSTMENT_TO_ZERO)
	    {
		dp=0;
	    }
	    else
	    {
		isell++;
		if(FORCE_FIRST_DPADJUSTMENT_TO_ZERO && isell<=0)
		{
		    dp=0;
		}
		else
		{
		    dp=((osell.price[isell]-(isell>0?osell.price[isell-1]:pcurrent_buy))*(osell.vol_total[isell]-m.vmatch))/(osell.vol_total[isell]-(isell>0?osell.vol_total[isell-1]:0));
		}
		if(PRINT_DEBUG)
		{
		    System.err.println(" dp="+dp+" ibuy="+ibuy+" isell="+isell);
		}
	    }

	    assym=-dp;
	}
	else if(isell>=0 && osell.vol_total[isell]==m.vmatch)
	{ // vbuy>vsell at level
	    // buy price adjustment
	    if(FORCE_DPADJUSTMENT_TO_ZERO)
	    {
		dp=0;
	    }
	    else
	    {
		ibuy++;
		if(FORCE_FIRST_DPADJUSTMENT_TO_ZERO && ibuy<=0)
		{
		    dp=0;
		}
		else
		{
		    dp=-((obuy.price[ibuy]-(ibuy>0?obuy.price[ibuy-1]:pcurrent_sell))*(obuy.vol_total[ibuy]-m.vmatch))/(obuy.vol_total[ibuy]-(ibuy>0?obuy.vol_total[ibuy-1]:0));
		}
		if(PRINT_DEBUG)
		{
		    System.err.println(" dp="+dp+" ibuy="+ibuy+" isell="+isell);
		}
	    }
	    assym=dp;
	}
	else
	{
	    throw new RuntimeException(" ibuy="+ibuy+" isell="+isell+" m.vmatch="+m.vmatch);
	}
	final double shift=pcurrent_buy-obuy.price[ibuy]+osell.price[isell]-pcurrent_sell-dp;
	final double delta=(osell.price[isell]-pcurrent_sell)-(pcurrent_buy-obuy.price[ibuy])+assym;

	if(Math.abs(shift-m.shift)>1e-9||Math.abs(delta-m.delta)>1e-9)
	{
	    throw new RuntimeException("Mismatch shift="+shift+
				       " m.shift="+m.shift+
				       " delta="+delta+
				       " m.delta="+m.delta+
				       " ibuy="+ibuy+
				       " isell="+isell+
				       " v_buy="+obuy.vol_total[ibuy]+
				       " v_sell="+osell.vol_total[isell]+
				       " m.vmatch="+m.vmatch+
				       " m.test_dp="+m.test_dp+
				       " dp="+dp+
				       " assym="+assym+
				       " s_buy="+s_buy+
				       " s_sell="+s_sell+
				       " type="+m.type);
	}

    }


    // unit test. option ModelParameters.CONSTRAINS_EXPENSIVE_CHECKS should be on
    public static void main(String [] args)
    {
	if(!OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    throw new IllegalArgumentException("the ModelParameters.CONSTRAINS_EXPENSIVE_CHECKS should be set to true");
	}
	final java.util.Random r=new java.util.Random(-991L);

  final OrdersFlow obuy=new OrdersFlow(true),osell=new OrdersFlow(false);
  final Distribution<MatchWithShift> mr=new Distribution<MatchWithShift>();
  final Distribution<BookItem>
      buy=new Distribution<BookItem>(),
      sell=new Distribution<BookItem>();
  for(int itest=0;itest<10000;itest++)
  {
        final long t0=System.currentTimeMillis();

        mr.clearAll(); // clear buffer to be re-used

	final int n_buy=100+r.nextInt(1910),n_sell=131+r.nextInt(1820);
	buy.clearAll();
	if(n_buy>=buy.getCapacity())
	{
	    buy.replaceBuffer(new BookItem [1+2*n_buy]);
	}
	sell.clearAll();
	if(n_sell>=sell.getCapacity())
	{
	    sell.replaceBuffer(new BookItem [1+2*n_sell]);
	}

	final MatchOrdersVariousShifts mo=new MatchOrdersVariousShifts();

	for(int i=0;i<n_buy;i++)
	{
	    buy.add(new BookItem(1-r.nextInt(100)/100.0,
				 100+r.nextInt(5000),
				 null,
				 BookItem.BOOK_BUY,
				 r.nextLong(),
				 -1,
				 null));
	}
	com.polytechnik.utils.TMPQSortAlgorithm.objectQuickSortPointers(buy.getBuffer(),0,n_buy-1,Distribution.cmp_by_price_lowest_down);


	for(int i=0;i<n_sell;i++)
	{
	    sell.add(new BookItem(1.01+r.nextInt(100)/100.0,
				  100+r.nextInt(1510),
				  null,
				  BookItem.BOOK_SELL,
				  r.nextLong(),
				  -1,
				  null));
	}
	com.polytechnik.utils.TMPQSortAlgorithm.objectQuickSortPointers(sell.getBuffer(),0,n_sell-1,Distribution.cmp_by_price_lowest_up);


	obuy.clear();
	osell.clear();
	mo.copyOrderedOrdersToFlow(buy,obuy);
	mo.copyOrderedOrdersToFlow(sell,osell);
	mo.matchWithVariousShifts(obuy,osell,mr);
	final MatchWithShift [] tb=mr.getBuffer();
	System.err.println("TEST PMATCH\tvmach\tdelta\tshift\ttype");
	for(int i=0,n=mr.getSize();i<n;i++)
	{
	    final  MatchWithShift t=tb[i];
	    System.err.println(String.valueOf(t.test_pmatch)+"\t"+
			       t.vmatch+"\t"+
			       t.delta+"\t"+
			       t.shift+"\t"+
			       t.type);
	    checkDP(t,obuy,osell);
	}
	final long t1=System.currentTimeMillis();
	System.err.println("Test "+(1+itest)+" OK returned matches="+mr.getSize()+
			   " n_buy="+n_buy+" n_sell="+n_sell+
			   " "+((t1-t0)/1000L)+" seconds for this test");

  }
    }
}
