package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** Orders and trades in one place.
 *  @version $Id: OrdersAndTrades.java,v 1.168 2016/03/23 23:49:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrdersAndTrades {

    /** Whether to turn on expensive constrains.
     */
    public static final boolean CONSTRAINS_EXPENSIVE_CHECKS=false;//true;

    public static final boolean USE_ORIGTIME=false;
    public static final boolean SAVE_NONDISPLAYABLE_TRADES=true;
    public static final boolean SAVE_EDGE_ORDERS=true;
    public static final boolean FLAG_PSEUDOTRADES=false;

    public final String ticker;
    public final RealtimeBook orderbook;
    public final TradeList tradebook;
    public final TradeList pseudotrades;

    OrdersAndTrades(final String ticker,
		    final com.polytechnik.utils.TObjectProcedure<Trade> tradelistcallback)
    {
	this.ticker=ticker;
	this.orderbook=new RealtimeBook(this);
	this.tradebook=new TradeList(305,tradelistcallback,this);
	this.pseudotrades=new TradeList(305,null,this);
    }

    public void clearBefore(final long t0)
    {
	this.tradebook.clearBefore(t0);
	this.pseudotrades.clearBefore(t0);
    }


    private boolean save_trades=true;
    public void setActive(final boolean save_trades)
    {
	this.save_trades=save_trades;
    }
    public boolean getActive()
    {
	return this.save_trades;
    }

    // for now with a warning
    private Object data=null;
    public <SD> void setData(SD data)
    {
	this.data=data;
    }
    public <SD> SD getData()
    {
	@SuppressWarnings("unchecked")
	final SD s=(SD)this.data;
	return s;
    }







/** Unit test to test no order left.
 * top run
 * java com/polytechnik/realtimebook/OrdersAndTrades\$TestBook__Add_Cancel_Execute /u3/itch/itch41/S092112-v41.txt.gz  >/tmp/log_book_err5.txt 2>&1&
 * Some days may fail where instruments with negative spread or other abnormalities exist.
 * For example CBF at  /u3/itch/itch41/S092012-v41.txt.gz
 */
static class TestBook__Add_Cancel_Execute extends com.polytechnik.itch.DummyTrader {
    static final long TAU_CLEAR=100*TraderInterface.TIME_SCALE;

static class IData {
    final String ticker;
    final OrdersAndTrades ot;
    long n_tests=0;
    IData(final String ticker,final OrdersAndTrades ot)
    {
	this.ticker=ticker;
	this.ot=ot;
    }
    class CountVol implements com.polytechnik.utils.TObjectProcedure<Trade>{
	long v_buy_abs=0,v_sell_abs=0;
	Trade processes=null,lasttrade=null;
	public boolean execute(final Trade b)
	{
	    if(processes==null) processes=b;
	    if(b==lasttrade){return false;}
	    if(b.buy_sell_indicator==BookItem.BOOK_BUY) v_buy_abs+=b.shares;
	    else if(b.buy_sell_indicator==BookItem.BOOK_SELL) v_sell_abs+=b.shares;
	    return true;
	}
	void resetLast()
	{
	    lasttrade=processes;
	    processes=null;
	}
    }

    final CountVol cvol=new CountVol(),cvolpseudo=new CountVol();
    void checkAndSetNextCheck(final long currenttime,final int eventid)
    {
	ot.tradebook.backwardForEachEntry(cvol,0);
	cvol.resetLast();

	ot.pseudotrades.backwardForEachEntry(cvolpseudo,0);
	cvolpseudo.resetLast();

	checkLevel0(currenttime,eventid);
	ot.clearBefore(currenttime-TAU_CLEAR);

	//System.err.println(ot.ticker+" test OK");
	n_tests++;
    }


    private void checkLevel0(final long currenttime,final int eventid)
    {
	if(eventid==UPD_EXECUTIONORDER_TRADE||eventid==UPD_EXECUTIONOTH_TRADE)
	{
	    final Trade t=ot.tradebook.getLastAddedElement();
	    if(t.buy_sell_indicator!=BookItem.BOOK_UNKNOWN)
	    {
		final BookItem level0=ot.orderbook.getBestOrder(t.buy_sell_indicator);
		if(level0!=null)
		if((t.buy_sell_indicator==BookItem.BOOK_BUY && t.price<level0.price)||
		   (t.buy_sell_indicator==BookItem.BOOK_SELL && t.price>level0.price))
		{
		    printDiag("WARNING: possible trade price mistmath to book",currenttime,t,level0,eventid);
		}
		final BookItem blevel0=ot.orderbook.getBestLevel(currenttime,t.buy_sell_indicator);
		if(level0!=null && (blevel0==null || level0.shares>blevel0.shares || blevel0.shares!=ot.orderbook.getVolBetterPrice(level0.price,t.buy_sell_indicator)||blevel0.price!=level0.price))
		    throw new RuntimeException("v shares mismatch:blevel0.shares="+(blevel0!=null?"price="+blevel0.price+" shares="+blevel0.shares:"??"));
	    }
	    if(t.order!=null && t.order.order!=null)
	    {
		final BookItem bexe=t.order,bo=bexe.order;
		if(bexe.buy_sell_indicator==bo.buy_sell_indicator) throw new RuntimeException("bexe.buy_sell_indicator="+bexe.buy_sell_indicator+" bo.buy_sell_indicator="+bo.buy_sell_indicator);
		if((bexe.buy_sell_indicator==BookItem.BOOK_BUY && !(bexe.price<=bo.price))||
		   (bexe.buy_sell_indicator==BookItem.BOOK_SELL && !(bexe.price>=bo.price)))
		{
		    printDiag("WARNING: possible orig order mistmath to book",currenttime,t,null,eventid);
		}
	    }
	}
	else if(eventid==UPD_ADD_ORDER)
	{

	}
	else if(eventid==UPD_CANCELREPLACE_ORDER)
	{

	}
    }

    void checkPseudotrades(final OrdersAndTrades ot,final long currenttime,final int eventid)
    {
	if(!FLAG_PSEUDOTRADES) return;
	final BookItem b=ot.orderbook.getBestOrder(BookItem.BOOK_BUY),
	    s=ot.orderbook.getBestOrder(BookItem.BOOK_SELL);
	final Trade t=ot.pseudotrades.getLastAddedElement();
	if(t!=null && t.best_buy!=null && t.best_sell!=null && !(t.best_sell.price>=t.best_buy.price))
	{
	    printDiag("NEGATIVE SPREAD",currenttime,t,null,eventid);
	    return;
	}
	//System.err.println("ticker="+ot.ticker+" eventid="+eventid+" t="+t);dumpPseudoTrades(ot);
	if(t!=null && b!=null && !(b.price==t.best_buy.price))
	    throw new RuntimeException("b.price="+b.price+" t.best_buy.price="+t.best_buy.price+" eventid="+eventid);
	if(t!=null && s!=null && !(s.price==t.best_sell.price))
	    throw new RuntimeException("s.price="+s.price+" t.best_sell.price="+t.best_sell.price+" eventid="+eventid);
	final Trade tl=ot.tradebook.getLastAddedElement();
	if(t!=null && (eventid==UPD_EXECUTIONORDER_TRADE||eventid==UPD_EXECUTIONOTH_TRADE))
	{
	    if(tl!=t)
	    {
		printDiag("Buy sell levels",currenttime,tl,null,eventid);
		throw new RuntimeException("Pseudotrade!=trade:\nt="+Trade.getTradeDescription(t)+"\ntl="+Trade.getTradeDescription(tl));
	    }
	}
	if(cvol.v_buy_abs!=cvolpseudo.v_buy_abs||cvol.v_sell_abs!=cvolpseudo.v_sell_abs)
	{
	    printDiag("Buy sell levels",currenttime,tl,null,eventid);
	    throw new RuntimeException("cvol.v_buy_abs="+cvol.v_buy_abs+" cvolpseudo.v_buy_abs="+cvolpseudo.v_buy_abs+
				       "\ncvol.v_sell_abs="+cvol.v_sell_abs+" cvolpseudo.v_sell_abs="+cvolpseudo.v_sell_abs);
	}
    }

    void checkPseudotradesDirectly(final OrdersAndTrades ot,final long currenttime,final int eventid)
    {
	//if(ot.ticker.equals("CBF")) dumpBook(ot);
	if(!FLAG_PSEUDOTRADES) return;
	final Trade t=ot.pseudotrades.getLastAddedElement(),
	    tl=ot.tradebook.getLastAddedElement();
	if(tl==null) return;
	if(t.best_buy!=null && t.best_sell!=null && !(t.best_sell.price>=t.best_buy.price))
	{
	    printDiag(" checkPseudotradesDirectly:NEGATIVE SPREAD",currenttime,t,null,eventid);
	    return;
	}
	if(!(tl.price==t.price)||!(currenttime==t.time))
	{
	    printDiag("Buy sell levels",currenttime,tl,null,eventid);
	    throw new RuntimeException("currenttime="+currenttime+
				       "\nt="+Trade.getTradeDescription(t)+
				       "\ntl="+Trade.getTradeDescription(tl));
	}
	final BookItem obest_buy=ot.orderbook.getBestOrder(BookItem.BOOK_BUY),
	    obest_sell=ot.orderbook.getBestOrder(BookItem.BOOK_SELL);
	final int vbuy=obest_buy==null?0:ot.orderbook.getVolBetterPrice(obest_buy.price,BookItem.BOOK_BUY),
	    vsell=obest_sell==null?0:ot.orderbook.getVolBetterPrice(obest_sell.price,BookItem.BOOK_SELL);
	//printDiag("Buy sell levels",currenttime,t,null,eventid);
	//System.err.println("obest_buy="+obest_buy+" t.best_buy="+t.best_buy+" tl="+tl);
	if(obest_buy!=null && (!(vbuy==t.best_buy.shares)))
	{
	    printDiag("Buy sell levels",currenttime,t,null,eventid);
	    throw new RuntimeException("vbuy="+vbuy+"\nt="+Trade.getTradeDescription(t)+
				       "\nobest_buy="+BookItem.getOrderDescription(obest_buy)+
				       "\nt.best_buy="+BookItem.getOrderDescription(t.best_buy));
	}
	//printDiag("Buy sell levels",currenttime,t,null,eventid);
	//System.err.println("obest_sell="+obest_sell+" t.best_sell="+t.best_sell+" tl="+tl);
	if(obest_sell!=null && (!(vsell==t.best_sell.shares)))
	{
	    printDiag("Buy sell levels",currenttime,t,null,eventid);
	    throw new RuntimeException("vsell="+vsell+"\nt="+Trade.getTradeDescription(t)+
				       "\nobest_sell="+BookItem.getOrderDescription(obest_sell)+
				       "\nt.best_sell="+BookItem.getOrderDescription(t.best_sell));
	}
    }

    private void printDiag(final String txt,final long currenttime,final Trade t,final BookItem level0,final int eventid)
    {
	System.err.println(ot.ticker+":"+eventid+"=============== "+txt+" ========================");
	dumpBook(ot);
	dumpTrades(ot);
	dumpPseudoTrades(ot);
	System.err.println(ot.ticker+":Mismatch in p: eventid="+eventid);
	System.err.println("TRADE="+Trade.getTradeDescription(t));
	System.err.println("LEVEL0="+BookItem.getOrderDescription(level0));
	System.err.println("=======================================");
	//throw new RuntimeException();
    }
} // end of class IData


	public void setInstrument(final OrdersAndTrades ot,
				  final long currenttime,
				  final boolean flag_statusOK,
				  final int round_lot_shares,
				  final boolean round_lot_only,
				  final String market)
	{
	    ot.setData(new IData(ot.ticker,ot));
	}


	static class DumpT<TRD extends Trade> implements com.polytechnik.utils.TObjectProcedure<TRD>
	{
	    public boolean execute(final TRD b)
	    {
		System.err.println(b.price+"\t"+b.shares+"\t"+b.time+"\t"+BookItem.getTimeHHMMSS(b.time)+"\t"+b.buy_sell_indicator+"\t"+
				   (b.order!=null?"ORD:"+b.order.price+";"+b.order.shares:"ORD N/A"));
		return true;
	    }
	}

	static void dumpBook(final OrdersAndTrades ot)
	{
	    System.err.println(ot.ticker+": BOOK");
	    for(final java.util.Iterator<BookItem> i=ot.orderbook.getBookIterator();i.hasNext();)
	    {
		final BookItem b=i.next();
		System.err.println(b.price+"\t"+b.shares+"\t"+b.time+"\t"+BookItem.getTimeHHMMSS(b.time)+"\t"+b.buy_sell_indicator+"\t"+b.orderreferencenumber);
	    }
	}
	static void dumpTrades(final OrdersAndTrades ot)
	{
	    System.err.println(ot.ticker+": TRADES");
	    ot.tradebook.backwardForEachEntry(new DumpT<Trade>(),0);
	}
	static void dumpPseudoTrades(final OrdersAndTrades ot)
	{
	    System.err.println(ot.ticker+": PSEUDOTRADES");
	    ot.pseudotrades.backwardForEachEntry(new DumpT<Trade>(),0);
	}


	public void update(final OrdersAndTrades ot,
			   final long currenttime,
			   final int eventid)
       {
	   final IData idd=ot.getData();
	   idd.checkAndSetNextCheck(currenttime,eventid);
	   idd.checkPseudotrades(ot,currenttime,eventid);
	   idd.checkPseudotradesDirectly(ot,currenttime,eventid);
       }


	public static void main(String [] args)
	    throws Exception
	{
	    if(args.length<=0)
	    {
		System.err.println("OrdersAndTrades.TestBook__Add_Cancel_Execute itch_file1.gz itch_file1.gz ...");
		System.exit(1);
	    }
	    for(String a:args)
	    {
		final TestBook__Add_Cancel_Execute p=new TestBook__Add_Cancel_Execute();
		final com.polytechnik.itch.ITCHParser parser=p.runParser(a);
		parser.overAll(new com.polytechnik.utils.TObjectProcedure<OrdersAndTrades>(){
			public boolean execute(final OrdersAndTrades ot)
			{
			    final IData id=ot.getData();
			    System.err.println(id.ot.ticker+"\ttests OK="+id.n_tests+
					       "\tv_buy_abs="+id.cvol.v_buy_abs+
					       "\tv_sell_abs="+id.cvol.v_sell_abs);
			    return true;
			}
		    });
	    }
	}
    } // END OF TestBook__Add_Cancel_Execute



}
