package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A real time book.
 *  @version $Id: RealtimeBook.java,v 1.71 2016/03/23 22:42:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class RealtimeBook extends com.polytechnik.utils.LongTreeValueMapTest<BookItem>
    implements BookCopyable<BookItem> {
    public final OrdersAndTrades ref;


    RealtimeBook(final OrdersAndTrades ref)
    {
	// we set load factor not too low, because we regulary
	// iterate through all elements.
	super(Distribution.cmp_orders_by_price_and_type);
	this.ref=ref;
    }

    public int copyData(final BookItem [] items)
    {
	return copyValues(items);
    }

    public int getSize()
    {
	return size();
    }

    java.util.Iterator<BookItem> getBookIterator()
    {
	return getInternalValuesTable().keySet().iterator();
    }

    public BookItem getBestOrder(final byte buy_sell_indicator)
    {
	final BookItem b;
	if(buy_sell_indicator==BookItem.BOOK_BUY)
	{
	    b=firstValue();
	}
	else if(buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    b=lastValue();
	}
	else
	{
	    throw new IllegalArgumentException("Wrong buy_sell_indicator="+buy_sell_indicator);
	}
	if(b!=null && b.buy_sell_indicator==buy_sell_indicator)
	{
	    return b;
	}
	return null;
    }

    public int getVolBetterPrice(final double p0,final byte buy_sell_indicator)
    {
	final java.util.Set<java.util.Map.Entry<BookItem,Object>> i;
	if(buy_sell_indicator==BookItem.BOOK_BUY)
	{
	    i=getInternalValuesTable().entrySet();
	}
	else if(buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    i=getInternalValuesTable().descendingMap().entrySet();
	}
	else
	{
	    throw new IllegalArgumentException("Wrong buy_sell_indicator="+buy_sell_indicator);
	}

	int shares=0;
	for(java.util.Map.Entry<BookItem,Object> entry: i)
	{
	    final BookItem b=entry.getKey();
	    if(b.buy_sell_indicator!=buy_sell_indicator)
	    {
		break;
	    }
	    if(buy_sell_indicator==BookItem.BOOK_BUY?b.price>=p0:b.price<=p0)
	    {
		shares+=b.shares;
	    }
	    else
	    {
		break;
	    }
	}
	return shares;
    }


    public BookItem getBestLevel(final long currenttime,final byte buy_sell_indicator)
    {
	final java.util.Set<java.util.Map.Entry<BookItem,Object>> i;
	if(buy_sell_indicator==BookItem.BOOK_BUY)
	{
	    i=getInternalValuesTable().entrySet();
	}
	else if(buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    i=getInternalValuesTable().descendingMap().entrySet();
	}
	else
	{
	    throw new IllegalArgumentException("Wrong buy_sell_indicator="+buy_sell_indicator);
	}

	int shares=0,no=0;
	double p0=Double.NaN;
	BookItem bl=null;
	for(java.util.Map.Entry<BookItem,Object> entry: i)
	{
	    final BookItem b=entry.getKey();
	    if(b.buy_sell_indicator!=buy_sell_indicator){break;}
	    bl=b;
	    if((no++)<=0) {p0=b.price;}
	    else if(!(buy_sell_indicator==BookItem.BOOK_BUY?b.price>=p0:b.price<=p0)) {break;}
	    shares+=b.shares;
	}
	return no<=0?null:new BookItem(p0,shares,null,buy_sell_indicator,currenttime,0,bl);
    }


public int copyOrdersSeparateBuySell(final Distribution<BookItem> buy,
				     final Distribution<BookItem> sell)
{
    // copy data to buy buffer
    final int nor=getSize();
    if(nor>=buy.getCapacity())
    {
	buy.replaceBuffer(new BookItem [2*nor+10]);
    }
    // save data from book to target array
    final int bs=copyData(buy.getBuffer());
    if(bs!=getSize())
    {
	throw new java.util.ConcurrentModificationException();
    }
    buy.setSize(bs);
    // split buy & sell
    copyOrders1SplitByType(buy,sell);
    return bs;
}

/** A kludgy way to copy the data.
 *  But this speeds the things up.
 */
private static void copyOrders1SplitByType(final Distribution<BookItem> buy,
					   final Distribution<BookItem> sell)
{
    if(sell.getCapacity()<buy.getCapacity())
    {
	sell.replaceBuffer(new BookItem [buy.getCapacity()]);
    }

    // sort orders by price and order type
    // a sorting without creation of array copy.
    final int no=buy.getSize();
    final BookItem [] buf=buy.getBuffer(),bufs=sell.getBuffer();
    // the data is already sorted in LongTreeValueMapTest
    //com.polytechnik.utils.TMPQSortAlgorithm.objectQuickSortPointers(buf,0,no-1,Distribution.cmp_orders_by_price_and_type); // earliest at 0

    // a slightly kludgy way to copy the data.
    // hopefully this will speed the things up.
    int ns=0;
    for(int i=no;--i>=0;)
    {
	final BookItem b=buf[i];
	if(b.buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    bufs[ns++]=b;
	    buf[i]=null;
	}
	else
	{
	    if(b.buy_sell_indicator!=BookItem.BOOK_BUY)
	    {
		throw new RuntimeException("Unknown buy_sell_indicator="+
					   b.buy_sell_indicator+
					   " time="+b.time);
	    }
	    break;
	}
    }
    buy.setSize(no-ns);
    sell.setSize(ns);
}

}
