package com.polytechnik.realtimebook;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** Book builder.
 *  @version $Id: TradesBuilder.java,v 1.54 2012/09/26 22:20:44 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TradesBuilder {

    private static final boolean PRINT_DIAG=true;

    private final BookBuilder bb;
    public TradesBuilder(BookBuilder bb)
    {
	this.bb=bb;
    }



    /** A map of matchnumber ID -> TradeList.
     */
    private final com.polytechnik.utils.LongHashMap<TradeList> globaltrademapping=
	new com.polytechnik.utils.LongHashMap<TradeList>(BookBuilder.LARGETABLE,BookBuilder.LOADFACTOR);
    private final com.polytechnik.utils.TObjectProcedure<Trade> callback=
	new com.polytechnik.utils.TObjectProcedure<Trade>(){
	public boolean execute(final Trade b)
	{
	    globaltrademapping.remove(b.matchnumber);
	    return true;
	}
    };
    /** A map of ticker -> TradeList.
     */
    private final java.util.HashMap<String,OrdersAndTrades> byticker=
	new java.util.HashMap<String,OrdersAndTrades>(BookBuilder.SMALLTABLE,BookBuilder.LOADFACTOR);

    public OrdersAndTrades initTicker(final String ticker)
    {
	// make sure we have ticker book
	OrdersAndTrades ot=this.byticker.get(ticker);
	if(ot==null)
	{ // create new book
	    ot=new OrdersAndTrades(ticker,this.callback);
	    this.byticker.put(ticker,ot);
	}
	return ot;
    }


    public TradeList idLookupTrades(final long matchnumber)
    {
	return this.globaltrademapping.get(matchnumber);
    }


    public TradeList addNewTrade(final TradeList tickerbook,
				 final Trade trade)
    {
	if(!tickerbook.ref.getActive())
	{
	    return null;
	}

	final TradeList test_book=this.globaltrademapping.put(trade.matchnumber,tickerbook);
	if(test_book!=null)
	{
	    if(PRINT_DIAG)
	    {
		System.err.println("The matchnumber="+trade.matchnumber+" ticker="+tickerbook.ref.ticker+
				   " is already in the globaltrademapping for ticker2="+
				   test_book.ref.ticker+" cannot add a new. Replacing by posted one");
	    }
	    // typically means wrong or duplicated records posted.
	    // use the last one
	    tickerbook.removeBrokenElement(new com.polytechnik.utils.TObjectProcedure<Trade>(){
		public boolean execute(final Trade b)
		{
		    return b.matchnumber!=trade.matchnumber;
		}
	    });
	}

	// add an item to the book
	tickerbook.add(trade);
	return tickerbook;
    }


    public TradeList removeTrade(final long matchnumber)
    {
	final TradeList tickerbook=this.globaltrademapping.remove(matchnumber);
	if(tickerbook==null)
	{
	    return null; // no such trade found
	}

	tickerbook.removeBrokenElement(new com.polytechnik.utils.TObjectProcedure<Trade>(){
		public boolean execute(final Trade b)
		{
		    return b.matchnumber!=matchnumber;
		}
	    });
	return tickerbook;
    }


    public final String getStats()
    {
	final StringBuilder buf=new StringBuilder();
	buf.append("STATS:");
	buf.append(this.getClass().getName());
	buf.append(" ver $Id: TradesBuilder.java,v 1.54 2012/09/26 22:20:44 mal Exp $");
	buf.append('\n');
	buf.append("MAPPINGS: matchid to ticker:");
	final int n_orders=this.bb.getNOrders();
	final int n_trades=this.globaltrademapping.size();
	buf.append(n_trades);
	buf.append('\n');

	int n_orders2=0;
	int n_trades2=0;
	for(OrdersAndTrades ot:this.byticker.values())
	{
	    final int no=ot.orderbook.getSize(),nt=ot.tradebook.getSize();
	    buf.append(ot.ticker);
	    buf.append(":orders:");
	    buf.append(no);
	    buf.append(":trades:");
	    buf.append(nt);
	    buf.append('\n');
	    n_orders2+=no;
	    n_trades2+=nt;
	}
	buf.append("MAPPINGS: Book. Order counter: N1,N2:");
	buf.append(n_orders);
	buf.append(',');
	buf.append(n_orders2);
	buf.append('\n');
	if(n_orders!=n_orders2)
	{
	    throw new RuntimeException("TOTAL ORDERS NUMBER MISMATCH: "+
				       n_orders+" and "+n_orders2);
	}
	buf.append("MAPPINGS: Trades. Trade counter: N1,N2:");
	buf.append(n_trades);
	buf.append(',');
	buf.append(n_trades2);
	buf.append('\n');
	if(n_trades!=n_trades2)
	{
	    throw new RuntimeException("TOTAL TRADES NUMBER MISMATCH: "+
				       n_trades+" and "+n_trades2);
	}
	buf.append('\n');
	return buf.toString();
    }


    public void overAll(final com.polytechnik.utils.TObjectProcedure<OrdersAndTrades> processor)
    {
	for(OrdersAndTrades e:this.byticker.values())
	{
	    processor.execute(e);
	}
    }


}
