package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */

/** A crude and simple bissection style root finder algorithm.
 *  Used for simple(non-degenerated) real roots finding only.
 *  Can work in an orthogonal polynomial basis.
 *  @version $Id: BissectionRootFinder.java,v 1.5 2015/07/06 06:47:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

public abstract class BissectionRootFinder {

public double doBissection(final double Xl,final double Xr)
{
    final int NMAX=3000;
    double xl=Xl,xr=Xr;
    double fl=getF(xl),fr=getF(xr);
    //System.err.println("xl="+xl+" fl="+fl);
    //System.err.println("xr="+xr+" fr="+fr);
    if(fl>0)
    {
	double t=xl;xl=xr;xr=t;
	t=fl;fl=fr;fr=t;
    }
    if(!(fr>=0)) return Double.NaN;
    for(int k=0;k<NMAX;k++)
    {
	final double x=0.5*(xl+xr),f=getF(x);
	if(f<0){fl=f;xl=x;}
	else {fr=f;xr=x;}
	//System.err.println("k="+k+" xl="+xl+" xr="+xr+" 0.5*(xl+xr)="+(0.5*(xl+xr))+" Math.abs(xl-xr)="+Math.abs(xl-xr)+" Math.ulp(x)="+(0.5*Math.ulp(x)));
	if(!(Math.abs(xl-xr)>Math.ulp(x))) {
	    //System.err.println("doBissection: k="+k+" xl="+xl+" xr="+xr+" fl="+fl+" fr="+fr+" dx="+Math.abs(xl-xr)+" ulp(x)="+Math.ulp(x));
	    return Math.abs(fl)<Math.abs(fr)?xl:xr;
	}
    }
    return Double.NaN;
}

abstract public double getF(final double x);

}
