package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Eigenvectors problem.
 *  @version $Id: EVSolver.java,v 1.6 2015/06/13 23:03:56 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EVSolver {
    private static final boolean USE_dsyevr=false;

/** Solve eigenvalues problem.
 */
public double [] getEV(final int n,final double [] A,final int lda_A,final boolean flag_ev)
{
    return getEV_Common(n,A,lda_A,flag_ev);
}
/** Solve thridiagonal eigenvalues problem. */
public double [] getEV_Thridiagonal(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev)
{
    return getEV_3DCommon(n,diag,diag_off,offdiag,offdiag_off,flag_ev);
}
/** Solve generalized eigenvalues problem.
 */
public double [] getGEV_Generalized(final int n,final double [] A,final int lda_A,final double [] B,final int lda_B,final boolean flag_ev)
{
    return getGEV_Common(n,A,lda_A,B,lda_B,flag_ev);
}

/** Eigenvalues problem. Calls Lapack Library.
 */
static double [] getEV_Common(final int n,final double [] QQ,final int lda_QQ,final boolean flag_ev)
{
    if(USE_dsyevr)
    {
	return com.polytechnik.lapack.Eigenvalues.getEV_dsyevr(n,QQ,lda_QQ,flag_ev);
    }
    else
    {
	return com.polytechnik.lapack.Eigenvalues.getEV_dsyevx(n,QQ,lda_QQ,flag_ev);
    }
}
/** Thridiagonal eigenvalues problem. Calls Lapack Library.
 */
static double [] getEV_3DCommon(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev)
{
    if(!flag_ev)
    {
	return com.polytechnik.lapack.Eigenvalues.getEV_dstebz(n,diag,diag_off,offdiag,offdiag_off,flag_ev);
    }
    return com.polytechnik.lapack.Eigenvalues.getEV_dstemr(n,diag,diag_off,offdiag,offdiag_off,flag_ev);
}
/** Generalized eigenvalues problem. Calls Lapack Library.
 */
static double [] getGEV_Common(final int n,final double [] A,final int lda_A,final double [] B,final int lda_B,final boolean flag_ev)
{
    if(USE_dsyevr)
    {
	return com.polytechnik.lapack.Eigenvalues.getGEV_dsygvx_dsyevr(n,A,lda_A,B,lda_B,flag_ev);
    }
    else
    {
	return com.polytechnik.lapack.Eigenvalues.getGEV_dsygvx(n,A,lda_A,B,lda_B,flag_ev);
    }
    // the dsygvx may still be broken. June, 13, 2015

    // Our conversion is still some kind broken. Use original jlapack call below.
    //return com.polytechnik.algorithms.EigenvaluesSymmetric.getGEV(n,A,n,B,n,true);
    // something is fixed in Dec, 19, 2012. but still not sure.
}

}
