package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
/** A class to ge one of many objects.
 *  @version $Id: GenericPool.java,v 1.9 2008/09/30 15:18:25 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

class GenericPool {

  private static final boolean PRINT_DEBUG=false;
  private int nfreeslots;
  private final int [] stack,positioninstack;
  private final boolean flag_wait_for_available;

    GenericPool(final int psize,final boolean flag_wait_for_available)
  {
      this.nfreeslots=psize;
      this.stack=new int [psize];
      this.positioninstack=new int [psize];
      for(int i=psize;i-- >0;)
      {
	  this.positioninstack[i]=this.stack[i]=i;
      }
      this.flag_wait_for_available=flag_wait_for_available;
  }


  /** Returns an available element. */
  public synchronized int getAvailableElement()
    {
      for(;;)
	{
	  // the low this.nfreeslots elements in stack are available
	  if(this.nfreeslots>0)
	  {
	      // take an element from stack on this.nfreeslots-1 position
	      final int element=this.stack[--this.nfreeslots];
	      return element;
	  }
	  // all slots are in use.
	  // sleeping until we get one
	  String d=null;
	  if(PRINT_DEBUG) {
	      d=new java.util.Date().toString();
	      System.err.println("GenericPool:: ["+d+
				 "] all slots are in use. sleeping until one will be avaiable."+
				 " total available="+this.nfreeslots+
				 " of "+this.positioninstack.length+" Thread="+
				 Thread.currentThread().getName());
	  }
	  if(this.flag_wait_for_available)
	  {
	      try{
		  wait();
	      }catch(InterruptedException iexc){
		  System.err.println("GenericPool:: ["+
				     new java.util.Date().toString()+
				     "]:: Interrupted."+
				     " total available="+this.nfreeslots+
				     " of "+this.positioninstack.length+" Thread="+
				     Thread.currentThread().getName());
	      }
	  }
	  else
	  {
	      // do not wait. return -1 as "not available"
	      return -1;
	  }
	  if(PRINT_DEBUG) {
	      System.err.println("GenericPool:: ["+new java.util.Date().toString()+
				 "] slots available, starting using it. Was requested at ["+d+"]."+
				 " total available="+this.nfreeslots+
				 " of "+this.positioninstack.length+" Thread="+
				 Thread.currentThread().getName());
	  }
	}
    }


  /** This method releases an element. */
  public synchronized void releaseElement(final int element)
  {
      if(PRINT_DEBUG)
      {
	  System.err.println("GenericPool:: ["+
			     new java.util.Date().toString()+
			     "] Slot: Element "+element+
			     " released."+
			     " total now available="+(this.nfreeslots+1)+
			     " of "+this.positioninstack.length+" Thread="+
			     Thread.currentThread().getName());
      }

      // move the element to the list of available
      final int position=this.positioninstack[element],
	  in1=this.stack[position],
	  in2=this.stack[this.nfreeslots];
      // now permutate position and this.nfreeslots elements in this.stack
      // and in1 and in2 elements in this.positioninstack
      final int x=this.stack[position];
      this.stack[position]=this.stack[this.nfreeslots];
      this.stack[this.nfreeslots]=x;
      final int y=this.positioninstack[in1];
      this.positioninstack[in1]=this.positioninstack[in2];
      this.positioninstack[in2]=y;
      this.nfreeslots++;
      if(PRINT_DEBUG)
      {
	  System.err.println("GenericPool:: ["+ new java.util.Date().toString()+
			     "] stack and pointers to stack nfreeslots="+
			     this.nfreeslots);
	  for(int j=0;j<this.positioninstack.length;j++)
	  {
	      System.err.println("EL"+j+"\t"+ this.stack[j]+"\t"+this.positioninstack[j]+"\t"+this.positioninstack[this.stack[j]]);
	  }
      }
      notify(); // notify those who wait for a slot
  }

}
