package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */

/** Find roots using LegendreShifted basis.
 *  @version $Id: LegendreShiftedPolynomialRootsFinder.java,v 1.11 2013/11/01 21:11:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LegendreShiftedPolynomialRootsFinder extends PolynomialSimpleRootsFinder {
    final LegendreShifted B=new LegendreShifted();
    public LegendreShiftedPolynomialRootsFinder(final int nmax)
    {
	super(nmax,100,17,1,1e-3);
    }

    public double f(final int n,final double x)
    {
	return B.calculateSumD0(n,coefs,x);
    }
    public double df(final int n,final double x)
    {
	return B.calculateSumD1(n,coefs,x);
    }
    public void doSdiv1AndScale(final int n,final double [] res,final double [] p,final double x0)
    {
	B.sdiv1(n,res,p,x0);
	final double s=res[n-1];
	for(int k=n;--k>=0;)
	{
	    res[k]/=s;
	}
	res[n]=0;
    }

    private static void test_LegendreShiftedRoots(final int nmax,final double eps)
    {
	final long t0=System.currentTimeMillis();
	final LegendreShiftedPolynomialRootsFinder rf=new LegendreShiftedPolynomialRootsFinder(nmax);
	final PolynomialRootJenkinsTraubORIG jrf=new PolynomialRootJenkinsTraubORIG(nmax);
	for(int n=1;n<=nmax;n++)
	{
	    final double [] pol=new double [n+1],re=new double [n],im=new double [n];
	    pol[n]=1;
	    final int nroots=rf.findRoots(n,pol,re,im);
	    SortArrays.sortArrayWithLoad(n,re,null);
	    final double [] jre=new double [n],jim=new double [n];
	    final int jnroots=jrf.findRoots(n,rf.B.convertToPolynomialFromBasis_Test(n,pol),jre,jim);
	    SortArrays.sortArrayWithLoad(n,jre,null);
	    if(nroots!=jnroots) throw new RuntimeException("nroots="+nroots+" jnroots="+jnroots+" n="+n);
	    UnitTests.checkDoubleArrayRel(re,"re",jre,"jre",eps);
	    System.err.println("test_LegendreShiftedRoots: n="+n+" matched OK:");
	}
	final long t1=System.currentTimeMillis();
	System.err.println("LegendreShiftedPolynomialRootsFinder:test_LegendreShiftedRoots: OK eps="+eps+
			   " in "+(t1-t0)+" milliseconds.");
    }

private static void printAllQuadratures(final int n_max,final java.util.Random r)
{
    for(int n=1;n<=n_max;n++)
    {
	final LegendreShiftedPolynomialRootsFinder rf=new LegendreShiftedPolynomialRootsFinder(n+1);
	final LegendreShifted.TestBasisLegendreShifted t=new LegendreShifted.TestBasisLegendreShifted();
	t.printQuadratures(rf,n);
	t.printRadauQuadratures2(rf,n,r);
	t.printDeltaFunctionRoots(rf,n);
    }
}

    public static void main(String [] args)
    {
	final java.util.Random r=new java.util.Random(2711);
	final double eps=1e-8;
	printAllQuadratures(25,r);
	test_LegendreShiftedRoots(12,eps);
    }


}
