package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */

/** Two Roots finder.
 *  Calculates ln(kt/kv) , (ln(kt/kv))' , (ln(kt/kv))''.
 *  @version $Id: LogPolynomialsRatio.java,v 1.1 2015/05/08 22:59:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LogPolynomialsRatio implements DerivativesCalculateable {
    final OrthogonalPolynomialsABasis M;
    final double [] kt,kv;
    public LogPolynomialsRatio(final double [] kv,final double [] kt,final OrthogonalPolynomialsABasis M)
    {
	this.M=M;
	this.kv=kv;
	this.kt=kt;
    }
    public double getF(final double x)
    {
	return Math.log(M.fx(kt,x)/M.fx(kv,x));
    }
    public double getDF(final double x)
    {
	return M.Dfx(kt,x)/M.fx(kt,x)-M.Dfx(kv,x)/M.fx(kv,x);
    }
    public double getD2F(final double x)
    {
	final double Ktx=M.fx(kt,x),Kvx=M.fx(kv,x),
	    DKtx=M.Dfx(kt,x),DKvx=M.Dfx(kv,x),
	    D2Ktx=M.D2fx(kt,x),D2Kvx=M.D2fx(kv,x);
	return (D2Ktx/Ktx-Math.pow(DKtx/Ktx,2))-(D2Kvx/Kvx-Math.pow(DKvx/Kvx,2));
    }
}
