package com.polytechnik.utils;


import java.util.*;

/**
 * An unoptimized TreeMap, just for testing.
 */

public class LongTreeValueMapTest<V> extends LongHashMap<V>
{
    private static final Object PRESENT=new Object();
    private final java.util.TreeMap<V,Object> data;
    public LongTreeValueMapTest(final Comparator<V> cmp)
    {
	super(50,0.6f);
	this.data=new java.util.TreeMap<V,Object>(cmp);
    }

    public V put(final long key,final V value)
    {
	final V p=super.put(key,value);
	if(p!=null)
	{
	    final Object ps=this.data.remove(p);
	    if(ps==null)
	    {
		throw new IllegalStateException("put:inconsistent Data or non-unique values:"+
						" key="+key+" value="+value+" p="+p+" ps="+ps);
	    }
	}
	final Object ps=this.data.put(value,PRESENT);
	final int n=size(),ns=this.data.size();
	if(ps!=null || n!=ns || value==null)
	{
	    throw new IllegalStateException("put:inconsistent Data or non-unique values:"+
					    " ns="+ns+" n="+n+" key="+key+" value="+value+" p="+p);
	}
	return p;
    }

    public V remove(final long key)
    {
	final V p=super.remove(key);
	if(p!=null)
	{
	    final Object ps=this.data.remove(p);
	    if(ps==null)
	    {
		throw new IllegalStateException("remove:inconsistent Data or non-unique values:"+
						" key="+key+" p="+p+" ps="+ps);
	    }
	}
	final int n=size(),ns=this.data.size();
	if(n!=ns)
	{
	    throw new IllegalStateException("remove:inconsistent Data or non-unique values:"+
					    " ns="+ns+" n="+n+" key="+key+" p="+p);
	}
	return p;
    }

    public void clear()
    {
	final int n=size(),ns=this.data.size();
	if(n!=ns)
	{
	    throw new IllegalStateException("clear:inconsistent Data or non-unique values:"+
					    " ns="+ns+" n="+n);
	}
	super.clear();
	this.data.clear();
    }

    public final int copyValues(final V[] valuesarr)
    {
	// this is for what we use this class: return sorted
	final int n=size(),ns=this.data.size();
	if(n!=ns||valuesarr.length<ns)
	{
	    throw new IllegalStateException("copyValues:inconsistent Data or non-unique values or insufficient array size:"+
					    " ns="+ns+" n="+n+" valuesarr.length="+valuesarr.length);
	}
	final V[] r=this.data.keySet().toArray(valuesarr);
	if(r!=valuesarr)
	{
	    throw new IllegalStateException("copyValues:Insufficient array size:"+
					    " ns="+ns+" n="+n+" valuesarr.length="+valuesarr.length);
	}
	return ns;
    }

    public V lastValue() {
	if(this.data.size()>0)
	{
	    return this.data.lastKey();
	}
	return null;
    }

    public V firstValue() {
	if(this.data.size()>0)
	{
	    return this.data.firstKey();
	}
	return null;
    }


    protected java.util.TreeMap<V,Object> getInternalValuesTable()
    {
	return this.data;
    }


//========================== THE TEST ==================================
// to run the test use the command
// java -Xloggc:/tmp/glog -Xmx2500000K com.polytechnik.utils.LongTreeMapTest

    static void test_order(final Random r)
    {
	final Comparator<Integer> ic=new Comparator<Integer>()
	{
	    public int compare(final Integer o1, final Integer o2)
	    {
		final int i1=o1.intValue(),i2=o2.intValue();
		if(i1<i2)
		{
		    return -1;
		}
		if(i1>i2)
		{
		    return 1;
		}
		return 0;
	    }
	};


	int v=100;
	final LongTreeValueMapTest<Integer> m=new LongTreeValueMapTest<Integer>(ic);
	for(int x=0;x<10000;x++)
	{
	    if(r.nextInt(10)<5)
	    {
		m.clear();
	    }
	    else
	    {
		m.remove(r.nextInt(1000));
	    }
	    for(int i=0;i<100;i++)
	    {
		m.put(r.nextInt(1000),new Integer(v++));
	    }

	    final Integer [] res=new Integer [m.size()];
	    m.copyValues(res);
	    for(int i=1;i<res.length;i++)
	    {
		if(res[i].intValue()<res[i-1].intValue())
		{
		    throw new RuntimeException("The data is not sorted");
		}
	    }
	}
	System.err.println("test_order: tests OK");
    }




    public static void main(String [] args)
    {
	test_order(new Random(178));
    }

//========================== END OF THE TEST ==================================


}
