package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Chebyshev polynomials as basis.
 *  @version $Id: OrthogonalPolynomialsChebyshevBasis.java,v 1.9 2015/10/25 15:28:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsChebyshevBasis extends OrthogonalPolynomialsABasis {

public OrthogonalPolynomialsChebyshevBasis(){super(new Chebyshev());}

/** Multiply by (1-x).
 */
public double [] getRadauMultiplied(final double [] nodes0)
{
    final double [] res=new double [nodes0.length+1];
    B.setaxbP(nodes0.length-1,res,nodes0,-1,1);
    return res;
}


static class TestChebyshevInMonomials extends TestMonomials {
    TestChebyshevInMonomials()
    {
	super("BasisChebyshev",
	      new OrthogonalPolynomialsChebyshevBasis(),
	      1e-6,
	      new java.util.Random(8831L),
	      -1,1);
    }
@Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
	{
	    final double [] res=new double [bnodes.length-1];
	    O.B.setXQnMomentsFromQnMoments(bnodes.length-1,res,bnodes);
	    for(int k=res.length;--k>=0;) {res[k]=bnodes[k]-res[k];}
	    UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	    System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
	}
} // END OF class TestChebyshevInMonomials

public static void main(String [] args)
{
    new TestChebyshevInMonomials().runTests(7);
}

}
