package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Hermite polynomials as basis.
 *  @version $Id: OrthogonalPolynomialsHermiteEBasis.java,v 1.12 2015/10/25 15:28:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsHermiteEBasis extends OrthogonalPolynomialsABasis {

public OrthogonalPolynomialsHermiteEBasis(){super(new HermiteE());}

/** Multiply by (1-x).
 * Radau is not applicable to Hermite (-inf, inf) interval.
 */
public double [] getRadauMultiplied(final double [] nodes0)
{
    final double [] res=new double [nodes0.length+1];
    B.setaxbP(nodes0.length-1,res,nodes0,-1,1);
    return res;
}


static class TestHermiteEInMonomials extends TestMonomials {
    TestHermiteEInMonomials()
    {
	super("BasisHermiteE",
	      new OrthogonalPolynomialsHermiteEBasis(),
	      1e-6,
	      new java.util.Random(1471L),
	      -1,1);
    }
    @Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
    {
	final double [] res=new double [bnodes.length-1];
	O.B.setXQnMomentsFromQnMoments(bnodes.length-1,res,bnodes);
	for(int k=res.length;--k>=0;) {res[k]=bnodes[k]-res[k];}
	UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
    }
} // END OF class TestHermiteEInMonomials

public static void main(String [] args)
{
    new TestHermiteEInMonomials().runTests(5);
}

}
