package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Monomials as basis.
 *  @version $Id: OrthogonalPolynomialsMonomialsBasis.java,v 1.34 2015/10/25 15:37:08 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsMonomialsBasis extends OrthogonalPolynomialsABasis {

public OrthogonalPolynomialsMonomialsBasis(){super(new Monomials());}

/** Multiply by (1-x).
 */
public double [] getRadauMultiplied(final double [] nodes0)
{
    final double [] res=new double [nodes0.length+1];
    Polynomials.set1mxP(res,nodes0,nodes0.length);
    return res;
}


/** Test  (-1)^{k+1} x^k/(k+3) basis to check scale.
 */
static class TestScaleNon1 extends OrthogonalPolynomialsABasis {

public TestScaleNon1(){super(null);}

static double en(final int k){return (k+3)*(k%2==0?-1:1);}
public double getSignToMatch(final int k){return (k%2==0?-1:1);}
static double [] convertBasisToMonomials(final double [] p)
{
    final double [] m=new double [p.length];
    for(int k=0;k<p.length;k++) m[k]=p[k]/en(k);
    //System.err.println("p="+java.util.Arrays.toString(p)+" m="+java.util.Arrays.toString(m));
    return m;
}
static void convertMonomialsToBasis(final double [] res,final double [] m,final int n)
{
    for(int k=0;k<n;k++) res[k]=m[k]*en(k);
}
public double [] getPQMultiplied(final double [] P,final double [] Q)
{
    final double [] m=Polynomials.getMult2Pol(convertBasisToMonomials(P),convertBasisToMonomials(Q));
    convertMonomialsToBasis(m,m,m.length);
    return m;
}
public void setQQCoefs(final int l,final int m,final double [] coefs)
{
    java.util.Arrays.fill(coefs,0,l+m+1,0);
    coefs[l+m]=en(l+m)/(en(l)*en(m));
}
public PolynomialRootsFinder getPolynomialRootsFinderInBasis()
{
    return new PolynomialRootsFinder(){
	public int findRoots(final int n,
			     final double [] polynomial_coefs,
			     final double [] re_root,
			     final double [] im_root){
	    return PolynomialRootsConfederateMatrixABasis.ABASISMONOMIALS.findRoots(n,convertBasisToMonomials(polynomial_coefs),re_root,im_root);
	}
    };
}
public void setxP(final double [] res,final double [] p)
{
    final double [] m=convertBasisToMonomials(p);
    Polynomials.setxP(res,m,m.length);
    convertMonomialsToBasis(res,res,m.length+1);
}
public double sdiv1(final double [] res,final double [] p,final double x0)
{
    final double [] m=convertBasisToMonomials(p);
    double r=Polynomials.sdiv1(res,m,m.length,x0);
    convertMonomialsToBasis(res,res,m.length-1);
    return r;
}
public double fx(final double [] m,final double x)
{
    return Polynomials.getD0NoScale(convertBasisToMonomials(m),x);
}
public double Dfx(final double [] m,final double x)
{
    return Polynomials.getD1NoScale(convertBasisToMonomials(m),x);
}
public double D2fx(final double [] m,final double x)
{
    return Polynomials.getD2NoScale(convertBasisToMonomials(m),x);
}
public double [] getRadauMultiplied(final double [] nodes0)
{
    final double [] res=new double [nodes0.length+1];
    final double [] m=convertBasisToMonomials(nodes0);
    Polynomials.set1mxP(res,m,m.length);
    convertMonomialsToBasis(res,res,m.length+1);
    return res;
}
} // END OF class TestScaleNon1

public static void main(String [] args)
{
    new TestMonomials("BasisMonomials",
		      new OrthogonalPolynomialsMonomialsBasis(),
		      1e-5,
		      new java.util.Random(8831L),
		      0,1)
    {
	@Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
	{
	    final double [] res=new double [bnodes.length-1];
	    O.B.setXQnMomentsFromQnMoments(bnodes.length-1,res,bnodes);
	    for(int k=res.length;--k>=0;) {res[k]=bnodes[k]-res[k];}
	    UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	    System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
	}
    }.runTests(6);



   new TestMonomials("BasisMonomialsNon1",
		      new TestScaleNon1(),
		      1e-5,
		      new java.util.Random(8831L),
		      0,1)
    {
	@Override double [] convertToMonomials(final double [] p)
	{
	    return TestScaleNon1.convertBasisToMonomials(p);
	}
	@Override double [] getRadauMultiplied(final double [] nodes)
	{
	    final double [] nnodes=new double [nodes.length];
	    TestScaleNon1.convertMonomialsToBasis(nnodes,nodes,nodes.length);
	    final double [] res=O.getRadauMultiplied(nnodes);
	    return TestScaleNon1.convertBasisToMonomials(res);
	}
    }.runTests(6);
}

}
