package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/* Test Polynomial roots.
 *
 *  @version $Id: PolynomialRootsTester.java,v 1.6 2011/03/06 11:09:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

public class PolynomialRootsTester  {

    private final PolynomialRootsFinder rootfinder;
    private final double eps;
    public PolynomialRootsTester(final PolynomialRootsFinder rootfinder,
				 final double eps)
    {
	this.rootfinder=rootfinder;
	this.eps=eps;
    }

    public void testPolynomialFromFile(final String fname) throws java.io.IOException
    {
	System.err.println("Reading polynomial from "+fname);
	final double [] p=readPolynomialFromFile(fname);
	final int n=p.length-1;
	final double [] rex=new double [n],imx=new double [n];
	testPolynom(n,p,rex,imx);
	System.err.println("N="+n+" read");
	for(int i=p.length;--i>=0;)
	{
	    System.err.println("pcoefs("+i+")="+p[i]+" read");
	}
	System.err.println("ROOTS:");
	for(int i=0;i<n;i++)
	{
	    System.err.println(rex[i]+"\t"+imx[i]);
	}
    }

    public double [] readPolynomialFromFile(final String fname) throws java.io.IOException
    {
	final java.io.LineNumberReader fr=new java.io.LineNumberReader(new java.io.FileReader(fname));
	final int n=Integer.valueOf(fr.readLine());
	final double [] p=new double [n+1];
	for(int i=p.length;--i>=0;)
	{
	    p[i]=Double.valueOf(fr.readLine());
	}
	fr.close();
	return p;
    }

    public void testPolynom(final int n,final double [] p,final double [] rex,final double [] imx)
    {
	int nr=rootfinder.findRoots(n,p,rex,imx);
	if(nr!=n)
	{
	    System.err.println("Some roots were not found :"+PolynomialRoot.getPolinomTXT(p)+"\nnr="+nr+" n="+n+"\t"+
			       "Roots:\n"+PolynomialRoot.getRootsTXT(nr,rex,imx));
	    throw new RuntimeException(PolynomialRoot.getPolinomTXT(p)+"\nnr="+nr+" n="+n);
	}

	for(int j=0;j<nr;j++)
	{
	    //System.err.println("j="+j);
	    PolynomialRoot.checkValues(p,n,rex[j],imx[j],eps,"test"/* " t="+i+" roots:"+PolynomialRoot.getRootsTXT(nr,rex,imx)*/);
	}
    }

    public void testRootsRandomPolynomial(final int n,
					  final int n_tests,
					  final java.util.Random rn)
    {
	final long t0=System.currentTimeMillis();

	final double [] p=new double [n+1];
	final double [] rex=new double [n],imx=new double [n];
	for(int i=0;i<n_tests;i++)
	{
	  for(int dg=n;dg-->-1;)
	  {
	    for(int dr=3;dr-->0;)
	    {
	      PolynomialRoot.setRandomP(p,n,rn);
	      for(int j=0;j<=dg;j++)
	      {
		  p[j]=0;
	      }
	      if(dr==0)
	      {
		  p[0]=-1+2.0*rn.nextDouble();
	      }
	      else if(dr==1)
	      {
		  p[0]=p[1]=0;
	      }

	      testPolynom(n,p,rex,imx);
	    }
	  }
	}
	System.err.println("testRoots(): n_tests="+n_tests+" OK, dim="+n);
	final long t1=System.currentTimeMillis();
	System.err.println("testRoots(): "+n_tests+" tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: PolynomialRootsTester.java,v 1.6 2011/03/06 11:09:58 mal Exp $");
    }
}
