package com.polytechnik.utils;

/** Converted from http://www.netlib.org/sfmm/quanc8.f to java
 *  $Id: QUANC8.java,v 1.15 2013/08/19 15:05:23 mal Exp $
 */
public class QUANC8 {
    public interface DoubleFunction {
	double f(final double x);
    }
    public final double a,b;
    public final double integral,errest,flag;
    public final int nofun;
    QUANC8(final double a,
	   final double b,
	   final double integral,
	   final double errest,
	   final int nofun,
	   final double flag)
    {
	this.a=a;
	this.b=b;
	this.integral=integral;
	this.errest=errest;
	this.nofun=nofun;
	this.flag=flag;
    }

    public static QUANC8 integrate(final DoubleFunction fun,
				   final double a,
				   final double b,
				   final double abserr,
				   final double relerr,
				   final int nomax)
    {

//
//
//   estimate the integral of fun(x) from a to b
//   to a user provided tolerance.
//   an automatic adaptive routine based on
//   the 8-panel newton-cotes rule.
//
//   input ..
//
//   fun     the name of the integrand function subprogram fun(x).
//   a       the lower limit of integration.
//   b       the upper limit of integration.(b may be less than a.)
//   relerr  a relative error tolerance. (should be non-negative)
//   abserr  an absolute error tolerance. (should be non-negative)
//
//   output ..
//
//   result  an approximation to the integral hopefully satisfying the
//           least stringent of the two error tolerances.
//   errest  an estimate of the magnitude of the actual error.
//   nofun   the number of function values used in calculation of result.
//   flag    a reliability indicator.  if flag is zero, then result
//           probably satisfies the error tolerance.  if flag is
//           xxx.yyy , then  xxx = the number of intervals which have
//           not converged and  0.yyy = the fraction of the interval
//           left to do when the limit on  nofun  was approached.
//


      final int
	  levmin = 1,
	  levmax0=60, // 30,
	  levout = 10
	  ;
      final double w0 =   3956.0 / 14175.0;
      final double w1 =  23552.0 / 14175.0;
      final double w2 =  -3712.0 / 14175.0;
      final double w3 =  41984.0 / 14175.0;
      final double w4 = -18160.0 / 14175.0;

      final double [] qright=new double [levmax0+1+1],f=new double [16+1],x=new double [16+1];
      final double [][] fsave=new double [8+1][levmax0+1],xsave=new double [8+1][levmax0+1];
      double area,x0,f0,stone,step,cor11;
      double qprev,qnow,qdiff,qleft,esterr,tolerr;
      double result, errest, flag;

      int nofun,nofin,lev,nim,levmax=levmax0;

//
//   trouble when nofun reaches nofin
//
      //nofin = nomax - 8*(levmax-levout+(int)Math.pow(2,levout+1));
      nofin=nomax;


//
//   initialize running sums to zero.
//
      flag = 0.0;
      result = 0.0;
      cor11  = 0.0;
      errest = 0.0;
      area   = 0.0;
      nofun = 0;
      if (a == b)
      {
	  return new QUANC8(a,b,result,errest,nofun,flag);
      }
//
//   ***   stage 2 ***   initialization for first interval
//
      lev = 0;
      nim = 1;
      x0 = a;
      x[16] = b;
      qprev  = 0.0;
      f0 = fun.f(x0);
      stone = (b - a) / 16.0;
      x[8]  =  (x0  + x[16]) / 2.0;
      x[4]  =  (x0  + x[8])  / 2.0;
      x[12] =  (x[8]  + x[16]) / 2.0;
      x[2]  =  (x0  + x[4])  / 2.0;
      x[6]  =  (x[4]  + x[8])  / 2.0;
      x[10] =  (x[8]  + x[12]) / 2.0;
      x[14] =  (x[12] + x[16]) / 2.0;
      for(int j = 2;j<=16;j+=2)
      {
	  f[j] = fun.f(x[j]);
      }
      nofun = 9;
//
//   ***   stage 3 ***   central calculation
//   requires qprev,x0,x2,x4,...,x16,f0,f2,f4,...,f16.
//   calculates x1,x3,...x15, f1,f3,...f15,qleft,qright,qnow,qdiff,area.
//
    label30:for(;;) { label70:{
	  x[1] = (x0 + x[2]) / 2.0;
	  f[1] = fun.f(x[1]);
	  for(int j = 3;j<= 15;j+=2)
	  {
	      x[j] = (x[j-1] + x[j+1]) / 2.0;
	      f[j] = fun.f(x[j]);
	  }
	  nofun += 8;
	  step = (x[16] - x0) / 16.0;
	  qleft  =  (w0*(f0 + f[8])  + w1*(f[1]+f[7])  + w2*(f[2]+f[6])
		     + w3*(f[3]+f[5])  +  w4*f[4]) * step;
	  qright[lev+1]=(w0*(f[8]+f[16])+w1*(f[9]+f[15])+w2*(f[10]+f[14])
			 + w3*(f[11]+f[13]) + w4*f[12]) * step;
	  qnow = qleft + qright[lev+1];
	  qdiff = qnow - qprev;
	  area = area + qdiff;
//
//   ***   stage 4 *** interval convergence test
//
	  esterr = Math.abs(qdiff) / 1023.0;
	  tolerr = Math.max(abserr,relerr*Math.abs(area)) * (step/stone);

	  if (!(lev <levmin))
	  {
	      if (lev >= levmax)
	      {
		  //
		  //   current level is levmax.
		  //
		  flag += 1.0;
		  break label70;
	      }
	      if (nofun > nofin)
	      {
		  //
		  //   ***   stage 6   ***   trouble section
		  //   number of function values is about to exceed limit.
		  //
		  nofin *= 2;
		  levmax = levout;
		  flag += (b - x0) / (b - a);
		  break label70;
	      }
	      if (esterr <= tolerr)
	      {
		  //System.err.println(" esterr="+esterr+" tolerr="+tolerr);
		  break label70;
	      }
	  }

//
//   ***   stage 5   ***   no convergence
//   locate next interval.
//
	  nim = 2*nim;
	  lev = lev+1;
//
//   store right hand elements for future use.
//

	  for(int i=1;i<=8;i++)
	  {
	      fsave[i][lev] = f[i+8];
	      xsave[i][lev] = x[i+8];
	  }
//
//   assemble left hand elements for immediate use.
//
	  qprev = qleft;
	  for(int i=1;i<=8;i++)
	  {
	      final int j = -i;
	      f[2*j+18] = f[j+9];
	      x[2*j+18] = x[j+9];
	  }
	  continue label30;

	  } // end label70
//
//   ***   stage 7   ***   interval converged
//   add contributions into running sums.
//
      result += qnow;
      errest += esterr;
      cor11  += qdiff / 1023.0;
//
//   locate next interval.
//
      for(;;)
      {
	  if (nim == 2*(nim/2))
	  {
	      nim++;
	      break;
	  }
	  nim /=2;
	  lev--;
      }
      if (!(lev<=0))
      {
//
//   assemble elements required for the next interval.
//
	  qprev = qright[lev];
	  x0 = x[16];
	  f0 = f[16];
	  for(int i = 1; i<=8 ;i++)
	  {
	      f[2*i] = fsave[i][lev];
	      x[2*i] = xsave[i][lev];
	  }
	  continue label30;
      }
//
//   ***   stage 8   ***   finalize and return
//
      result += cor11;
//
//   make sure errest not less than roundoff level.
//
      if (errest != 0.0)
      {
	  for(int jj2=0;jj2<2048;jj2++)
	  {
	      final double temp = Math.abs(result) + errest;
	      if (!(temp == Math.abs(result)))
	      {
		  break;
	      }
	      errest *= 2;
	  }
      }
      break label30;}
      return new QUANC8(a,b,result,errest,nofun,flag);
    }



    // tests
    static final class FSIN implements DoubleFunction {
	final double W0=12.4;
	public double f(final double x)
	{
	    return Math.sin(x*W0);
	}
	double getExact(final double a,final double b)
	{
	    return -(Math.cos(b*W0)-Math.cos(a*W0))/W0;
	}
    }

	static void test(final FSIN fun)
	{
	    final double abserr=1e-7,relerr=1e-7,eps=1e-5;
	    final int nomax=500000;
	    int n=0;
	    for(double x=-2;x<2;x+=0.1)
	    {
		for(double y=-3;y<3;y+=0.1)
		{
		    final QUANC8 r=integrate(fun,x,y,abserr,relerr,nomax);
		    final double exact_res=fun.getExact(x,y);
		    UnitTests.checkDouble(r.integral,"r.integral flag="+r.flag+
					  " nofun="+r.nofun+" ",exact_res,"exact_res",eps);
		    n++;
		}
	    }
	    System.err.println("QUANC8.test():"+fun.getClass().getName()+" N="+n+" tests OK");
	}


    static final class CompareToFortran implements DoubleFunction {
	public double f(final double x)
	{
	    return Math.sin(1/x)/x;
	}
	static void printValues()
	{
	    final double a=0.001,b=111,abserr=1e-7,relerr=1e-7;
	    final int nomax=50000;
	    final QUANC8 r=integrate(new CompareToFortran(),a,b,abserr,relerr,nomax);
	    System.err.println("a="+a+" b="+b+" abserr="+abserr+" relerr="+relerr+
			       " nomax="+nomax);
	    System.err.println("result="+r.integral);
	    System.err.println("errest="+r.errest);
	    System.err.println("nofun="+r.nofun);
	    System.err.println("flag="+r.flag);
	}
    }




    public static void main(final String [] args)
    {
	test(new FSIN());
	CompareToFortran.printValues();
    }



}
