package com.polytechnik.utils;

import java.io.StreamTokenizer;
import java.io.Reader;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.PropertyResourceBundle;

/** A class with a variety of helper functions.
 * @author Vladislav Malyshkin mal@gromco.com
 * @version $Id: ReadWriteUtils.java,v 1.42 2011/10/14 16:18:15 mal Exp $
 */
public abstract class ReadWriteUtils {

    /** System property to add a perfix to resources.
     */
    public static final String PROPERTYPREFIX="com.polytechnik.utils.ReadWriteUtils.PROPERTYPREFIX";

  /** Create stream tokenizer with the settings we need. */
  public static final StreamTokenizer getTokenizer(String separators,Reader br)
    {
	final StreamTokenizer st=new StreamTokenizer(br);
	st.resetSyntax();
	st.wordChars(' ','\u00FF');
	st.eolIsSignificant(true);
	st.whitespaceChars('\r','\r'); // treat \r as a space
        final int sep_len=separators.length();
	for(int i=0;i<sep_len;i++)
	  {
	    final char c=separators.charAt(i);
	    st.ordinaryChar(c);
	  }
	return st;
    }


  /** This method reads one line from a stream.
   *  @param tk The StreamTokenizer to read data from.
   *  @param arraytosave An array to store read data.
   *  @return The number of words in line.
   */
  public static final int readOneLine(StreamTokenizer tk,String [] arraytosave)
             throws java.io.IOException
    {
	int nsaved=0,ntotal=0,ttype=StreamTokenizer.TT_EOL;
	boolean prev_word=false;
	// System.err.println("LINE----");
	for(;;)
	    {
		ttype=tk.nextToken();
		// System.err.println("ttype="+ttype+" sval="+tk.sval);
		if(ttype!=StreamTokenizer.TT_EOF &&
		   ttype!=StreamTokenizer.TT_EOL)
		    {
			if(tk.sval!=null)
			    { // word
				if(nsaved<arraytosave.length)
				    arraytosave[nsaved++]=tk.sval;
				ntotal++;
				prev_word=true;
			    }
			else
			    { // separator
				if(!prev_word)
				    { // an empty string in between two separators
					if(nsaved<arraytosave.length)
					    arraytosave[nsaved++]="";
					ntotal++;
				    }
				prev_word=false;
			    }
		    }
		else
		    { // handle last symbol,
		      // (in case we have a separator in the end).
			if(ntotal>0 && !prev_word)
			    {
				if(nsaved<arraytosave.length)
				    arraytosave[nsaved++]="";
				ntotal++;
			    }
			break;
		    }
	}
	// System.err.println("readOneLine: READ "+ntotal+" strings.");
	return ntotal;
    }




  /** This method reads one line from a stream.
   *  @param tk The StreamTokenizer to read data from.
   *  @return Read line.
   */
    public static final String [] readOneLine(StreamTokenizer tk)
	throws java.io.IOException
    {
	String [] arraytosave=new String [2];
	int nsaved=0,ttype=StreamTokenizer.TT_EOL;
	boolean prev_word=false;
	// System.err.println("LINE----");
	for(;;)
	{
	    ttype=tk.nextToken();
	    // System.err.println("ttype="+ttype+" sval="+tk.sval);
	    if(ttype!=StreamTokenizer.TT_EOF &&
	       ttype!=StreamTokenizer.TT_EOL)
	    {
		if(tk.sval!=null)
		{ // word
		    if(nsaved>=arraytosave.length)
		    {
			final String [] a2=new String [2*arraytosave.length+10];
			System.arraycopy(arraytosave,0,a2,0,nsaved);
			arraytosave=a2;
		    }
		    arraytosave[nsaved++]=tk.sval;
		    prev_word=true;
		}
		else
		{ // separator
		    if(!prev_word)
		    { // an empty string in between two separators
			if(nsaved>=arraytosave.length)
			{
			    final String [] a2=new String [2*arraytosave.length+10];
			    System.arraycopy(arraytosave,0,a2,0,nsaved);
			    arraytosave=a2;
			}
			arraytosave[nsaved++]="";
		    }
		    prev_word=false;
		}
	    }
	    else
	    { // handle last symbol,
		// (in case we have a separator in the end).
		if(nsaved>0 && !prev_word)
		{
		    if(nsaved>=arraytosave.length)
		    {
			final String [] a2=new String [2*arraytosave.length+10];
			System.arraycopy(arraytosave,0,a2,0,nsaved);
			arraytosave=a2;
		    }
		    arraytosave[nsaved++]="";
		}
		break;
	    }
	}
	// System.err.println("readOneLine: READ "+nsaved+" strings.");
	return shrinkArray(nsaved,arraytosave);
    }



    public static int processStreamByLines(
	final String stream_id,
	final StreamTokenizer tk,
	final com.polytechnik.utils.TObjectProcedure<String []> header_processor,
	final com.polytechnik.utils.TObjectProcedure<String []> processor)
	throws java.io.IOException
    {

	final String [] sread=ReadWriteUtils.readOneLine(tk);
	// check header
	if(!header_processor.execute(sread))
	{
	    throw new RuntimeException("Invalid Header in stream="+stream_id);
	}

	// process body
	int nlinesread=1,npointsread=0;
	for(;tk.ttype!=StreamTokenizer.TT_EOF;)
	{
	    final int nrecordsinline=ReadWriteUtils.readOneLine(tk,sread);
	    nlinesread++;
            // System.err.println(" LINE READ, "+nrecordsinline+" records.");
	    if(tk.ttype==StreamTokenizer.TT_EOF && nrecordsinline<=0)
	    {
		// EOF after LF
		break;
	    }
            if(nrecordsinline!=sread.length)
	    {
                if(nrecordsinline>0) // non-empty line
		{
		    System.err.println(
			"Invalid line #"+nlinesread+" STR="+
			ReadWriteUtils.mergeElements(ReadWriteUtils.shrinkArray(nrecordsinline,sread))+
			" read from data file stream="+stream_id+
			" with nrecordsinline="+nrecordsinline+
			" should be="+sread.length);
		}
		else
		{
		    System.err.println("Empty line #"+nlinesread+
				       " read from data file stream="+stream_id);
		}
		throw new RuntimeException(
			"Invalid line #"+nlinesread+" STR="+
			ReadWriteUtils.mergeElements(ReadWriteUtils.shrinkArray(nrecordsinline,sread))+
			" read from data file stream="+stream_id);
	    }
            else
	    { // line OK, process it
		if(!processor.execute(sread))
		{
		    throw new RuntimeException(
			"Failed to process stream="+stream_id+
			" line #"+nlinesread+" recordnum="+nlinesread+" STR="+
			ReadWriteUtils.mergeElements(ReadWriteUtils.shrinkArray(nrecordsinline,sread))+
			" read from data file stream="+stream_id);
		}
	    }
	}
	return npointsread;
    }


  /** Extract names. */
  public static final String [] extractnames(String str,String delim)
    {
      final StringTokenizer tk= delim!=null ?
	new StringTokenizer(str,delim):new StringTokenizer(str);
      final ArrayList<String> v=new ArrayList<String>();
      while(tk.hasMoreTokens())
	v.add(tk.nextToken());
      final String [] rarr=new String [v.size()];
      v.toArray(rarr);
      return rarr;
    }

    /** The function merges the list of array elements as a string.
     */
    public static final String mergeElements(String [] elements)
    {
	final StringBuilder buf=new StringBuilder();
	for(int i=0;i<elements.length;i++)
	{
	    if(i>0)
		buf.append(" ");
	    buf.append(elements[i]);
	}
	return buf.toString();
    }

    /** Reads properties.
     *  A prefix may be added to
     *  property name if a special system property is set.
     */
    public static final ResourceBundle getProperties(String fname)
	throws java.util.MissingResourceException,java.io.IOException
    {
	final String prefix=System.getProperty(PROPERTYPREFIX);
	final String xfname=prefix!=null?prefix+fname:fname;
	return new PropertyResourceBundle(
	    new java.io.FileReader(fname));
    }

  /** Reads parameters from config using a given prefix.
   */
  public static final String [] readParameters(String prefix,
					       String [] parameters,
					       ResourceBundle resources)
    throws java.util.MissingResourceException
    {
      final String [] res=new String[parameters.length];
      for(int i=0;i<parameters.length;i++)
	{
	  if(parameters[i]!=null)
	    res[i]=getparameter(prefix+parameters[i],resources,true);
	}
      return res;
    }

   public static final class ReadNameValue {
	public final String name;
	public final String [] values;
	ReadNameValue(String name,
		      String [] values)
	{
	    this.name=name;
	    this.values=values;
	}
    }
    /** Reads given parameters from a properties. */
    public static final ReadNameValue [] readAllParameters(String allparameters,
							   String [] namestoread,
							   ResourceBundle resources)
	throws java.util.MissingResourceException
    {
      final StringTokenizer tk=new StringTokenizer(allparameters);
      final ArrayList<ReadNameValue> v=new ArrayList<ReadNameValue>();
      while (tk.hasMoreTokens())
	{
	  final String n=tk.nextToken();
	  final String [] f=
	      ReadWriteUtils.readParameters(n+".",namestoread,resources);
	  v.add(new ReadNameValue(n,f));
	}
      final ReadNameValue [] r=new ReadNameValue[v.size()];
      v.toArray(r);
      return r;
    }


  /** Reads a parameter from a configurartion file. */
   public static final String getparameter(String name,
					   ResourceBundle resources,
					   boolean mustbeset)
			throws java.util.MissingResourceException
    {
      if(mustbeset)
	{
	  return resources.getString(name);
	}
      else
	{
	  try {
	    return resources.getString(name);
	  } catch (java.util.MissingResourceException mre) {}
	}
      return null;
    }

  /** Checkes for a given element in an array. */
  public static final boolean hasElement(int [] array, int element)
    {
      // check we have this field in serie
      for(int j=0;j<array.length;j++)
	{
	  if(array[j]==element)
	    return true;
	}
      return false;
    }

  /** Checkes for a given element in an array. */
  public static final int elementIndex(int [] array, int element)
    {
      // check we have this field in serie
      for(int j=0;j<array.length;j++)
	{
	  if(array[j]==element)
	    return j;
	}
      return -1;
    }

    /** Checkes for a given element in an array. */
    public static final int elementIndex(String [] array,
					 String element)
    {
      // check we have this field in serie
      for(int j=0;j<array.length;j++)
	{
	  if(array[j].equals(element))
	    return j;
	}
      return -1;
    }

  /** This method extracts integer numbers from a string array. */
   public static final int [] extractNumbers(String [] array)
     throws NumberFormatException
    {
      final int [] a2=new int [array.length];
      for(int i=0;i<a2.length;i++)
	a2[i]=new Integer(array[i]).intValue();
      return a2;
    }


    /** Checkes for a given element in an array. */
    public static final int elementIndex(long [] array, long element)
    {
	// check we have this field in serie
	for(int j=0,n=array.length;j<n;j++)
	{
	    if(array[j]==element)
		return j;
	}
	return -1;
    }

    /** This method shrinks an array. */
   public static final int [] shrinkArray(final int nelements,final int [] array)
    {
	if(nelements>=array.length)
	    return array;
	final int [] ar=new int [nelements];
	System.arraycopy(array,0,ar,0,nelements);
	return ar;
    }


   /** This method shrinks an array. */
   public static final long [] shrinkArray(final int nelements,final long [] array)
    {
	if(nelements>=array.length)
	    return array;
	final long [] ar=new long [nelements];
	System.arraycopy(array,0,ar,0,nelements);
	return ar;
    }


   /** This method shrinks an array. */
   public static final double [] shrinkArray(final int nelements,final double [] array)
    {
	if(nelements>=array.length)
	    return array;
	final double [] ar=new double [nelements];
	System.arraycopy(array,0,ar,0,nelements);
	return ar;
    }


    /** This method shrinks an array. */
    public static final String [] shrinkArray(final int nelements,final String [] array)
    {
	if(nelements>=array.length)
	    return array;
	final String [] ar=new String [nelements];
	System.arraycopy(array,0,ar,0,nelements);
	return ar;
    }


    /** Encodes an array of integers as bits. */
    public static final int getArrayAsBits(int [] data,
					   int [] bits)

    {
	int u=0;
	for(int i=0;i<data.length;i++)
	    {
		u|=bits[data[i]];
	    }
	return u;
    }

    /** Decodes an integer to array. */
    public static final int [] getBitsAsArray(int data,
					      int [] bits)
    {
	final int [] u=new int [bits.length];
	int nSaved=0;
	for(int i=0;i<u.length;i++)
	    {
		if((data&bits[i])!=0)
		    u[nSaved++]=i;
	    }
	if(nSaved>=u.length)
	    return u;
	final int [] u1=new int [nSaved];
	for(nSaved=0;nSaved<u1.length;nSaved++)
	    {
		u1[nSaved]=u[nSaved];
	    }
	return u1;
    }

    public static final String PARAMSEPARATOR=", \t";


    public static final String [] getStreamContent(java.io.InputStream [] streams)
	throws java.io.IOException
    {
	final StringBuilder [] bufs=new StringBuilder [streams.length];
	final java.io.InputStreamReader [] sout=
	    new java.io.InputStreamReader [streams.length];
	final String [] res=new String [streams.length];
	final int [] indexes=new int [streams.length];
	int i,n_left=sout.length;
	final char [] c=new char [1024*32];
	for(i=0;i<sout.length;i++)
	    {
		bufs[i]=new StringBuilder(c.length+1);
		sout[i]=new java.io.InputStreamReader(streams[i]);
		indexes[i]=i;
	    }
	for(;n_left>0;)
	{
	    for(i=0;i<n_left;i++)
		{
		    final int n=indexes[i];
		    final int syms=sout[n].read(c);
		    if(syms>0)
			{
			    bufs[n].append(c,0,syms);
			}
		    else if(syms<0)
			{
			    sout[n].close();
			    streams[n].close();
			    sout[n]=null;
			    streams[n]=null;
			    res[n]=bufs[n].toString();
			    bufs[n]=null;
			    n_left--;
			    indexes[i]=indexes[n_left];
			    indexes[n_left]=n;
			}
		}
	}
	return res;
    }

    /** This method reads all the data from a file.
     */
    public static final String getFileContent(String filename)
	throws java.io.IOException
    {
	return getStreamContent(
	     new java.io.InputStream []{
                 new java.io.FileInputStream(new java.io.File(filename))
		     })[0];
    }


    /** This method reads all the data from a file.
     */
    public static final String getFileContent(java.io.File file)
	throws java.io.IOException
    {
	return getStreamContent(
	     new java.io.InputStream []{
                 new java.io.FileInputStream(file)
		     })[0];
    }


    public static final byte [] getStreamBytes(java.io.InputStream stream)
	throws java.io.IOException
    {
	final java.io.ByteArrayOutputStream ba=new java.io.ByteArrayOutputStream();
	final byte [] b=new byte [1024];
	for(;;)
	{
	    final int syms=stream.read(b);
	    if(syms>0)
	    {
		ba.write(b,0,syms);
	    }
	    else if(syms<0)
	    {
		break;
	    }
	}
	return ba.toByteArray();
    }

    public static final byte [] getFileBytes(String filename)
	throws java.io.IOException
    {
	return getStreamBytes(new java.io.FileInputStream(new java.io.File(filename)));
    }

  /** Extracts long from a string.
   * This method extrcats long value from a string.
   * it uses defaultvalue if the string is not a number.
   */
  public static final long getLong(String s,long defaultvalue)
    {
      long l=defaultvalue;
      try {
	if(s!=null)
	  l=new Long(s).longValue();
      }catch(Exception ee){}
      return l;
    }

    /** Extracts double from a string.
     */
    public static final double getDouble(String s,double defaultvalue)
    {
      double d=defaultvalue;
      try {
	if(s!=null)
	    {
		d=new Double(s).doubleValue();
		if(Double.isInfinite(d))
		    {
			d=defaultvalue;
		    }
	    }

      }catch(Exception ee){}
      return d;
    }

    /** Extracts integer from a string.
     */
    public static final int getInteger(String s,int defaultvalue)
    {
      int i=defaultvalue;
      try {
	if(s!=null)
	    i=new Integer(s).intValue();
      }catch(Exception ee){}
      return i;
    }

}
