package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */

/** Skewness estimator.
 *  @version $Id: Skewness.java,v 1.8 2015/08/07 16:13:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class Skewness {
    public final double xa,xm,sk;
    Skewness(final double xa,
	     final double xm,
	     final double sk)
    {
	this.xa=xa;
	this.xm=xm;
	this.sk=sk;
    }

    /** Calculate modified skewness given 0..3 moments in arbitrary basis.
     */
    public static Skewness getSkewness2(final double m0,final double m1,final double m2,final double m3,final OrthogonalPolynomialsABasis M)
    {
	final double [] moms=new double []{m0,m1,m2,m3};
	final double [] P1=M.getSingleOrthogonalPolynomial(1,moms);
	final double xa=-M.fx(P1,0)/M.Dfx(P1,0);
	final double [] re=M.getXNodes(M.getSingleOrthogonalPolynomial(2,moms));
	final double xm=0.5*(re[0]+re[1]),sk=2*(xa-xm)/(re[0]-re[1]);
	return new Skewness(xa,xm,sk);
    }

    /** Same as getSkewness2, but different algorithm.
     */
    public static Skewness getSkewness(final double m0,final double m1,final double m2,final double m3,final OrthogonalPolynomialsABasis M)
    {
	final double [] moms=new double []{m0,m1,m2,m3};
	final double [][] Q=M.getQuadraturesFromMoments_EV(2,moms);
	final double [] x=Q[0],w=Q[1];
	final double nr=w[0]+w[1],xa=(x[0]*w[0]+x[1]*w[1])/nr,
	    xm=0.5*(x[0]+x[1]),sk=(w[0]-w[1])/nr;
	return new Skewness(xa,xm,sk);
    }


    /** Calculate modified skewness given 0..3 moments in monomials basis.
     */
    public static Skewness getSkewnessFromMonomialMoments(final double m0,final double m1,final double m2,final double m3)
    {
	return getSkewness(m0,m1,m2,m3,MB);
    }
    public static Skewness getSkewnessFromMonomialMoments2(final double m0,final double m1,final double m2,final double m3)
    {
	return getSkewness2(m0,m1,m2,m3,MB);
    }
    static final OrthogonalPolynomialsMonomialsBasis MB=new OrthogonalPolynomialsMonomialsBasis();

public static void matchSkewness(final Skewness a,final Skewness b,final double eps)
{
    UnitTests.checkDoubleRel(a.xa,"a.xa",b.xa,"b.xa",eps);
    UnitTests.checkDoubleRel(a.xm,"a.xm",b.xm,"b.xm",eps);
    UnitTests.checkDoubleRel(a.sk,"a.sk",b.sk,"b.sk",eps);

    System.err.println("Skewness:matchSkewness test OK eps="+eps);
}

}
