package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Unit test utils.
 *  @version $Id: UnitTests.java,v 1.19 2015/10/23 11:06:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UnitTests {
public static void checkDouble(final double v1,final String s1,
				    final double v2,final String s2)
    {
	if(Double.isNaN(v1)!=Double.isNaN(v2)||
	   (!Double.isNaN(v1) && v2!=v1))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2);
	}
    }

    public static boolean checkDoubleNoFail(final double v1,final double v2,final double eps)
    {
	if(Double.isNaN(v1)!=Double.isNaN(v2)||
	   (!Double.isNaN(v1) && Math.abs(v2-v1)>eps))
	{
	    return true;
	}
	return false;
    }

    public static void checkDouble(final double v1,final String s1,
			    final double v2,final String s2,
			    final double eps)
    {
	if(checkDoubleNoFail(v1,v2,eps))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2+" eps="+eps);
	}
    }
    public static void checkDoubleRel(final double v1,final String s1,
			       final double v2,final String s2,
			       final double eps)
    {
	if(checkDoubleRelNoFail(v1,v2,eps))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2+" eps="+eps);
	}
    }
    public static boolean checkDoubleRelNoFail(final double v1,final double v2,final double eps)
    {
	if(v1==0||v2==0/* ||Math.abs(v1)<1 */)
	{
	    return checkDoubleNoFail(v1,v2,eps);
	}
	else if(Double.isNaN(v1)!=Double.isNaN(v2)||
		(!Double.isNaN(v1) && Math.abs((v2-v1)/Math.max(v1,v2))>eps))
	{
	    return true;
	}
	return false;
    }


public static void checkDoubleArrayRel1(final double [] av1,final String s1,
				final double [] av2,final String s2,
				final double eps)
{
    if(av1.length!=av2.length)
    {
	throw new RuntimeException("Mismatch: "+s1+".length="+av1.length+" "+s2+".length="+av2.length);
    }
    checkDoubleArrayRel1Firstindex(av1.length,av1,s1,av2,s2,eps);
}

public static void checkDoubleArrayRel(final double [] av1,final String s1,
				final double [] av2,final String s2,
				final double eps)
{
    if(av1.length!=av2.length)
    {
	throw new RuntimeException("Mismatch: "+s1+".length="+av1.length+" "+s2+".length="+av2.length);
    }
    checkDoubleArrayRelFirstindex(av1.length,av1,s1,av2,s2,eps);
}

public static void checkDoubleArrayRelFirstindex(final int len,
					  final double [] av1,final String s1,
					  final double [] av2,final String s2,
					  final double eps)
{
    boolean f=false;
    final StringBuilder buf=new StringBuilder();
    for(int k=0;k<len;k++)
    {
	if(checkDoubleRelNoFail(av1[k],av2[k],eps))
	{
	    f=true;
	    buf.append("mismatch:ind="+k+"\t"+s1+"["+k+"]="+av1[k]+"\t"+s2+"["+k+"]="+av2[k]+"\n");
	}
    }
    if(f)
    {
	final StringBuilder s=new StringBuilder();
	s.append(" array mismatch, len="+len+" eps="+eps+"\n");
	s.append(buf.toString());
	s.append("\n");
	for(int i=0;i<len;i++)
	{
	    s.append(s1+"["+i+"]="+av1[i]+"\t"+s2+"["+i+"]="+av2[i]+"\n");
	}
	throw new RuntimeException("Mismatch: \n"+s.toString()+" eps="+eps);
    }
}


public static void checkDoubleArrayRel1Firstindex(final int len,
					  final double [] av1,final String s1,
					  final double [] av2,final String s2,
					  final double eps)
    {
	/*
	System.err.println("-----");
	for(int i=0;i<Math.max(av1.length,av2.length);i++)
	{
	    System.err.println("av1["+i+"]="+(i<av1.length?String.valueOf(av1[i]):"<>")+"\t"+
			       "av2["+i+"]="+(i<av2.length?String.valueOf(av2[i]):"<>"));
	}
	*/

	int ind=-1;
	for(int i=0;i<len;i++)
	{
	    final double v1=av1[i],v2=av2[i];
	    if(Double.isNaN(v1)!=Double.isNaN(v2))
	    {
		ind=i;
		break;
	    }
	    else if(v1==0||v2==0 ||Math.abs(v1)<1 )
	    {
		if(!Double.isNaN(v1) && Math.abs(v2-v1)>eps)
		{
		    ind=i;
		    break;
		}
	    }
	    else if(!Double.isNaN(v1) && Math.abs((v2-v1)/Math.max(v1,v2))>eps)
	    {
		ind=i;
		break;
	    }
	}
	if(ind>=0)
	{
	    final StringBuilder s=new StringBuilder();
	    s.append(" array mismatch, ind="+ind+" len="+len+"\n");
	    for(int i=0;i<len;i++)
	    {
		s.append(s1+"["+i+"]="+av1[i]+"\t"+s2+"["+i+"]="+av2[i]+"\n");
	    }
	    throw new RuntimeException("Mismatch: \n"+s.toString()+" eps="+eps);
	}
    }


public static void main(String [] args)
{
 System.err.println("UnitTests::main(): starting ALL tests; ver=$Id: UnitTests.java,v 1.19 2015/10/23 11:06:16 mal Exp $");
 final long t0=System.currentTimeMillis();
 System.err.println("Version");
 System.err.println("Started="+new java.util.Date(t0));
 System.getProperties().list(System.err);
 System.err.println("END Version");

 CircularList.main(args);
 EMoments.main(args);
 ExponentialInterpolation.main(args);
 FMoments.main(args);
 Laguerre.main(args);
 Chebyshev.main(args);
 Legendre.main(args);
 HermiteE.main(args);
 LegendreShifted.main(args);
 Monomials.main(args);
 Linsystems.main(args);
 LongHashMap.main(args);
 LongTreeValueMapTest.main(args);
 PolynomialRoot.main(args);
 PolynomialRootJenkinsTraubORIG.main(new String [0]);
 Polynomials.main(args);
 Progonka3.main(args);
 Progonka5.main(args);
 TMatr.main(args);
 SortArrays.main(args);
 TMPQSortAlgorithm.main(args);
 QUANC8.main(args);
 Vandermonde2Interpolation.main(args);
 VandermondeInterpolation.main(args);
 LegendreShiftedPolynomialRootsFinder.main(args);
 LaguerrePolynomialRootsFinder.main(args);
 OrthogonalPolynomialsLegendreShiftedBasis.main(args);
 OrthogonalPolynomialsLaguerreBasis.main(args);
 OrthogonalPolynomialsMonomialsBasis.main(args);
 OrthogonalPolynomialsChebyshevBasis.main(args);
 OrthogonalPolynomialsLegendreBasis.main(args);
 OrthogonalPolynomialsHermiteEBasis.main(args);
 RayleighQuotient.main(args);
 PolynomialRootsConfederateMatrixABasis.main(args);

 //WorkersPool.main(new String []{"5"}); // calls System.exit()

 final long t1=System.currentTimeMillis();
 System.err.println("Version");
 System.err.println("Finished="+new java.util.Date(t1)+" Started="+new java.util.Date(t0));
 System.getProperties().list(System.err);
 System.err.println("END Version");
 System.err.println("UnitTests::main(): ALL tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: UnitTests.java,v 1.19 2015/10/23 11:06:16 mal Exp $");
}


}
