package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A class that performs some work in a separate thread.
 *  @version $Id: Worker.java,v 1.21 2012/01/15 17:29:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class Worker extends Thread {


    private static final boolean PRINT_DEBUG=false;
    private final GenericPool pool;
    private boolean sleeping=false;
    private boolean startedok=false;

    Worker(final String n,final GenericPool pool)
    {
	super(n);
	this.pool=pool;
	//setDaemon(true);
	if(PRINT_DEBUG)
	{
	    System.err.println("Worker:: ["+new java.util.Date()+
			       "] new thread created. Thread="+getName());
	}
	synchronized(this)
	{
	    start();
	    // wait until we are notified by created thread.
	    try {
		if(PRINT_DEBUG)
		{
		    System.err.println("Worker:: ["+new java.util.Date()+
				       "] constructor is waiting for notification. Thread="+getName());
		}
		wait();
	    } catch (final InterruptedException ie) {
		if(PRINT_DEBUG)
		{
		    System.err.println("Worker:: ["+new java.util.Date()+
				       "] constructor InterruptedException notified. Thread="+getName());
		}
	    }
	    if(PRINT_DEBUG)
	    {
		System.err.println("Worker:: ["+new java.util.Date()+
				   "] constructor notified. Thread="+getName());
	    }
	    if(!sleeping || !startedok)
	    {
		throw new IllegalStateException("The worker Thread="+getName()+
						" is not sleeping while in constructor.");
	    }
	}
    }

    private WorkerTask currenttask=null;
    private int pool_element=-1;
    void performWork(final WorkerTask task,final int pool_element)
    {
	if(this.currenttask!=null)
	{
	    throw new IllegalStateException("An attempt to put to work a busy Worker. Thread="+
					    getName());
	}
	synchronized(this)
	{
	    this.currenttask=task;
	    this.pool_element=pool_element;
	    if(!sleeping || !isAlive())
	    {
		throw new IllegalStateException("The worker Thread="+getName()+
						" is not sleeping.");
	    }
	    notifyAll();
	}
    }

    public void run()
    {
	for(;;)
	{
	    try {
		synchronized(this)
		{
		    if(PRINT_DEBUG)
		    {
			System.err.println("Worker:: ["+new java.util.Date()+
					   "] went to sleep. Thread="+getName());
		    }
		    sleeping=true;
		    if(!startedok)
		    {
			startedok=true;
			notifyAll(); // notify constructor we are ready
		    }

		    final int pe=this.pool_element;
		    this.pool_element=-1;
		    if(pe>=0)
		    {
			final WorkerTask pecurrenttask=this.currenttask;
			this.currenttask=null;
			synchronized(this.pool)
			{ // make sure no initializeTask() is running concurrently
			    // cleanup
			    if(PRINT_DEBUG)
			    {
				System.err.println("Worker:: ["+new java.util.Date()+
						   "] doing cleanup. Thread="+getName());
			    }
			    pecurrenttask.cleanupTask(pe);
			    this.pool.releaseElement(pe);
			}
		    }
		    else
		    {
			this.currenttask=null;
		    }
		    try {
			// for loop to guard against spurious wake-up
			for(;this.currenttask==null;)
			{
			    wait();
			    if(this.currenttask==null)
			    {
				System.err.println("Worker:: ["+new java.util.Date()+
						   "] spurious wake-up. Thread="+getName());
			    }
			}
		    } catch (final InterruptedException ie) {
		    }
		    if(PRINT_DEBUG)
		    {
			System.err.println("Worker:: ["+new java.util.Date()+
					   "] waked up. Thread="+getName());
		    }
		    sleeping=false;
		}
		if(PRINT_DEBUG)
		{
		    System.err.println("Worker:: ["+new java.util.Date()+
				       "] job started, run asynchroneously. Thread="+getName());
		}
		this.currenttask.performTask(this.pool_element);
		if(PRINT_DEBUG)
		{
		    System.err.println("Worker:: ["+new java.util.Date()+
				       "] job ended. Thread="+getName());
		}
	    } catch (final Throwable e) {
		System.err.println("Worker:: ["+new java.util.Date()+
				   "] failed to process. Thread="+getName());
		e.printStackTrace(System.err);
	    }
	    if(PRINT_DEBUG)
	    {
		System.err.println("Worker:: ["+new java.util.Date()+
				   "] before loop end. Thread="+getName());
	    }
	}
    }

}
