package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A group of workers.
 *  @version $Id: WorkersPool.java,v 1.7 2011/10/08 18:29:01 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class WorkersPool {


    private final com.polytechnik.utils.GenericPool POOL;
    private final Worker [] workers;
    private final boolean flag_singlethreaded;
    public WorkersPool(final int n_workers,
		       final boolean flag_wait_for_available)
    {
	this.POOL=new com.polytechnik.utils.GenericPool(n_workers,flag_wait_for_available);
	this.workers=new Worker [n_workers];
	for(int i=n_workers;i-- >0;)
	{
	    this.workers[i]=new Worker("Worker "+i,this.POOL);
	}
	if(n_workers<=0)
	{
	    this.flag_singlethreaded=true;
	}
	else
	{
	    this.flag_singlethreaded=false;
	}
    }

    public boolean performWork(final WorkerTask task)
    {
	if(this.flag_singlethreaded)
	{
	   task.initializeTask(0);
	   task.performTask(0);
	   task.cleanupTask(0);
	}
	else
	{
	    final int pool_element;
	    synchronized(this.POOL)
	    { // make sure no cleanupTask() is running concurrently
	    pool_element=this.POOL.getAvailableElement();
	    if(pool_element<0)
	    {
		// no worker available, skip task
		// depending on flag_wait_for_available
		// in getAvailableElement(): we either return -1 or wait until a slot become available
		return false;
	    }
	    task.initializeTask(pool_element);
	    }
	    final Worker w=this.workers[pool_element];
	    w.performWork(task,pool_element);
	}
	return true;
    }



    public static void main(String [] args)
    {
	if(args.length!=1)
	{
	    System.err.println("main: n_workers");
	    System.exit(1);
	}
	final int n_tests=2000;
	System.err.println("WorkersPool: starting "+n_tests+" tests, ver=$Id: WorkersPool.java,v 1.7 2011/10/08 18:29:01 mal Exp $");
	final int n_workers=Integer.parseInt(args[0]);
	final WorkersPool w=new WorkersPool(n_workers,true);
	for(int i=0;i<n_tests;i++)
	{
	    final int j=i;
	    w.performWork(new WorkerTask(){
		    int status=0;
		    public void initializeTask(int el)
		    {
			if(status!=0) throw new RuntimeException("status="+status);
			status=1;
			// make a random delay to see how monitor works
			try {Thread.sleep(10L+(int)(1000*Math.random()));} catch(final InterruptedException ie) {}
			System.err.println("================TASK "+j+" INITIALIZED DONE EL="+el+
					   " Thread="+Thread.currentThread().getName());
		    }
		    public void cleanupTask(final int el)
		    {
			if(status!=2) throw new RuntimeException("status="+status);
			status=3;
			// make a random delay to see how monitor works
			try {Thread.sleep(10L+(int)(1000*Math.random()));} catch(final InterruptedException ie) {}
			System.err.println("================TASK "+j+" CLEANUP DONE EL="+el+
					   " Thread="+Thread.currentThread().getName());
		    }
		    public void performTask(int el)
		    {
			if(status!=1) throw new RuntimeException("status="+status);
			status=2;
			System.err.println("================TASK "+j+" STARTED EL="+el+
					   " Thread="+Thread.currentThread().getName());
			// make a random delay to see how monitor works
			try {Thread.sleep(2000L+(int)(10000*Math.random()));} catch(final InterruptedException ie) {}
			System.err.println("================TASK "+j+" ENDED EL="+el+
					   " Thread="+Thread.currentThread().getName());
		    }
		    protected void finalize() throws Throwable {
			if(status!=3) throw new RuntimeException("status="+status);
			super.finalize();
		    }
		});
	}
	try {Thread.sleep(10000L);} catch(final InterruptedException ie) {}
	System.err.println("WorkersPool: all tests ("+n_tests+") finished OK ver=$Id: WorkersPool.java,v 1.7 2011/10/08 18:29:01 mal Exp $");
	System.exit(0);
    }


}
